#ifdef GRANDSTREAM_NETWORKS
/*
 * Asterisk -- An open source telephony toolkit.
 *
 * Copyright (C) 2016-2017, yuancheng pang <ychpang@grandstream.cn>
 *
 * app_apitimeout.c adapted from the api user login timeout check
 *
 * See http://www.asterisk.org for more information about
 * the Asterisk project. Please do not directly contact
 * any of the maintainers of this project for assistance;
 * the project provides a web site, mailing lists and IRC
 * channels for your use.
 *
 * This program is free software, distributed under the terms of
 * the GNU General Public License Version 2. See the LICENSE file
 * at the top of the source tree.
 */

/*** MODULEINFO
	<depend>sqlite3</depend>
	<support_level>core</support_level>
 ***/

#include "asterisk.h"

#include <stdio.h>
#include <string.h>
#include <curl/curl.h>
#include <stdio.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <string.h>
#include <stdlib.h>
#include <unistd.h>
#include <stdbool.h>

#include "asterisk/pbx.h"
#include "asterisk/module.h"
#include "asterisk/app.h"
#include "asterisk/channel.h"
#include "asterisk/dsp.h"
#include <sys/un.h>

#include <time.h>
#include <sys/time.h>
#include "asterisk/astdb.h"


#define OUTTIME 600
static char *app = "CHECK_API_TIMEOUT";

AST_MODULE_LOG("app_checkapi");
/*** DOCUMENTATION
	<application name="CHECK_API_TIMEOUT" language="en_US">
		<synopsis>
			To check the api user login timeout
		</synopsis>
	</application>
 ***/

static int check_timeout_exe(struct ast_channel *chan, const char *data)
{
	char login_time[32];
	ast_ucm_db_get("API", "LOGINTIME", login_time, sizeof(login_time));
	ast_log(LOG_DEBUG, "login_time:%s", login_time);

	if(ast_strlen_zero(login_time)){
		/*if the login time is null, regard as no one is using cdrapi*/
		ast_channel_lock(chan);
		pbx_builtin_setvar_helper(chan, "CALL_ALLOW", "yes");
		ast_channel_unlock(chan);
		return 0;
	}

	struct timeval now;
	struct tm tm_login_time;
	time_t tmp_login_time = 0;

	gettimeofday(&now, NULL);
	memset(&tm_login_time, 0, sizeof(struct tm));
	strptime(login_time, "%Y-%m-%d %H:%M:%S", &tm_login_time);
	tmp_login_time = mktime(&tm_login_time);

	if (difftime(now.tv_sec,tmp_login_time)>600.0){
		ast_channel_lock(chan);
		pbx_builtin_setvar_helper(chan, "CALL_ALLOW", "yes");
		ast_channel_unlock(chan);
	}
	return 0;
}

static int unload_module(void)
{
	int res;
	res = ast_unregister_application(app);
	return res;
}

static int load_module(void)
{
	int res;
	res = ast_register_application_xml(app, check_timeout_exe);

	return res;
}

AST_MODULE_INFO(ASTERISK_GPL_KEY, AST_MODFLAG_LOAD_ORDER, "Check cdrapi login timeout",
	.support_level = AST_MODULE_SUPPORT_CORE,
	.load = load_module,
	.unload = unload_module,
);

#endif
