#ifdef GRANDSTREAM_NETWORKS


#include "asterisk.h"

#include <regex.h>

#include "asterisk/module.h"
#include "asterisk/channel.h"
#include "asterisk/pbx.h"
#include "asterisk/utils.h"
#include "asterisk/app.h"
#include "asterisk/astdb.h"
#include "asterisk/file.h"
#include "asterisk/chanvars.h"
#include "asterisk/devicestate.h"
#include "asterisk/dial.h"
#ifdef USE_DB_MYSQL
#include "asterisk/ucm_dbi.h"
#ifdef HAVE_UNIXODBC
#include "asterisk/core_odbc.h"
#endif
#include "asterisk/features.h"
#endif




AST_MODULE_LOG("callinggroup");
static const char * const app_callinggroup= "CallingGroup";
AST_RWLOCK_DEFINE_STATIC(callinggroup_mutex);

#define CALLGROUP_MAX_MEMBERS 256
#define MAX_CALL_GROUP_NAME 256
#define UCM_CONFIG_DB     "ucm_config"
#define AST_SAFE_FREE(ptr) if (ptr) {\
    ast_free(ptr);\
    ptr = NULL; }


int is_local_number(const char *number);


/*************************************************/

struct generic_prepare_struct {
	char *sql;
	int argc;
	char **argv;
};

static SQLHSTMT generic_prepare(struct odbc_obj *obj, void *data)
{
	struct generic_prepare_struct *gps = data;
	int res, i;
	SQLHSTMT stmt;

	res = SQLAllocHandle(SQL_HANDLE_STMT, obj->con, &stmt);
	if (!SQL_SUCCEEDED(res)) {
		ast_log(AST_LOG_WARNING, "SQL Alloc Handle failed!\n");
		return NULL;
	}
	res = SQLPrepare(stmt, (unsigned char *) gps->sql, SQL_NTS);
	if (!SQL_SUCCEEDED(res)) {
		ast_log(AST_LOG_WARNING, "SQL Prepare failed![%s]\n", gps->sql);
		SQLFreeHandle(SQL_HANDLE_STMT, stmt);
		return NULL;
	}
	for (i = 0; i < gps->argc; i++)
		SQLBindParameter(stmt, i + 1, SQL_PARAM_INPUT, SQL_C_CHAR, SQL_CHAR, strlen(gps->argv[i]), 0, gps->argv[i], 0, NULL);

	return stmt;
}

/*nodes of members number*/
struct numbers
{
    char members[CALLGROUP_MAX_MEMBERS];  //CALLGROUP_MAX_MEMBERS
    struct numbers *next;
};

/*list of head numbers*/
struct headnum_list
{
    char headnum[AST_MAX_EXTENSION];                /*this can be defined as ivr-id or paging group number or so*/
    char headname[MAX_CALL_GROUP_NAME];             /*MAX_CALL_GROUP_NAME,the name of callinggroup,not very useful*/
    char enable[255];
    struct numbers *extension;
    struct headnum_list *next;
};

/*a global param used for data transfer*/
struct headnum_list *g_cg_list = NULL;  //callinggroup list
struct headnum_list *g_al_list = NULL;  //allownumber list

/*init caller id numbers list, insert first number*/
static struct numbers *malloc_numbers(const char *members)
{
    if (members == NULL) {
        return NULL;
    }
    struct numbers *ext = NULL;
    ext = ast_malloc(sizeof(struct numbers));
    if (ext != NULL) {
        strncpy(ext->members, members, sizeof(ext->members));
        ext->next = NULL;
    }
    return ext;
}

/*create head number list*/
static struct numbers *create_numbers_list(struct headnum_list *list, const char *extension)
{
    if (list == NULL || extension== NULL) {
        return NULL;
    }

    struct numbers *node_ext = malloc_numbers(extension);
    struct numbers *tmp_ext = list->extension;
    if (list->extension == NULL) {
        list->extension = node_ext;
    }
    else {
        while (tmp_ext->next) {
            tmp_ext = tmp_ext->next;
        }
        tmp_ext->next = node_ext;
    }

    return node_ext;
}

/*init ivr list*/
static struct headnum_list* malloc_list(const char *head_num)
{
    if (head_num == NULL) {
        return NULL;
    }

    struct headnum_list *list = NULL;
    list = ast_malloc(sizeof(struct headnum_list));
    if (list) {
        strncpy(list->headnum, head_num, sizeof(list->headnum));
        list->extension = NULL;
        list->next = NULL;
    }

    return list;
}

/*create head number list*/
static struct headnum_list *create_list(struct headnum_list **list, const char *headnum)
{
    if (list == NULL || headnum == NULL) {
        return NULL;
    }

    struct headnum_list *tmp_list = *list, *tmp = NULL;
    tmp = malloc_list(headnum);
    if (tmp_list == NULL) {
        *list = tmp;
    }
    else {
        while( tmp_list->next ) {
            tmp_list = tmp_list->next;
        }
        tmp_list->next = tmp;
    }

    return tmp;
}

/*to see wether global saved right*/
static void print_global_list(void)
{
    struct headnum_list *tmp_list = g_cg_list;
    struct numbers *tmp_ext = NULL;
    ast_log(LOG_DEBUG, "start to show callinggroup members");
    while (tmp_list) {                                  //print member conf
        struct headnum_list *tmp_a_list = g_al_list;
        struct numbers *tmp_ald = NULL;
        ast_log(LOG_DEBUG, "[%s]", tmp_list->headnum);
        ast_log(LOG_DEBUG, "group = %s", tmp_list->headname);
        ast_log(LOG_DEBUG, "enable = %s", tmp_list->enable);//11.16
        if (0 == strcasecmp(tmp_list->enable,"no")){
            tmp_list = tmp_list->next;
            continue;
        }
        if(NULL != tmp_list->extension ){
            tmp_ext = tmp_list->extension;
            while (tmp_ext) {
                ast_log(LOG_DEBUG,"members = %s", tmp_ext->members);
                tmp_ext = tmp_ext->next;
            }
        }
        if (NULL != tmp_a_list->extension){            //print allowed number
            while(tmp_a_list) {
                if (strncmp(tmp_a_list->headnum,tmp_list->headnum,sizeof(tmp_list->headnum)) == 0){
                    tmp_ald = tmp_a_list->extension;
                    while (tmp_ald) {
                        ast_log(LOG_DEBUG,"allowed = %s", tmp_ald->members);
                        tmp_ald = tmp_ald->next;
                    }
                    break;
                }
                tmp_a_list = tmp_a_list->next;
            }
        }
    /*  if (NULL != tmp_list->next) {
            ast_log(LOG_DEBUG,"There is next group!");
        }else{
            ast_log(LOG_DEBUG,"There is no more group!");
        }*/
        tmp_list = tmp_list->next;
    }
/*  while (tmp_a_list) {                              //another mode to print allowed conf
        ast_debug(1, "[%s]", tmp_a_list->headnum);
        if(NULL != tmp_a_list->extension ){
            tmp_ald = tmp_a_list->extension;
            while (tmp_ald) {
                ast_log(LOG_DEBUG,"allowed = %s", tmp_ald->members);
                tmp_ald = tmp_ald->next;
            }
        }
        tmp_a_list = tmp_a_list->next;
    }*/

    ast_log(LOG_DEBUG, "end to show callinggroup members");
}

static void unload_list(void)
{
    struct headnum_list *tmp_list = g_cg_list, *prev_list = NULL;
    struct numbers *tmp_ext = NULL, *prev_ext = NULL;
    while (tmp_list) {
        if(tmp_list->headnum != NULL){
            tmp_ext = tmp_list->extension;
        }

        while (tmp_ext) {
            prev_ext = tmp_ext;
            tmp_ext = tmp_ext->next;
            ast_free(prev_ext);
        }
        prev_list = tmp_list;
        tmp_list = tmp_list->next;
        ast_free(prev_list);
    }
    g_cg_list = NULL;

    tmp_list = g_al_list;
    while (tmp_list) {
        if(tmp_list->headnum != NULL){
            tmp_ext = tmp_list->extension;
        }

        while (tmp_ext) {
            prev_ext = tmp_ext;
            tmp_ext = tmp_ext->next;
            ast_free(prev_ext);
        }
        prev_list = tmp_list;
        tmp_list = tmp_list->next;
        ast_free(prev_list);
    }
    g_al_list = NULL;

}

static char *conf_runsql_get_varchar_field(char *sql, const char *key)
{
	char *result = NULL;
	char tmp[512] = {0};
	SQLHSTMT stmt;
	struct generic_prepare_struct gps;
	struct odbc_obj *conf_obj = NULL;
	if (ast_strlen_zero(sql) || ast_strlen_zero(key)) {
		return NULL;
	}
	conf_obj = ast_core_odbc_request_obj(UCM_CONFIG_DB, 0);
	if (!conf_obj) {
		ast_log(AST_LOG_WARNING, "Failed to obtain database object for '%s'!\n", UCM_CONFIG_DB);
		return NULL;
	}

	gps.sql = sql;
	gps.argc = 0;
	gps.argv = NULL;

	stmt = ast_core_odbc_prepare_and_execute(conf_obj, generic_prepare, &gps);
	if (!stmt) {
		ast_log(LOG_ERROR, "SQL Execute error!\n[%s]\n\n", sql);
		goto end_clean;
	} else {

		int res = SQLFetch(stmt);
		if (!SQL_SUCCEEDED(res)) {
			ast_log(LOG_WARNING, "SQL Fetch error!\n[%s]\n\n", sql);
			SQLFreeHandle(SQL_HANDLE_STMT, stmt);
			goto end_clean;
		}
		res = SQLGetData(stmt, 1, SQL_CHAR, tmp, sizeof(tmp), NULL);
		if (SQL_SUCCEEDED(res)) {
			result = ast_strdup(tmp);
		}
		SQLFreeHandle(SQL_HANDLE_STMT, stmt);
	}
	ast_log(LOG_DEBUG, "SQL Execute success!\n[%s]\n\n", sql);

end_clean:
	ast_core_odbc_release_obj(conf_obj);
	conf_obj = NULL;
	return result;
}

int is_local_number(const char *number)
{
    char cmd[512] = {0};
    char* extension = NULL;
    int res = 0;

    snprintf(cmd,sizeof(cmd)-1,"SELECT extension FROM numbers WHERE extension = '%s'", number);
    extension = conf_runsql_get_varchar_field(cmd,"extension");
    if (extension != NULL && extension[0] != '\0')
    {
        res = 1;
    }
    else
    {
        res = 0;
    }
    AST_SAFE_FREE(extension);

    return res;
}

/*
static char *get_members(struct numbers *ext)
{
    if (ext == NULL) {
        ast_log(LOG_DEBUG,"ext error");
        return NULL;
    }
    struct numbers *tmp_ext = ext;
    while (tmp_ext != NULL) {
        if (NULL != tmp_ext->members) {
            ast_log(LOG_DEBUG, "here find members: %s", tmp_ext->members);
            break;
        }
        tmp_ext = tmp_ext->next;
    }

    return tmp_ext ? tmp_ext->members : NULL;
}*/


/*find list by head number
static char *find_members_by_headnum(struct headnum_list *list, const char *headnum)
{
    if (list == NULL || headnum == NULL) {
        return NULL;
    }
    struct headnum_list *tmp_list = list;
    while (tmp_list != NULL) {
        if (!strncmp(tmp_list->headnum, headnum, sizeof(tmp_list->headnum))) {
            ast_log(LOG_DEBUG, "here find callinggroup: %s", tmp_list->headnum);
            break;
        }
        tmp_list = tmp_list->next;
    }

    return tmp_list ? get_members(tmp_list->extension) : NULL;
}
*/

/*************************************************/


static int callinggroup_exec(struct ast_channel *chan, const char *data)
{
    const char *tmp_caller = ast_channel_caller_num(chan);
    ast_log(LOG_DEBUG, "callinggroup_exec Enter.............!\n");
    ast_log(LOG_DEBUG, "caller = %s",tmp_caller);
    if (tmp_caller == NULL || tmp_caller[0] == '\0') {
        /*BEGIN: Mod by weidu, for memory use after free*/
        ast_channel_lock(chan);
        tmp_caller = ast_strdupa(S_COR(ast_channel_caller(chan)->id.number.valid, ast_channel_caller(chan)->id.number.str, ""));
        ast_channel_unlock(chan);
        /*END: Mod by weidu*/
        ast_log(LOG_DEBUG, "channel caller = %s",tmp_caller);
    }

    const char *tmp_callee = pbx_builtin_getvar_helper(chan,"CALLEEEXTEN");
    ast_log(LOG_DEBUG, "callee = %s",tmp_callee);

    //bug 201297,UCM should not allow to seamless transfer when set A dial B and A operation *44
    const char *seamless = pbx_builtin_getvar_helper(chan,"SEAMLESS_TRANSFER_NUMBER");
    int is_local = 1;
    if (NULL != seamless) {
        ast_log(LOG_DEBUG, "seamless trans to = %s",seamless);
        tmp_callee = seamless;
        is_local = is_local_number(seamless);
        if (is_local == 0)
        {
            ast_log(LOG_DEBUG, "Callee maybe not local number!");
            pbx_builtin_setvar_helper(chan, "CALLPERMISSION", "allow");
            return 0;
        }
    }

    const char *cf_user = pbx_builtin_getvar_helper(chan,"EXTTOCALL");
    const char *phone_user = pbx_builtin_getvar_helper(chan,"ORIG_CALLEE");
    const char *dial_by_name = pbx_builtin_getvar_helper(chan,"FROM_DIRECTORY");
    const char *speed_dial = pbx_builtin_getvar_helper(chan,"TRANSPARENTSDP");
    const char *from_ivr = pbx_builtin_getvar_helper(chan,"FROM_IVR");
    const char *from_spd_custom = pbx_builtin_getvar_helper(chan,"FROM_SPD_CUSTOM");
    if ((NULL != seamless && '\0' != seamless[0])||(NULL != from_ivr && 0 == strcmp(from_ivr, "operator")))
    {
        //caller should not change
    }
    else if (NULL != cf_user && NULL == seamless) {
        ast_log(LOG_DEBUG, "call trans from = %s",cf_user);
        tmp_caller = cf_user;
    }
    else if((NULL != phone_user) && (0 != strcmp(tmp_caller,phone_user)) && (0 != strcmp(tmp_callee,phone_user)) && (dial_by_name == NULL || 0 != strcmp(dial_by_name,"yes")) && (speed_dial == NULL || 0 != strcmp(speed_dial,"yes")) && (from_spd_custom == NULL || 0 != strcmp(from_spd_custom,"yes"))){
        ast_log(LOG_DEBUG, "phone trans from = %s",phone_user);
        pbx_builtin_setvar_helper(chan, "PHONETRANSFLAG", "1");
        const char *vm_main = pbx_builtin_getvar_helper(chan,"IS_VMMAIN");
        if (vm_main != NULL && strcmp(vm_main, "yes") == 0)
        {
            ast_log(LOG_DEBUG, "But is VM main.");
        }
        else
        {
            tmp_caller = phone_user;
        }
    }

    const char *trans_exten = pbx_builtin_getvar_helper(chan,"TRANSEXT");
    if (NULL != trans_exten) {
        ast_log(LOG_DEBUG, "trans from = %s",trans_exten);
        tmp_caller = trans_exten;
    }

    if ((NULL == tmp_caller) || (NULL == tmp_callee)) {
        ast_log(LOG_DEBUG, "caller or callee is null");
        return 0;
    }

    /*print_global_list();*/

    if (ast_strlen_zero(data)) {
        ast_log(LOG_WARNING, "This application requires at least one argument (destination(s) to call)\n");
        return -1;
    }
    if (strcasecmp(tmp_callee,"h") == 0) {
        return 0;
    }
    if (strncmp(tmp_caller,tmp_callee,strlen(tmp_caller)) == 0) {
        return 0;
    }

    pbx_builtin_setvar_helper(chan, "CALLPERMISSION", "allow");


    ast_rwlock_rdlock(&callinggroup_mutex);
    struct headnum_list *mem_list = g_cg_list;
    struct headnum_list *ald_list = g_al_list;

    int caller_in_callinggroup_flag = 0;

    struct numbers *mem_ext = NULL;
    struct numbers *ald_ext = NULL;
    if(mem_list == NULL)
        ast_log(LOG_DEBUG, "mem_list == NULL");
    while (NULL != mem_list) {                                                                              //travel all group
        if ((NULL == mem_list->enable) || (0 == strcasecmp(mem_list->enable,"no"))
            || (NULL == mem_list->extension) || (NULL == mem_list->extension->members)) {
            mem_list = mem_list->next;
            //ast_log(LOG_DEBUG, "group %s is empty or not enable.",mem_list->headnum);
            continue;
        }
        mem_ext = mem_list->extension;
        while (mem_ext->members != NULL) {
            if (strncmp(tmp_caller,mem_ext->members,strlen(tmp_caller)) == 0) {                                   //find caller in group
                caller_in_callinggroup_flag = 1;
                //ast_log(LOG_DEBUG, "find member = %s in",mem_ext->members);
                //ast_log(LOG_DEBUG, "group = %s",mem_list->headnum);
                while (NULL != ald_list) {
                    if ((ald_list->headnum != NULL)
                    && (strncmp(ald_list->headnum,mem_list->headnum, strlen(mem_list->headnum)) == 0)) {    //find allowed_list
                        if ((NULL == ald_list->extension) || (NULL == ald_list->extension->members)) {      //if is a empty list
                            break;
                        }
                        ald_ext = ald_list->extension;
                        while (ald_ext->members != NULL) {
                            if (strncmp(tmp_callee,ald_ext->members,strlen(tmp_callee)) == 0) {
                                //ast_log(LOG_DEBUG, "In allowed group = %s",ald_list->headnum);
                                //ast_log(LOG_DEBUG, "allowed = %s",ald_ext->members);
                                ast_rwlock_unlock(&callinggroup_mutex);
                                return 0;
                            }
                            ald_ext = ald_ext->next;
                        }
                        break;
                    }
                    ald_list = ald_list->next;
                }//callee not in allowed_list
                //find caller in group but callee not in allowed_list
            }
            mem_ext = mem_ext->next;
        }//caller not in this group
        mem_list = mem_list->next;
    }
    ast_rwlock_unlock(&callinggroup_mutex);

    if (1 == caller_in_callinggroup_flag) {
        ast_log(LOG_DEBUG, "This call rejected by Restrict Calls!");
        pbx_builtin_setvar_helper(chan, "CALLPERMISSION", "reject");
    }

    return 0;
}

static int unload_module(void)
{
    unload_list();

    return ast_unregister_application(app_callinggroup);
}

static int load_config(int reload)
{
    int res = 0;
    char *tmp = NULL;
    char *tmp_a = NULL;
    struct ast_config *cfg = NULL;
    struct ast_variable *var = NULL;
    struct headnum_list *tmp_list = NULL;
    struct ast_flags config_flags = { reload ? CONFIG_FLAG_FILEUNCHANGED : 0 };

    if (!(cfg = ast_config_load("callinggroup_members.conf", config_flags))) {//load member conf,return ast_config
        ast_log(LOG_NOTICE, "No config file callinggroup_members.conf\n");
        return 0;
    } else if (cfg == CONFIG_STATUS_FILEUNCHANGED) {
        ast_log(LOG_DEBUG, "Config file callinggroup_members.conf is unchanged.\n");
        return 0;
    } else if (cfg == CONFIG_STATUS_FILEINVALID) {
        ast_log(LOG_ERROR, "Config file callinggroup_members.conf is in an invalid format.  Aborting.\n");
        return 1;
    }
    unload_list();

    while ((tmp = ast_category_browse(cfg, tmp)) ) {
        tmp_list = create_list(&g_cg_list, tmp);
        for (var = ast_variable_browse(cfg, tmp); var; var = var->next) {
            if (!strcasecmp(var->name, "name")) {   /*add group name to struct*/
                strncpy(tmp_list->headname,var->value,sizeof(tmp_list->headname));
            }
            if (!strcasecmp(var->name, "enable")) {   /*this group isn't out of service?*/
                //ast_log(LOG_DEBUG, "zff:ENABLE: %s", var->value);
                strncpy(tmp_list->enable,var->value,sizeof(tmp_list->enable));
            }
            if (!strcasecmp(var->name, "member")) {
                //ast_log(LOG_DEBUG, "CALLING GROUP: %s | MEMBER NUMBERS : %s", tmp, var->value);
                create_numbers_list(tmp_list, var->value);
            }
        }
    }

    tmp_list = NULL;
    while ((tmp_a = ast_category_browse(cfg, tmp_a)) ) {
        tmp_list = create_list(&g_al_list, tmp_a);
        for (var = ast_variable_browse(cfg, tmp_a); var; var = var->next) {
            if (!strcasecmp(var->name, "allowed")) {
                //ast_log(LOG_DEBUG, "CALLING GROUP: %s | ALLOWED NUMBERS : %s", tmp_a, var->value);
                create_numbers_list(tmp_list, var->value);
            }
        }
    }
    print_global_list();

    ast_config_destroy(cfg);

    return res;
}


static int load_module(void)
{

    if (load_config(0)) {
        return AST_MODULE_LOAD_DECLINE;
    }

    return ast_register_application_xml(app_callinggroup, callinggroup_exec);
}

static int reload(void)
{
    int res = 0;

    ast_log(LOG_DEBUG, "reload module callinggroup\n");
    ast_rwlock_wrlock(&callinggroup_mutex);
    res = load_config(1);
    ast_rwlock_unlock(&callinggroup_mutex);

    return res;
}

AST_MODULE_INFO(ASTERISK_GPL_KEY, AST_MODFLAG_DEFAULT, "Callinggroup Application",
    .support_level = AST_MODULE_SUPPORT_CORE,
    .load = load_module,
    .unload = unload_module,
    .reload = reload,
);


#endif
