/*** MODULEINFO
	<depend>ucmsecret</depend>
	<defaultenabled>yes</defaultenabled>
	<support_level>core</support_level>
 ***/
#ifdef GRANDSTREAM_NETWORKS

#include "asterisk.h"

#include <regex.h>

#include "asterisk/module.h"
#include "asterisk/channel.h"
#include "asterisk/pbx.h"
#include "asterisk/utils.h"
#include "asterisk/app.h"
#include "asterisk/astdb.h"
#include "asterisk/file.h"
#include "asterisk/chanvars.h"
#include "asterisk/devicestate.h"
#include "asterisk/dial.h"
#ifdef USE_DB_MYSQL
#include "asterisk/ucm_dbi.h"
#ifdef HAVE_UNIXODBC
#include "asterisk/core_odbc.h"
#endif
#include "asterisk/features.h"
#endif
#ifdef HAVE_UCMSECRET
#include "ucm_crypt.h"
#endif
AST_MODULE_LOG("getcidname");
static const char * const app_getcidname= "GetCidName";

struct generic_prepare_struct {
	char *sql;
	int argc;
	char **argv;
};

static SQLHSTMT generic_prepare(struct odbc_obj *obj, void *data)
{
	struct generic_prepare_struct *gps = data;
	int res, i;
	SQLHSTMT stmt;

	res = SQLAllocHandle(SQL_HANDLE_STMT, obj->con, &stmt);
	if (!SQL_SUCCEEDED(res)) {
		ast_log(AST_LOG_WARNING, "SQL Alloc Handle failed!\n");
		return NULL;
	}
	res = SQLPrepare(stmt, (unsigned char *) gps->sql, SQL_NTS);
	if (!SQL_SUCCEEDED(res)) {
		ast_log(AST_LOG_WARNING, "SQL Prepare failed![%s]\n", gps->sql);
		SQLFreeHandle(SQL_HANDLE_STMT, stmt);
		return NULL;
	}
	for (i = 0; i < gps->argc; i++)
		SQLBindParameter(stmt, i + 1, SQL_PARAM_INPUT, SQL_C_CHAR, SQL_CHAR, strlen(gps->argv[i]), 0, gps->argv[i], 0, NULL);

	return stmt;
}

/* need free */
static char* ucm_config_get_one_varchar(char *cmd)
{
	char *result = NULL;
	struct odbc_obj *getcidname_obj = NULL;

	if (ast_strlen_zero(cmd)) {
		return NULL;
	}

	getcidname_obj = ast_core_odbc_request_obj("ucm_config", 0);
	if (!getcidname_obj) {
		ast_log(AST_LOG_WARNING, "Failed to obtain database object for 'ucm_config'!\n");
		return NULL;
	}

	SQLHSTMT stmt;
	struct generic_prepare_struct gps;

	gps.sql = cmd;
	gps.argc = 0;
	gps.argv = NULL;

	stmt = ast_core_odbc_prepare_and_execute(getcidname_obj, generic_prepare, &gps);
	if (!stmt) {
		ast_log(LOG_ERROR, "SQL Execute error!\n[%s]\n\n", cmd);
		goto end_clean;
	} else {
		char tmp[256];
		memset(tmp, 0, sizeof(tmp));
		int res = SQLFetch(stmt);
		if (!SQL_SUCCEEDED(res)) {
			ast_log(LOG_ERROR, "SQL Fetch error!\n[%s]\n\n", cmd);
			SQLFreeHandle(SQL_HANDLE_STMT, stmt);
			goto end_clean;
		}
		res = SQLGetData(stmt, 1, SQL_CHAR, tmp, sizeof(tmp), NULL);
		if (SQL_SUCCEEDED(res)) {
			result = ast_strdup(tmp);
		}
		SQLFreeHandle(SQL_HANDLE_STMT, stmt);
	}
	ast_log(LOG_DEBUG, "SQL Execute success!\n[%s]\n\n", cmd);

end_clean:
	ast_core_odbc_release_obj(getcidname_obj);
	getcidname_obj = NULL;
	return result;
}

static int getcidname_exec(struct ast_channel *chan, const char *data)
{
	ast_log(LOG_DEBUG, "getcidname_exec Enter.............!\n");
	char cmd[1024] = { 0 };
	char *caller_id_name = NULL;
	const char *org_caller_id_num = NULL;
	char *file_key = get_encrypted_key();

	if (strstr(ast_channel_name(chan), "DAHDI/"))
	{
		org_caller_id_num = ast_channel_caller(chan)->id.number.str;
		ast_log(LOG_DEBUG, "DAHDI org_caller_id_num = %s", org_caller_id_num);
	}
	else
	{
		org_caller_id_num = ast_channel_caller_num(chan);
		ast_log(LOG_DEBUG, "PJSIP org_caller_id_num = %s", org_caller_id_num);
	}

	snprintf(cmd, sizeof(cmd), "SELECT CAST(AES_DECRYPT(UNHEX(substring(calleridname,7)), '%s') AS CHAR CHARSET utf8) AS calleridname FROM phonebooks WHERE AES_DECRYPT(UNHEX(substring(accountnumber,7)), '%s') = %s", file_key, file_key, org_caller_id_num);
	caller_id_name = ucm_config_get_one_varchar(cmd);
	if (caller_id_name && strlen(caller_id_name) != 0)
	{
		ast_log(LOG_DEBUG, "caller_id_name = %s", caller_id_name);
		pbx_builtin_setvar_helper(chan, "CALLERID(name)", caller_id_name);
		/* The flag is currently replaced, need not replace agin */
		pbx_builtin_setvar_helper(chan, "IS_REPLACE", "yes");
		/* This flag indicates that remote database queries are not required -by yjzhao */
		pbx_builtin_setvar_helper(chan, "REMOTE_QUERY", "no");
	}
	if (caller_id_name)
	{
		ast_free(caller_id_name);
		caller_id_name = NULL;
	}
	if (file_key)
	{
		ast_free(file_key);
		file_key = NULL;
	}

	return 0;
}

static int unload_module(void)
{
	return ast_unregister_application(app_getcidname);
}

static int load_module(void)
{
	return ast_register_application_xml(app_getcidname, getcidname_exec);
}

AST_MODULE_INFO(ASTERISK_GPL_KEY, AST_MODFLAG_DEFAULT, "GetCidName Application",
	.support_level = AST_MODULE_SUPPORT_CORE,
	.load = load_module,
	.unload = unload_module,
);


#endif
