#ifdef GRANDSTREAM_NETWORKS
/*
 * Asterisk -- An open source telephony toolkit.
 *
 * Copyright (C) 2016-2017, kmye <kmye@grandstream.cn>
 *
 * app_reseteventlist.c adapted from the eventlist state reset
 *
 * See http://www.asterisk.org for more information about
 * the Asterisk project. Please do not directly contact
 * any of the maintainers of this project for assistance;
 * the project provides a web site, mailing lists and IRC
 * channels for your use.
 *
 * This program is free software, distributed under the terms of
 * the GNU General Public License Version 2. See the LICENSE file
 * at the top of the source tree.
 */

/*** MODULEINFO
	<depend>sqlite3</depend>
	<support_level>core</support_level>
 ***/

/*** DOCUMENTATION
	<application name="ResetInboundMode" language="en_US">
		<synopsis>
			reset BLF state for inbound mode.
		</synopsis>
		<syntax>
			<parameter name="options">
				<optionlist>
					<option name="r">
						<para> reset subscription of inbound mode </para>
					</option>
				</optionlist>
			</parameter>
		</syntax>
	</application>

 ***/



#include "asterisk.h"

#include <stdio.h>
#include <string.h>
#include <stdio.h>
#include <sys/types.h>
#include <sqlite3.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <stdlib.h>
#include <unistd.h>

#include "asterisk/pbx.h"
#include "asterisk/module.h"
#include "asterisk/app.h"
#include "asterisk/channel.h"

static char *app = "ResetInboundMode";

enum {
	OPTION_SUBSCRIBE = (1 << 0),
	OPTION_UNSUBSCRIBE = (1 << 1),
	OPTION_RESET = (1 << 2),
};

AST_APP_OPTIONS(app_opts,{
	AST_APP_OPTION('r', OPTION_RESET),
});

static int reset_eventlist(struct ast_channel *chan, const char *data)
{
	if (NULL == chan || NULL == data) {
		return -1;
	}
	char *parse;
	struct ast_flags flags = { 0, };
	char *opts[0];
	const char* options = NULL;
	RAII_VAR(struct stasis_message *, message, NULL, ao2_cleanup);
	RAII_VAR(struct ast_json_payload *, payload, NULL, ao2_cleanup);
	RAII_VAR(struct ast_json *, to_send_json_object, NULL, ast_json_unref);
	struct stasis_topic *reset_eventlist_topic;
	AST_DECLARE_APP_ARGS(args,
		AST_APP_ARG(exten);
		AST_APP_ARG(options);
	);
	if (ast_strlen_zero(data)) {
		ast_log(LOG_WARNING, "Reset Eventlist requires an argument\n");
		return -1;
	}
	parse = ast_strdupa(data);
	AST_STANDARD_APP_ARGS(args, parse);
	if (args.argc == 2)
		ast_app_parse_options(app_opts, &flags, opts, args.options);

	if (ast_test_flag(&flags, OPTION_RESET)) {
		options = "reset_inbound_mode";
	} else {
		options = "UNKNOWN";
	}
	to_send_json_object = ast_json_pack("{s: s, s: s}",
		"extension", args.exten,
		"option", options
	);
	if (NULL == to_send_json_object)
		return -1;
	if (NULL == (payload = ast_json_payload_create(to_send_json_object)))
		return -1;
	if (NULL == (message = stasis_message_create(ast_reset_eventlist_type(), payload)))
		return -1;
	reset_eventlist_topic = ast_reset_eventlist_topic_all();
	if (!reset_eventlist_topic) {
		return -1;
	}
	stasis_publish(reset_eventlist_topic, message);
	return 0;
}
static int unload_module(void)
{
	int res;
	res = ast_unregister_application(app);
	return res;
}

static int load_module(void)
{
	int res;
	res = ast_register_application_xml(app, reset_eventlist);

	return res;
}

AST_MODULE_INFO_STANDARD(ASTERISK_GPL_KEY, "Trivial Reset Inbound Mode Application");

#endif

