#ifdef GRANDSTREAM_NETWORKS
/*
 * Asterisk -- An open source telephony toolkit.
 *
 * Copyright (C) 2019, Grandstream Networks.
 *
 *
 * See http://www.asterisk.org for more information about
 * the Asterisk project. Please do not directly contact
 * any of the maintainers of this project for assistance;
 * the project provides a web site, mailing lists and IRC
 * channels for your use.
 *
 * This program is free software, distributed under the terms of
 * the GNU General Public License Version 2. See the LICENSE file
 * at the top of the source tree.
 *
 * Please follow coding guidelines
 * http://svn.digium.com/view/asterisk/trunk/doc/CODING-GUIDELINES
 */

/*! \file
 *
 * \brief Confbridge resource manager
 *
 * \author\Grandstream Networks
 *
 * This file contains functions that are used for managing the media
 * resource in sfu mode
 *
 * \ingroup applications
 */

/*** MODULEINFO
	<support_level>core</support_level>
 ***/

#ifndef _CONF_RESOURCE_H_
#define _CONF_RESOURCE_H_

enum resource_type {
	RESOURCE_TYPE_AUDIO,
	RESOURCE_TYPE_VIDEO,  /*main video*/
	RESOURCE_TYPE_SLIDES, /*slides video*/
	RESOURCE_TYPE_END
};

enum video_resolution {
	VIDEO_RESOLUTION_NONE,
	VIDEO_RESOLUTION_360P,
	VIDEO_RESOLUTION_720P,
	VIDEO_RESOLUTION_1080P,
	VIDEO_RESOLUTION_MAX
};

enum conf_control_frame_type {
	CONF_CONTROL_OPEN_CAMERA,
	CONF_CONTROL_CLOSE_CAMERA
};

typedef struct _conf_net_info
{
    int haon;
    int harole; /* 1- master, 0 - slaver */
    int mode;
    char wan[16];
    char lan[16];
    char lan1[16];
    char lan2[16];
    char default_if[16];
    char ip_hafloat[64];
}conf_net_info;

extern int ast_conf_get_audio_resource(const char *conf_number, struct ast_channel *chan);
extern int ast_conf_put_audio_resource(const char *conf_number, struct ast_channel *chan);
extern int ast_conf_get_main_video_resource(const char *conf_number, struct ast_channel *chan, int resolution);
extern int ast_conf_put_main_video_resource(const char *conf_number, struct ast_channel *chan);
extern int ast_conf_get_slides_video_resource(const char *conf_number, struct ast_channel *chan, int share_audio);
extern int ast_conf_put_slides_video_resource(const char *conf_number, struct ast_channel *chan);
/*Free the object after use*/
extern char *ast_conf_get_slider(const char *conf_number);
/*Free the object after use by ast_channel_unref*/
extern struct ast_channel *ast_conf_get_slider_channel(const char *conf_number);
extern struct ast_channel *ast_conf_get_sfu_moderator(struct ast_channel *chan, const char *conf_number);
extern struct ast_channel *ast_conf_get_sfu_focus(struct ast_channel *chan, const char *conf_number);
extern int ast_conf_set_slides_type(const char *conf_number, int type);
extern int ast_conf_get_slides_type(const char *conf_number);
extern int ast_conf_set_slider_name(const char *conf_number, char *slider_name);
extern char *ast_conf_get_slider_name(const char *conf_number);
extern int ast_conf_switch_main_video_resolution(struct ast_channel *chan, const char *conf_number, int new_resolution);
extern void ast_conf_indicate(struct ast_channel *chan, const char *conf_number, int condition, void *data);
extern int ast_conf_set_video_codec(struct ast_channel *chan, const char *conf_number, const char *codec);
/*Free the object after use*/
extern char *ast_conf_get_video_codec(struct ast_channel *chan, const char *conf_number);
extern int ast_conf_mic_mute_unmute_helper(int mute, const char *conference_name, const char *chan_name);
extern int ast_conf_user_hand_up_helper(int hand_up, const char *conference_name, const char *chan_name, struct ast_channel *op_chan);
/*Free the object after use*/
extern char *ast_conf_get_hd_chan_name(const char *conf_number);
extern enum video_resolution ast_conf_get_hd_resolution(const char *conf_number);
extern void ast_conf_cei_video_resolution_update(struct ast_channel *chan, const char *conf_number, int new_resolution);

enum video_resolution conf_user_get_video_quality(const char *channel_name, const char *conf_number);
const char *video_resolution_type_to_name(int type);
int video_resolution_name_to_type(const char *name);
int load_conf_resource(void);
int unload_conf_resource(void);
int reload_conf_resource(void);
int sfu_max_capability(void);
int sfu_max_video_record_rooms_capability(void);
int sfu_max_video_encode_capability(void);
int sfu_video_encode_frame(void);
int sfu_slides_encode_frame(void);
int sfu_focus_encode_frame(void);
extern void ast_conf_remove_conference(const char *conf_number);
extern void ast_conf_net_init(conf_net_info *info);
extern void ast_conf_get_net_info(struct ast_json *net_json, conf_net_info *info);
#endif
#endif
