/*!
 * \file
 * \brief chan_rtsp header file
 *
 * \author zyxia <zyxia@grandstream.cn>
 *
 */

#ifndef _ASTERISK_CHAN_RTSP_H
#define _ASTERISK_CHAN_RTSP_H

#include "asterisk.h"
#include "asterisk/channel.h"
#include "asterisk/rtp_engine.h"
#include "asterisk/netsock2.h"
#include "asterisk/avs_controller.h"


#define AUDIO_SDP             0
#define VIDEO_SDP             1

#define AST_1080P_LEVEL_ID 40                             /* 28 hex according Dec is 40 */
#define AST_720P_LEVEL_ID 31                              /* 1f hex according Dec is 31 */

/**
 ** Structures
 **/
typedef struct {
	char url[MAX_URL_SIZE];
	char scheme[20]; /* scheme: http, rtsp, ...*/
	char host[80];
	int port;
	char rep[MAX_URL_SIZE];
} url_t;

typedef struct {
	char control[MAX_URL_SIZE];
	int num;
} rtspSDPMedia;

typedef struct {
	char protocol[256];		// RTP/AVP
	char type_cast[256];	// unicast broadcast multicast
	char server_ip[40];
	int client_RTP_port;	//  client RTP port
	int client_RTCP_port;	// client RTCP port
	int server_RTP_port;	// server RTP port
	int server_RTCP_port;	// server RTCP port
} rtsp_setup_t;

typedef struct {
	url_t		   url;		/* URL Information */  
	int			   sock;		/* Socket */
	unsigned int	cseq_tx;	/* Packet count (RTSP server) */
	int			   connected;	/* Is connected? 0/1 */
	float			duration;	/* File duration time */
	int			   session;	/* session id */
	rtspSDPMedia   sdp[2];  /* 0 - audio, 1 - video */
} rtspsession_t;

typedef struct {
   char name[256];
   char value[256];
} header_t;

typedef struct {
/*____NUM_OPT____*/
	int 		NbCommand;
/*___ALL METHODS___*/
	int 		Blocksize;
	int   	CSeq;
	char     Location[MAX_URL_SIZE];
	long int 		Session;
/*_____PLAY______*/
	char 		PLAY_Range[128];
/*_____SETUP_____*/
	char 		SETUP_Transport[256]; 
} t_reqcmd;

struct rtsp_pvt {
	struct ast_channel *owner;              /* Who owns us */
	char callid[80];                        /* Global CallID */
	char cid_num[256];                      /* Cid number*/
	/*! \brief Caller ID name from an incoming call. */
	char cid_name[AST_MAX_EXTENSION];
	int amaflags;                           /* AMA Flags */
	struct ast_sockaddr sa;                 /* Our peer */
	struct ast_sockaddr ourip;              /* Our IP */

	struct ast_format_cap *caps;            /*!< Special capability (codec) */
	struct ast_format_cap *jointcaps;       /*!< Supported capability at both ends (codecs) */

	struct ast_rtp_instance *rtp;           /* RTP Session */
	struct ast_rtp_instance *vrtp;
	char stream_id[MAX_UNIQUE_ID];

	int has_video;
	int has_audio;

	long int port_group_id;
	//void* rtsp_client_state;
	char rtsp_url[MAX_URL_SIZE];
	char rtsp_username[MAX_RTSP_USERNAME_SIZE];
	char rtsp_password[MAX_RTSP_PWD_SIZE];
	char transport_protocol[MAX_PROTOCOL_LEN];
	char sdp_info[RECV_BUFFER_SIZE];
};

/*! \brief ast_rtsp_user descriptor, as read from the config file.
 *
 * \note It is still missing some fields -- e.g. we can have multiple permit and deny
 * lines which are not supported here, and readperm/writeperm/writetimeout
 * are not stored.
 */
struct ast_rtsp_user {
	char account[80];
	char *username;
	char *password;
	char *host;
	char* transport_protocol;
	char* streaming_protocol;
	char* fullname;
	int  port;
	char *path;
	int qualify_frequency;
	int keepalive_sched_id;
	int keepalive_status;
	char info_hash[33];
	int is_exist;
	AST_RWLIST_ENTRY(ast_rtsp_user) list;
};


enum rtsp_status {
	RTSP_UNMONITORNED=0,
	RTSP_REACHABLE,
	RTSP_UNREACHABLE
};

#endif	/* _ASTERISK_CHAN_RTSP_H */

