#ifdef GRANDSTREAM_NETWORKS
/*********************************************
 * File Name : bfcp.h
 * Author: bfdu
 * Date:   2014.02.09
 * Description: for bfcp
 * Changes: Initial
********************************************/

#ifndef _ASTERISK_BFCP_H
#define _ASTERISK_BFCP_H

#include "asterisk/network.h"
#include "asterisk/frame.h"
#include "asterisk/io.h"
#include "asterisk/sched.h"
#include "asterisk/channel.h"
#include "asterisk/netsock2.h"
#include "asterisk/tcptls.h"
#include "asterisk/pbx.h"

//#include <bfcp_common.h>
#include <bfcp_list.h>
#include <bfcp_mem.h>
#include <bfcp_msg.h>
#include <bfcp_struct.h>

#ifndef FALSE
#define FALSE    0
#endif

#ifndef TRUE
#define TRUE     1
#endif

#if defined(HAVE_FUNOPEN)
#define HOOK_T int
#define LEN_T int
#else
#define HOOK_T ssize_t
#define LEN_T size_t
#endif

#define BFCP_MAX_SETUP_SIZE 		16
#define BFCP_MAX_FLOORCTRL_SIZE 	8
#define BFCP_MAX_CONNECTION_SIZE 	16
#define BFCP_MAX_FINGERPRINT_SIZE 	64
#define BFCP_MAX_FLOORID_SIZE 		64
#define BFCP_MAX_CONTENT_SIZE		8
#define BFCP_MAX_DATAGRAM_LIMIT     1400
#define BFCP_XMIT_ERROR     		-2
#define DEFAULT_BFCP_AUTHLIMIT      100
#define DEFAULT_BFCP_AUTHTIMEOUT    30
#define BFCP_MAX_ADDRESS_LEN        48

#if defined(__cplusplus) || defined(c_plusplus)
extern "C" {
#endif

enum bfcp_info_action {
	BFCP_INFO_ACTION_FLOOR_REQUEST        =  1,
	BFCP_INFO_ACTION_FLOOR_RELEASE        =  2,
	BFCP_INFO_ACTION_FLOOR_CANCEL         =  3,
	BFCP_INFO_ACTION_FLOOR_STATUS         =  4,
	BFCP_INFO_ACTION_FLOOR_REQUEST_STATUS =  5,
	BFCP_INFO_ACTION_MAX                  =  6,
};

enum bfcp_denied_reason {
	BFCP_DENIED_REASON_NONE,
	BFCP_DENIED_REASON_CPU_LIMIT,
	BFCP_DENIED_REASON_NO_RESOURCE,
	BFCP_DENIED_REASON_NO_VIDEO_ROOM,
	BFCP_DENIED_REASON_MAX,
};

struct ast_bfcp_info
{
	uint32_t              conf_id;			/* Used for BFCP msg */
	uint16_t              user_id;			/* Used for BFCP msg */
	unsigned int 		  floor_requst_id;
	unsigned int 		  floor_id;
	char                  info_conf_id[128];
	char                  account_id[128];
	char                  floor_owner_id[128];
	char                  msid[128];
	char                  slider_name[128];
	char                  share_type[16];
	char                  share_audio[16];
	enum bfcp_prim        primitive;
	enum bfcp_reqstatus   status;
	int                   transaction_id;
	int 				  received_transaction_id;
	enum bfcp_info_action action;
	enum bfcp_denied_reason denied_reason;
	unsigned int 		  closed_by_host;
};

struct ast_bfcp_udp {
	int fd;
	char resp;
	struct ast_frame f;
	unsigned char rawdata[8192];
	int nat;
	int flags;
	struct ast_sockaddr us;
	struct ast_sockaddr them;
	int *ioid;
	struct ast_sched_context *sched;
	struct io_context *io;
	void *data;
	char *tag;
	struct ast_bfcp_info bfcp_info;
	struct ast_channel *channel;
	int sched_id;
	int sending_msg;
	int retransmit_count;
	struct bfcp_buffer timeout_buffer;
	struct bfcp_buffer bfcp_recv_buffer;
	struct bfcp_buffer bfcp_send_buffer;
	struct ast_bfcp_engine *engine;
	char turn_relay_address[BFCP_MAX_ADDRESS_LEN];
	unsigned int turn_relay_port;
	struct ast_sockaddr remote_addr;
	unsigned int media_turn_relay;
};

/*! ao2 object used for the FILE stream fopencookie()/funopen() cookie. */
struct ast_bfcp_tcptls_stream {
	/*! SSL state if not NULL */
	SSL *ssl;
	/*!
	 * \brief Start time from when an I/O sequence must complete
	 * by struct ast_bfcp_tcptls_stream.timeout.
	 *
	 * \note If struct ast_bfcp_tcptls_stream.start.tv_sec is zero then
	 * start time is the current I/O request.
	 */
	struct timeval start;
	/*!
	 * \brief The socket returned by accept().
	 *
	 * \note Set to -1 if the stream is closed.
	 */
	int fd;
	/*!
	 * \brief Timeout in ms relative to struct ast_bfcp_tcptls_stream.start
	 * to wait for an event on struct ast_bfcp_tcptls_stream.fd.
	 *
	 * \note Set to -1 to disable timeout.
	 * \note The socket needs to be set to non-blocking for the timeout
	 * feature to work correctly.
	 */
	int timeout;
	/*! TRUE if stream can exclusively wait for fd input. */
	int exclusive_input;
};

struct ast_bfcp_tcptls_session_instance {
	FILE *f;    /*!< fopen/funopen result */
	int fd;     /*!< the socket returned by accept() */
	SSL *ssl;   /*!< ssl state */
	int client;
	struct ast_sockaddr remote_address;
	struct ast_bfcp_tcp *bfcp_tcp;
	struct ast_tcptls_session_args *parent;
	/* Sometimes, when an entity reads TCP data, multiple
	 * logical messages might be read at the same time. In such
	 * a circumstance, there needs to be a place to stash the
	 * extra data.
	 */
	struct ast_str *overflow_buf;
	/*! ao2 FILE stream cookie object associated with f. */
	struct ast_bfcp_tcptls_stream *stream_cookie;
	/*! ao2 object private data of parent->worker_fn */
	void *private_data;
};

struct ast_bfcp_tcp {
	struct ast_tcptls_session_args bfcp_tcp_desc;
	struct ast_bfcp_tcptls_session_instance *tcptls_session;
	struct ast_sockaddr remote_address;
	struct ast_bfcp_info bfcp_info;
	struct ast_channel *channel;
	unsigned int tcptls_session_destoryed;
};

struct ast_bfcp_para {
	unsigned int has_udp_bfcp;			/* for UDP BFCP flag */
	unsigned int has_tcp_bfcp;		/* for TCP BFCP flag */
	int confid;
	int userid;
	char floorid[BFCP_MAX_FLOORID_SIZE];
	char local_floorctrl[BFCP_MAX_FLOORCTRL_SIZE];
	char remote_floorctrl[BFCP_MAX_FLOORCTRL_SIZE];
	char local_setup[BFCP_MAX_SETUP_SIZE];
	char remote_setup[BFCP_MAX_SETUP_SIZE];
	char connection[BFCP_MAX_CONNECTION_SIZE];
	char fingerprint[BFCP_MAX_FINGERPRINT_SIZE];
};

struct bfcp_conf_resource_manager_funcs
{
	int (*conf_get_slides_video_resource)(const char *conf_number, struct ast_channel *chan, int share_audio);

	int (*conf_put_slides_video_resource)(const char *conf_number, struct ast_channel *chan);

	struct ast_channel *(*conf_get_slider_channel)(const char *conf_number);

	struct ast_channel *(*conf_get_sfu_moderator)(struct ast_channel *chan, const char *conf_number);

	int (*conf_set_slider_name)(const char *conf_number, char *slider_name);
};

struct ast_bfcp_engine {
	const char *name;
	struct ast_module *mod;
	int (*new)(struct ast_bfcp_udp *bfcp_udp);
	int (*destory)(struct ast_bfcp_udp *bfcp_udp);
	int (*get_turn_relay_address)(struct ast_bfcp_udp *bfcp_udp);
	int (*turn_packet_send)(struct ast_bfcp_udp *bfcp_udp, void *pkt, int size);
	int (*create_turn_permission)(struct ast_bfcp_udp *bfcp_udp, const char *peer_address);
	int (*set_peer_address)(struct ast_bfcp_udp *bfcp_udp, struct ast_sockaddr *remote_addr);
	void (*set_udp_reachable)(int value);
	/*! Linked list information */
	AST_RWLIST_ENTRY(ast_bfcp_engine) entry;
};

void ast_bfcp_setnat(struct ast_bfcp_udp *bfcp, int nat);
int ast_bfcp_fd(const struct ast_bfcp_udp *bfcp);
void ast_bfcp_set_peer(struct ast_bfcp_udp *bfcp, const struct ast_sockaddr *them);
void ast_bfcp_get_us(const struct ast_bfcp_udp *bfcp, struct ast_sockaddr *us);
void ast_bfcp_stop(struct ast_bfcp_udp *bfcp);
void ast_bfcp_destroy(struct ast_bfcp_udp *bfcp);
struct ast_bfcp_udp *ast_bfcp_udp_new_with_bindaddr(const char *bfcp_protocol, struct ast_sched_context *sched,
	struct io_context *io, struct ast_sockaddr *addr, const char *engine_name, const char *remote_address, unsigned int media_turn_relay);
struct ast_frame *ast_bfcp_read(struct ast_bfcp_udp *bfcp);
int ast_bfcp_write(struct ast_bfcp_udp *s, struct ast_frame *f);
int ast_bfcp_setqos(struct ast_bfcp_udp *bfcp, unsigned int tos, unsigned int cos);
int ast_bfcp_get_and_cmp_requested_target_address(struct ast_bfcp_udp *bfcp, struct ast_sockaddr *address);
void ast_bfcp_set_channel_slides_status(struct ast_channel *channel, const struct ast_bfcp_info *bfcp_info, unsigned int is_recv_bfcp_msg);

struct ast_bfcp_tcp *ast_bfcp_tcp_new_with_bindaddr(struct ast_sockaddr *addr);
void ast_bfcp_tcptls_server_start(struct ast_bfcp_tcp *bfcp_tcp);
void *handle_bfcp_tcptls_connection(void *data);
void *ast_bfcp_tcptls_server_root(void *data);
void bfcp_init_tcp_desc(struct ast_tcptls_session_args *bfcp_tcp_desc);
struct ast_bfcp_tcptls_session_instance *ast_bfcp_tcptls_client_start(struct ast_bfcp_tcp *bfcp_tcp);
struct ast_bfcp_tcptls_session_instance *ast_bfcp_tcptls_client_create(struct ast_tcptls_session_args *desc);
void bfcp_tcptls_client_args_destructor(void *obj);
int bfcp_check_authtimeout(time_t start);
void *_bfcp_tcp_helper_thread(struct ast_bfcp_tcp *bfcp_tcp);
void *bfcp_tcp_worker_fn(void *data);
int ast_bfcp_tcptls_server_read(struct ast_bfcp_tcptls_session_instance *tcptls_session, void *buf, int count);
int ast_bfcp_tcptls_server_write(struct ast_bfcp_tcptls_session_instance *tcptls_session, const void *buf, int count);
void ast_bfcp_tcp_write(struct ast_bfcp_tcp *bfcp_tcp, const void *data, int datalen);
void ast_bfcp_tcp_get_us(const struct ast_bfcp_tcp *bfcp_tcp, struct ast_sockaddr *us);
void ast_bfcp_tcp_set_peer(struct ast_bfcp_tcp *bfcp_tcp, const struct ast_sockaddr *them);
int ast_bfcp_tcp_get_and_cmp_requested_target_address(struct ast_bfcp_tcp *bfcp_tcp, struct ast_sockaddr *address);
void ast_bfcp_tcptls_server_stop(struct ast_bfcp_tcp *bfcp_tcp);
void ast_bfcp_tcp_destroy(struct ast_bfcp_tcp *bfcp_tcp);
int ast_send_bfcp_msg(struct ast_bfcp_udp *bfcp);
int ast_send_bfcp_tcp_msg(struct ast_bfcp_tcp *bfcp_tcp);
int ast_bfcp_init(void);
int ast_bfcp_register_conf_funcs(struct bfcp_conf_resource_manager_funcs *funcs);
int ast_bfcp_unregister_conf_funcs(void);
void ast_bfcp_set_data(struct ast_bfcp_udp *bfcp, void *data);
void *ast_bfcp_get_data(struct ast_bfcp_udp *bfcp_udp);
int ast_bfcp_engine_register(struct ast_bfcp_engine *engine, struct ast_module *module);
int ast_bfcp_engine_unregister(struct ast_bfcp_engine *engine);
void ast_bfcp_udp_reachable_set(int value, const char *engine_name);
#if defined(__cplusplus) || defined(c_plusplus)
}
#endif

#endif

#endif

