/*
 * Asterisk -- An open source telephony toolkit.
 *
 * Copyright (C) 2023, Grandstream, Inc.
 *
 * See http://www.asterisk.org for more information about
 * the Asterisk project. Please do not directly contact
 * any of the maintainers of this project for assistance;
 * the project provides a web site, mailing lists and IRC
 * channels for your use.
 *
 * This program is free software, distributed under the terms of
 * the GNU General Public License Version 2. See the LICENSE file
 * at the top of the source tree.
 */

#ifndef _FWD_TIMING_H
#define _FWD_TIMING_H

#include "asterisk/astobj2.h"
#include "asterisk/_private.h"
#include "asterisk/module.h"
#include "asterisk/config.h"
#include "asterisk/channel.h"
#include "asterisk/utils.h"


enum ast_fwd_service_type {
	FWD_CFU = 0,
	FWD_CFN = 1,
	FWD_CFB = 2,
	FWD_RCFU = 3,
	FWD_RCFN = 4,
	FWD_RCFB = 5,
    FWD_BUTT,
};

enum ast_fwd_dst_type {
	FWD_DST_NONE     = 0,
	FWD_DST_USER     = 1,
	FWD_DST_CUSTOM   = 2,
	FWD_DST_VM       = 3,
	FWD_DST_RINGROUP = 4,
	FWD_DST_QUEUE    = 5,
	FWD_DST_VMG      = 6,
	FWD_DST_PROMPT   = 7,
    FWD_DST_BUTT,
};

enum ast_fwd_time_type {
	TIME_ALL         = 0, /* all time, no need to check timing*/
	TIME_OFFICE      = 1, /* officetime */
	TIME_NON_OFFICE  = 2, /* non-officetime */
	TIME_HOLIDAY     = 3, /* holidaytime */
	TIME_NON_HOLIDAY = 4, /* non-holidaytime */
	TIME_NON_OFFICE_OR_HOLIDAY = 5,   /* non-officetime OR holidaytime */
	TIME_SPECIFIC              = 6,   /* specific time */
	TIME_SPECIFIC_AND_NON_HOLIDAY = 7,/* specific time AND non-holidaytime */
	TIME_OFFICE_AND_NON_HOLIDAY  = 8, /* officetime and non-holidaytime */
	TIME_NON_SPECIFIC            = 9, /* non-specifictime */
	TIME_NON_SPECIFIC_OR_HOLIDAY = 10,/* non-specifictime OR holidaytime */
};

struct ast_fwd_setting {
    int  enabled;
    enum ast_fwd_time_type  time_type;
    char dst_number[80];
};

struct ast_fwd_status {
    int  active;
    int  update;
	char dst_num[80];
};

struct ast_fwd_active_status {
    struct ast_fwd_status cfu_status;
    struct ast_fwd_status cfn_status;
    struct ast_fwd_status cfb_status;
    struct ast_fwd_status rcfu_status;
    struct ast_fwd_status rcfn_status;
    struct ast_fwd_status rcfb_status;
};

struct ast_fwd_ext_status {
	char   ext[80];
    struct ast_fwd_active_status last_status;
    struct ast_fwd_active_status cur_status;
	AST_LIST_ENTRY(ast_fwd_ext_status) entry;
};

struct ast_fwd_notify_info {
	char   ext[80];
    struct ast_fwd_active_status notify_status;
};


int ast_fwd_active_status_update_set(struct ast_fwd_active_status *active_status, int flag);
int ast_fwd_active_status_cpy(struct ast_fwd_active_status *status_src, struct ast_fwd_active_status *status_dst);
int ast_fwd_notify_info_cpy(struct ast_fwd_notify_info *info_src, struct ast_fwd_notify_info *info_dst);
int ast_fwd_ext_get_notify_data(char *ext, struct ast_fwd_active_status *active_status, int full);
const char *ast_fwd_type_to_str(enum ast_fwd_service_type stype);
enum ast_fwd_service_type ast_fwd_type_get(char *text);
enum ast_fwd_dst_type ast_fwd_dst_type_get(char *ext);
enum ast_presence_state ast_fwd_ext_get_presence_status(char *ext);
int ast_fwd_timing_init(void);

int ast_fwd_timetype_check(char *ext, int time_type);

#endif /* _FWD_TIMING_H */
