/*
 * Asterisk -- An open source telephony toolkit.
 *
 * Copyright (C) 2014, Digium, Inc.
 *
 * Mark Michelson <mmichelson@digium.com>
 *
 * See http://www.asterisk.org for more information about
 * the Asterisk project. Please do not directly contact
 * any of the maintainers of this project for assistance;
 * the project provides a web site, mailing lists and IRC
 * channels for your use.
 *
 * This program is free software, distributed under the terms of
 * the GNU General Public License Version 2. See the LICENSE file
 * at the top of the source tree.
 */

#ifndef _RES_PJSIP_BFCP_INFO_H
#define _RES_PJSIP_BFCP_INFO_H

#define BFCP_INFO_INVALID      0XFFFF
#define BFCP_INFO_SUB_TYPE     "x-grandstream-slides-video-control+xml"
#define BFCP_INFO_CONF_SUB_TYPE     "x-grandstream-confcontrol+xml"

const char * const bfcp_info_reqstatus_strmap[][1] = {
	{ /* 0 NULL          */      ""          },
	{ /* 1 BFCP_PENDING  */      "Pending"   },
	{ /* 2 BFCP_ACCEPTED */      "Accepted"  },
	{ /* 3 BFCP_GRANTED  */      "Granted"   },
	{ /* 4 BFCP_DENIED   */      "Denied"    },
	{ /* 5 BFCP_CANCELLED*/      "Cancelled" },
	{ /* 6 BFCP_RELEASED */      "Released"  },
	{ /* 7 BFCP_REVOKED  */      "Revoked"   },
};

const char * const bfcp_info_prim_strmap[][1] = {
	{ /* 0 BFCP_KEEP_ALIVE           */      "KeepAlive"           },
	{ /* 1 BFCP_FLOOR_REQUEST        */      "FloorRequest"        },
	{ /* 2 BFCP_FLOOR_RELEASE        */      "FloorRelease"        },
	{ /* 3 BFCP_FLOOR_REQUEST_QUERY  */      "FloorRequestQuery"   },
	{ /* 4 BFCP_FLOOR_REQUEST_STATUS */      "FloorRequestStatus"  },
	{ /* 5 BFCP_USER_QUERY           */      "UserQuery"           },
	{ /* 6 BFCP_USER_STATUS          */      "UserStatus"          },
	{ /* 7 BFCP_FLOOR_QUERY          */      "FloorQuery"          },
	{ /* 8 BFCP_FLOOR_STATUS         */      "FloorStatus"         },
	{ /* 9 BFCP_CHAIR_ACTION         */      "ChairAction"         },
	{ /* 10 BFCP_CHAIR_ACTION_ACK    */      "ChairActionAck"      },
	{ /* 11 BFCP_HELLO               */      "Hello"               },
	{ /* 12 BFCP_HELLO_ACK           */      "HelloAck"            },
	{ /* 13 BFCP_ERROR               */      "Error"               },
	{ /* 14 BFCP_FLOOR_REQ_STATUS_ACK*/      "FloorReqStatusAck"   },
	{ /* 15 BFCP_ERROR_ACK           */      "ErrorAck"            },
	{ /* 16 BFCP_FLOOR_STATUS_ACK    */      "FloorStatusAck"      },
	{ /* 17 BFCP_GOODBYE             */      "GoodBye"             },
	{ /* 18 BFCP_GOODBYE_ACK         */      "GoodByeAck"          },
	{ /* 19 BFCP_FLOOR_CANCEL        */      "FloorCancel"         },
};

const char * const bfcp_info_action_strmap[][1] = {
	{ /* 0 BFCP_INFO_ACTION_UNKNOW               */      "unknow"              },
	{ /* 1 BFCP_INFO_ACTION_FLOOR_REQUEST        */      "FloorRequest"        },
	{ /* 2 BFCP_INFO_ACTION_FLOOR_RELEASE        */      "FloorRelease"        },
	{ /* 3 BFCP_INFO_ACTION_FLOOR_CANCEL         */      "FloorCancel"         },
	{ /* 4 BFCP_INFO_ACTION_FLOOR_STATUS         */      "FloorStatus"         },
	{ /* 5 BFCP_INFO_ACTION_FLOOR_REQUEST_STATUS */      "FloorRequestStatus"  },
};

enum bfcp_info_share_type {
	BFCP_INFO_SHARE_TYPE_UNKNOW        =  0,
	BFCP_INFO_SHARE_TYPE_SCREEN        =  1,
	BFCP_INFO_SHARE_TYPE_WINDOW        =  2,
	BFCP_INFO_SHARE_TYPE_TAB           =  3,
	BFCP_INFO_SHARE_TYPE_WHITEBOARD    =  4,
	BFCP_INFO_SHARE_TYPE_DOCUMENT      =  5,
};

const char * const bfcp_info_share_type_strmap[][1] = {
	{ /* 0 BFCP_INFO_SHARE_TYPE_UNKNOW        */      "unknow"        },
	{ /* 1 BFCP_INFO_SHARE_TYPE_SCREEN        */      "screen"        },
	{ /* 2 BFCP_INFO_SHARE_TYPE_WINDOW        */      "window"        },
	{ /* 3 BFCP_INFO_SHARE_TYPE_TAB           */      "tab"           },
	{ /* 4 BFCP_INFO_SHARE_TYPE_WHITEBOARD    */      "whiteboard"    },
	{ /* 5 BFCP_INFO_SHARE_TYPE_DOCUMENT      */      "document"      },
};

const char * const bfcp_denied_reason_strmap[][1] = {
	{ /* 0 NULL                            */      ""                          },
	{ /* 1 BFCP_DENIED_REASON_CPU_LIMIT     */      "CPU limit"                },
	{ /* 2 BFCP_DENIED_REASON_NO_RESOURCE   */      "No slides video reource"  },
	{ /* 3 BFCP_DENIED_REASON_NO_VIDEO_ROOM */      "No video room"  },
};

#endif /* _RES_PJSIP_BFCP_INFO_H */
