/*
 * Asterisk -- An open source telephony toolkit.
 *
 * Copyright (C) 2014, Digium, Inc.
 *
 * Mark Michelson <mmichelson@digium.com>
 *
 * See http://www.asterisk.org for more information about
 * the Asterisk project. Please do not directly contact
 * any of the maintainers of this project for assistance;
 * the project provides a web site, mailing lists and IRC
 * channels for your use.
 *
 * This program is free software, distributed under the terms of
 * the GNU General Public License Version 2. See the LICENSE file
 * at the top of the source tree.
 */

#ifndef _RES_PJSIP_BODY_GENERATOR_TYPES_H
#define _RES_PJSIP_BODY_GENERATOR_TYPES_H

#include "asterisk/pbx.h"
#ifdef GRANDSTREAM_NETWORKS
#include "asterisk/pjsip_cei.h"
#include "asterisk/app.h"
#include "asterisk/fwd_timing.h"
#endif

/*! \brief Forward declaration for ao2_container so full astobj2.h is not required */
struct ao2_container;

#ifdef GRANDSTREAM_NETWORKS
enum ast_extension_type  {
	EN_EXTEN_LOCAL   = 0,
	EN_EXTEN_REMOTE  = 1,
	EN_EXTEN_SPECIAL = 2,
	EN_EXTEN_VMG     = 3,
};
#endif

/*!
 * \brief structure used for presence XML bodies
 *
 * This is used for the following body types:
 * \li application/pidf+xml
 * \li application/xpidf+xml
 * \li application/cpim-pidf+xml
 * \li application/dialog-info+xml
 */
struct ast_sip_exten_state_data {
	/*! The extension of the current state change */
	const char *exten;
	/*! The extension state of the change */
	enum ast_extension_states exten_state;
	/*! The presence state of the change */
	enum ast_presence_state presence_state;
	/*! The presence subtype of the change */
	char *presence_subtype;
	/*! The presence message of the change */
	char *presence_message;
	/*! Subscriber user agent */
	char *user_agent;
	/*! Current device state information */
	struct ao2_container *device_state_info;
	/*! Local dialog URI */
	char local[PJSIP_MAX_URL_SIZE];
	/*! Remote dialog URI */
	char remote[PJSIP_MAX_URL_SIZE];
#ifdef GRANDSTREAM_NETWORKS
	char display_name[AST_MAX_EXTENSION];
	enum ast_extension_type exten_type;
#endif
	/*! Optional subscription */
	struct ast_sip_subscription *sub;
	/*! A datastores container to persist datastores */
	struct ao2_container *datastores;
	/*! Allocation pool */
	pj_pool_t *pool;
};

/*!
 * \brief Message counter used for message-summary XML bodies
 *
 * This is used for application/simple-message-summary bodies.
 */
struct ast_sip_message_accumulator {
	/*! Number of old messages */
	int old_msgs;
	/*! Number of new messages */
	int new_msgs;
	/*! Message-Account */
	char message_account[PJSIP_MAX_URL_SIZE];
#ifdef GRANDSTREAM_NETWORKS
	/*! Number of urgent messages */
	int urgent_msgs;
	char exten[64];
	struct ast_mwi_external_data external_data;
#endif
};

#ifdef GRANDSTREAM_NETWORKS
/*!
 * \brief structure used for conference-info+xml XML bodies
 *
 * This is used for application/conference-info+xml bodies.
 */
struct pjsip_cei_notify_data {
	char *channelName;
	char *conference;
	char ceiURI[PJSIP_MAX_URL_SIZE];
	char webHost[PJSIP_MAX_URL_SIZE];
	int notifyVer;
	int isTerminated;
	/* XML template for generating XML body */
	char *content;
	enum cei_event_type type;
	enum ast_terminal_type terminal_type;
	int ucmProtNotifyVer;
	int isUcmProtNotify;
};

/* Support as-feature-event subscription, added by yfyang */
/*!
 * \brief structure used for as-feature-event XML bodies
 * \li application/x-as-feature-event+xml
 */
struct ast_sip_feature_event_data {
	char exten[80];  	     	    	/*! The extension of the current state change */
	struct ast_fwd_active_status fwd_state; /*! The feature_event state of the change */
};

#endif

#endif /* _RES_PJSIP_BODY_GENERATOR_TYPES_H */
