/*
 * Asterisk -- An open source telephony toolkit.
 *
 * Copyright (C) 2020, Digium, Inc.
 *
 * bfdu <bfdu@grandstream.cn>
 *
 * See http://www.asterisk.org for more information about
 * the Asterisk project. Please do not directly contact
 * any of the maintainers of this project for assistance;
 * the project provides a web site, mailing lists and IRC
 * channels for your use.
 *
 * This program is free software, distributed under the terms of
 * the GNU General Public License Version 2. See the LICENSE file
 * at the top of the source tree.
 */

#ifndef _RES_PJSIP_CAMERA_CONTROL_H
#define _RES_PJSIP_CAMERA_CONTROL_H

#define CAMERA_CONTROL_INFO_INVALID      0XFFFF
#define CAMERA_CONTROL_INFO_SUB_TYPE     "x-grandstream-camera-confcontrol+xml"

enum ast_camera_control_action {
	CAMERA_CONTROL_ACTION_UNKNOW                           =  0,
	CAMERA_CONTROL_ACTION_OPEN_PARTICIPANT_CAMERA          =  1,
	CAMERA_CONTROL_ACTION_CLOSE_PARTICIPANT_CAMERA         =  2,
	CAMERA_CONTROL_ACTION_OPEN_OWN_CAMERA                  =  3,
	CAMERA_CONTROL_ACTION_CLOSE_OWN_CAMERA                 =  4,
	CAMERA_CONTROL_ACTION_OPEN_PARTICIPANT_CAMERA_STATUS   =  5,
	CAMERA_CONTROL_ACTION_CLOSE_PARTICIPANT_CAMERA_STATUS  =  6,
	CAMERA_CONTROL_ACTION_OPEN_OWN_CAMERA_STATUS           =  7,
	CAMERA_CONTROL_ACTION_CLOSE_OWN_CAMERA_STATUS          =  8,
	CAMERA_CONTROL_ACTION_REPORT_CAMERA_CAPABILITY         =  9,
	CAMERA_CONTROL_ACTION_MAX,
};

enum ast_camera_control_status {
	CAMERA_CONTROL_STATUS_UNKNOW     = 0,
	CAMERA_CONTROL_STATUS_SUCCEED    = 1,
	CAMERA_CONTROL_STATUS_FAILED     = 2,
	CAMERA_CONTROL_STATUS_MAX,
};

struct ast_camera_control_info
{
	char participant_channel_id[128];
	char failed_reason[128];
	enum ast_camera_control_action action;
	enum ast_camera_control_status status;
	char camera_capability;
};

const char * const camera_control_action_strmap[][1] = {
	{ /* 0 CAMERA_CONTROL_ACTION_UNKNOW                             */      "unknow"                        },
	{ /* 1 CAMERA_CONTROL_ACTION_OPEN_PARTICIPANT_CAMERA            */      "OpenParticipantCamera"         },
	{ /* 2 CAMERA_CONTROL_ACTION_CLOSE_PARTICIPANT_CAMERA           */      "CloseParticipantCamera"        },
	{ /* 3 CAMERA_CONTROL_ACTION_OPEN_OWN_CAMERA                    */      "OpenOwnCamera"                 },
	{ /* 4 CAMERA_CONTROL_ACTION_CLOSE_OWN_CAMERA                   */      "CloseOwnCamera"                },
	{ /* 5 CAMERA_CONTROL_ACTION_OPEN_PARTICIPANT_CAMERA_STATUS     */      "OpenParticipantCameraStatus"   },
	{ /* 6 CAMERA_CONTROL_ACTION_CLOSE_PARTICIPANT_CAMERA_STATUS    */      "CloseParticipantCameraStatus"  },
	{ /* 7 CAMERA_CONTROL_ACTION_OPEN_OWN_CAMERA_STATUS             */      "OpenOwnCameraStatus"           },
	{ /* 8 CAMERA_CONTROL_ACTION_CLOSE_OWN_CAMERA_STATUS            */      "CloseOwnCameraStatus"          },
	{ /* 9 CAMERA_CONTROL_ACTION_REPORT_CAMERA_CAPABILITY           */      "ReportCameraCapability"        },
};

const char * const camera_control_status_strmap[][1] = {
	{ /* 0 CAMERA_CONTROL_STATUS_UNKNOW             */      "unknow"        },
	{ /* 1 CAMERA_CONTROL_STATUS_SUCCEED            */      "Succeed"       },
	{ /* 2 CAMERA_CONTROL_STATUS_FAILED             */      "Failed"        },
};

#endif /* _RES_PJSIP_CAMERA_CONTROL_H */
