/*
 * asterisk -- An open source telephony toolkit.
 *
 * Copyright (C) 2017, Grandstream Networks.
 *
 * Yang Yanfei <yfyang@grandstream.cn>
 *
 * See http://www.asterisk.org for more information about
 * the Asterisk project. Please do not directly contact
 * any of the maintainers of this project for assistance;
 * the project provides a web site, mailing lists and IRC
 * channels for your use.
 *
 * This program is free software, distributed under the terms of
 * the GNU General Public License Version 2. See the LICENSE file
 * at the top of the source tree.
 */

#ifndef _RES_PJSIP_SCA_H
#define _RES_PJSIP_SCA_H

struct ast_sip_line_seize_handler {
    /* to notify the seized state has terminated */
    int (*seize_terminated)(char* sharedline, int line_index);   
	AST_LIST_ENTRY(ast_sip_line_seize_handler) next;
};

struct ast_sip_call_info_handler {
    /* to notify the call-info subscription need to terminated */
    int (*call_info_terminated)(char* sharedline, char* exten);   
	AST_LIST_ENTRY(ast_sip_call_info_handler) next;
};

/*!
 * Check line index and seize one line
 * 
 * \param sharedline : the name of the shared line
 * \param local_info : the local party in the call
 * \param line_index : the index of the line to seize
 * 
 * \retval 0 Success
 * \retval -1 Failure
 */
int ast_sip_line_check_and_seize(char* sharedline, char* local_info, int line_index);


/*!
 * To get the detailed callinfo string
 * 
 * \param sharedline : the name of the shared line 
 * \param local_host : the host name
 * \param buf : in/out param, the buffer to record the callinfo string
 * \param buf_size : the size of the buffer
 * 
 * \retval 0 Success
 * \retval -1 Failure
 */
int ast_sip_get_call_info(char* sharedline, char* local_host, char* buf, int buf_size);


/*!
 * To end the seized state of the line when the line-seize subscription terminated
 * 
 * \param sharedline : the name of the shared line
 * \param local_info : the local party in the call
 * \param line_index : the index of the line to unseize
 * 
 * \retval 0 Success
 * \retval -1 Failure
 */
int ast_sip_line_unseize(char* sharedline, char* local_info, int line_index);


/*!
 * \brief Register a handler to SIP SCA line seize processing
 *
 * \param handler: The handler to register
 * \retval 0 Success
 * \retval -1 Failure
 */
int ast_sip_register_line_seize_handler(struct ast_sip_line_seize_handler *handler);

/*!
 * \brief Unregister a handler to SIP SCA line seize processing
 *
 * \param handler: The handler to unregister
 */
void ast_sip_unregister_line_seize_handler(struct ast_sip_line_seize_handler *handler);


int ast_sip_register_call_info_handler(struct ast_sip_call_info_handler *handler);

void ast_sip_unregister_call_info_handler(struct ast_sip_call_info_handler *handler);


#endif /* _RES_PJSIP_SCA_H */
