/*** MODULEINFO
	<support_level>core</support_level>
 ***/

#include "asterisk.h"

#include <stdio.h>
#include <stdlib.h>
#include <errno.h>
#include <string.h>
#include <netdb.h>
#include <sys/types.h>
#include <netinet/in.h>
#include <sys/socket.h>
#include <unistd.h>
#include <strings.h>
#include <sqlite3.h>
#include "asterisk.h"
#include <sys/un.h>
#include <ifaddrs.h>  

#include "asterisk/paths.h"
#include "asterisk/channel.h"
#include "asterisk/indications.h"
#include "asterisk/file.h"
#include "asterisk/pbx.h"
#include "asterisk/app.h"
#include "asterisk/multi_media.h"
#include "asterisk/astdb.h"
#include "asterisk/select.h"
#include "asterisk/lock.h"
#include "asterisk/module.h"
#include "asterisk/avs_controller.h"
#include "asterisk/haserver.h"
#include "asterisk/linkedlists.h"


#ifdef HAVE_UNIXODBC
#include "asterisk/ucm_dbi.h"
#include "asterisk/core_odbc.h"
#include "asterisk/app.h"
#include "asterisk/cdr.h"


AST_MODULE_LOG("core");
#endif

struct slave_media_info {
	char ip[MAX_IPADDR_V6_LEN];
	SLAVE_STATUS_E    status;
	AST_LIST_ENTRY(slave_media_info) entry;
};

#define SLAVE_MEDIA_INFO_DB "slave_media_info"
static AST_LIST_HEAD_STATIC(slave_media_list, slave_media_info);


static MEDIA_ROLE_E g_media_role = MEDIA_ROLE_UNKNOWN;
static pthread_t ast_multiavs_rsync_phread = AST_PTHREADT_NULL;
char g_ast_avs_rsync_flag[AST_MULTI_AVS_FILE_BACKUP_MAX] = {0};
static NETWORK_MODE_E network_mode = SWITCH_MODE;


AST_MUTEX_DEFINE_STATIC(totalrecordlock);
static int total_mediarecords;


const char * ast_g_BackupType[AST_MULTI_AVS_FILE_BACKUP_MAX] = 
{
	AST_MULTI_AVS_FAX_LOCAL_FILE,
	AST_MULTI_AVS_VOICE_MAIL_LOCAL_FILE,
	AST_MULTI_AVS_VOICE_PROMPT_LOCAL_FILE,
	AST_MULTI_AVS_VOICE_MOH_LOCAL_FILE,
	AST_MULTI_AVS_DUMP_LOCAL_FILE,
	AST_MULTI_AVS_SLAVE_RECORDING_FILE,
};

const char * ast_g_BackupPath[AST_MULTI_AVS_FILE_BACKUP_MAX] = 
{
	AST_MULTI_AVS_FAX_FILE_PATH,
	AST_MULTI_AVS_VOICE_MAIL_FILE_PATH,
	AST_VOICE_PROMPT_FILE_PATH,
	AST_VOICE_MOH_FILE_PATH,
	AST_AVS_DUMP_FILE_PATH,
	AST_MULTI_AVS_SLAVE_RECORDING_FILE_PATH,
};

MEDIA_ROLE_E ast_media_role_get(void)
{
	return g_media_role;
}

void ast_create_record_file(const char* file_name){

	char cmd[512] = {0};
	ast_log(LOG_DEBUG, "ast_create_record_file file_name:%s",file_name);

	if(ast_strlen_zero(file_name))
		return;

	snprintf(cmd, sizeof(cmd), "touch %s ", file_name);
	ast_log(LOG_DEBUG, "ast_create_record_file cmd:%s",cmd);
	ast_safe_system2(cmd);
}

void ast_increase_media_records(void){
	ast_mutex_lock(&totalrecordlock);
	total_mediarecords++;
	ast_log(LOG_DEBUG, "ast_increase_media_records totalrecords:%d \n",total_mediarecords);
	ast_mutex_unlock(&totalrecordlock);
}

void ast_decrease_media_records(int decrease){
	ast_mutex_lock(&totalrecordlock);
	total_mediarecords = total_mediarecords - decrease; 
	if(total_mediarecords < 0)
		total_mediarecords = 0;
	ast_log(LOG_DEBUG, "ast_decrease_media_records totalrecords:%d \n",total_mediarecords);
	ast_mutex_unlock(&totalrecordlock);
}

void ast_media_role_init(void)
{
	char media_role[8] = {0};
	ast_safe_nvram_get("mediaRole", media_role, sizeof(media_role) - 1);
	if (strlen(media_role) && 0 == strcmp(media_role, "0")) {
		g_media_role = MEDIA_ROLE_SLAVE;
	} else if (strlen(media_role) && 0 == strcmp(media_role, "1")) {
		g_media_role = MEDIA_ROLE_MASTER;
	}
}

int ast_sync_record_from_slave(const           char* cmd_prefix,const char* avs_address,char *src_path, char *dst_path,char *module)
{
	char backup_user[16] = "backup";
	char cmd[512] = {0};

	if (!avs_address || ast_strlen_zero(module) 
		|| ast_strlen_zero(src_path) || ast_strlen_zero(dst_path)
		|| ast_strlen_zero(cmd_prefix)) {
		ast_log(LOG_ERROR, "file_path is null");
		return -1;
	}

	snprintf(cmd, sizeof(cmd), "%s '%s@%s::%s%s' '%s' ", cmd_prefix,backup_user, ast_strlen_zero(avs_address)?"127.0.0.1" : avs_address, module,src_path,dst_path);
	ast_log(LOG_DEBUG, "ast_sync_record_from_slave cmd:%s,src:%s,dst:%s",cmd,src_path,dst_path);
	ast_safe_system2(cmd);

	return 0;
}

/*Start by zyxia for  for Bug 285267,2024.2.5*/
int ast_sync_record_from_slave_and_move(const           char* cmd_prefix,const char* avs_address,char *src_path, char *dst_path,char *module)
{
	char backup_user[16] = "backup";
	char cmd[512] = {0};
	ast_log(LOG_DEBUG, "ast_sync_record_from_slave_and_move cmd");

	if (!avs_address || ast_strlen_zero(module) 
		|| ast_strlen_zero(src_path) || ast_strlen_zero(dst_path)
		|| ast_strlen_zero(cmd_prefix)) {
		ast_log(LOG_ERROR, "file_path is null");
		return -1;
	}

	/*1. move first,make the file exist first*/
		/*
	memset(cmd,0,sizeof(cmd));
	snprintf(cmd, sizeof(cmd), "mv %s %s", src_path,dst_path);
	ast_log(LOG_DEBUG, "ast_sync_record_from_slave_and_move  mv cmd:%s,src:%s,dst:%s",cmd,src_path,dst_path);
	ast_safe_system2(cmd);
	*/

	/*2 rsync again,edit by zyxia for Bug 303476,rsync to real path,not tmp path*/
	memset(cmd,0,sizeof(cmd));
	snprintf(cmd, sizeof(cmd), "%s %s@%s::%s%s %s ", cmd_prefix,backup_user, ast_strlen_zero(avs_address)?"127.0.0.1" : avs_address, module,src_path,dst_path);
	ast_log(LOG_DEBUG, "ast_sync_record_from_slave_and_move cmd:%s,src:%s,dst:%s",cmd,src_path,src_path);
	ast_safe_system2(cmd);

	/*3 rm the new file,but need run in background */
	/*
	memset(cmd,0,sizeof(cmd));
	snprintf(cmd, sizeof(cmd), "rm  %s &", src_path);
	ast_log(LOG_DEBUG, "ast_sync_record_from_slave_and_move  rm cmd:%s,src:%s",cmd,src_path);
	ast_safe_system2(cmd);
	*/
	return 0;
}
/*End by zyxia for  for Bug 285267,2024.2.5*/

int ast_sync_file_to_slave(const char* cmd_prefix,const char* avs_address,char *src_path, char *dst_path,char *module)
{
	char backup_user[16] = "backup";
	char cmd[512] = {0};
	ast_log(LOG_DEBUG, "ast_sync_file_to_slave cmd");

	if (!avs_address || ast_strlen_zero(module)
		|| ast_strlen_zero(src_path) || ast_strlen_zero(dst_path)
		|| ast_strlen_zero(cmd_prefix)) {
		ast_log(LOG_ERROR, "file_path is null");
		return -1;
	}

	snprintf(cmd, sizeof(cmd), "%s %s  %s@%s::%s%s", cmd_prefix,src_path,backup_user, ast_strlen_zero(avs_address)?"127.0.0.1" : avs_address, module,dst_path);
	ast_log(LOG_DEBUG, "ast_sync_file_to_slave cmd:%s,src:%s,dst:%s,avs:%s",cmd,src_path,dst_path,avs_address);
	ast_safe_system2(cmd);

	return 0;
}

/*
file_name: delete filename
dst_path: local dir
*/
int ast_sync_delete_file(const char* cmd_prefix,const char* avs_address,char *file_name, char *dst_path,char *module)
{
	char backup_user[16] = "backup";
	char cmd[512] = {0};
	ast_log(LOG_DEBUG, "ast_sync_delete_file cmd");

	if (!avs_address || ast_strlen_zero(module)
		|| ast_strlen_zero(file_name) || ast_strlen_zero(dst_path)
		|| ast_strlen_zero(cmd_prefix)) {
		ast_log(LOG_ERROR, "file_path is null");
		return -1;
	}

	snprintf(cmd, sizeof(cmd), "%s --include=%s '--exclude=*' %s  %s@%s::%s", cmd_prefix,file_name,dst_path,backup_user, ast_strlen_zero(avs_address)?"127.0.0.1" : avs_address, module);
	ast_log(LOG_DEBUG, "ast_sync_delete_file cmd:%s",cmd);
	ast_safe_system2(cmd);
	return 0;
}

int ast_multi_avs_sync_record(char *file_path)
{
	if (!file_path) {
		ast_log(LOG_ERROR, "file_path is null");
	}

	if (strcasestr(file_path, "sounds")) {
		usleep(500000);
		ast_multi_avs_rsync_file(AST_MULTI_AVS_VOICE_PROMPT_FILE_ENUM,1);
	} else if (strcasestr(file_path, "moh")) {
		usleep(500000);
		ast_multi_avs_rsync_file(AST_MULTI_AVS_VOICE_MOH_FILE_ENUM,1);
	}

	return 0;
}

int ast_multi_avs_rsync_file(int i,int opt)
{
	if (AST_MULTI_AVS_FILE_BACKUP_MAX <= i || AST_MULTI_AVS_FAX_FILE_ENUM > i)
	{
		ast_log(LOG_DEBUG, "ast_multi_avs_rsync_file input is invalid.");
		return -1;
	}

	g_ast_avs_rsync_flag[i] = opt;

	return 0;
}

static int ast_multi_avs_rsync_task(int i)
{
	char backup_user[16] = "backup";
	struct slave_media_info *tmp_slave = NULL;
	char dest_path[64] = {0};	
	char cmd[512] = {0};
//	char full_path[256] = {0};

	AST_LIST_LOCK(&slave_media_list);
	AST_LIST_TRAVERSE(&slave_media_list, tmp_slave, entry) {
		if (tmp_slave && !ast_strlen_zero(tmp_slave->ip)) {
			snprintf(dest_path, sizeof(dest_path), "%s@%s::%s", backup_user, tmp_slave->ip, ast_g_BackupType[i]);
			if(i == AST_MULTI_AVS_SLAVE_RECORDING_FILE_ENUM || i == AST_MULTI_AVS_DUMP_FILE_ENUM) {//rsyn from slave
				snprintf(cmd, sizeof(cmd), "%s %s %s", AST_MULTI_AVS_RSYNC_CMD,  dest_path,ast_g_BackupPath[i]);
			}
			else if (i == AST_MULTI_AVS_VOICE_MAIL_FILE_ENUM){
				snprintf(cmd, sizeof(cmd), "%s %s %s", AST_MULTI_AVS_RSYNC_DEL_CMD, ast_g_BackupPath[i], dest_path);//Bug 312085  voicemail 
			}
			else
			{
				snprintf(cmd, sizeof(cmd), "%s %s %s", AST_MULTI_AVS_RSYNC_CMD, ast_g_BackupPath[i], dest_path);
			}
			ast_safe_system2(cmd);
			ast_log(LOG_DEBUG, "ast_multi_avs_rsync_task cmd %s",cmd);
		}
	}
	AST_LIST_UNLOCK(&slave_media_list);
	return 0;
}

static void *ast_multi_avs_sync_file_task(void *data)
{
	int i = 0;
	int retry = 5;
	int reset_sec = 0;
	char cmd[512] = {0};

	while(1)
	{
		if(total_mediarecords > 0 ) {
			ast_multi_avs_rsync_file(AST_MULTI_AVS_SLAVE_RECORDING_FILE_ENUM,0);//rsync all time
			reset_sec = 30;
		}
		else {
			ast_multi_avs_rsync_file(AST_MULTI_AVS_SLAVE_RECORDING_FILE_ENUM,0);
			/*Bug 303476,Start added by zyxia ,2024.6.18 */
			if(reset_sec > 0) {
				reset_sec --;
				if(reset_sec == 0)
				{
					snprintf(cmd, sizeof(cmd), "rm %s/*", AST_MULTI_AVS_SLAVE_RECORDING_FILE_PATH);
					ast_log(LOG_DEBUG, "ast_multi_avs_sync_file_task cmd:%s",cmd);
					ast_safe_system2(cmd);
				}
			}
			/*Bug 303476,End added by zyxia ,2024.6.18 */
		}
		
		for (i = 0; i < AST_MULTI_AVS_FILE_BACKUP_MAX; i++)
		{
			if (1 == g_ast_avs_rsync_flag[i])
			{
				retry = 5;
				if(i != AST_MULTI_AVS_SLAVE_RECORDING_FILE_ENUM && i != AST_MULTI_AVS_DUMP_FILE_ENUM)
					g_ast_avs_rsync_flag[i] = 0;
				ast_log(LOG_DEBUG, "ast_multi_avs_sync_file_task [%d].",i);
				while (retry > 0 && 0 != ast_multi_avs_rsync_task(i))
				{
					sleep(1);
					retry--;
					ast_log(LOG_ERROR, "ast_multi_avs_sync_file_task error, left try %d!", retry);
				}
			}
		}

		sleep(1);
	}

	return NULL;
}
static void multi_media_shutdown(void)
{
	struct slave_media_info *tmp_slave = NULL;

	while ((tmp_slave = AST_LIST_REMOVE_HEAD(&slave_media_list, entry))) {
		ast_free(tmp_slave);
	}

}
#ifdef HAVE_UNIXODBC

struct generic_prepare_struct {
	char *sql;
	int argc;
	char **argv;
};

static SQLHSTMT generic_prepare(struct odbc_obj *obj, void *data)
{
	struct generic_prepare_struct *gps = data;
	int res, i;
	SQLHSTMT stmt;

	res = SQLAllocHandle(SQL_HANDLE_STMT, obj->con, &stmt);
	if (!SQL_SUCCEEDED(res)) {
		ast_log(AST_LOG_WARNING, "SQL Alloc Handle failed!\n");
		return NULL;
	}
	res = SQLPrepare(stmt, (unsigned char *) gps->sql, SQL_NTS);
	if (!SQL_SUCCEEDED(res)) {
		ast_log(AST_LOG_WARNING, "SQL Prepare failed![%s]\n", gps->sql);
		SQLFreeHandle(SQL_HANDLE_STMT, stmt);
		return NULL;
	}
	for (i = 0; i < gps->argc; i++)
		SQLBindParameter(stmt, i + 1, SQL_PARAM_INPUT, SQL_C_CHAR, SQL_CHAR, strlen(gps->argv[i]), 0, gps->argv[i], 0, NULL);

	return stmt;
}

void ast_media_slave_list_init(void){
		struct slave_media_info *tmp_slave = NULL;

		char *sqlcmd = NULL;
		struct odbc_obj *obj = NULL;
		SQLHSTMT stmt = NULL;
		char row_ip[MAX_IPADDR_V6_LEN] = {0};
		ast_log(LOG_ERROR, "ast_media_slave_list_init ");

		int res = -1;
		ast_asprintf(&sqlcmd,"select slave_media_ip from %s",SLAVE_MEDIA_INFO_DB);

		struct generic_prepare_struct gps = { .sql = sqlcmd };

		obj = ast_core_odbc_request_obj(UCM_CONFIG_DB, 0);
		if (!obj) {
			ast_log(LOG_ERROR, "ast_media_slave_list_init Failed to obtain database object for '%s'!\n", UCM_CONFIG_DB);
			return ;
		}
		stmt = ast_core_odbc_prepare_and_execute(obj, generic_prepare, &gps);
		if (!stmt) {
			ast_log(LOG_ERROR, "ast_media_slave_list_init SQL Execute error!\n[%s]\n\n", sqlcmd);
			goto bail;
		}
		while((res = SQLFetch(stmt)) != SQL_NO_DATA && SQL_SUCCEEDED(res)) {
			memset(row_ip, 0, sizeof(row_ip));
			/*member_extension in database can be NULL!*/
			SQLGetData(stmt, 1, SQL_CHAR, row_ip, sizeof(row_ip), NULL);
			ast_log(LOG_DEBUG, "row_ip:%s ", row_ip);
			if (row_ip[0] == '\0')
			{
				continue;
			}
			tmp_slave = ast_calloc(1, sizeof(struct slave_media_info));
			ast_copy_string(tmp_slave->ip, row_ip, sizeof(tmp_slave->ip));

			AST_LIST_LOCK(&slave_media_list);
			AST_LIST_INSERT_TAIL(&slave_media_list, tmp_slave, entry);
			AST_LIST_UNLOCK(&slave_media_list);
		}

		SQLFreeHandle(SQL_HANDLE_STMT, stmt);
	bail:
		ast_core_odbc_release_obj(obj);
		AST_SAFE_FREE(sqlcmd);

}
#endif

NETWORK_MODE_E ast_network_mode_get(void)
{
	return network_mode;
}

void ast_network_mode_init(void)
{
	char network_pvalue[8] = { 0 };
	ast_safe_nvram_get("231", network_pvalue, sizeof(network_pvalue) - 1);

	if(!strcasecmp(network_pvalue,"1"))//switch
		network_mode = SWITCH_MODE;
	else
		network_mode = MULTI_MODE;
	ast_log(LOG_DEBUG, "ast_network_mode_get :%d\n",network_mode);
	return ;
}

int ast_multi_media_init(void)
{
	ast_media_role_init();
	ast_network_mode_init();
	if(g_media_role == MEDIA_ROLE_MASTER) {
		ast_media_slave_list_init();
		if (ast_pthread_create_background(&ast_multiavs_rsync_phread, NULL, ast_multi_avs_sync_file_task, NULL) < 0) 
		{
			ast_log(LOG_ERROR, "Could not create ast_ha_sync_file_task thread\n");
		}
	}

	ast_register_cleanup(multi_media_shutdown);

	return 0;
}



