/****************************************************************************
 *
 * FILENAME:        $RCSfile: database.h,v $
 *
 * LAST REVISION:   $Revision: 1.0 $
 * LAST MODIFIED:   $Date: 2013/06/03 09:24:42 $
 *
 * DESCRIPTION:
 *
 * vi: set ts=4:
 *
 * Copyright (c) 2012-2013 by Grandstream Networks, Inc.
 * All rights reserved.
 *
 * This material is proprietary to Grandstream Networks, Inc. and,
 * in addition to the above mentioned Copyright, may be
 * subject to protection under other intellectual property
 * regimes, including patents, trade secrets, designs and/or
 * trademarks.
 *
 * Any use of this material for any purpose, except with an
 * express license from Grandstream Networks, Inc. is strictly
 * prohibited.
 *
 ***************************************************************************/
#include "asterisk.h"
	
	
#include "asterisk/paths.h"	/* use ast_config_AST_CONFIG_DIR */
#include "asterisk/network.h"	/* we do some sockaddr manipulation here */
#include <time.h>
#include <sys/stat.h>
	
#include <math.h>	/* HUGE_VAL */
#include <regex.h>
	
#define AST_INCLUDE_GLOB 1
	
#include "asterisk/config.h"
#include "asterisk/cli.h"
#include "asterisk/lock.h"
#include "asterisk/utils.h"
#include "asterisk/channel.h"
#include "asterisk/app.h"
#include "asterisk/astobj2.h"
#include "asterisk/strings.h"	/* for the ast_str_*() API */
#include "asterisk/netsock2.h"
#include <asterisk/ucm_dbi.h>

AST_MODULE_LOG("dbi")

int sqlContainComment(const char* sql);
char* stripSqlAfterSemicolon(const char* sqlcmd);


int sqlContainComment(const char* sql)
{
	char* str = NULL;

	char *comment = strstr(sql, "--");
	if (NULL == comment)
	{
	    return 0;
	}

	char* begin_quote = strchr(sql, '\'');
	if (NULL == begin_quote)
	{
	    return 1;
	}

	char *end_quote = strchr(begin_quote+1, '\'');
	if (NULL == end_quote)
	{
	    //cgilog(SYSLOG_DEBUG, "invalid sql:%s!", sql);
	    return 1;
	}

	while(1)
	{
		if (comment > end_quote)
		{
			str = end_quote+1;
			if (*str != '\0')
			{
				begin_quote = strchr(str, '\'');
				if (NULL == begin_quote)
				{
				    return 1;
				}
				end_quote = strchr(begin_quote+1, '\'');
				if (NULL == end_quote)
				{
				    //cgilog(SYSLOG_DEBUG, "invalid sql:%s!", sql);
				    return 1;
				}
			}
			else
			{
				return 1;
			}
		}

		if (comment > begin_quote && comment < end_quote)
		{
			str = end_quote+1;
			if (*str != '\0')
			{
			comment = strstr(str, "--");
			}
			else
			{
			return 0;
			}
			if (NULL == comment)
			{
			return 0;
			}
		}

		if (comment < begin_quote)
		{
			return 1;
		}
	}
}

/* need to free */
 char* stripSqlAfterSemicolon(const char* sqlcmd)
{
	char* str = NULL;

	char* sql = ast_strdup(sqlcmd);
	char *semicolon = strchr(sql, ';');
	if (NULL == semicolon)
	{
		return sql;
	}

	char* begin_quote = strchr(sql, '\'');
	if (NULL == begin_quote)
	{
		*semicolon = 0;
		return sql;
	}

	char *end_quote = strchr(begin_quote+1, '\'');
	if (NULL == end_quote)
	{
		ast_log(LOG_DEBUG, "invalid sql:%s!", sql);
		return sql;
	}

	while(1)
	{
		if (semicolon > end_quote)
		{
			str = end_quote+1;
			if (*str != '\0')
			{
				begin_quote = strchr(str, '\'');
				if (NULL == begin_quote)
				{
					*semicolon = 0;
					break;
				}
				end_quote = strchr(begin_quote+1, '\'');
				if (NULL == end_quote)
				{
					ast_log(LOG_DEBUG, "invalid sql:%s!", sql);
					break;
				}
			}
			else
			{
				break;
			}
		}

		if (semicolon > begin_quote && semicolon < end_quote)
		{
			str = end_quote+1;
			if (*str != '\0')
			{
				semicolon = strchr(str, ';');
			}
			else
			{
				break;
			}
			if (NULL == semicolon)
			{
				break;
			}
		}

		if (semicolon < begin_quote)
		{
			*semicolon = 0;
			break;
		}
	}
	return sql;
}

#if 0
 dbi_conn ast_dbi_connect_db(const char *dbName, dbi_inst* pInst)
{
	dbi_conn conn = NULL;

	if (dbName == NULL || dbName[0] == '\0' )
	{
		ast_log(LOG_ERROR, "DataBase parameters are invalid!!." );
		return NULL;
	}

	char dbdPath[256] = { 0 };

	snprintf(dbdPath, sizeof(dbdPath) - 1, "%s/lib/dbd", DBI_DBD_PATH_PREFIX);
	dbi_initialize_r(dbdPath, pInst);

	conn = dbi_conn_new_r("mysql", *pInst);
	if ( conn == NULL )
	{
		ast_log(LOG_ERROR, "DBI: new connection error !" );
		return NULL;
	}

	dbi_conn_set_option(conn, "host", MYSQL_HOST);
	dbi_conn_set_option(conn, "username", MYSQL_USER_NAME);
	dbi_conn_set_option(conn, "password", MYSQL_USER_PASSWD);
	dbi_conn_set_option(conn, "mysql_unix_socket", MYSQL_SOCK_PATH);

	dbi_conn_set_option(conn, "dbname", dbName);
	dbi_conn_set_option(conn, "encoding", "UTF-8");
	dbi_conn_set_option_numeric(conn, "timeout", 60*1000);


	if(dbi_conn_connect(conn) < 0)
	{
		const char *errorMsg = NULL;
		int errorNum = dbi_conn_error(conn, &errorMsg);
		ast_log(LOG_ERROR, "Could not connect. Please check the option settings, Error code is %d, Msg:{%s}",
		       errorNum, errorMsg == NULL ? "" : errorMsg);
		dbi_shutdown_r(*pInst);
		return NULL;
	}


	ast_log(LOG_DEBUG, "ast_dbi_connect_db connect to %s", dbName);

	return conn;

}
int ast_dbi_runSQLCmd(dbi_conn connection, const char *sqlCmd)
{
	if (sqlCmd == NULL || sqlCmd[0] == '\0' )
	{
		ast_log(LOG_ERROR, "ast_runSQLCmd parametrs error!");
		return -1;
	}

	int ret = -1;

	ast_log(LOG_DEBUG, "Enter runSQLCmd : cmd is {%s}", sqlCmd);

	dbi_result result = NULL;
	if (sqlContainComment(sqlCmd))
	{
		/* don't allow "--", may be sql injection attack. see bug 32406. */
		ast_log(LOG_ERROR, "Detected sql injection attack!");
		return ret;
	}
	/* only allow the first sql statement. */
	char* sql = stripSqlAfterSemicolon(sqlCmd);

	result = dbi_conn_query(connection, sql);
	if (result)
	{
		dbi_result_free(result);
		 ast_log(LOG_DEBUG, "Update success !");
		ret = 0;
	}
	else
	{
		/*Error*/
		const char *errorMsg = NULL;
		int errorNum = dbi_conn_error(connection, &errorMsg);
		ast_log(LOG_ERROR, "runSQLCmd : error message is %s, ID is %d,sql:%s", errorMsg ? errorMsg : "", errorNum,sql);
	}
	if (sql)
	{
		ast_free(sql);
	}

	return ret;
}

int ast_dbi_disconnect_db(dbi_conn connection, dbi_inst* pInst)
{
	if (connection != NULL)
	{
		dbi_conn_close(connection);
		connection = NULL;
	}

	dbi_shutdown_r(*pInst);
	return 0;

}

int ast_dbi_rollbackTransaction(dbi_conn connection)
{

	if ( connection != NULL )
	{
		return dbi_conn_transaction_rollback(connection);
	}

	ast_log(LOG_ERROR, "Transaction Rollback --> failed !");

	return -1;

}
int ast_dbi_beginTransaction(dbi_conn connection)
{
	int ret = -1;
	int retry = 0;

	if (connection != NULL)
	{
		//while(CGICODE_SUCCESS != (ret = runSQLCmd( "BEGIN IMMEDIATE", task->idx )))
		while(ret)
		{
			ret = ast_dbi_runSQLCmd(connection, BEGIN_TRANSACTIONS);
			if ( retry < 5)
			{
				/* there may be another process operating the database.  */
				usleep(300000); // 0.3 second
			}
			else
			{
				/* max retry times is 5 */
				break;
			}
			retry++;
		}

		if (ret)
		{
			ast_log(LOG_ERROR, "Transaction Begin --> failed !");
		}
			return ret;
	}

	return -1;
}


int ast_dbi_commitTransaction(dbi_conn connection)
{
	//cgilog(SYSLOG_DEBUG, "Transaction Commit !");
	int ret = -1;
	if ( connection != NULL )
	{
		ret = dbi_conn_transaction_commit(connection);
		if (ret < 0)
		{
		/*Error*/
		const char *errorMsg = NULL;
		int errorNum = dbi_conn_error(connection, &errorMsg);
		ast_log(LOG_ERROR, "Transaction Commit : error message is %s, ID is %d", errorMsg ? errorMsg : "", errorNum);
		}

		return 0;
	}

	ast_log(LOG_ERROR, "Transaction Commit --> failed !");

	return ret;

}

int ast_dbi_update_db(dbi_conn connection,const char *sql)
{
	if(sql == NULL || connection == NULL)
	{

		ast_log(LOG_ERROR, "ast_dbi_update_db sql or connection is NULL !\n");
		return -1;
	}
	if(ast_dbi_beginTransaction(connection))
	{
		ast_log(LOG_ERROR, "ast_dbi_update_db,begin error!\n");
		return -1;
	}
	ast_log(LOG_VERBOSE, "ast_dbi_update_db sql command: %s \n", sql);

	if(ast_dbi_runSQLCmd(connection, sql))
	{
		ast_log(LOG_WARNING, "ast_dbi_update_db failed!\n");
		return -1;
	}

	if(ast_dbi_commitTransaction(connection))
	{
		ast_log(LOG_WARNING, "ast_dbi_update_db commit error!\n");
		ast_dbi_rollbackTransaction(connection);
		return -1;
	}
	return 0;
}

int ast_dbi_getItemNumberInternal(dbi_conn connection,char *cmd,char *key)
{
	int item = 0;
	char* sql;
	dbi_result result = NULL;
	
	if (sqlContainComment(cmd))
	{
		/* don't allow "--", may be sql injection attack. see bug 32406. */
		ast_log(LOG_ERROR, "Detected sql injection attack!");
		return -1;
	}
	/* only allow the first sql statement. */
	sql = stripSqlAfterSemicolon(cmd);

	result = dbi_conn_query(connection, sql);
	if (result)
	{
		while (dbi_result_next_row(result))
		{
			item = dbi_result_get_int(result, key);
			break;
		}

		dbi_result_free(result);
	}
	else
	{
		/*Error*/
		const char *errorMsg = NULL;
		int errorNum = dbi_conn_error(connection, &errorMsg);
		ast_log(LOG_ERROR, "ast_dbi_get_ItemNumberInternal : error message is %s, ID is %d,sql:%s", errorMsg ? errorMsg : "", errorNum,sql);
	}

	ast_log(LOG_DEBUG, "ast_dbi_getItemNumberInternal item is {%d}", item );

	if (sql)
	{
		ast_free(sql);
	}

	return item;
}


/* get the total item in this table */
int ast_dbi_getItemNumber_v2( dbi_conn connection, const char *table, const char *parameter)
{
	if (table == NULL)
	{
		ast_log(LOG_ERROR, "ast_dbi_getItemNumber_v2 parametrs error,table is NULL!");
		return -1;
	}

	char cmd[2048] = {0};
	int item = 0;

	snprintf(cmd, sizeof(cmd), "SELECT * FROM %s %s", table, parameter ? parameter : "");

	ast_log(LOG_DEBUG, "ast_dbi_getItemNumber_v2  : cmd is {%s}", cmd );

	dbi_result result = NULL;

	if (sqlContainComment(cmd))
	{
		/* don't allow "--", may be sql injection attack. see bug 32406. */
		ast_log(LOG_ERROR,"Detected sql injection attack!");
		return -1;
	}
	/* only allow the first sql statement. */
	char* sql = stripSqlAfterSemicolon(cmd);

	result = dbi_conn_query(connection, sql);
	if (result)
	{
		item = dbi_result_get_numrows(result);
		dbi_result_free(result);
	}
	else
	{
		/*Error*/
		const char *errorMsg = NULL;
		int errorNum = dbi_conn_error(connection, &errorMsg);
		ast_log(LOG_ERROR, "ast_dbi_getItemNumber_v2 : error message is %s, ID is %d,sql:%s", errorMsg ? errorMsg : "", errorNum,sql);
	}

	ast_log(LOG_DEBUG, "ast_dbi_getItemNumber_v2 item is {%d}", item );

	if (sql)
	{
		ast_free(sql);
	}

	return item;
}

 int ast_dbi_getOneIntField( dbi_conn connection, const char *sqlCmd, char* fieldName )
{
	int ret = -1;
	dbi_result result = NULL;

	if ( connection == NULL || sqlCmd == NULL )
	{
		ast_log(LOG_ERROR, "ast_dbi_getOneIntField : param error!");
		return -1;
	}


	if (sqlContainComment(sqlCmd))
	{
		/* don't allow "--", may be sql injection attack. see bug 32406. */
		ast_log(LOG_ERROR, "Detected sql injection attack!");
		return -1;
	}
	/* only allow the first sql statement. */
	char* sql = stripSqlAfterSemicolon(sqlCmd);

	result = dbi_conn_query(connection, sql);
	if (result)
	{
		while ( dbi_result_next_row(result) )
		{
			ret = (int)dbi_result_get_longlong(result, fieldName);
			break;
		}

		dbi_result_free(result);
	}
	else
	{
		/*Error*/
		const char *errorMsg = NULL;
		int errorNum = dbi_conn_error(connection, &errorMsg);
		ast_log(LOG_ERROR, "ast_dbi_getOneIntField : error message is %s, ID is %d,sql:%s", errorMsg ? errorMsg : "", errorNum,sql);
	}
	if(sql)
	{
		ast_free(sql);
	}

	return ret;
}

/*Need to free the return pointer*/
char *ast_dbi_getOneVarcharField( dbi_conn connection, const char *sqlCmd, const char* fieldName )
{
	char *ret = NULL;
	dbi_result result = NULL;

	if ( connection == NULL || sqlCmd == NULL )
	{
		ast_log(LOG_ERROR, "ast_dbi_getOneVarcharField : param error!");
		return ret;
	}

	ast_log(LOG_DEBUG, "Enter ast_dbi_getOneVarcharField : cmd is {%s}", sqlCmd );

	if (sqlContainComment(sqlCmd))
	{
		/* don't allow "--", may be sql injection attack. see bug 32406. */
		ast_log(LOG_ERROR, "Detected sql injection attack!");

		return ret;
	}
	/* only allow the first sql statement. */
	char* sql = stripSqlAfterSemicolon(sqlCmd);

	result = dbi_conn_query(connection, sql);
	if (result)
	{
	while ( dbi_result_next_row(result) )
	{
		ret = dbi_result_get_as_string_copy(result, fieldName);
		break;
	}

	dbi_result_free(result);
	}
	else
		{
		/*Error*/
		const char *errorMsg = NULL;
		int errorNum = dbi_conn_error(connection, &errorMsg);
		ast_log(LOG_ERROR, "runSQLGetOneVarcharField : error message is %s, ID is %d,sql:%s", errorMsg ? errorMsg : "", errorNum,sql);
	}

	if(sql) {
		ast_free(sql);
	}

	return ret;
}

/* get the total item in this table */
int ast_dbi_getItemNumber(dbi_conn connection, const char *sqlCmd)
{
	int item = 0;
	dbi_result result = NULL;

	if (NULL == connection || NULL == sqlCmd)
	{
		ast_log(LOG_ERROR, "parametrs error!");
		return -1;
	}

	if (sqlContainComment(sqlCmd))
	{
		/* don't allow "--", may be sql injection attack. see bug 32406. */
		ast_log(LOG_ERROR, "Detected sql injection attack!");
		return -1;
	}
	/* only allow the first sql statement. */
	char* sql = stripSqlAfterSemicolon(sqlCmd);

	result = dbi_conn_query(connection, sql);
	if (result)
	{
		item = dbi_result_get_numrows(result);
		dbi_result_free(result);
	}
	else
	{
		/*Error*/
		const char *errorMsg = NULL;
		int errorNum = dbi_conn_error(connection, &errorMsg);
		ast_log(LOG_ERROR, "ast_dbi_getItemNumber : error message is %s, ID is %d,sql:%s", errorMsg, errorNum,sql);
	}

	if (sql)
	  ast_free(sql);

	return item;
}


int time_t2Timestamp(time_t time, char *timestamp, int stamp_len)
{
	if (timestamp != NULL)
	{
		struct tm ptm;
		gmtime_r(&time, &ptm);
		snprintf(timestamp, stamp_len, "%d-%02d-%02d %02d:%02d:%02d", (1900 + ptm.tm_year)
			 , (1 + ptm.tm_mon)
			 , ptm.tm_mday
			 , ptm.tm_hour
			 , ptm.tm_min
			 , ptm.tm_sec);
		ast_log(LOG_DEBUG, "time_t2Timestamp: The timestamp is: %s.", timestamp);

		return 0;
	}
	else
	{
		ast_log(LOG_ERROR, "time_t2Timestamp: The timestamp is: NULL.");
	}
	return -1;
}


/* get the total item in this table */
int ast_dbi_get_table(dbi_conn connection,const char *sqlCmd,struct ast_json **resultJson,int *pnRow,int *pnColumn)
{
	int numRow = 0;
	int numColumns= 0;
	dbi_result result = NULL;
	time_t time;
	char *tmp = NULL;
	if( pnColumn ) *pnColumn = 0;
	if( pnRow ) *pnRow = 0; 

	if (NULL == connection || NULL == sqlCmd || resultJson == NULL)
	{
		ast_log(LOG_ERROR, "parametrs error!");
		return -1;
	}

	if (sqlContainComment(sqlCmd))
	{
		/* don't allow "--", may be sql injection attack. see bug 32406. */
		ast_log(LOG_ERROR, "Detected sql injection attack!");
		return -1;
	}
	/* only allow the first sql statement. */
	char* sql = stripSqlAfterSemicolon(sqlCmd);
	result = dbi_conn_query(connection, sql);
	if (result)
	{
		numRow = dbi_result_get_numrows(result);
		numColumns = dbi_result_get_numfields(result);
		char *fieldName[numColumns + 1];
		int fieldType[numColumns + 1];
		
		memset(fieldName, 0, (numColumns + 1) * sizeof(char*));
		memset(fieldType, 0, (numColumns + 1) * sizeof(int));
		
		/*Show the field name*/
		int i = 1;
		for (i=1; i <= numColumns; i++)
		{
			fieldName[i] = ast_strdup(dbi_result_get_field_name(result, i));
			fieldType[i] = dbi_result_get_field_type_idx(result, i);
		}

		*resultJson = ast_json_array_create();
		int row=0;
		while (dbi_result_next_row(result)) {

				for (i = 1; i <= numColumns; i++) {
					if ( 1 == dbi_result_field_is_null( result, fieldName[i] ) )
					{
						ast_json_array_append(*resultJson, ast_json_string_create(""));
					    continue;
					}
					int attribs = dbi_result_get_field_attribs( result, fieldName[i] );

					switch (fieldType[i]) {
						case DBI_TYPE_INTEGER:
						if (attribs == DBI_INTEGER_SIZE8) {
							ast_json_array_append(*resultJson, ast_json_integer_create((int)dbi_result_get_longlong(result, fieldName[i])));
						}
						else {
							ast_json_array_append(*resultJson, ast_json_integer_create(dbi_result_get_int(result, fieldName[i])));
						}
						break;
					case DBI_TYPE_DECIMAL:
						if (attribs == DBI_DECIMAL_SIZE8) {
							//ast_json_object_set(subJson, fieldName[i], json_object_new_double(dbi_result_get_double(result, fieldName[i])));
						}
						else {
							//ast_json_object_set(subJson, fieldName[i], json_object_new_double((double)dbi_result_get_float(result, fieldName[i])));
						}

						//cgilog(SYSLOG_DEBUG, "[DECIMAL]Result ====> (%s): [%f]", fieldName[i], dbi_result_get_float(result, fieldName[i]));
						break;
					case DBI_TYPE_STRING:
						tmp = dbi_result_get_string(result, fieldName[i]);
						ast_json_array_append(*resultJson, ast_json_string_create(tmp == NULL ? "" : tmp));
					//dbi_result_get_binary(result, fieldName[i]);
					break;
					case DBI_TYPE_DATETIME:
						time = dbi_result_get_datetime(result, fieldName[i]);
						char timestamp[128] = {0};
						if (0 == time)
						{
							snprintf(timestamp, sizeof(timestamp), "--");
						}
						else
						{
							time_t2Timestamp(time, timestamp, sizeof( timestamp ));
						}
						ast_json_array_append(*resultJson, ast_json_string_create(timestamp));
					//ast_json_object_set(subJson, fieldName[i], ast_json_string_create(timestamp));
					break;
					}
				}
		}
		dbi_result_free(result);
		int t =1;
		for (t = 1; t <= numColumns; t++)
		{
			if (fieldName[t] != NULL)
			{
				ast_free(fieldName[t]);
				fieldName[t] = NULL;
			}
		}

#if 0
		int j=0;
		//struct ast_json *module_name = ast_json_object_get(*resultJson, "module_name");

		ast_log(LOG_ERROR,"jspon_arra size: %d",ast_json_array_size(*resultJson));
		for (j = 0; j < ast_json_array_size(*resultJson); ++j) {
			tmp = ast_json_string_get(ast_json_array_get(*resultJson, j));
			ast_log(LOG_ERROR, "Exit runSQLGetAllItem : result[%d] is <%s>", j,tmp == NULL ? "empty" : tmp );
		}
#endif
	}
	else
	{
		/*Error*/
		const char *errorMsg = NULL;
		int errorNum = dbi_conn_error(connection, &errorMsg);
		ast_log(LOG_ERROR, "ast_dbi_getItemNumber : error message is %s, ID is %d,sql:%s", errorMsg, errorNum,sql);
	}

	if (sql)
		ast_free(sql);

	if( pnColumn ) 
		*pnColumn = numColumns;

	if( pnRow )
		*pnRow = numRow;

	return 0;
}


#if 0
/*Need to free the return resultJson*/
/* can get all the item */
int ast_dbi_getAllItem( dbi_conn connection, const char *sqlCmd, char***pazResult , int *pnRow,int *pnColumn)
{
	if ( connection == NULL || resultJson == NULL || sqlCmd == NULL )
	{
	    ast_log(LOG_ERROR, "ast_dbi_getAllItem : param error!");
	    return -1;
	}
	const char *tmp = NULL;
	time_t time;
	int ret = -1;
	*pazResult = 0
	TabResult res;

	ast_log(LOG_DEBUG, "Enter ast_dbi_getAllItem : cmd is {%s}", sqlCmd );

	dbi_result result = NULL;

	if (sqlContainComment(sqlCmd))
	{
	    /* don't allow "--", may be sql injection attack. */
	    ast_log(LOG_DEBUG, "Detected sql injection attack!");
	    ret = -1;
	    return ret;
	}
	/* only allow the first sql statement. */
	char* sql = stripSqlAfterSemicolon(sqlCmd);

	result = dbi_conn_query(connection, sql);

	if (result)
	{
		int numColumns = dbi_result_get_numfields(result);
		char *fieldName[numColumns + 1];
		int fieldType[numColumns + 1];

		memset(fieldName, 0, (numColumns + 1) * sizeof(char*));
		memset(fieldType, 0, (numColumns + 1) * sizeof(int));

		/*Show the field name*/
		for (int i = 1; i <= numColumns; i++)
		{
			fieldName[i] = ast_strdup(dbi_result_get_field_name(result, i));
			fieldType[i] = dbi_result_get_field_type_idx(result, i);
		}
		res.nRow = 0;
		res.nColumn = 0;
		res.nData = 1;
		res.nAlloc = 20;
		res.azResult = malloc(sizeof(char*)*res.nAlloc ); 
		if( res.azResult==0 )
			ast_log(LOG_ERROR, "ast_dbi_getAllItem malloc \n");
			return -1;
		}
		res.azResult[0] = 0;

        while (dbi_result_next_row(result))
        {


            for (int i = 1; i <= numColumns; i++)
            {
                if ( 1 == dbi_result_field_is_null( result, fieldName[i] ) )
                {
                    json_object_object_add(subJson, fieldName[i], NULL);
                    continue;
                }
                int attribs = dbi_result_get_field_attribs( result, fieldName[i] );

                switch (fieldType[i])
                {
                   case DBI_TYPE_INTEGER:
                        if (attribs == DBI_INTEGER_SIZE8)
                        {
                            json_object_object_add(subJson, fieldName[i], json_object_new_int((int)dbi_result_get_longlong(result, fieldName[i])));
                        }
                        else
                        {
                            json_object_object_add(subJson, fieldName[i], json_object_new_int(dbi_result_get_int(result, fieldName[i])));
                        }

                        //cgilog(SYSLOG_DEBUG, "[INTEGER]Result ====> (%s): [%d]", fieldName[i], dbi_result_get_int(result, fieldName[i]));
                        break;
                    case DBI_TYPE_DECIMAL:
                        if (attribs == DBI_DECIMAL_SIZE8)
                        {
                            json_object_object_add(subJson, fieldName[i], json_object_new_double(dbi_result_get_double(result, fieldName[i])));
                        }
                        else
                        {
                            json_object_object_add(subJson, fieldName[i], json_object_new_double((double)dbi_result_get_float(result, fieldName[i])));
                        }

                        //cgilog(SYSLOG_DEBUG, "[DECIMAL]Result ====> (%s): [%f]", fieldName[i], dbi_result_get_float(result, fieldName[i]));
                        break;
                    case DBI_TYPE_STRING:
                        tmp = dbi_result_get_string(result, fieldName[i]);
                        json_object_object_add(subJson, fieldName[i], json_object_new_string(tmp == NULL ? "" : tmp));
                        //cgilog(SYSLOG_DEBUG, "[STRING]Result ====> (%s): [%d]", fieldName[i], tmp == NULL ? "" : tmp);
                        break;
                    case DBI_TYPE_BINARY:
                        //dbi_result_get_binary(result, fieldName[i]);
                        break;
                    case DBI_TYPE_DATETIME:
                        time = dbi_result_get_datetime(result, fieldName[i]);
                        char timestamp[128] = {0};
                        if (0 == time)
                        {
                            snprintf(timestamp, sizeof(timestamp), "--");
                        }
                        else
                        {
                            time_t2Timestamp(time, timestamp, sizeof( timestamp ));
                        }
                        json_object_object_add(subJson, fieldName[i], json_object_new_string(timestamp));
                        break;
                }
            }
        }

        dbi_result_free(result);

        for (int t = 1; t <= numColumns; t++)
        {
            if (fieldName[t] != NULL)
            {
                free(fieldName[t]);
                fieldName[t] = NULL;
            }
        }

        tmp = json_object_get_string(*resultJson);
        cgilog(SYSLOG_DEBUG, "Exit runSQLGetAllItem : result is <%s>", tmp == NULL ? "empty" : tmp );
        ret = CGICODE_SUCCESS;
    }
    else
    {
        /*Error*/
        const char *errorMsg = NULL;
        int errorNum = dbi_conn_error(connection, &errorMsg);
        cgilog(SYSLOG_DEBUG, "runSQLGetAllItem : error message is %s, ID is %d", errorMsg, errorNum );
        handle_constraint_error(&ret, errorNum);
    }
    cgilog(SYSLOG_DEBUG, "EXIT runSQLGetAllItem" );
	if(sql)
     fsree(sql);

    return ret;
}
#endif
#endif

