#ifdef GRANDSTREAM_NETWORKS
#include "asterisk.h"

#include "asterisk/channel.h"
#include "asterisk/bridge.h"
#include "asterisk/stasis.h"
#include "asterisk/stasis_channels.h"
#include "asterisk/stasis_bridges.h"
#include "asterisk/manager.h"
#include "asterisk/stasis_message_router.h"
#include "asterisk/app.h"
#include "asterisk/astdb.h"
#include "asterisk/pjsip_cei.h"
#include "asterisk/stream.h"
#include "include/confbridge.h"
#include "include/conf_resource.h"
#include "asterisk/cli.h"

AST_MODULE_LOG("Bridge");

#define SOFTBRIDGE_VIDEO_DEST_PREFIX "softbridge_dest"
#define SOFTBRIDGE_VIDEO_DEST_LEN strlen(SOFTBRIDGE_VIDEO_DEST_PREFIX)
#define TALKING_USER_MAX_NUM (5)


char dev_info[MAX_DEV_NAME] = {0};

struct cei_video {
	char code_name[8];
	int frame_rate;
	enum video_resolution resolution;
};

struct cei_media {
	int video_capable;
	int video_muted;
	enum ast_media_type type;
	struct cei_video video_info;
};

/* STRUCTS */
struct cei_user {
	enum cei_user_type user_type;
	int user_num;                    //Meetme user number
	char *entity;                    //Entity string
	char cid_name[258];              //support utf-8 code
	char cid_number[33];
	char channel_name[101];
	char conference_id[41];          // conference joined
	char self_conf_id[41];           // seld conference id, if it's a remote user
	char user_hash[41];
	int admin;
	int user_role;
	int muted;
	int mic_muted;
	int user_status;
	int can_invite;
	int talking;
	int talking_rms;
	int sharing;
	int share_audio;
	int isfocus;
	time_t hand_up_time;
	struct cei_media media;
	enum ast_terminal_type term_type;
	char extension[33];
	char msid[AST_UUID_STR_LEN];
	char related_ext[MAX_RELATED_EXT];            /*!< The related exten for confXXXX exten */
	AST_LIST_ENTRY(cei_user) list;
};

struct cei_conference {
	int video_conference;
	char conference_num[100];         //Name of this conference, as configured in Meetme
	char conference_id[41];           //Conference hash
	char internal_publisher[41];      //Internal publisher's conference hash
	int locked;
	int muted;
	int local_size;
	int remote_size;                  //If remote, parsed local user-count
	int local_user_size;
	int remote_user_size;
	char record_type[MAX_RECORD_TYPE];
	time_t record_start_utc_timestamp;
	char *entity;
	int mcb_count;
	AST_LIST_ENTRY(cei_conference) list;
	int refcount;
	struct ao2_container *remote_container;
	struct ao2_container *user_container;
	int stop_propagating;
	int version;                      //If local, indicates the document version since a MCB entered the room. If remote, indicates last version received
	int version_reset;
	char start_time[32];
	char *web_page;
	char *feature_id;                //feature id in operation log
	struct cei_user *sharing_user;
	struct cei_conference* host_conf;
	char *subject;
	char *conf_config;               //configuration of the conference
	char *schedule_id;
	char *im_service_prefix;
	AST_LIST_HEAD_NOLOCK(, cei_talking_user) talking_user_list;
	unsigned short talking_users;
	char conf_layout[MAX_CONF_LAYOUT];
	int allow_user_unmute;
	int is_virtial_trunk_conf;
	int maximum_user_count;
	int act_event_type;
	char *conf_creator;
};
/* END STRUCTS */

struct cei_talking_user
{
	struct cei_user *user;
	AST_LIST_ENTRY(cei_talking_user) list;
};

struct cei_schedule_user {
	int sigined_in;
	char extension[AST_MAX_EXTENSION];
	char related_info[256];
	AST_LIST_ENTRY(cei_schedule_user) entry;
};

struct cei_schedule_conf {
	char conf_name[AST_MAX_EXTENSION];
	char schedule_id[256];
	AST_LIST_HEAD_NOLOCK(, cei_schedule_user) cei_schedule_user_list;
	AST_LIST_ENTRY(cei_schedule_conf) entry;
};
AST_LIST_HEAD_STATIC(cei_schedule_conf_list, cei_schedule_conf);

/* VARIABLES */
static AST_LIST_HEAD_STATIC(confs, cei_conference);
#define TRUE 1
#define FALSE 0
#define CEI_EVENT_USER_LEN 512
#define CEI_EVENT_DATA_LEN 9500
#define MAX_CONFNUM 80
//static int room_count = 0;
static int conf_max_capability = 0;
static struct stasis_message_router *bridge_state_router;
static struct stasis_message_router *channel_state_router;
static struct stasis_message_router *cei_mcb_process_router;
static struct stasis_message_router *cei_etc_process_router;
static struct stasis_message_router *cei_sub_router = NULL;
/* END VARIABLES */

/* FUNCTION DECLARATION */
static struct cei_conference* build_conf_by_num(const char *conf_no);
static int user_hash_cmp(void *obj, void *arg, int flags);
static int conference_id_cmp(void *obj, void *arg, int flags);
static void hash_user(struct cei_user *user);
static void poll_cei_conference(struct cei_conference *conf,struct ast_str **extra_text);
static void poll_cei_partial_notify(struct cei_conference *conf, struct cei_conference *remote_conf, struct cei_user *user,
										struct ast_str **data_string, enum cei_event_type event_type );
static void conf_free_all(void);
static int conf_free(struct cei_conference *conf);
static void clear_conf_users(struct cei_conference *conf);
static void process_remote_xml(struct cei_conference *conf, const char *publisher_private_num, const char *xml, struct ast_str **data_string);
static int check_xml_text_node(struct ast_xml_node *node);
static void process_remote_xml_users(struct cei_conference *conf, struct cei_conference *remote_conference, struct ast_xml_node *node_users);
static struct cei_user *process_remote_xml_user(struct cei_conference *host_conf, struct ast_xml_node *node_user);
static struct cei_conference* get_conf_by_id(const char *conf_id);
static void delete_remote_confs_by_publisher(struct cei_conference *host_conf, const char *publisher);
static void delete_remote_conf_by_id(struct cei_conference *host_conf, const char *remote_conf_id);
 struct cei_conference* get_conf_by_num(const char *conf_num);
static struct cei_user* get_user_by_cid_num(struct cei_conference *conf, const char *cid_num);
static struct cei_user* get_user_by_user_num(struct cei_conference *conf, int user_num);
static struct cei_user* get_user_by_channel_name(struct cei_conference *conf, const char *channel_name, int strict_cmp);
static void process_remote_string(struct cei_conference *conf,const char *publisher_num, const char *publisher_id,
										char *content, struct ast_str **data_string);
static int get_conf_max_capability(void);
static int check_role_as_conference(struct cei_user *user);
extern int ast_sip_cei_same_channel(const char* channel, const char* mychannel, const char *cid_number);
extern int ast_sip_get_nvram_value(char *pvalue, char* buf, int len );
static char *conf_get_http_uri(void);
static void remove_conference(struct cei_conference *conf);
static void process_trunk_conf_xml(struct cei_conference *conf, const char *publisher_private_num, const char *xml, struct ast_str **data_string,int *full);

/* END FUNCTION DECLARATION */

static void cei_publish_manager_event(struct ast_json * json_object)
{
	RAII_VAR(struct stasis_message *, msg, NULL, ao2_cleanup);
	RAII_VAR(struct ast_json_payload *, payload, NULL, ao2_cleanup);

	// Sanity check
	if (json_object == NULL) {
		ast_debug(LOG_ERROR, "json_object IS NULL");
		return;
	}

	const char *conference_name = ast_json_string_get(ast_json_object_get(json_object, "conference"));

	if (conference_name == NULL) {
		ast_debug(LOG_ERROR, "conference_name IS NULL");
		return;
	}

	payload = ast_json_payload_create(json_object);
	if(!payload) {
		return;
	}

	msg = stasis_message_create(ast_cei_notify_type(), payload);
	if (msg == NULL) {
		return;
	}

	stasis_publish(ast_cei_topic(conference_name), msg);
}

static void cei_update_media_info(struct cei_user *user, const char *conf_number)
{
	RAII_VAR(struct ast_channel *, chan, NULL,  ao2_cleanup);
	struct ast_stream_topology *topology = NULL;
	int i = 0;

	if (user->user_type != CEI_TYPE_NORMAL_USER) {
		return;
	}

	user->media.type = AST_MEDIA_TYPE_AUDIO;

	chan = ast_channel_get_by_name(user->channel_name);
	if (!chan) {
		ast_log(LOG_WARNING, "Couldn't find the channel '%s'", user->channel_name);
		return;
	}

	ast_channel_lock(chan);
	topology =  ast_channel_get_stream_topology(chan);
	if (!topology) {
		ast_channel_unlock(chan);
		return;
	}

	for (i = 0; i < ast_stream_topology_get_count(topology); ++i) {
		struct ast_stream *stream = ast_stream_topology_get_stream(topology, i);

		if (ast_stream_get_type(stream) == AST_MEDIA_TYPE_VIDEO
			&& ast_stream_get_sub_type(stream) == AST_MEDIA_SUB_VIDEO_TYPE_MAIN) {
			user->media.video_capable = 1;
			if (ast_stream_get_state(stream) != AST_STREAM_STATE_REMOVED &&
				ast_stream_get_state(stream) != AST_STREAM_STATE_SENDONLY &&
				ast_stream_get_state(stream) != AST_STREAM_STATE_INACTIVE) {
				struct ast_format *format = ast_format_cap_get_format(ast_stream_get_formats(stream), 0);
				if (format) {
					user->media.type = AST_MEDIA_TYPE_VIDEO;
					ast_copy_string(user->media.video_info.code_name, ast_format_get_name(format), sizeof(user->media.video_info.code_name));
					ao2_ref(format, -1);
				}
			}
			break;
		}
	}

	ast_channel_unlock(chan);

	if (user->media.type == AST_MEDIA_TYPE_VIDEO) {
		user->media.video_info.resolution = conf_user_get_video_quality(user->channel_name, conf_number);
	}
}

static const char *cei_terminate_type_string(enum ast_terminal_type type)
{
	switch (type) {
		case AST_TERMINAL_TYPE_SIP:
		case AST_TERMINAL_TYPE_ICE_TRUNK:
			return "normal";
		case AST_TERMINAL_TYPE_GVC:
			return "gvc";
		case AST_TERMINAL_TYPE_WEBRTC:
			return "gswave";
		case AST_TERMINAL_TYPE_GSWAVE_MOBILE:
			return "waveapp";
		/*start added by zyxia for rtsp,2021.9.15*/
		case AST_TERMINAL_TYPE_IPC:
			return "ipc";
		/*end added by zyxia for rtsp,2021.9.15*/
		case AST_TERMINAL_TYPE_UNKNOW:
		default:
			return "unknown";
	}
}

static enum ast_terminal_type cei_terminate_type_by_string(const char *string)
{
	if (ast_strlen_zero(string)) {
		return AST_TERMINAL_TYPE_UNKNOW;
	}

	if (!strcasecmp(string, "normal")) {
		return AST_TERMINAL_TYPE_SIP;
	} else if (!strcasecmp(string, "gvc")) {
		return AST_TERMINAL_TYPE_GVC;
	} else if (!strcasecmp(string, "gswave")) {
		return AST_TERMINAL_TYPE_WEBRTC;
	} else if (!strcasecmp(string, "waveapp")) {
		return AST_TERMINAL_TYPE_GSWAVE_MOBILE;
	}

	return AST_TERMINAL_TYPE_UNKNOW;
}

static enum ast_terminal_type cei_get_terminate_type(const char *channel_name)
{
	RAII_VAR(struct ast_channel *, c, NULL, ao2_cleanup);

	c = ast_channel_get_by_name(channel_name);
	if (!c) {
		ast_log(LOG_WARNING, "Couldn't find the channel '%s'", channel_name);
		return AST_TERMINAL_TYPE_UNKNOW;
	}

	return ast_channel_get_terminate_type(c);
}

static int cei_get_extension(const char *channel_name, char *buf, int size)
{
	const char *start = NULL;
	const char *end =  NULL;

	memset(buf, 0, size);

	if (!strncasecmp(channel_name, "PJSIP/", 6)) {
		start = channel_name + 6;
		end = strrchr(start, '-');
		if (!end) {
			return -1;
		}
		strncpy(buf, start, size > (end - start) ? (end - start) : size);
	} else if (!strncasecmp(channel_name, "Local/", 6)) {
		start = channel_name + 6;
		end = strrchr(start, '@');
		if (!end) {
			return -1;
		}
		strncpy(buf, start, size > (end - start) ? (end - start) : size);
		char * token = strrchr(buf, '-');
		if (token) {
			*token = '\0';
		}
	} else {
		return -1;
	}

	return 0;
}

static int cei_get_msid(const char *channel_name, char *buf, int size)
{
	RAII_VAR(struct ast_channel *, c, NULL, ao2_cleanup);
	struct ast_stream_topology *topology = NULL;
	int i = 0;
	int setted_msid = 0;
	struct ast_stream *auio_stream = NULL;

	memset(buf, 0, size);
	if (strncasecmp(channel_name, "PJSIP/", 6)  && strncasecmp(channel_name, "RTSP/", 5)) {
		return -1;
	}

	c = ast_channel_get_by_name(channel_name);
	if (!c) {
		ast_log(LOG_WARNING, "Couldn't find the channel '%s'", channel_name);
		return -1;
	}

	topology = ast_channel_get_stream_topology(c);
	if (!topology) {
		return -1;
	}

	for (i = 0; i < ast_stream_topology_get_count(topology); ++i) {
		struct ast_stream *stream = ast_stream_topology_get_stream(topology, i);
		if (ast_stream_get_sub_type(stream) == AST_MEDIA_SUB_AUDIO_TYPE) {
			auio_stream = stream;
		}
		if (ast_stream_get_sub_type(stream) == AST_MEDIA_SUB_VIDEO_TYPE_MAIN
			&& ast_stream_get_state(stream) != AST_STREAM_STATE_REMOVED
			&& ast_stream_get_state(stream) != AST_STREAM_STATE_SENDONLY
			&& ast_stream_get_state(stream) != AST_STREAM_STATE_INACTIVE
			&& strncmp(ast_stream_get_name(stream), SOFTBRIDGE_VIDEO_DEST_PREFIX,
				SOFTBRIDGE_VIDEO_DEST_LEN)) {
			snprintf(buf, size, "%s", ast_stream_get_metadata(stream, "MSID:MSLABEL"));
			setted_msid = 1;
			break;
		}
	}

	if (!setted_msid) {
		snprintf(buf, size, "%s", ast_stream_get_metadata(auio_stream, "MSID:MSLABEL"));
	}

	ast_log(LOG_DEBUG, "cei_get_msid channel:%s. msid:%s",channel_name,buf);

	return 0;
}

static int cei_get_call_type(const char *channel_name, const char *related_info)
{
	char family[256] = {0};
	char user_type[64] = {0};

	if (ast_strlen_zero(channel_name)) {
		ast_log(LOG_DEBUG, "cei_get_local_call channel name null!");
		return AST_CALL_TYPE_UNKNOWN;
	}
	if (strcasestr(channel_name, "DAHDI/o")) {
		return AST_CALL_TYPE_DAHDIO_TRUNK;
	} else if (strcasestr(channel_name, "DAHDI/i")) {
		return AST_CALL_TYPE_DAHDII_TRUNK;
	} else if (strcasestr(channel_name, "IAX2/trunk")) {
		return AST_CALL_TYPE_IAX2_TRUNK;
	} else if (strcasestr(channel_name, "PJSIP/trunk")) {
		return AST_CALL_TYPE_IAX2_TRUNK;
	} else if (strcasestr(channel_name, "PJSIP/conf")) {
		if (!ast_strlen_zero(related_info)) {
			snprintf(family, sizeof(family)-1, "AAUSER/%s", related_info);
			ast_ucm_db_get(family, "TYPE", user_type, sizeof(user_type));
			if (!ast_strlen_zero(user_type) && !strcasecmp(user_type, "user")) {
				ast_log(LOG_DEBUG, "related_info %s is local exten!", related_info);
				return AST_CALL_TYPE_LOCAL;
			}
		}
		return AST_CALL_TYPE_ANONYMOUS;
	} else {
		return AST_CALL_TYPE_LOCAL;
	}
}
static void cei_sche_conf_destroy(void *obj)
{
	struct cei_schedule_conf *conference = obj;
	struct cei_schedule_user *user=  NULL;

	ast_log(LOG_DEBUG, "cei schedule conference '%s' is destroying.\n", conference->conf_name);


	AST_LIST_TRAVERSE_SAFE_BEGIN(&conference->cei_schedule_user_list, user, entry){
		AST_LIST_REMOVE_CURRENT(entry);
		ao2_cleanup(user);
	}
	AST_LIST_TRAVERSE_SAFE_END;
}

static struct cei_schedule_conf *cei_sche_conf_fetch_add(const char *conf_number, int add, const char *schedule_id)
{
	struct cei_schedule_conf *conference = NULL;

	if (ast_strlen_zero(conf_number) || ast_strlen_zero(schedule_id)) {
		return NULL;
	}

	AST_LIST_LOCK(&cei_schedule_conf_list);

	AST_LIST_TRAVERSE(&cei_schedule_conf_list, conference, entry) {
		if (!strcmp(conference->conf_name, conf_number) && !strcmp(conference->schedule_id, schedule_id)) {
			ao2_ref(conference, +1);
			break;
		}
	}

	if (add && !conference) {
		if ((conference = ao2_alloc(sizeof(*conference), cei_sche_conf_destroy))) {
			ast_log(LOG_DEBUG, "Add a new cei schedule conference '%s'.\n", conf_number);
			ast_copy_string(conference->conf_name, conf_number, sizeof(conference->conf_name) - 1);
			ast_copy_string(conference->schedule_id, schedule_id, sizeof(conference->schedule_id) - 1);
			AST_LIST_INSERT_TAIL(&cei_schedule_conf_list, conference, entry);
			ao2_ref(conference, +1);
		}
	}
	AST_LIST_UNLOCK(&cei_schedule_conf_list);

	return conference;
}

static void cei_sche_user_destroy(void *obj)
{
	struct cei_schedule_user *user = obj;

	ast_log(LOG_DEBUG, "schedule attendees user '%s|%s' is destroying.\n", user->extension , user->related_info);
}

static int cei_sche_user_add(struct cei_schedule_conf *conference, const char *extension, const char *related_info, int user_signed)
{
	struct cei_schedule_user *user = NULL;

	if (!conference || (ast_strlen_zero(related_info) && ast_strlen_zero(extension))) {
		return -1;
	}

	user = ao2_alloc(sizeof(*user), cei_sche_user_destroy);
	if (!user) {
		return -1;
	}
	ast_copy_string(user->related_info, related_info?related_info:"", sizeof(user->related_info) - 1);
	ast_copy_string(user->extension, extension?extension:"", sizeof(user->extension) - 1);
	user->sigined_in = user_signed;
	ast_log(LOG_DEBUG, "Add schedule attendees '%s|%s|%d'.\n", user->extension, user->related_info, user->sigined_in);
	ao2_lock(conference);
	AST_LIST_INSERT_TAIL(&conference->cei_schedule_user_list, user, entry);
	ao2_unlock(conference);

	return 0;
}

static struct cei_schedule_user * cei_sche_conf_user_find(struct cei_schedule_conf *conference, const char *extension, const char *related_info)
{
	int anonymous = 0;
	struct cei_schedule_user *user = NULL;

	if (!conference || (ast_strlen_zero(related_info) && ast_strlen_zero(extension))) {
		ast_log(LOG_WARNING, "parament is null");
		return NULL;
	}
	if (ast_strlen_zero(extension) || strstr(extension, "conf")) {
		anonymous = 1;
	}

	AST_LIST_TRAVERSE(&conference->cei_schedule_user_list, user, entry) {
		if ((!ast_strlen_zero(extension) && !strcmp(user->extension, extension))
			|| (anonymous && !ast_strlen_zero(related_info) && (!strcmp(user->related_info, related_info) || !strcmp(user->extension, related_info)))) {
			ao2_ref(user, +1);
			break;
		}
	}

	return user;
}

static void cei_sche_user_signed(const char *conference, const char *schedule_id, const char *extension, const char *related_info, const char *schedule_utc_time)
{
	RAII_VAR(struct cei_schedule_user *, conference_user, NULL, ao2_cleanup);
	RAII_VAR(struct cei_schedule_conf *, conference_obj, NULL, ao2_cleanup);

	if (ast_strlen_zero(conference) || ast_strlen_zero(schedule_id)) {
		ast_log(LOG_DEBUG, "cei_sche_user_signed only record schedule meeting!");
		return;
	}
	if (ast_strlen_zero(related_info) && ast_strlen_zero(extension)) {
		return;
	}
	if (ast_strlen_zero(schedule_utc_time)) {
		ast_log(LOG_DEBUG, "may be random now!");
	}
	conference_obj = cei_sche_conf_fetch_add(conference, 0, schedule_id);
	if (!conference_obj) {
		ast_log(LOG_DEBUG, "Failed to fetch the schedule conference '%s'\n", conference);
		return;
	}
	conference_user = cei_sche_conf_user_find(conference_obj, extension, related_info);
	if (conference_user) {
		ao2_lock(conference_user);
		conference_user->sigined_in = 1;
		ao2_unlock(conference_user);
		ast_log(LOG_DEBUG, "sigined in user %s|%s", extension, related_info);
		ast_log(LOG_DEBUG, "sigined in user list %s|%s", conference_user->extension, conference_user->related_info);
		conf_record_schedule_users_signed(conference, schedule_id, extension, related_info, schedule_utc_time);
	} else {
		ast_log(LOG_DEBUG, "Failed to fetch the schedule user %s|%s!", extension, related_info);
	}
}

void conf_cei_sche_user_load(const char *conf_name, const char *scheduel_id, const char *extension, const char *related_info, int user_signed)
{
	RAII_VAR(struct cei_schedule_conf *, conference_obj, NULL, ao2_cleanup);

	if (ast_strlen_zero(conf_name) || ast_strlen_zero(scheduel_id)
		|| (ast_strlen_zero(related_info) && ast_strlen_zero(extension))) {
		ast_log(LOG_WARNING, "parament is null");
		return;
	}

	conference_obj = cei_sche_conf_fetch_add(conf_name, 1, scheduel_id);
	if (!conference_obj) {
		ast_log(LOG_DEBUG, "Failed to fetch the schedule conference '%s'\n", conf_name);
		return;
	}

	ao2_lock(conference_obj);
	if (cei_sche_user_add(conference_obj, extension, related_info, user_signed) < 0) {
		ast_log(LOG_WARNING, "Failed to add attends '%s' in conference '%s'\n", extension, conf_name);
		ao2_unlock(conference_obj);
		return;
	}
	ao2_unlock(conference_obj);

}

static void cei_sche_conf_user_remove(const char *conf_number)
{
	struct cei_schedule_conf *conference = NULL;

	if (ast_strlen_zero(conf_number)) {
		return;
	}

	ast_log(LOG_DEBUG, "Remove the cei schedule conference '%s'.\n", conf_number);
	AST_LIST_LOCK(&cei_schedule_conf_list);

	AST_LIST_TRAVERSE_SAFE_BEGIN(&cei_schedule_conf_list, conference, entry)
	{
		if (!strcasecmp(conf_number, "all") || !strcmp(conference->conf_name, conf_number)) {
			AST_LIST_REMOVE_CURRENT(entry);
			ao2_cleanup(conference);
			break;
		}
	}
	AST_LIST_TRAVERSE_SAFE_END;

	AST_LIST_UNLOCK(&cei_schedule_conf_list);
}

static int cei_talking_user_find(struct cei_conference *conference, struct cei_user *user)
{
	struct cei_talking_user *talking_user = NULL;
	int found = 0;

	AST_LIST_TRAVERSE(&conference->talking_user_list, talking_user, list) {
		if (talking_user->user == user) {
			found = 1;
			break;
		}
	}

	return found;
}

static void cei_talking_user_insert(struct cei_conference *conference, struct cei_user *user)
{
	struct cei_talking_user *talking_user = NULL;

	if (cei_talking_user_find(conference, user)) {
		ast_log(LOG_DEBUG, "user %s-%s is already in talking user list.", user->cid_number, user->channel_name);
		return;
	}

	if (conference->talking_users >= TALKING_USER_MAX_NUM) {
		talking_user = AST_LIST_REMOVE_HEAD(&conference->talking_user_list, list);
		ast_free(talking_user);
		conference->talking_users--;
	}

	talking_user = ast_calloc(1, sizeof(*talking_user));
	if (!talking_user) {
		return;
	}

	talking_user->user = user;
	AST_LIST_INSERT_TAIL(&conference->talking_user_list, talking_user, list);
	conference->talking_users++;
}

static int cei_talking_user_remove(struct cei_conference *conference, struct cei_user *user)
{
	int removed = 0;
	struct cei_talking_user *talking_user = NULL;

	AST_LIST_TRAVERSE_SAFE_BEGIN(&conference->talking_user_list, talking_user, list) {
		if (talking_user->user == user) {
			AST_LIST_REMOVE_CURRENT(list);
			ast_free(talking_user);
			conference->talking_users--;
			removed = 1;
			break;
		}
	}
	AST_LIST_TRAVERSE_SAFE_END;

	return removed;
}

static struct ast_json *cei_talking_notify_build_conference(struct cei_conference *conference)
{
	RAII_VAR(struct ast_json *, json_conf, NULL, ast_json_unref);
	RAII_VAR(struct ast_json *, json_obj, NULL, ast_json_unref);
	struct cei_talking_user *talking_user = NULL;
	char user_id[AST_MAX_EXTENSION] = {0};

	json_conf = ast_json_pack("{s: s, s: s}",
		"ext", conference->conference_num,
		"id", conference->conference_id
	);

	if (!json_conf) {
		return NULL;
	}

	ao2_lock(conference->user_container);

	/*user*/
	json_obj = ast_json_array_create();
	if (!json_obj) {
		ao2_unlock(conference->user_container);
		return NULL;
	}
	AST_LIST_TRAVERSE(&conference->talking_user_list, talking_user, list) {
		if (!talking_user->user->muted && (!talking_user->user->mic_muted || (talking_user->user->mic_muted && talking_user->user->share_audio))) {
			snprintf(user_id, sizeof(user_id), "%s-%d", talking_user->user->cid_number, talking_user->user->user_num);
			ast_json_array_append(json_obj, ast_json_string_create(user_id));
		}
	}

	ast_json_ref(json_obj);
	ast_json_object_set(json_conf, "user", json_obj);

	ao2_unlock(conference->user_container);

	ast_json_ref(json_conf);
	return json_conf;
}

static struct ast_json *poll_cei_talking_notify(struct cei_conference *conference)
{
	RAII_VAR(struct ast_json *, json_conf, ast_json_object_create(), ast_json_unref);
	RAII_VAR(struct ast_json *, json_conf_array, ast_json_array_create(), ast_json_unref);

	if (!conference) {
		return NULL;
	}

	/*local conference*/
	ast_json_array_append(json_conf_array, cei_talking_notify_build_conference(conference));

	/*remote conference*/
	ao2_lock(conference->remote_container);
	if (ao2_container_count(conference->remote_container)) {
		struct ao2_iterator remote_conf_iter = ao2_iterator_init(conference->remote_container, 0);
		struct cei_conference *remote_conf = NULL;
		while ((remote_conf = ao2_iterator_next(&remote_conf_iter))) {
			ast_json_array_append(json_conf_array, cei_talking_notify_build_conference(remote_conf));
			ao2_ref(remote_conf, -1);
		}
		ao2_iterator_destroy(&remote_conf_iter);
	}
	ao2_unlock(conference->remote_container);

	ast_json_ref(json_conf_array);
	ast_json_object_set(json_conf, "conf", json_conf_array);

	ast_json_ref(json_conf);
	return json_conf;
}

static void cei_talking_notify(struct cei_conference *conference, struct cei_user *user)
{
	RAII_VAR(struct ast_json *, json_object, NULL, ast_json_unref);
	RAII_VAR(struct ast_json *, json_talk, NULL, ast_json_unref);
	RAII_VAR(char *, json_str, NULL, ast_json_free);

	json_talk = poll_cei_talking_notify(conference);
	if (!json_talk) {
		return;
	}
	json_str = ast_json_dump_string(json_talk);

	if (user) {
		json_object = ast_json_pack("{s: i, s: s, s: i, s: i, s: s, s: s, s: s, s: s}",
			"user_type", user->user_type,
			"conference", conference->conference_num,
			"version_reset", conference->version_reset,
			"user_role", user->user_role,
			"user_name", check_role_as_conference(user) ? user->self_conf_id : user->channel_name,
			"full_text", json_str,
			"event_type", CEI_NOTIFY_EVENT_TALK_FOR_WAVE,
			"cid_number", user->cid_number
		);
	} else {
		json_object = ast_json_pack("{s: s, s: i, s: s, s: s}",
			"conference", conference->conference_num,
			"version_reset", conference->version_reset,
			"full_text", json_str,
			"event_type", CEI_NOTIFY_EVENT_TALK_FOR_WAVE
		);
	}
	if (!json_object) {
		return;
	}

	cei_publish_manager_event(json_object);
}

static void cei_user_destroy(void *obj)
{
#if 0
	ast_debug(LOG_DEBUG, "[WL] cei user destroy");
#endif
}

static void cei_join_cb(void *data, struct stasis_subscription *sub,
	struct stasis_message *message)
{
	struct cei_conference *conf;
	struct cei_user *user = NULL;
	struct ast_str *full_text = ast_str_create(CEI_EVENT_DATA_LEN);
	struct ast_str *partial_text = ast_str_create(CEI_EVENT_DATA_LEN);

	const struct ast_bridge_blob *blob = stasis_message_data(message);
	unsigned int user_count= ast_json_integer_get(ast_json_object_get(blob->blob, "user_count"));
	unsigned int user_number = ast_json_integer_get(ast_json_object_get(blob->blob, "user_number"));
	const char *conference_name = ast_json_string_get(ast_json_object_get(blob->blob, "conference"));
	const char *cid_number = ast_json_string_get(ast_json_object_get(blob->blob, "caller_id_number"));
	const char *cid_name = ast_json_string_get(ast_json_object_get(blob->blob, "caller_id_name"));
	const char *channel_name = ast_json_string_get(ast_json_object_get(blob->blob, "channel_name"));
	const char *mcb_type = ast_json_string_get(ast_json_object_get(blob->blob, "mcb_type"));
	const char *mcb_remote_conf_id = ast_json_string_get(ast_json_object_get(blob->blob, "mcb_remote_conf_id"));
	const char *conference_type = ast_json_string_get(ast_json_object_get(blob->blob, "conference_type"));
	const char *feature_id = ast_json_string_get(ast_json_object_get(blob->blob, "feature_id"));
	const char *subject = ast_json_string_get(ast_json_object_get(blob->blob, "subject"));
	const char *conference_config = ast_json_string_get(ast_json_object_get(blob->blob, "conference_config"));
	const char *record_type = ast_json_string_get(ast_json_object_get(blob->blob, "record_type"));
	const char *related_info = ast_json_string_get(ast_json_object_get(blob->blob, "related_info"));
	const char *schedule_id = ast_json_string_get(ast_json_object_get(blob->blob, "schedule_id"));
	time_t record_start_utc_timestamp = ast_json_integer_get(ast_json_object_get(blob->blob, "record_start_timestamp_utc"));
	const char *im_service_prefix = ast_json_string_get(ast_json_object_get(blob->blob, "im_service_prefix"));
	const char *allow_user_unmute = ast_json_string_get(ast_json_object_get(blob->blob, "allow_user_unmute"));
	const char *schedule_utc_time = ast_json_string_get(ast_json_object_get(blob->blob, "schedule_time"));
	const char *sca_shareline = ast_json_string_get(ast_json_object_get(blob->blob, "sca_shareline"));
	const char *conf_creator = ast_json_string_get(ast_json_object_get(blob->blob, "creator"));

	unsigned int admin_status = 0;
	if (ast_json_is_true(ast_json_object_get(blob->blob, "admin"))) {
			admin_status = 1;
	}
	unsigned int mute_status = ast_json_integer_get(ast_json_object_get(blob->blob, "muted"));
	unsigned int mic_mute_status = ast_json_integer_get(ast_json_object_get(blob->blob, "mic_muted"));
	unsigned int can_invite = ast_json_integer_get(ast_json_object_get(blob->blob, "can_invite"));
	unsigned int invited_status = ast_json_integer_get(ast_json_object_get(blob->blob, "invited"));
	unsigned int isfocus = ast_json_integer_get(ast_json_object_get(blob->blob, "isfocus"));

	if (conference_name == NULL || cid_number == NULL || cid_name == NULL ||
		channel_name == NULL || mcb_type == NULL) {
		ast_debug(LOG_DEBUG, "[WL] EVENT INCOMPLETE");
		ast_free(full_text);
		ast_free(partial_text);
		return;
	}

	// get cei_conference
	conf = build_conf_by_num(conference_name);
	if (!conf) {
		ast_free(full_text);
		ast_free(partial_text);
		return;
	}

	ao2_lock(conf);
	if (ast_strlen_zero(conf->start_time)) {
		conf->video_conference = !strcmp(S_OR(conference_type, ""), "video") ? 1 : 0;
		ast_copy_string(conf->start_time, ast_json_string_get(ast_json_object_get(blob->blob, "conf_starttime")), sizeof(conf->start_time));
	}

	if (!conf->web_page) {
		conf->web_page = conf_get_http_uri();
	}

	if (!conf->feature_id) {
		conf->feature_id = ast_strdup(feature_id);
	}
	if (!conf->subject) {
		conf->subject = ast_strdup(subject);
	}
	if (!conf->conf_config) {
		conf->conf_config = ast_strdup(conference_config);
	}
	if (!conf->schedule_id) {
		conf->schedule_id = ast_strdup(schedule_id);
	}
	if (!conf->im_service_prefix) {
		ast_debug(LOG_DEBUG, "cei set im_service_prefix %s", im_service_prefix);
		conf->im_service_prefix = ast_strdup(im_service_prefix);
	}
	if (!ast_strlen_zero(allow_user_unmute) && ast_true(allow_user_unmute)) {
		conf->allow_user_unmute = 1;
	}
	if (!conf->conf_creator && !ast_strlen_zero(conf_creator)) {
		conf->conf_creator = ast_strdup(conf_creator);
	}

	strncpy(conf->record_type, record_type, sizeof(conf->record_type));
	conf->record_start_utc_timestamp = record_start_utc_timestamp;

	ao2_unlock(conf);

	ast_debug(LOG_DEBUG, "[WL] Create new user struct with user_count: %d, user_num:%d, cid_name:%s, cid_num:%s, chan_name:%s, conf_name:%s, admin_status:%d, muted_status:%d, can_invite: %d, conf_type:%s, subject:%s, mic_mute_status:%d, conference_config %s, related_info %s isfocus %d conf_creator %s",
												user_count,
												user_number,
												cid_name,
												cid_number,
												channel_name,
												conf->conference_id,
												admin_status,
												mute_status,
												can_invite,
												conference_type,
												conf->subject,
												mic_mute_status,
												conference_config,
												related_info,
												isfocus,
												conf_creator
	);

	if (!(user = ao2_alloc(sizeof(struct cei_user), cei_user_destroy))) {
		ast_free(full_text);
		ast_free(partial_text);
		ast_debug(LOG_DEBUG, "[WL] CANNOT CREATE NEW USER");
		/*-1 for using build_conf_by_num*/
		ast_atomic_fetchadd_int(&conf->refcount, -1);
		ast_debug(LOG_DEBUG, "cei_join_cb leave refcount %d", conf->refcount);
		remove_conference(conf);
		return;
	}

	if (user_count != (conf->local_size + 1)) {
		ast_debug(LOG_DEBUG, "[WL] CONF SIZE MISMATCH %d vs %d",
							 conf->local_size, user_count);
	}

	//add info to cei_user
	user->user_num = (int) user_number;
	ast_copy_string(user->cid_number, cid_number, sizeof(user->cid_number));
	ast_copy_string(user->channel_name, channel_name, sizeof(user->channel_name));
	ast_copy_string(user->conference_id, conf->conference_id, sizeof(user->conference_id));
	hash_user(user);
	user->admin = (int) admin_status;
	user->muted = (int) mute_status;
	user->mic_muted = (int) mic_mute_status;
	user->isfocus = (int) isfocus;
	user->can_invite = (int) can_invite;
	user->media.type = AST_MEDIA_TYPE_AUDIO;
	user->term_type = cei_get_terminate_type(channel_name);
	if (conf->video_conference) {
		cei_get_extension(user->channel_name, user->extension, sizeof(user->extension)-1);
		cei_get_msid(user->channel_name, user->msid, sizeof(user->msid)-1);
	} else if (strcmp(mcb_type, CEI_MCB_TYPE_NONE) == 0) {
		cei_get_extension(user->channel_name, user->extension, sizeof(user->extension)-1);
	}
	if (ast_strlen_zero(user->extension)) {
		ast_copy_string(user->extension, cid_number, sizeof(user->extension)-1);
	}
	/*user signed in*/
	char signed_exten[AST_MAX_EXTENSION] = {0};
	conf_get_exten_by_channel(user->channel_name, signed_exten, AST_MAX_EXTENSION - 1);
	if (ast_strlen_zero(signed_exten)) {
		ast_copy_string(signed_exten, user->extension, AST_MAX_EXTENSION - 1);
	}
	if (ast_strlen_zero(signed_exten) || !strncasecmp(signed_exten, "trunk", 5)) {
		ast_copy_string(signed_exten, cid_number, AST_MAX_EXTENSION - 1);
	}
	if (!ast_strlen_zero(sca_shareline)) {
		memset(signed_exten, 0, sizeof(signed_exten));
		ast_copy_string(signed_exten, sca_shareline, AST_MAX_EXTENSION - 1);
		ast_debug(LOG_DEBUG, "user join get sca shareline %s", signed_exten);
	}
	if (!ast_strlen_zero(related_info)) {
		/*the exten set email,join through email link,but related_info is local exten,set im_service_prefix*/
		if (cei_get_call_type(channel_name, related_info) == AST_CALL_TYPE_LOCAL && !ast_strlen_zero(conf->im_service_prefix) && !strcasestr(related_info, "@")) {
			snprintf(user->related_ext, sizeof(user->related_ext)-1, "%s%s", conf->im_service_prefix, related_info);
		} else {
			/*The cross domain exten joins the  conference room through the assistant message*/
			ast_copy_string(user->related_ext, related_info, sizeof(user->related_ext));
		}
	} else {
		/*The local extension calls directly into the conference room,set im_service_prefix*/
		if (cei_get_call_type(channel_name, NULL) == AST_CALL_TYPE_LOCAL && !ast_strlen_zero(conf->im_service_prefix)) {
			snprintf(user->related_ext, sizeof(user->related_ext)-1, "%s%s", conf->im_service_prefix, signed_exten);
		} else if (cei_get_call_type(channel_name, NULL) == AST_CALL_TYPE_LOCAL && ast_strlen_zero(conf->im_service_prefix)) {
			snprintf(user->related_ext, sizeof(user->related_ext)-1, "%s", signed_exten);
		} else {
			ast_copy_string(user->related_ext, user->cid_number, sizeof(user->related_ext));
		}
	}

	// User type, User role, cid name
	// Modify by bfdu for Bug 288575, 2024.6.27
	if (strcmp(mcb_type, CEI_MCB_TYPE_NONE) == 0 || ast_strlen_zero(mcb_type)) {
		user->user_type = CEI_TYPE_NORMAL_USER;
		ast_copy_string(user->cid_name, cid_name, sizeof(user->cid_name));

		if (user->admin) {
			user->user_role = CEI_ROLE_ADMIN;
		}
		else if (user->can_invite) {
			user->user_role = CEI_ROLE_USER_INVITE;
		}
		else {
			user->user_role = CEI_ROLE_USER_NORMAL;
		}

		cei_update_media_info(user, conference_name);
	} else if (strcmp(mcb_type, CEI_MCB_TYPE_EXTERNAL) == 0) {
		user->user_type = CEI_TYPE_EXTERNAL_MCB;
		user->user_role = CEI_ROLE_CONFERENCE;
		ast_copy_string(user->cid_name, CEI_USER_ROLE_CONFERENCE, sizeof(user->cid_name));
		if (mcb_remote_conf_id != NULL) {
			ast_copy_string(user->self_conf_id, mcb_remote_conf_id, sizeof(user->self_conf_id));
		}

	} else if (strcmp(mcb_type, CEI_MCB_TYPE_INTERNAL) == 0) {
		user->user_type = CEI_TYPE_INTERNAL_MCB;
		user->user_role = CEI_ROLE_CONFERENCE;
		ast_copy_string(user->cid_name, CEI_USER_ROLE_CONFERENCE, sizeof(user->cid_name));
		if (mcb_remote_conf_id != NULL) {
			ast_copy_string(user->self_conf_id, mcb_remote_conf_id, sizeof(user->self_conf_id));
		}

	} else {
		ast_copy_string(user->cid_name, cid_name, sizeof(user->cid_name));
		user->user_type = CEI_TYPE_UNKNOWN;
	}

	ao2_lock(conf->user_container);
	ao2_link(conf->user_container, user);
	ast_atomic_fetchadd_int(&conf->local_size, 1);
	ast_atomic_fetchadd_int(&conf->local_user_size, (!check_role_as_conference(user))? 1 : 0);
	ast_atomic_fetchadd_int(&conf->refcount, 1);
	ao2_unlock(conf->user_container);

	poll_cei_conference(conf, &full_text);
	poll_cei_partial_notify(conf, NULL, user, &partial_text, CEI_EVENT_JOIN);
	/*user signed in*/
	cei_sche_user_signed(conference_name, schedule_id, signed_exten, related_info, schedule_utc_time);

	// Prepare the stasis data
	RAII_VAR(struct ast_json *, json_object, NULL, ast_json_unref);

	json_object = ast_json_pack("{s: i, s: s, s: i, s: i, s: i, s: s, s: s, s: s, s: s, s: s, s: s}",
		"user_type", user->user_type,
		"conference", conference_name,
		"preverified_user", invited_status,
		"version_reset", conf->version_reset,
		"user_role", user->user_role,
		"user_name", check_role_as_conference(user) ? user->self_conf_id : channel_name,
		"full_text", ast_str_buffer(full_text),
		"partial_text", ast_str_buffer(partial_text),
		"event_type", CEI_NOTIFY_EVENT_JOIN,
		"cid_number", user->cid_number,
		"conference_type", conference_type
	);

	cei_publish_manager_event(json_object);

	if (user->user_type == CEI_TYPE_NORMAL_USER) {
		cei_talking_notify(conf, user);
	}

	/*-1 for using build_conf_by_num*/
	ast_atomic_fetchadd_int(&conf->refcount, -1);
	ast_debug(LOG_DEBUG, "cei_join_cb leave refcount %d", conf->refcount);

	ast_free(partial_text);
	ast_free(full_text);
}

static void cei_leave_cb(void *data, struct stasis_subscription *sub,
	struct stasis_message *message)
{
	enum cei_user_type user_type = CEI_TYPE_UNKNOWN;
	char self_conf_id[41] = {0};
	struct cei_conference *conf;
	RAII_VAR(struct cei_user *, user, NULL, ao2_cleanup);
	struct ast_str *extra_text = ast_str_create(CEI_EVENT_DATA_LEN);
	struct ast_str *full_text = ast_str_create(CEI_EVENT_DATA_LEN);

	const struct ast_bridge_blob *blob = stasis_message_data(message);
	unsigned int user_count= ast_json_integer_get(ast_json_object_get(blob->blob, "user_count"));
	unsigned int user_number = ast_json_integer_get(ast_json_object_get(blob->blob, "user_number"));
	const char *conference_name = ast_json_string_get(ast_json_object_get(blob->blob, "conference"));
	const char *cid_number = ast_json_string_get(ast_json_object_get(blob->blob, "caller_id_number"));
	const char *channel_name = ast_json_string_get(ast_json_object_get(blob->blob, "channel_name"));

	ast_debug(LOG_DEBUG, "[WL] Delete a user : user_count: %d, user_num:%d, cid_num:%s, conf_name:%s, channel_name:%s",
												user_count,
												user_number,
												cid_number,
												conference_name,
												channel_name
	);

	if (conference_name == NULL || cid_number == NULL || channel_name == NULL) {
		ast_debug(LOG_DEBUG, "[WL] EVENT INCOMPLETE");
		ast_free(extra_text);
		ast_free(full_text);
		return;
	}

	// get cei_conference
	conf = build_conf_by_num(conference_name);

	if (!conf) {
		ast_free(extra_text);
		ast_free(full_text);
		return;
	}

	if (user_count != (conf->local_size - 1)) {
		ast_debug(LOG_DEBUG, "[WL] CONF SIZE MISMATCH %d vs %d", conf->local_size, user_count);
	}

	ao2_lock(conf->user_container);

	/*! begin: Modified by pyzhong@grandstream.cn for bug[63462], 2016-10-24*/
	//user = get_user_by_cid_num(conf, cid_number);
	user = get_user_by_channel_name(conf, channel_name, TRUE);
	/*! end: Modified by pyzhong@grandstream.cn for bug[63462], 2016-10-24*/
	if (user) {
		/*! begin: Modified by pyzhong@grandstream.cn for bug[71632], 2017-3-31*/
		user->user_status = CEI_STATUS_DISCONNECTED;
		/*Here for audio conference*/
		if (!conf->video_conference) {
			poll_cei_conference(conf, &full_text);
			/*! end: Modified by pyzhong@grandstream.cn for bug[71632], 2017-3-31*/
		}

		user_type = user->user_type;
		if (check_role_as_conference(user)) {
			ast_copy_string(self_conf_id, user->self_conf_id, sizeof(self_conf_id));
		}
		ast_atomic_fetchadd_int(&conf->local_size, -1);
		ast_atomic_fetchadd_int(&conf->local_user_size, (!check_role_as_conference(user))? -1 : 0);

		poll_cei_partial_notify(conf, NULL, user, &extra_text, CEI_EVENT_PART);

		ao2_unlink(conf->user_container, user);
		ast_atomic_fetchadd_int(&conf->refcount, -1);

		/*Here for video conference*/
		if (conf->video_conference) {
			poll_cei_conference(conf, &full_text);
		}

		/*remove from talking list*/
		if (cei_talking_user_remove(conf, user)) {
			cei_talking_notify(conf, NULL);
		}

		ast_debug(LOG_DEBUG, "[WL] REMOVED USER WITH ADDRESS %p", user);
		/*Check if it is the sharing user*/
		if (conf->sharing_user == user) {
			ao2_ref(conf->sharing_user, -1);
			conf->sharing_user = NULL;
		}
		/*-1 for ao2_alloc (join)*/
		ao2_ref(user, -1);
	}
	else {
		ast_debug(LOG_DEBUG, "[WL] ERROR, CANNOT FIND USER NUMBER %d IN CONFERENCE %s", user_number, conf->conference_num);
		ao2_unlock(conf->user_container);
		ast_free(extra_text);
		ast_free(full_text);
		/*-1 for using build_conf_by_num*/
		ast_atomic_fetchadd_int(&conf->refcount, -1);
		ast_debug(LOG_DEBUG, "cei_leave_cb leave refcount %d", conf->refcount);
		remove_conference(conf);
		return;
	}

	ao2_unlock(conf->user_container);

	// Prepare the stasis data
	RAII_VAR(struct ast_json *, json_object, NULL, ast_json_unref);

	json_object = ast_json_pack("{s: i, s: s, s: i, s: s, s: s, s: s, s: s, s: s}",
		"user_type", user_type,
		"conference", conference_name,
		"version_reset", conf->version_reset,
		"user_name", (!ast_strlen_zero(self_conf_id)) ? self_conf_id : channel_name,
		"full_text", ast_str_buffer(full_text),
		"partial_text", ast_str_buffer(extra_text),
		"event_type", CEI_NOTIFY_EVENT_LEAVE,
		"cid_number", cid_number
	);

	cei_publish_manager_event(json_object);
	/*-1 for using build_conf_by_num*/
	ast_atomic_fetchadd_int(&conf->refcount, -1);
	ast_debug(LOG_DEBUG, "cei_leave_cb leave refcount %d", conf->refcount);

#if 0
	reset_conference(conf);
#endif
	remove_conference(conf);
	ast_free(extra_text);
	ast_free(full_text);
}


static void cei_lock_status_cb(void *data, struct stasis_subscription *sub,
	struct stasis_message *message)
{
	struct cei_conference *conf;
	struct ast_str *extra_text = ast_str_create(CEI_EVENT_DATA_LEN);
	struct ast_str *full_text = ast_str_create(CEI_EVENT_DATA_LEN);

	const struct ast_bridge_blob *blob = stasis_message_data(message);
	const char *lock_status= ast_json_string_get(ast_json_object_get(blob->blob, "status"));
	const char *conference_name = ast_json_string_get(ast_json_object_get(blob->blob, "conference"));

	if (lock_status == NULL || conference_name == NULL) {
		ast_debug(LOG_DEBUG, "[WL] EVENT INCOMPLETE");
		ast_free(extra_text);
		ast_free(full_text);
		return;
	}

	// get cei_conference
	conf = build_conf_by_num(conference_name);
	ast_debug(LOG_DEBUG, "[WL] Lock conference: lock_status: %s, conf_name:%s;",
												lock_status,
												conference_name
	);
	if (!conf) {
		ast_free(extra_text);
		ast_free(full_text);
		return;
	}

	conf->locked = !strcmp(lock_status, "locked") ? TRUE : FALSE;

	poll_cei_conference(conf, &full_text);
	poll_cei_partial_notify(conf, NULL, NULL, &extra_text, CEI_EVENT_TOGGLE_LOCK);

	// Prepare the stasis data
	RAII_VAR(struct ast_json *, json_object, NULL, ast_json_unref);

	json_object = ast_json_pack("{s: s, s: i, s: s, s: s, s: s}",
		"conference", conference_name,
		"version_reset", conf->version_reset,
		"full_text", ast_str_buffer(full_text),
		"partial_text", ast_str_buffer(extra_text),
		"event_type", CEI_NOTIFY_EVENT_LOCK
	);

	cei_publish_manager_event(json_object);
	/*-1 for using build_conf_by_num*/
	ast_atomic_fetchadd_int(&conf->refcount, -1);
	ast_debug(LOG_DEBUG, "cei_lock_status_cb leave refcount %d", conf->refcount);

	ast_free(extra_text);
	ast_free(full_text);
}

static void cei_toggle_mute_cb(void *data, struct stasis_subscription *sub,
	struct stasis_message *message)
{
	struct cei_conference *conf;
	struct ast_str *extra_text = ast_str_create(CEI_EVENT_DATA_LEN);
	struct ast_str *full_text = ast_str_create(CEI_EVENT_DATA_LEN);
	int update_talking = 0;

	const struct ast_bridge_blob *blob = stasis_message_data(message);
	unsigned int user_number = ast_json_integer_get(ast_json_object_get(blob->blob, "user_number"));
	const char *conference_name = ast_json_string_get(ast_json_object_get(blob->blob, "conference"));
	unsigned int mute_status = ast_json_integer_get(ast_json_object_get(blob->blob, "muted"));
	unsigned int mic_mute_status = ast_json_integer_get(ast_json_object_get(blob->blob, "mic_muted"));
	unsigned int wait = ast_json_integer_get(ast_json_object_get(blob->blob, "wait"));
	const char *talk_status = ast_json_string_get(ast_json_object_get(blob->blob, "talking_status"));
	unsigned int admin_status = 0;
	if (ast_json_is_true(ast_json_object_get(blob->blob, "admin"))) {
			admin_status = 1;
	}
	const char *cid_number = ast_json_string_get(ast_json_object_get(blob->blob, "caller_id_number"));

	ast_debug(LOG_DEBUG, "[WL] Mute user: user_num:%d, cid:%s, conf_name:%s, mute_status:%d, mic_mute_status:%d, admin_status:%d, wait:%d, talking_status:%s",
												user_number,
												cid_number,
												conference_name,
												mute_status,
												mic_mute_status,
												admin_status,
												wait,
												talk_status
	);

	if (conference_name == NULL) {
		ast_debug(LOG_DEBUG, "[WL] EVENT INCOMPLETE");
		ast_free(extra_text);
		ast_free(full_text);
		return;
	}

	conf = build_conf_by_num(conference_name);
	if (!conf) {
		return;
	}

	ao2_lock(conf->user_container);

	RAII_VAR(struct cei_user *, user, get_user_by_user_num(conf, (int)user_number), ao2_cleanup);
	if (user) {
		user->muted = mute_status;
		user->mic_muted = mic_mute_status;
		user->talking = !strcmp(talk_status, "on") ? 1 : 0;
		if (user->muted || (user->mic_muted && !user->share_audio)) {
			update_talking = cei_talking_user_remove(conf, user) ? 1 : 0;
		} else {
			if (user->talking) {
				cei_talking_user_insert(conf, user);
				update_talking = 1;
			}
		}
	} else {
		ast_debug(LOG_WARNING, "[WL] ERROR, CANNOT FIND USER NUMBER %d IN CONFERENCE %s", user_number, conf->conference_num);
		ao2_unlock(conf->user_container);
		ast_free(extra_text);
		ast_free(full_text);
		/*-1 for using build_conf_by_num*/
		ast_atomic_fetchadd_int(&conf->refcount, -1);
		ast_debug(LOG_DEBUG, "cei_toggle_mute_cb leave refcount %d", conf->refcount);
		return;
	}
	ao2_unlock(conf->user_container);

	if (wait) {
		ast_free(extra_text);
		ast_free(full_text);
		/*-1 for using build_conf_by_num*/
		ast_atomic_fetchadd_int(&conf->refcount, -1);
		ast_debug(LOG_DEBUG, "cei_toggle_mute_cb leave refcount %d", conf->refcount);
		return;
	}

	poll_cei_conference(conf, &full_text);
	poll_cei_partial_notify(conf, NULL, user, &extra_text, CEI_EVENT_TOGGLE_MUTE);

	// Prepare the stasis data
	RAII_VAR(struct ast_json *, json_object, NULL, ast_json_unref);

	json_object = ast_json_pack("{s: i, s: s, s: i, s: s, s: s, s: s}",
		"user_type", user->user_type,
		"conference", conference_name,
		"version_reset", conf->version_reset,
		"full_text", ast_str_buffer(full_text),
		"partial_text", ast_str_buffer(extra_text),
		"event_type", CEI_NOTIFY_EVENT_MUTE
	);

	cei_publish_manager_event(json_object);

	/*talking for wave*/
	ao2_lock(conf->user_container);
	if (update_talking) {
		cei_talking_notify(conf, NULL);
	}
	ao2_unlock(conf->user_container);
	/*-1 for using build_conf_by_num*/
	ast_atomic_fetchadd_int(&conf->refcount, -1);
	ast_debug(LOG_DEBUG, "cei_toggle_mute_cb leave refcount %d", conf->refcount);

	ast_free(extra_text);
	ast_free(full_text);
}

static void cei_toggle_record_cb(void *data, struct stasis_subscription *sub,
		struct stasis_message *message)
{
	struct cei_conference *conf;
	struct ast_str *extra_text = ast_str_create(CEI_EVENT_DATA_LEN);
	struct ast_str *full_text = ast_str_create(CEI_EVENT_DATA_LEN);

	const struct ast_bridge_blob *blob = stasis_message_data(message);
	const char *type= ast_json_string_get(ast_json_object_get(blob->blob, "type"));  /* type: audio/video/focus/none */
	const char *status= ast_json_string_get(ast_json_object_get(blob->blob, "status"));  /* status: on/off */
	const char *conference_name = ast_json_string_get(ast_json_object_get(blob->blob, "conference"));
	time_t record_start_utc_timestamp = ast_json_integer_get(ast_json_object_get(blob->blob, "start_timestamp_utc"));

	if (ast_strlen_zero(type) || ast_strlen_zero(status)
		|| ast_strlen_zero(conference_name)) {
		ast_debug(LOG_DEBUG, "[WL] EVENT INCOMPLETE");
		ast_free(extra_text);
		ast_free(full_text);
		return;
	}

	// get cei_conference
	conf = build_conf_by_num(conference_name);
	ast_debug(LOG_DEBUG, "[WL] toggle conference[%s] record: type: %s, status:%s;",
				conference_name, type, status);

	if (!conf) {
		ast_free(extra_text);
		ast_free(full_text);
		return;
	}

	if (!strcmp(status, "on")) {
		strncpy(conf->record_type, type, sizeof(conf->record_type) -  1);
	} else {
		strncpy(conf->record_type, "none", sizeof(conf->record_type));
	}
	conf->record_start_utc_timestamp = record_start_utc_timestamp;

	poll_cei_conference(conf, &full_text);
	poll_cei_partial_notify(conf, NULL, NULL, &extra_text, CEI_EVENT_TOGGLE_RECORD);

	// Prepare the stasis data
	RAII_VAR(struct ast_json *, json_object, NULL, ast_json_unref);

	json_object = ast_json_pack("{s: s, s: i, s: s, s: s, s: s}",
		"conference", conference_name,
		"version_reset", conf->version_reset,
		"full_text", ast_str_buffer(full_text),
		"partial_text", ast_str_buffer(extra_text),
		"event_type", CEI_NOTIFY_EVENT_RECORD
	);

	cei_publish_manager_event(json_object);
	/*-1 for using build_conf_by_num*/
	ast_atomic_fetchadd_int(&conf->refcount, -1);
	ast_debug(LOG_DEBUG, "cei_toggle_record_cb leave refcount %d", conf->refcount);

	ast_free(extra_text);
	ast_free(full_text);
}

static void cei_talk_status_cb(void *data, struct stasis_subscription *sub,
	struct stasis_message *message)
{
	struct cei_conference *conf = NULL;
	struct ast_str *extra_text = ast_str_create(CEI_EVENT_DATA_LEN);

	const struct ast_bridge_blob *blob = stasis_message_data(message);
	unsigned int user_number = ast_json_integer_get(ast_json_object_get(blob->blob, "user_number"));
	const char *conference_name = ast_json_string_get(ast_json_object_get(blob->blob, "conference"));
	const char *talk_status = ast_json_string_get(ast_json_object_get(blob->blob, "talking_status"));
	const char *cid_number = ast_json_string_get(ast_json_object_get(blob->blob, "caller_id_number"));
	int talk_rms = ast_json_integer_get(ast_json_object_get(blob->blob, "talking_rms"));

	ast_debug(LOG_DEBUG, "[WL] talking detect user: user_num:%d, conf_name %s, cid_number:%s talking:%s",
												user_number,
												conference_name,
												cid_number,
												talk_status
	);

	if (conference_name == NULL) {
		ast_debug(LOG_DEBUG, "[WL] EVENT INCOMPLETE");
		ast_free(extra_text);
		return;
	}

	conf = build_conf_by_num(conference_name);
	if (!conf) {
		ast_debug(LOG_WARNING, "[WL] Couldn't find the conference '%s'", conference_name);
		ast_free(extra_text);
		return;
	}

	ao2_lock(conf->user_container);

	RAII_VAR(struct cei_user *, user, get_user_by_user_num(conf, (int)user_number), ao2_cleanup);
	if (user) {
		int last_talk_status = user->talking;

		user->talking = !strcmp(talk_status, "on") ? 1 : 0;
		user->talking_rms = talk_rms;
		if (user->talking != last_talk_status) {
			if (user->talking) {
				cei_talking_user_insert(conf, user);
			} else {
				cei_talking_user_remove(conf, user);
			}
		}
	} else {
		ast_debug(LOG_WARNING, "[WL] ERROR, CANNOT FIND USER NUMBER %d IN CONFERENCE %s", user_number, conf->conference_num);
		ao2_unlock(conf->user_container);
		ast_free(extra_text);
		/*-1 for using build_conf_by_num*/
		ast_atomic_fetchadd_int(&conf->refcount, -1);
		ast_debug(LOG_DEBUG, "cei_talk_status_cb leave refcount %d", conf->refcount);
		return;
	}
	ao2_unlock(conf->user_container);

	poll_cei_partial_notify(conf, NULL, user, &extra_text, CEI_EVENT_TALKING);

	// Prepare the stasis data
	RAII_VAR(struct ast_json *, json_object, NULL, ast_json_unref);

	json_object = ast_json_pack("{s: i, s: s, s: i, s: s, s: s}",
			"user_type", user->user_type,
			"conference", conference_name,
			"version_reset", conf->version_reset,
			"partial_text", ast_str_buffer(extra_text),
			"event_type", CEI_NOTIFY_EVENT_TALK
	);

	cei_publish_manager_event(json_object);

	/*talking for wave*/
	cei_talking_notify(conf, NULL);

	/*-1 for using build_conf_by_num*/
	ast_atomic_fetchadd_int(&conf->refcount, -1);
	ast_debug(LOG_DEBUG, "cei_talk_status_cb leave refcount %d", conf->refcount);

	ast_free(extra_text);
}

// UNSUB for both ext MCB and int MCB
static void cei_mcb_process_unsub_cb(void *data, struct stasis_subscription *sub,
	struct stasis_message *message)
{
	ast_log(LOG_DEBUG, "[TK] Enter...%s \n", __FUNCTION__);
	struct cei_conference	*host_conf  = NULL;
	struct ast_str			*extra_text = ast_str_create(CEI_EVENT_DATA_LEN);
	struct ast_json_payload *payload    = NULL;
	struct ast_json			*json_object = NULL;

	payload = stasis_message_data(message);
	if (NULL == payload) {
		ast_debug(LOG_DEBUG, "Got NULL payload from stasis message.");
		return;
	}

	json_object = payload->json;
	if (json_object == NULL) {
		ast_debug(LOG_DEBUG, "[WL] Imcomplete message.");
		return;
	}

	const char *conf_id = ast_json_string_get(ast_json_object_get(json_object, "conf_id"));
	const char *publisher = ast_json_string_get(ast_json_object_get(json_object, "publisher"));


	if (conf_id == NULL || publisher == NULL) {
		ast_debug(LOG_DEBUG, "[WL] Imcomplete data. \n");
		return;
	}

	ast_debug(LOG_DEBUG, "[WL] Got mcb leave data : %s,%s",
												conf_id,
												publisher);

	// Get host conference
	host_conf = get_conf_by_id(conf_id);
	if (host_conf == NULL) {
		ast_free(extra_text);
		return;
	}

	delete_remote_confs_by_publisher(host_conf, publisher);

	poll_cei_conference(host_conf, &extra_text);

	// Prepare the stasis data
	RAII_VAR(struct ast_json *, to_send_json_object, NULL, ast_json_unref);

	to_send_json_object = ast_json_pack("{s: s, s: i, s: s, s: s}",
		"conference", host_conf->conference_num,
		"version_reset", host_conf->version_reset,
		"full_text", ast_str_buffer(extra_text),
		"event_type", CEI_NOTIFY_EVENT_MCB_LEAVE
	);

	cei_publish_manager_event(to_send_json_object);
	/*-1 for using get_conf_by_id*/
	ast_atomic_fetchadd_int(&host_conf->refcount, -1);
	ast_debug(LOG_DEBUG, "cei_mcb_process_unsub_cb leave refcount %d", host_conf->refcount);

	ast_free(extra_text);
}

static void cei_ext_mcb_process_xml_cb(void *data, struct stasis_subscription *sub,
	struct stasis_message *message)
{
	ast_log(LOG_DEBUG, "[TK] Enter...%s \n", __FUNCTION__);
	struct cei_conference *host_conf = NULL;
	struct ast_str *extra_text = ast_str_create(CEI_EVENT_DATA_LEN);

	struct ast_json_payload *payload     = NULL;
	struct ast_json			*json_object = NULL;

	payload = stasis_message_data(message);
	if (NULL == payload) {
		ast_debug(LOG_DEBUG, "Got NULL payload from stasis message.");
		return;
	}

	json_object = payload->json;
	if (json_object == NULL) {
		ast_debug(LOG_DEBUG, "[WL] Imcomplete message.");
		return;
	}
	const char *conf_no = ast_json_string_get(ast_json_object_get(json_object, "conf_num"));
	const char *xml = ast_json_string_get(ast_json_object_get(json_object, "xml"));
	const char *publisher_id = ast_json_string_get(ast_json_object_get(json_object, "publisher_id"));
	const char *publisher_private_num = ast_json_string_get(ast_json_object_get(json_object, "publisher_private_cid"));

	if (conf_no == NULL || xml == NULL || publisher_id == NULL || publisher_private_num == NULL) {
		ast_debug(LOG_DEBUG, "[WL] Imcomplete data. \n");
		return;
	}
	ast_debug(LOG_DEBUG, "[WL] Got mcb data : conf %s, publisher %s %s, content %s",
												conf_no,
												publisher_id,
												publisher_private_num,
												xml
												);

	// Get host conference
	host_conf = get_conf_by_num(conf_no);
	if (host_conf == NULL) {
		// No user in conf ever before MCB
		host_conf = build_conf_by_num(conf_no);
	}

	if (host_conf == NULL) {
		ast_free(extra_text);
		return;
	}

	process_remote_xml(host_conf, publisher_private_num, xml, &extra_text);

	// Prepare the stasis data
	RAII_VAR(struct ast_json *, to_send_json_object, NULL, ast_json_unref);


	to_send_json_object = ast_json_pack("{s: i, s: s, s: i, s: s, s: s, s: s}",
		"user_type", CEI_TYPE_EXTERNAL_MCB,
		"conference", conf_no,
		"version_reset", host_conf->version_reset,
		"user_name", publisher_id,
		"partial_text", ast_str_buffer(extra_text),
		"event_type", CEI_NOTIFY_EVENT_EXT_MCB
	);

	cei_publish_manager_event(to_send_json_object);

	/*-1 for using get_conf_by_num or build_conf_by_num*/
	ast_atomic_fetchadd_int(&host_conf->refcount, -1);
	ast_debug(LOG_DEBUG, "cei_ext_mcb_process_xml_cb leave refcount %d", host_conf->refcount);
	ast_free(extra_text);
}

static void cei_int_mcb_process_string_cb(void *data, struct stasis_subscription *sub,
	struct stasis_message *message)
{
	ast_log(LOG_DEBUG, "[TK] Enter...%s \n", __FUNCTION__);
	struct cei_conference *host_conf = NULL;
	struct cei_conference *remote_conf = NULL;
	struct ast_str *extra_text = ast_str_create(CEI_EVENT_DATA_LEN);

	struct ast_json_payload *payload     = NULL;
	struct ast_json			*json_object = NULL;

	payload = stasis_message_data(message);
	if (NULL == payload) {
		ast_log(LOG_WARNING, "Payload from stasis message is NULL");
		return;
	}

	json_object = payload->json;
	if (json_object == NULL) {
		ast_debug(LOG_DEBUG, "[WL] Imcomplete message.");
		return;
	}
	const char *conf_no = ast_json_string_get(ast_json_object_get(json_object, "conf_num"));
	const char *content = ast_json_string_get(ast_json_object_get(json_object, "content"));
	const char *publisher_id = ast_json_string_get(ast_json_object_get(json_object, "publisher_id"));

	if (conf_no == NULL || content == NULL || publisher_id == NULL) {
		ast_debug(LOG_DEBUG, "[WL] Imcomplete data. \n");
		return;
	}
	ast_debug(LOG_DEBUG, "[WL] Got mcb data : conf %s, publisher %s,%s",
												conf_no,
												publisher_id,
												content
												);

	// Get host conference
	host_conf = get_conf_by_num(conf_no);
	if (host_conf == NULL) {
		host_conf = build_conf_by_num(conf_no);
	}

	if (host_conf == NULL) {
		ast_free(extra_text);
		return;
	}

	remote_conf = get_conf_by_id(publisher_id);
	if (remote_conf == NULL) {
		ast_free(extra_text);
		/*-1 for using build_conf_by_num*/
		ast_atomic_fetchadd_int(&host_conf->refcount, -1);
		ast_debug(LOG_DEBUG, "cei_int_mcb_process_string_cb leave refcount %d", host_conf->refcount);
		return;
	}

	//change to string
	char *parsing_content = ast_strdup(content);
	process_remote_string(host_conf, remote_conf->conference_num, publisher_id, parsing_content, &extra_text);

	// Prepare the stasis data
	RAII_VAR(struct ast_json *, to_send_json_object, NULL, ast_json_unref);

	to_send_json_object = ast_json_pack("{s: i, s: s, s: i, s: s, s: s, s: s}",
		"user_type", CEI_TYPE_INTERNAL_MCB,
		"conference", conf_no,
		"version_reset", host_conf->version_reset,
		"user_name", publisher_id,
		"partial_text", ast_str_buffer(extra_text),
		"event_type", CEI_NOTIFY_EVENT_INT_MCB
	);

	cei_publish_manager_event(to_send_json_object);

	/*-1 for using build_conf_by_num*/
	ast_atomic_fetchadd_int(&host_conf->refcount, -1);
	ast_debug(LOG_DEBUG, "cei_int_mcb_process_string_cb leave refcount %d", host_conf->refcount);
	/*-1 for using get_conf_by_id*/
	ast_atomic_fetchadd_int(&remote_conf->refcount, -1);
	ast_debug(LOG_DEBUG, "cei_int_mcb_process_string_cb leave refcount %d", remote_conf->refcount);

	// Clean up
	ast_free(extra_text);
	ast_free(parsing_content);
}

static void cei_ext_trunk_conf_process_xml_cb(void *data, struct stasis_subscription *sub,
	struct stasis_message *message)
{
	ast_log(LOG_DEBUG, "[TK] Enter...%s \n", __FUNCTION__);
	struct cei_conference *host_conf = NULL;
	struct ast_str *extra_text = ast_str_create(CEI_EVENT_DATA_LEN);

	struct ast_json_payload *payload	 = NULL;
	struct ast_json 		*json_object = NULL;

	payload = stasis_message_data(message);
	if (NULL == payload) {
		ast_debug(LOG_DEBUG, "Got NULL payload from stasis message.");
		return;
	}

	json_object = payload->json;
	if (json_object == NULL) {
		ast_debug(LOG_DEBUG, "[WL] Imcomplete message.");
		return;
	}

	const char *remote_conf_num = ast_json_string_get(ast_json_object_get(json_object, "remote_conf_num"));
	const char *xml = ast_json_string_get(ast_json_object_get(json_object, "xml"));
	const char *local_virtual_conf_num = ast_json_string_get(ast_json_object_get(json_object, "local_virtual_conf_num"));

	if (remote_conf_num == NULL || xml == NULL || local_virtual_conf_num ==NULL) {
		ast_debug(LOG_DEBUG, "[WL] Imcomplete data. \n");
		return;
	}
	ast_debug(LOG_DEBUG, "[WL] Got trunk conf data : local conf %s,remote:%s  content %s",
												local_virtual_conf_num,
												remote_conf_num,
												xml
												);

	// Get host conference
	host_conf = get_conf_by_num(local_virtual_conf_num);
	if (host_conf == NULL) {
		// No user in conf ever before etc
		host_conf = build_conf_by_num(local_virtual_conf_num);
	}

	if (host_conf == NULL) {
		ast_free(extra_text);
		return;
	}
	host_conf ->is_virtial_trunk_conf = 1;
	int full_text = 0;
	process_trunk_conf_xml(host_conf, remote_conf_num, xml, &extra_text,&full_text);

	// Prepare the stasis data
	RAII_VAR(struct ast_json *, to_send_json_object, NULL, ast_json_unref);

	if (full_text) {
		to_send_json_object = ast_json_pack("{s: i, s: s, s: i, s: s, s: s, s: s}",
			"user_type", CEI_TYPE_TRUNK_CONF_USER,
			"conference", local_virtual_conf_num,
			"version_reset", host_conf->version_reset,
			"user_name", remote_conf_num,
			"full_text",ast_str_buffer(extra_text),
			"event_type", CEI_NOTIFY_EVENT_TRUNK_CONF
		);
	} else {
		to_send_json_object = ast_json_pack("{s: i, s: s, s: i, s: s, s: s, s: s}",
			"user_type", CEI_TYPE_TRUNK_CONF_USER,
			"conference", local_virtual_conf_num,
			"version_reset", host_conf->version_reset,
			"user_name", remote_conf_num,
			"partial_text", ast_str_buffer(extra_text),
			"event_type", CEI_NOTIFY_EVENT_TRUNK_CONF
		);
	}
	ast_debug(LOG_DEBUG, "[WL] Send cei data : %s",
												ast_str_buffer(extra_text)
												);

	cei_publish_manager_event(to_send_json_object);

	/*-1 for using get_conf_by_num or build_conf_by_num*/
	ast_atomic_fetchadd_int(&host_conf->refcount, -1);
	ast_debug(LOG_DEBUG, "cei_ext_trunk_conf_process_xml_cb leave refcount %d", host_conf->refcount);
	ast_free(extra_text);
}

static void cei_ext_trunk_conf_remove_cb(void *data, struct stasis_subscription *sub,
	struct stasis_message *message)
{
	ast_log(LOG_DEBUG, "[TK] Enter...%s \n", __FUNCTION__);

	struct ast_json_payload *payload	 = NULL;
	struct ast_json 		*json_object = NULL;

	payload = stasis_message_data(message);
	if (NULL == payload) {
		ast_debug(LOG_DEBUG, "Got NULL payload from stasis message.");
		return;
	}

	json_object = payload->json;
	if (json_object == NULL) {
		ast_debug(LOG_DEBUG, "[WL] Imcomplete message.");
		return;
	}

	const char *local_virtual_conf_num = ast_json_string_get(ast_json_object_get(json_object, "local_virtual_conf_num"));

	if (local_virtual_conf_num ==NULL) {
		ast_debug(LOG_DEBUG, "[WL] Imcomplete data. \n");
		return;
	}
	ast_debug(LOG_DEBUG, "[WL] cei_ext_trunk_conf_remove_cb : local conf %s",
												local_virtual_conf_num
												);

	ast_conf_remove_conference(local_virtual_conf_num);
	
}

/* !begin Modified by pyzhong@grandstream.cn for bug[65207], 2016-11-3*/
static void cei_refresh_cb(void *data, struct stasis_subscription *sub,
	struct stasis_message *message)
{
	struct ast_json			*json_object	= NULL;
	struct ast_json_payload *payload		= NULL;
	const char				*conference_name = NULL;
	const char				*user_channel	= NULL;
	struct cei_conference *conf = NULL;
	RAII_VAR(struct cei_user *, user, NULL, ao2_cleanup);
	struct ast_str *full_text = NULL;

	// Sanity check
	if (message == NULL) {
		ast_log(LOG_WARNING, "Stasis message is NULL");
		return;
	}

	payload = stasis_message_data(message);
	if (payload == NULL) {
		ast_debug(LOG_WARNING, "Got NULL payload from stasis message.");
		return;
	}

	json_object = payload->json;
	if (json_object == NULL) {
		ast_log(LOG_WARNING, "JSON object from stasis message is NULL");
		return;
	}

	conference_name = ast_json_string_get(ast_json_object_get(json_object, "conference"));
	user_channel = ast_json_string_get(ast_json_object_get(json_object, "user_channel"));
	if (conference_name == NULL || user_channel == NULL) {
		ast_log(LOG_WARNING, "Incomplete JSON object data");
		return;
	}

	conf = get_conf_by_num(conference_name);
	if (conf == NULL) {
		ast_log(LOG_WARNING, "couldn't find the cei conference information for %s", conference_name);
		return;
	}

	user = get_user_by_channel_name(conf, user_channel, FALSE);
	if (user == NULL) {
		ast_log(LOG_WARNING, "couldn't find the cei user information for %s", user_channel);
		/*-1 for using get_conf_by_num or build_conf_by_num*/
		ast_atomic_fetchadd_int(&conf->refcount, -1);
		ast_debug(LOG_DEBUG, "cei_refresh_cb leave refcount %d", conf->refcount);
		return;
	}

	full_text = ast_str_create(CEI_EVENT_DATA_LEN);
	if (full_text == NULL){
		/*-1 for using get_conf_by_num or build_conf_by_num*/
		ast_atomic_fetchadd_int(&conf->refcount, -1);
		ast_debug(LOG_DEBUG, "cei_refresh_cb leave refcount %d", conf->refcount);
		return;
	}

	poll_cei_conference(conf, &full_text);

	// Prepare the stasis data
	RAII_VAR(struct ast_json *, to_send_json_object, NULL, ast_json_unref);

	to_send_json_object = ast_json_pack("{s: i, s: s, s: i, s: i, s: s, s: s, s: s, s: s}",
		"user_type", user->user_type,
		"conference", conference_name,
		"version_reset", conf->version_reset,
		"user_role", user->user_role,
		"user_name", check_role_as_conference(user) ? user->self_conf_id : user->channel_name,
		"full_text", ast_str_buffer(full_text),
		"event_type", CEI_NOTIFY_EVENT_REFRESH_CEI,
		"cid_number", user->cid_number
	);

	cei_publish_manager_event(to_send_json_object);

	if (user->user_type == CEI_TYPE_NORMAL_USER) {
		cei_talking_notify(conf, user);
	}
	/*-1 for using get_conf_by_num or build_conf_by_num*/
	ast_atomic_fetchadd_int(&conf->refcount, -1);
	ast_debug(LOG_DEBUG, "cei_refresh_cb leave refcount %d", conf->refcount);

	ast_free(full_text);
}
/* !end Modified by pyzhong@grandstream.cn for bug[65207], 2016-11-3*/

static void cei_sub_cb(void *data, struct stasis_subscription *sub,
	struct stasis_message *message)
{
	struct ast_json 		*json_object	= NULL;
	struct ast_json_payload *payload		= NULL;
	const char				*conference_name = NULL;
	const char				*user_channel	= NULL;
	struct cei_conference *conf = NULL;
	RAII_VAR(struct cei_user *, user, NULL, ao2_cleanup);
	struct ast_str *full_text = NULL;
	// Prepare the stasis data
	RAII_VAR(struct ast_json *, to_send_json_object, NULL, ast_json_unref);

	// Sanity check
	if (message == NULL) {
		ast_log(LOG_WARNING, "Stasis message is NULL");
		return;
	}

	payload = stasis_message_data(message);
	if (payload == NULL) {
		ast_debug(LOG_WARNING, "Got NULL payload from stasis message.");
		return;
	}

	json_object = payload->json;
	if (json_object == NULL) {
		ast_log(LOG_WARNING, "JSON object from stasis message is NULL");
		return;
	}

	conference_name = ast_json_string_get(ast_json_object_get(json_object, "conference"));
	user_channel = ast_json_string_get(ast_json_object_get(json_object, "user_channel"));
	if (conference_name == NULL || user_channel == NULL) {
		ast_log(LOG_WARNING, "Incomplete JSON object data");
		return;
	}

	conf = get_conf_by_num(conference_name);
	if (conf == NULL) {
		ast_log(LOG_WARNING, "couldn't find the cei conference information for %s", conference_name);
		return;
	}

	if(conf->is_virtial_trunk_conf) {

		full_text = ast_str_create(CEI_EVENT_DATA_LEN);
		if (full_text == NULL){
			/*-1 for using get_conf_by_num or build_conf_by_num*/
			ast_atomic_fetchadd_int(&conf->refcount, -1);
			ast_debug(LOG_DEBUG, "cei_sub_cb leave refcount %d", conf->refcount);
			return;
		}

		poll_cei_conference(conf, &full_text);

		to_send_json_object = ast_json_pack("{s: s, s: i, s: s, s: s, s: i, s: s}",
			"conference", conference_name,
			"version_reset", conf->version_reset,
			"full_text", ast_str_buffer(full_text),
			"event_type", CEI_NOTIFY_EVENT_TRUNK_CONF,
			"preverified_user", 1,
			"conference_type", conf->video_conference ? "video" : "audio"
		);
		cei_publish_manager_event(to_send_json_object);

	}
	else {
		user = get_user_by_channel_name(conf, user_channel, FALSE);
		if (user == NULL) {
			ast_log(LOG_WARNING, "couldn't find the cei user information for %s", user_channel);
			/*-1 for using get_conf_by_num or build_conf_by_num*/
			ast_atomic_fetchadd_int(&conf->refcount, -1);
			ast_debug(LOG_DEBUG, "cei_sub_cb leave refcount %d", conf->refcount);
			return;
		}

		if (user->user_type != CEI_TYPE_NORMAL_USER) {
			ast_log(LOG_DEBUG, "User '%s' isn't the normal user.", user->cid_number);
			/*-1 for using get_conf_by_num or build_conf_by_num*/
			ast_atomic_fetchadd_int(&conf->refcount, -1);
			ast_debug(LOG_DEBUG, "cei_sub_cb leave refcount %d", conf->refcount);
			return;
		}

		full_text = ast_str_create(CEI_EVENT_DATA_LEN);
		if (full_text == NULL){
			/*-1 for using get_conf_by_num or build_conf_by_num*/
			ast_atomic_fetchadd_int(&conf->refcount, -1);
			ast_debug(LOG_DEBUG, "cei_sub_cb leave refcount %d", conf->refcount);
			return;
		}

		poll_cei_conference(conf, &full_text);


		to_send_json_object = ast_json_pack("{s: i, s: s, s: i, s: i, s: s, s: s, s: s, s: s, s: i, s: s}",
			"user_type", user->user_type,
			"conference", conference_name,
			"version_reset", conf->version_reset,
			"user_role", user->user_role,
			"user_name", check_role_as_conference(user) ? user->self_conf_id : user->channel_name,
			"full_text", ast_str_buffer(full_text),
			"event_type", CEI_NOTIFY_EVENT_SUBCRIBE_CEI,
			"cid_number", user->cid_number,
			"preverified_user", 1,
			"conference_type", conf->video_conference ? "video" : "audio"
		);

		cei_publish_manager_event(to_send_json_object);

		if (user->user_type == CEI_TYPE_NORMAL_USER) {
			cei_talking_notify(conf, user);
		}
	}


	/*-1 for using get_conf_by_num or build_conf_by_num*/
	ast_atomic_fetchadd_int(&conf->refcount, -1);
	ast_debug(LOG_DEBUG, "cei_sub_cb leave refcount %d", conf->refcount);

	ast_free(full_text);
}

static void cei_reset_role_admin(struct cei_conference *conf)
{
	struct cei_user *user = NULL;

	ao2_lock(conf->user_container);
	struct ao2_iterator user_iter = ao2_iterator_init(conf->user_container, 0);
	while ((user = ao2_iterator_next(&user_iter))) {
		if (user->admin) {
			user->admin = 0;
			if (user->can_invite) {
				user->user_role = CEI_ROLE_USER_INVITE;
 			} else {
 				user->user_role = CEI_ROLE_USER_NORMAL;
 			}
 		}
		ao2_ref(user, -1);
	}
	ao2_iterator_destroy(&user_iter);
	ao2_unlock(conf->user_container);
}

static void cei_moderator_cb(void *data, struct stasis_subscription *sub,
	struct stasis_message *message)
{
	struct cei_conference *conf = NULL;
	RAII_VAR(struct cei_user *, user, NULL, ao2_cleanup);
	struct ast_str *full_text = ast_str_create(CEI_EVENT_DATA_LEN);
	struct ast_str *partial_text = ast_str_create(CEI_EVENT_DATA_LEN);

	const struct ast_bridge_blob *blob = stasis_message_data(message);
	unsigned int user_number = ast_json_integer_get(ast_json_object_get(blob->blob, "user_number"));
	const char *conference_name = ast_json_string_get(ast_json_object_get(blob->blob, "conference"));
	const char *channel_name = ast_json_string_get(ast_json_object_get(blob->blob, "channel_name"));
	unsigned int admin_status = 0;
	if (ast_json_is_true(ast_json_object_get(blob->blob, "admin"))) {
			admin_status = 1;
	}
	const char *cid_number = ast_json_string_get(ast_json_object_get(blob->blob, "caller_id_number"));

	ast_debug(LOG_DEBUG, "[WL] moderator update: user_num:%d, cid_number:%s, channel_name:%s, conf_name:%s, admin_status:%d",
												user_number,
												cid_number,
												channel_name,
												conference_name,
												admin_status
	);

	if (conference_name == NULL || channel_name == NULL) {
		ast_debug(LOG_DEBUG, "[WL] EVENT INCOMPLETE");
		ast_free(full_text);
		ast_free(partial_text);
		return;
	}

	conf = build_conf_by_num(conference_name);

	ao2_lock(conf->user_container);

	cei_reset_role_admin(conf);

	user = get_user_by_channel_name(conf, channel_name, 1);
	if (user) {
		user->admin = (int)admin_status;
		if (user->admin) {
			user->user_role = CEI_ROLE_ADMIN;
		} else if (user->can_invite) {
			user->user_role = CEI_ROLE_USER_INVITE;
		} else {
			user->user_role = CEI_ROLE_USER_NORMAL;
		}
	} else {
		ast_debug(LOG_WARNING, "[WL] ERROR, CANNOT FIND USER NUMBER %d IN CONFERENCE %s", user_number, conf->conference_num);
		ast_free(full_text);
		ast_free(partial_text);
		ao2_unlock(conf->user_container);
		/*-1 for using build_conf_by_num*/
		ast_atomic_fetchadd_int(&conf->refcount, -1);
		ast_debug(LOG_DEBUG, "cei_moderator_cb leave refcount %d", conf->refcount);
		return;
	}
	ao2_unlock(conf->user_container);

	poll_cei_conference(conf, &full_text);
	poll_cei_partial_notify(conf, NULL, user, &partial_text, CEI_EVENT_MODERATOR_UPDATE);

	// Prepare the stasis data
	RAII_VAR(struct ast_json *, json_object, NULL, ast_json_unref);

	json_object = ast_json_pack("{s: s, s: i, s: s, s: s, s: s}",
			"conference", conference_name,
			"version_reset", conf->version_reset,
			"full_text", ast_str_buffer(full_text),
			"partial_text", ast_str_buffer(partial_text),
			"event_type", CEI_NOTIFY_EVENT_MODERATOR_UPDATE
	);

	cei_publish_manager_event(json_object);

	/*-1 for using build_conf_by_num*/
	ast_atomic_fetchadd_int(&conf->refcount, -1);
	ast_debug(LOG_DEBUG, "cei_moderator_cb leave refcount %d", conf->refcount);

	ast_free(full_text);
	ast_free(partial_text);
}

static void cei_refresh_all_cb(void *data, struct stasis_subscription *sub,
	struct stasis_message *message)
{
	struct cei_conference *conf = NULL;
	struct ast_str *full_text = ast_str_create(CEI_EVENT_DATA_LEN);

	const struct ast_bridge_blob *blob = stasis_message_data(message);
	const char *conference_name = ast_json_string_get(ast_json_object_get(blob->blob, "conference"));
	unsigned int update_talking_only = ast_json_integer_get(ast_json_object_get(blob->blob, "update_talking_only"));

	// get cei_conference
	conf = get_conf_by_num(conference_name);
	if (!conf) {
		ast_free(full_text);
		return;
	}

	if (update_talking_only) {
		cei_talking_notify(conf, NULL);
		ast_free(full_text);
		/*-1 for using get_conf_by_num or build_conf_by_num*/
		ast_atomic_fetchadd_int(&conf->refcount, -1);
		ast_debug(LOG_DEBUG, "cei_refresh_all_cb leave refcount %d", conf->refcount);
		return;
	}

	poll_cei_conference(conf, &full_text);

	// Prepare the stasis data
	RAII_VAR(struct ast_json *, json_object, NULL, ast_json_unref);

	json_object = ast_json_pack("{s: s, s: i, s: s, s: s}",
		"conference", conference_name,
		"version_reset", conf->version_reset,
		"full_text", ast_str_buffer(full_text),
		"event_type", CEI_NOTIFY_EVENT_MEDIA_UPDATE
	);

	cei_publish_manager_event(json_object);

	cei_talking_notify(conf, NULL);
	/*-1 for using get_conf_by_num or build_conf_by_num*/
	ast_atomic_fetchadd_int(&conf->refcount, -1);
	ast_debug(LOG_DEBUG, "cei_refresh_all_cb leave refcount %d", conf->refcount);

	ast_free(full_text);
}

static void cei_mute_status_cb(void *data, struct stasis_subscription *sub,
	struct stasis_message *message)
{
	struct cei_conference *conf = NULL;
	struct ast_str *extra_text = ast_str_create(CEI_EVENT_DATA_LEN);
	struct ast_str *full_text = ast_str_create(CEI_EVENT_DATA_LEN);

	const struct ast_bridge_blob *blob = stasis_message_data(message);
	unsigned int mute_status = ast_json_integer_get(ast_json_object_get(blob->blob, "muted"));
	const char *conference_name = ast_json_string_get(ast_json_object_get(blob->blob, "conference"));

	if (conference_name == NULL) {
		ast_debug(LOG_DEBUG, "[WL] EVENT INCOMPLETE");
		ast_free(extra_text);
		ast_free(full_text);
		return;
	}

	// get cei_conference
	conf = build_conf_by_num(conference_name);
	ast_debug(LOG_DEBUG, "[WL] Mute conference: mute_status: %u, conf_name:%s;",
												mute_status,
												conference_name);
	if (!conf) {
		ast_free(extra_text);
		ast_free(full_text);
		return;
	}

	conf->muted = mute_status ? TRUE : FALSE;

	poll_cei_conference(conf, &full_text);
	poll_cei_partial_notify(conf, NULL, NULL, &extra_text, CEI_EVENT_TOGGLE_LOCK);

	// Prepare the stasis data
	RAII_VAR(struct ast_json *, json_object, NULL, ast_json_unref);

	json_object = ast_json_pack("{s: s, s: i, s: s, s: s, s: s}",
		"conference", conference_name,
		"version_reset", conf->version_reset,
		"full_text", ast_str_buffer(full_text),
		"partial_text", ast_str_buffer(extra_text),
		"event_type", CEI_NOTIFY_EVENT_LOCK
	);

	cei_publish_manager_event(json_object);

	/*-1 for using build_conf_by_num*/
	ast_atomic_fetchadd_int(&conf->refcount, -1);
	ast_debug(LOG_DEBUG, "cei_mute_status_cb leave refcount %d", conf->refcount);

	ast_free(extra_text);
	ast_free(full_text);
}

static void cei_role_update_cb(void *data, struct stasis_subscription *sub,
	struct stasis_message *message)
{
	struct cei_conference *conf = NULL;
	RAII_VAR(struct cei_user *, user, NULL, ao2_cleanup);
	struct ast_str *extra_text = ast_str_create(CEI_EVENT_DATA_LEN);

	const struct ast_bridge_blob *blob = stasis_message_data(message);
	unsigned int user_number = ast_json_integer_get(ast_json_object_get(blob->blob, "user_number"));
	const char *conference_name = ast_json_string_get(ast_json_object_get(blob->blob, "conference"));
	const char *channel_name = ast_json_string_get(ast_json_object_get(blob->blob, "channel_name"));
	unsigned int wait = ast_json_integer_get(ast_json_object_get(blob->blob, "wait"));
	unsigned int muted = ast_json_integer_get(ast_json_object_get(blob->blob, "muted"));
	unsigned int mic_muted = ast_json_integer_get(ast_json_object_get(blob->blob, "mic_muted"));
	unsigned int can_invite = ast_json_integer_get(ast_json_object_get(blob->blob, "can_invite"));

	int admin = 0;
	if (ast_json_is_true(ast_json_object_get(blob->blob, "admin"))) {
		admin = 1;
	}
	const char *cid_number = ast_json_string_get(ast_json_object_get(blob->blob, "caller_id_number"));

	if (conference_name == NULL || channel_name == NULL) {
		ast_debug(LOG_DEBUG, "[WL] EVENT INCOMPLETE");
		ast_free(extra_text);
		return;
	}

	ast_debug(LOG_DEBUG, "[WL] role update: user_num:%d, cid_number:%s, channel_name:%s, conf_name:%s, muted:%d, mic_muted:%d",
												user_number,
												cid_number,
												channel_name,
												conference_name,
												muted,
												mic_muted
	);

	// get cei_conference
	conf = build_conf_by_num(conference_name);
	if (!conf) {
		ast_free(extra_text);
		return;
	}

	ao2_lock(conf->user_container);

	user = get_user_by_channel_name(conf, channel_name, 1);
	if (!user) {
		ast_debug(LOG_WARNING, "[WL] ERROR, CANNOT FIND USER NUMBER %d IN CONFERENCE %s", user_number, conf->conference_num);
		ao2_unlock(conf->user_container);
		ast_free(extra_text);
		/*-1 for using build_conf_by_num*/
		ast_atomic_fetchadd_int(&conf->refcount, -1);
		ast_debug(LOG_DEBUG, "cei_role_update_cb leave refcount %d", conf->refcount);
		return;
	}

	user->muted = muted;
	user->mic_muted = mic_muted;
	user->admin = admin;
	user->can_invite = can_invite;
	if (user->admin) {
		user->user_role = CEI_ROLE_ADMIN;
	} else if (user->can_invite) {
		user->user_role = CEI_ROLE_USER_INVITE;
	} else {
		user->user_role = CEI_ROLE_USER_NORMAL;
	}

	ao2_unlock(conf->user_container);

	if (wait) {
		ast_free(extra_text);
		/*-1 for using build_conf_by_num*/
		ast_atomic_fetchadd_int(&conf->refcount, -1);
		ast_debug(LOG_DEBUG, "cei_role_update_cb leave refcount %d", conf->refcount);
		return;
	}

	poll_cei_partial_notify(conf, NULL, user, &extra_text, CEI_EVENT_ROLE_CHANGE);

	// Prepare the stasis data
	RAII_VAR(struct ast_json *, json_object, NULL, ast_json_unref);

	json_object = ast_json_pack("{s: i, s: s, s: i, s: s, s: s}",
		"user_type", user->user_type,
		"conference", conference_name,
		"version_reset", conf->version_reset,
		"partial_text", ast_str_buffer(extra_text),
		"event_type", CEI_NOTIFY_EVENT_ROLE_CHANGE
	);

	cei_publish_manager_event(json_object);

	/*talking for wave. Bug 191753 Bug 194443*/
	ao2_lock(conf->user_container);
	cei_talking_notify(conf, NULL);
	ao2_unlock(conf->user_container);

	/*-1 for using build_conf_by_num*/
	ast_atomic_fetchadd_int(&conf->refcount, -1);
	ast_debug(LOG_DEBUG, "cei_role_update_cb leave refcount %d", conf->refcount);

	ast_free(extra_text);
}

static void cei_name_update_cb(void *data, struct stasis_subscription *sub,
	struct stasis_message *message)
{
	struct cei_conference *conf = NULL;
	struct ast_str *extra_text = ast_str_create(CEI_EVENT_DATA_LEN);
	struct ast_str *full_text = ast_str_create(CEI_EVENT_DATA_LEN);

	const struct ast_bridge_blob *blob = stasis_message_data(message);
	unsigned int user_number = ast_json_integer_get(ast_json_object_get(blob->blob, "user_number"));
	const char *channel_name = ast_json_string_get(ast_json_object_get(blob->blob, "channel_name"));
	const char *conference_name = ast_json_string_get(ast_json_object_get(blob->blob, "conference"));
	const char *name = ast_json_string_get(ast_json_object_get(blob->blob, "name"));

	ast_debug(LOG_DEBUG, "[WL] update user name: user_num:%d, channel_name:%s, conf_name %s, name:%s",
												user_number,
												channel_name,
												conference_name,
												name
	);

	if (conference_name == NULL) {
		ast_debug(LOG_DEBUG, "[WL] EVENT INCOMPLETE");
		ast_free(extra_text);
		ast_free(full_text);
		return;
	}

	conf = build_conf_by_num(conference_name);
	if (!conf) {
		ast_debug(LOG_WARNING, "[WL] Couldn't find the conference '%s'", conference_name);
		ast_free(extra_text);
		ast_free(full_text);
		return;
	}

	ao2_lock(conf->user_container);

	RAII_VAR(struct cei_user *, user, get_user_by_user_num(conf, (int)user_number), ao2_cleanup);
	if (user) {
		ast_copy_string(user->cid_name, name, sizeof(user->cid_name));
	} else {
		ast_debug(LOG_WARNING, "[WL] ERROR, CANNOT FIND USER NUMBER %d IN CONFERENCE %s", user_number, conf->conference_num);
		ao2_unlock(conf->user_container);
		ast_free(extra_text);
		ast_free(full_text);
		/*-1 for using build_conf_by_num*/
		ast_atomic_fetchadd_int(&conf->refcount, -1);
		ast_debug(LOG_DEBUG, "cei_name_update_cb leave refcount %d", conf->refcount);
		return;
	}
	ao2_unlock(conf->user_container);

	poll_cei_conference(conf, &full_text);
	poll_cei_partial_notify(conf, NULL, user, &extra_text, CEI_EVENT_TALKING);

	// Prepare the stasis data
	RAII_VAR(struct ast_json *, json_object, NULL, ast_json_unref);

	json_object = ast_json_pack("{s: i, s: s, s: i, s: s, s: s, s: s}",
			"user_type", user->user_type,
			"conference", conference_name,
			"version_reset", conf->version_reset,
			"full_text", ast_str_buffer(full_text),
			"partial_text", ast_str_buffer(extra_text),
			"event_type", CEI_NOTIFY_EVENT_OTHER
	);

	cei_publish_manager_event(json_object);

	/*-1 for using build_conf_by_num*/
	ast_atomic_fetchadd_int(&conf->refcount, -1);
	ast_debug(LOG_DEBUG, "cei_name_update_cb leave refcount %d", conf->refcount);

	ast_free(extra_text);
	ast_free(full_text);
}

static void cei_focus_status_cb(void *data, struct stasis_subscription *sub,
	struct stasis_message *message)
{
	struct cei_conference *conf = NULL;
	struct ast_str *extra_text = ast_str_create(CEI_EVENT_DATA_LEN);
	struct ast_str *full_text = ast_str_create(CEI_EVENT_DATA_LEN);

	const struct ast_bridge_blob *blob = stasis_message_data(message);
	unsigned int user_number = ast_json_integer_get(ast_json_object_get(blob->blob, "user_number"));
	const char *channel_name = ast_json_string_get(ast_json_object_get(blob->blob, "channel_name"));
	const char *conference_name = ast_json_string_get(ast_json_object_get(blob->blob, "conference"));
	unsigned int isfocus = ast_json_integer_get(ast_json_object_get(blob->blob, "isfocus"));
	const char *conf_layout= ast_json_string_get(ast_json_object_get(blob->blob, "layout"));

	ast_debug(LOG_DEBUG, "[WL] focus status cb: user_num:%d, channel_name:%s, conf_name %s, isfocus:%d, conf_layout:%s",
												user_number,
												channel_name,
												conference_name,
												isfocus,
												conf_layout
	);

	if (conference_name == NULL || conf_layout == NULL) {
		ast_debug(LOG_DEBUG, "[WL] EVENT INCOMPLETE");
		ast_free(extra_text);
		ast_free(full_text);
		return;
	}

	conf = build_conf_by_num(conference_name);
	if (!conf) {
		ast_debug(LOG_WARNING, "[WL] Couldn't find the conference '%s'", conference_name);
		ast_free(extra_text);
		ast_free(full_text);
		return;
	}

	ast_copy_string(conf->conf_layout, conf_layout, sizeof(conf->conf_layout));

	ao2_lock(conf->user_container);

	RAII_VAR(struct cei_user *, user, get_user_by_user_num(conf, (int)user_number), ao2_cleanup);
	if (user) {
		user->isfocus = isfocus;
	} else {
		ast_debug(LOG_WARNING, "[WL] ERROR, CANNOT FIND USER NUMBER %d IN CONFERENCE %s", user_number, conf->conference_num);
		ao2_unlock(conf->user_container);
		ast_free(extra_text);
		ast_free(full_text);
		/*-1 for using build_conf_by_num*/
		ast_atomic_fetchadd_int(&conf->refcount, -1);
		ast_debug(LOG_DEBUG, "cei_focus_status_cb leave refcount %d", conf->refcount);
		return;
	}
	ao2_unlock(conf->user_container);

	poll_cei_conference(conf, &full_text);
	poll_cei_partial_notify(conf, NULL, user, &extra_text, CEI_EVENT_TALKING);

	// Prepare the stasis data
	RAII_VAR(struct ast_json *, json_object, NULL, ast_json_unref);

	json_object = ast_json_pack("{s: i, s: s, s: i, s: s, s: s, s: s}",
			"user_type", user->user_type,
			"conference", conference_name,
			"version_reset", conf->version_reset,
			"full_text", ast_str_buffer(full_text),
			"partial_text", ast_str_buffer(extra_text),
			"event_type", CEI_NOTIFY_EVENT_OTHER
	);

	cei_publish_manager_event(json_object);

	/*-1 for using build_conf_by_num*/
	ast_atomic_fetchadd_int(&conf->refcount, -1);
	ast_debug(LOG_DEBUG, "cei_focus_status_cb leave refcount %d", conf->refcount);

	ast_free(extra_text);
	ast_free(full_text);
}

static void cei_layout_status_cb(void *data, struct stasis_subscription *sub,
	struct stasis_message *message)
{
	struct cei_conference *conf;
	struct ast_str *extra_text = ast_str_create(CEI_EVENT_DATA_LEN);
	struct ast_str *full_text = ast_str_create(CEI_EVENT_DATA_LEN);

	const struct ast_bridge_blob *blob = stasis_message_data(message);
	const char *conf_layout= ast_json_string_get(ast_json_object_get(blob->blob, "layout"));
	const char *conference_name = ast_json_string_get(ast_json_object_get(blob->blob, "conference"));

	if (conf_layout == NULL || conference_name == NULL) {
		ast_debug(LOG_DEBUG, "[WL] EVENT INCOMPLETE");
		ast_free(extra_text);
		ast_free(full_text);
		return;
	}

	// get cei_conference
	conf = build_conf_by_num(conference_name);
	ast_debug(LOG_DEBUG, "[WL] Conference layout: conf_layout: %s, conf_name:%s;",
												conf_layout,
												conference_name
	);
	if (!conf) {
		ast_free(extra_text);
		ast_free(full_text);
		return;
	}
	ao2_lock(conf);
	memset(conf->conf_layout, 0, sizeof(conf->conf_layout));
	ast_copy_string(conf->conf_layout, conf_layout, sizeof(conf->conf_layout));
	ao2_unlock(conf);

	poll_cei_conference(conf, &full_text);
	poll_cei_partial_notify(conf, NULL, NULL, &extra_text, CEI_EVENT_TOGGLE_LOCK);

	// Prepare the stasis data
	RAII_VAR(struct ast_json *, json_object, NULL, ast_json_unref);

	json_object = ast_json_pack("{s: s, s: i, s: s, s: s, s: s}",
		"conference", conference_name,
		"version_reset", conf->version_reset,
		"full_text", ast_str_buffer(full_text),
		"partial_text", ast_str_buffer(extra_text),
		"event_type", CEI_NOTIFY_EVENT_LOCK
	);

	cei_publish_manager_event(json_object);
	/*-1 for using build_conf_by_num*/
	ast_atomic_fetchadd_int(&conf->refcount, -1);
	ast_debug(LOG_DEBUG, "cei_layout_status_cb leave refcount %d", conf->refcount);

	ast_free(extra_text);
	ast_free(full_text);
}

static void cei_user_permission_cb(void *data, struct stasis_subscription *sub,
	struct stasis_message *message)
{
	struct cei_conference *conf;
	struct ast_str *extra_text = ast_str_create(CEI_EVENT_DATA_LEN);
	struct ast_str *full_text = ast_str_create(CEI_EVENT_DATA_LEN);

	const struct ast_bridge_blob *blob = stasis_message_data(message);
	const char *permission_type= ast_json_string_get(ast_json_object_get(blob->blob, "permission_type"));
	const char *conference_name = ast_json_string_get(ast_json_object_get(blob->blob, "conference"));
	const char *allow= ast_json_string_get(ast_json_object_get(blob->blob, "allow"));

	if (permission_type == NULL || conference_name == NULL || allow == NULL) {
		ast_debug(LOG_DEBUG, "[WL] EVENT INCOMPLETE");
		ast_free(extra_text);
		ast_free(full_text);
		return;
	}

	// get cei_conference
	conf = build_conf_by_num(conference_name);
	ast_debug(LOG_DEBUG, "[WL] User permission: permission_type: %s, conf_name:%s, allow:%s;",
												permission_type,
												conference_name,
												allow);
	if (!conf) {
		ast_free(extra_text);
		ast_free(full_text);
		return;
	}

	ao2_lock(conf);
	if (!strcasecmp(permission_type, "mute")) {
		if (ast_true(allow)) {
			conf->allow_user_unmute = 1;
		} else {
			conf->allow_user_unmute = 0;
		}
	}
	ao2_unlock(conf);

	poll_cei_conference(conf, &full_text);
	poll_cei_partial_notify(conf, NULL, NULL, &extra_text, CEI_EVENT_TOGGLE_LOCK);

	// Prepare the stasis data
	RAII_VAR(struct ast_json *, json_object, NULL, ast_json_unref);

	json_object = ast_json_pack("{s: s, s: i, s: s, s: s, s: s}",
		"conference", conference_name,
		"version_reset", conf->version_reset,
		"full_text", ast_str_buffer(full_text),
		"partial_text", ast_str_buffer(extra_text),
		"event_type", CEI_NOTIFY_EVENT_LOCK
	);

	cei_publish_manager_event(json_object);
	/*-1 for using build_conf_by_num*/
	ast_atomic_fetchadd_int(&conf->refcount, -1);
	ast_debug(LOG_DEBUG, "cei_user_permission_cb leave refcount %d", conf->refcount);

	ast_free(extra_text);
	ast_free(full_text);
}

static void cei_gesture_status_cb(void *data, struct stasis_subscription *sub,
	struct stasis_message *message)
{
	struct cei_conference *conf = NULL;
	RAII_VAR(struct cei_user *, user, NULL, ao2_cleanup);
	struct ast_str *extra_text = ast_str_create(CEI_EVENT_DATA_LEN);
	struct ast_str *full_text = ast_str_create(CEI_EVENT_DATA_LEN);

	const struct ast_bridge_blob *blob = stasis_message_data(message);
	unsigned int user_number = ast_json_integer_get(ast_json_object_get(blob->blob, "user_number"));
	const char *channel_name = ast_json_string_get(ast_json_object_get(blob->blob, "channel_name"));
	const char *conference_name = ast_json_string_get(ast_json_object_get(blob->blob, "conference"));
	time_t hand_up_time = ast_json_integer_get(ast_json_object_get(blob->blob, "hand_up_time"));

	ast_debug(LOG_DEBUG, "[WL] focus gesture cb: user_num:%d, channel_name:%s, conf_name %s, hand_up_time:%ld",
												user_number,
												channel_name,
												conference_name,
												hand_up_time
	);

	if (conference_name == NULL) {
		ast_debug(LOG_DEBUG, "[WL] EVENT INCOMPLETE");
		ast_free(extra_text);
		ast_free(full_text);
		return;
	}

	conf = build_conf_by_num(conference_name);
	if (!conf) {
		ast_debug(LOG_WARNING, "[WL] Couldn't find the conference '%s'", conference_name);
		ast_free(extra_text);
		ast_free(full_text);
		return;
	}

	ao2_lock(conf->user_container);

	if (ast_strlen_zero(channel_name)) {
		user = get_user_by_user_num(conf, (int)user_number);
	} else {
		user = get_user_by_channel_name(conf, channel_name, TRUE);
	}
	if (user) {
		user->hand_up_time = hand_up_time;
	} else {
		ast_debug(LOG_WARNING, "[WL] ERROR, CANNOT FIND USER NUMBER %d OR CHANNEL NAME %s IN CONFERENCE %s", user_number, channel_name ? channel_name : "", conf->conference_num);
		ao2_unlock(conf->user_container);
		ast_free(extra_text);
		ast_free(full_text);
		/*-1 for using build_conf_by_num*/
		ast_atomic_fetchadd_int(&conf->refcount, -1);
		ast_debug(LOG_DEBUG, "cei_gesture_status_cb leave refcount %d", conf->refcount);
		return;
	}
	ao2_unlock(conf->user_container);

	poll_cei_conference(conf, &full_text);
	poll_cei_partial_notify(conf, NULL, user, &extra_text, CEI_EVENT_TALKING);

	// Prepare the stasis data
	RAII_VAR(struct ast_json *, json_object, NULL, ast_json_unref);

	json_object = ast_json_pack("{s: i, s: s, s: i, s: s, s: s, s: s}",
			"user_type", user->user_type,
			"conference", conference_name,
			"version_reset", conf->version_reset,
			"full_text", ast_str_buffer(full_text),
			"partial_text", ast_str_buffer(extra_text),
			"event_type", CEI_NOTIFY_EVENT_OTHER
	);

	cei_publish_manager_event(json_object);

	/*-1 for using build_conf_by_num*/
	ast_atomic_fetchadd_int(&conf->refcount, -1);
	ast_debug(LOG_DEBUG, "cei_gesture_status_cb leave refcount %d", conf->refcount);

	ast_free(extra_text);
	ast_free(full_text);
}

static enum cei_act_event_type conf_act_2_cei_type(enum conf_act_type act_type)
{
	int i = 0;

	for (i = 0; i < CEI_ACT_TYPE_ALL; i++) {
		if (i == act_type) {
			return (enum cei_act_event_type)i;
		}
	}

	return CEI_ACT_TYPE_UNKNOWN;
}

static void cei_act_event_cb(void *data, struct stasis_subscription *sub,
	struct stasis_message *message)
{
	struct cei_conference *conf;
	struct ast_str *extra_text = ast_str_create(CEI_EVENT_DATA_LEN);
	struct ast_str *full_text = ast_str_create(CEI_EVENT_DATA_LEN);

	const struct ast_bridge_blob *blob = stasis_message_data(message);
	const char *conference_name = ast_json_string_get(ast_json_object_get(blob->blob, "conference"));
	unsigned int act_event_type = ast_json_integer_get(ast_json_object_get(blob->blob, "act_event_type"));

	if (conference_name == NULL) {
		ast_debug(LOG_DEBUG, "[WL] EVENT INCOMPLETE");
		ast_free(extra_text);
		ast_free(full_text);
		return;
	}

	// get cei_conference
	conf = build_conf_by_num(conference_name);
	ast_debug(LOG_DEBUG, "[WL] Conference layout: act_event_type: %d, conf_name:%s;",
												act_event_type,
												conference_name
	);
	if (!conf) {
		ast_free(extra_text);
		ast_free(full_text);
		return;
	}
	ao2_lock(conf);
	conf->act_event_type = conf_act_2_cei_type((enum conf_act_type)act_event_type);
	ao2_unlock(conf);

	if (conf->act_event_type == CEI_ACT_TYPE_PUT_DOWN_ALL_HANDS) {
		struct cei_user *user = NULL;

		ao2_lock(conf->user_container);
		struct ao2_iterator user_iter = ao2_iterator_init(conf->user_container, 0);
		while ((user = ao2_iterator_next(&user_iter))) {
			user->hand_up_time = 0;
			ao2_ref(user, -1);
		}
		ao2_iterator_destroy(&user_iter);
		ao2_unlock(conf->user_container);
	}
	poll_cei_conference(conf, &full_text);
	poll_cei_partial_notify(conf, NULL, NULL, &extra_text, CEI_EVENT_TOGGLE_LOCK);

	// Prepare the stasis data
	RAII_VAR(struct ast_json *, json_object, NULL, ast_json_unref);

	json_object = ast_json_pack("{s: s, s: i, s: s, s: s, s: s}",
		"conference", conference_name,
		"version_reset", conf->version_reset,
		"full_text", ast_str_buffer(full_text),
		"partial_text", ast_str_buffer(extra_text),
		"event_type", CEI_NOTIFY_EVENT_LOCK
	);

	cei_publish_manager_event(json_object);

	ao2_lock(conf);
	conf->act_event_type = CEI_ACT_TYPE_UNKNOWN;
	ao2_unlock(conf);

	/*-1 for using build_conf_by_num*/
	ast_atomic_fetchadd_int(&conf->refcount, -1);
	ast_debug(LOG_DEBUG, "cei_layout_status_cb leave refcount %d", conf->refcount);

	ast_free(extra_text);
	ast_free(full_text);
}
STASIS_MESSAGE_TYPE_DEFN(cei_join_type);
STASIS_MESSAGE_TYPE_DEFN(cei_leave_type);
STASIS_MESSAGE_TYPE_DEFN(cei_lock_status_type);
STASIS_MESSAGE_TYPE_DEFN(cei_toggle_mute_type);
STASIS_MESSAGE_TYPE_DEFN(cei_toggle_record_type);
STASIS_MESSAGE_TYPE_DEFN(cei_talk_status_type);
STASIS_MESSAGE_TYPE_DEFN(cei_moderator_type);
STASIS_MESSAGE_TYPE_DEFN(cei_refresh_all_type);
STASIS_MESSAGE_TYPE_DEFN(cei_mute_status_type);
STASIS_MESSAGE_TYPE_DEFN(cei_role_update_type);
STASIS_MESSAGE_TYPE_DEFN(cei_name_update_type);
STASIS_MESSAGE_TYPE_DEFN(cei_focus_status_type);
STASIS_MESSAGE_TYPE_DEFN(cei_layout_status_type);
STASIS_MESSAGE_TYPE_DEFN(cei_user_permission_type);
STASIS_MESSAGE_TYPE_DEFN(cei_gesture_status_type);
STASIS_MESSAGE_TYPE_DEFN(cei_act_event_type);

void manager_cei_shutdown(void) {
	STASIS_MESSAGE_TYPE_CLEANUP(cei_join_type);
	STASIS_MESSAGE_TYPE_CLEANUP(cei_leave_type);
	STASIS_MESSAGE_TYPE_CLEANUP(cei_lock_status_type);
	STASIS_MESSAGE_TYPE_CLEANUP(cei_toggle_mute_type);
	STASIS_MESSAGE_TYPE_CLEANUP(cei_toggle_record_type);
	STASIS_MESSAGE_TYPE_CLEANUP(cei_talk_status_type);
	STASIS_MESSAGE_TYPE_CLEANUP(cei_moderator_type);
	STASIS_MESSAGE_TYPE_CLEANUP(cei_refresh_all_type);
	STASIS_MESSAGE_TYPE_CLEANUP(cei_mute_status_type);
	STASIS_MESSAGE_TYPE_CLEANUP(cei_role_update_type);
	STASIS_MESSAGE_TYPE_CLEANUP(cei_name_update_type);
	STASIS_MESSAGE_TYPE_CLEANUP(cei_focus_status_type);
	STASIS_MESSAGE_TYPE_CLEANUP(cei_layout_status_type);
	STASIS_MESSAGE_TYPE_CLEANUP(cei_user_permission_type);
	STASIS_MESSAGE_TYPE_CLEANUP(cei_gesture_status_type);
	STASIS_MESSAGE_TYPE_CLEANUP(cei_act_event_type);

	if (bridge_state_router) {
		stasis_message_router_unsubscribe(bridge_state_router);
		bridge_state_router = NULL;
	}

	if (channel_state_router) {
		stasis_message_router_unsubscribe(channel_state_router);
		channel_state_router = NULL;
	}

	if (cei_mcb_process_router) {
		stasis_message_router_unsubscribe(cei_mcb_process_router);
		cei_mcb_process_router = NULL;
	}

	/* !begin Modified by pyzhong@grandstream.cn for bug[65207], 2016-11-3*/
	if (cei_sub_router) {
		stasis_message_router_unsubscribe(cei_sub_router);
		cei_sub_router = NULL;
	}
	if(cei_etc_process_router) {
		stasis_message_router_unsubscribe(cei_etc_process_router);
		cei_etc_process_router = NULL;
	}
	/* !end Modified by pyzhong@grandstream.cn for bug[65207], 2016-11-3*/

	/*Clear up all struct*/
	conf_free_all();
}

static char *handle_cli_schedule_user_list(struct ast_cli_entry *e, int cmd, struct ast_cli_args *a)
{
	struct cei_schedule_user *user = NULL;
	RAII_VAR(struct cei_schedule_conf*, conference, NULL, ao2_cleanup);

	switch (cmd) {
	case CLI_INIT:
		e->command = "confbridge schedule signed list";
		e->usage =
			"Usage: confbridge schedule signed list [<confnum>] [<scheduleid>]\n"
			"		list cei schedule user info.\n";
		return NULL;
	case CLI_GENERATE:
		return NULL;
	}

	if (a->argc != 6) {
		return CLI_SHOWUSAGE;
	}

	conference = cei_sche_conf_fetch_add(a->argv[4], 0, a->argv[5]);
	if (conference) {
		ao2_lock(conference);
		/*user list*/
		ast_cli(a->fd, "User==============    Related Info==============   Sigined==============\n");
		AST_LIST_TRAVERSE(&conference->cei_schedule_user_list, user, entry) {
			ast_cli(a->fd, "%-21s %-28s %-20d\n", user->extension, user->related_info, user->sigined_in);
		}
		ao2_unlock(conference);
		return CLI_SUCCESS;
	} else {
		ast_cli(a->fd, "no cei schedule conference found!\n");
		return CLI_SUCCESS;
	}
	return CLI_SHOWUSAGE;
}


static struct ast_cli_entry cli_confbridge_cei[] = {
	AST_CLI_DEFINE(handle_cli_schedule_user_list, "List conference bridges and participants."),
};

int manager_cei_init(void)
{
	STASIS_MESSAGE_TYPE_INIT(cei_join_type);
	STASIS_MESSAGE_TYPE_INIT(cei_leave_type);
	STASIS_MESSAGE_TYPE_INIT(cei_lock_status_type);
	STASIS_MESSAGE_TYPE_INIT(cei_toggle_mute_type);
	STASIS_MESSAGE_TYPE_INIT(cei_toggle_record_type);
	STASIS_MESSAGE_TYPE_INIT(cei_talk_status_type);
	STASIS_MESSAGE_TYPE_INIT(cei_moderator_type);
	STASIS_MESSAGE_TYPE_INIT(cei_refresh_all_type);
	STASIS_MESSAGE_TYPE_INIT(cei_mute_status_type);
	STASIS_MESSAGE_TYPE_INIT(cei_role_update_type);
	STASIS_MESSAGE_TYPE_INIT(cei_name_update_type);
	STASIS_MESSAGE_TYPE_INIT(cei_focus_status_type);
	STASIS_MESSAGE_TYPE_INIT(cei_layout_status_type);
	STASIS_MESSAGE_TYPE_INIT(cei_user_permission_type);
	STASIS_MESSAGE_TYPE_INIT(cei_gesture_status_type);
	STASIS_MESSAGE_TYPE_INIT(cei_act_event_type);

	bridge_state_router = stasis_message_router_create(
		ast_bridge_topic_all_cached());

	if (!bridge_state_router) {
		manager_cei_shutdown();
		return -1;
	}

	if (ast_cli_register_multiple(cli_confbridge_cei, ARRAY_LEN(cli_confbridge_cei))) {
		manager_cei_shutdown();
		return -1;
	}

	if (stasis_message_router_add(bridge_state_router,
			cei_join_type(),
			cei_join_cb,
			NULL)) {
		manager_cei_shutdown();
		return -1;
	}
	if (stasis_message_router_add(bridge_state_router,
			cei_leave_type(),
			cei_leave_cb,
			NULL)) {
		manager_cei_shutdown();
		return -1;
	}
	if (stasis_message_router_add(bridge_state_router,
			cei_lock_status_type(),
			cei_lock_status_cb,
			NULL)) {
		manager_cei_shutdown();
		return -1;
	}
	if (stasis_message_router_add(bridge_state_router,
			cei_toggle_mute_type(),
			cei_toggle_mute_cb,
			NULL)) {
		manager_cei_shutdown();
		return -1;
	}

	if (stasis_message_router_add(bridge_state_router,
			cei_toggle_record_type(),
			cei_toggle_record_cb,
			NULL)) {
		manager_cei_shutdown();
		return -1;
	}

	if (stasis_message_router_add(bridge_state_router,
			cei_talk_status_type(),
			cei_talk_status_cb,
			NULL)) {
		manager_cei_shutdown();
		return -1;
	}

	if (stasis_message_router_add(bridge_state_router,
			cei_moderator_type(),
			cei_moderator_cb,
			NULL)) {
		manager_cei_shutdown();
		return -1;
	}
	if (stasis_message_router_add(bridge_state_router,
			cei_refresh_all_type(),
			cei_refresh_all_cb,
			NULL)) {
		manager_cei_shutdown();
		return -1;
	}

	if (stasis_message_router_add(bridge_state_router,
			cei_mute_status_type(),
			cei_mute_status_cb,
			NULL)) {
		manager_cei_shutdown();
		return -1;
	}

	if (stasis_message_router_add(bridge_state_router,
			cei_role_update_type(),
			cei_role_update_cb,
			NULL)) {
		manager_cei_shutdown();
		return -1;
	}

	if (stasis_message_router_add(bridge_state_router,
			cei_name_update_type(),
			cei_name_update_cb,
			NULL)) {
		manager_cei_shutdown();
		return -1;
	}

	if (stasis_message_router_add(bridge_state_router,
			cei_focus_status_type(),
			cei_focus_status_cb,
			NULL)) {
		manager_cei_shutdown();
		return -1;
	}

	if (stasis_message_router_add(bridge_state_router,
			cei_layout_status_type(),
			cei_layout_status_cb,
			NULL)) {
		manager_cei_shutdown();
		return -1;
	}

	if (stasis_message_router_add(bridge_state_router,
			cei_user_permission_type(),
			cei_user_permission_cb,
			NULL)) {
		manager_cei_shutdown();
		return -1;
	}

	if (stasis_message_router_add(bridge_state_router,
			cei_gesture_status_type(),
			cei_gesture_status_cb,
			NULL)) {
		manager_cei_shutdown();
		return -1;
	}

	if (stasis_message_router_add(bridge_state_router,
			cei_act_event_type(),
			cei_act_event_cb,
			NULL)) {
		manager_cei_shutdown();
		return -1;
	}

	channel_state_router = stasis_message_router_create(
		ast_channel_topic_all_cached());

	if (!channel_state_router) {
		manager_cei_shutdown();
		return -1;
	}

	if (stasis_message_router_add(channel_state_router,
		  cei_join_type(),
			cei_join_cb,
			NULL)) {
		manager_cei_shutdown();
		return -1;
	}
	if (stasis_message_router_add(channel_state_router,
			cei_leave_type(),
			cei_leave_cb,
			NULL)) {
		manager_cei_shutdown();
		return -1;
	}
	if (stasis_message_router_add(channel_state_router,
			cei_lock_status_type(),
			cei_lock_status_cb,
			NULL)) {
		manager_cei_shutdown();
		return -1;
	}
	if (stasis_message_router_add(channel_state_router,
			cei_toggle_mute_type(),
			cei_toggle_mute_cb,
			NULL)) {
		manager_cei_shutdown();
		return -1;
	}
	if (stasis_message_router_add(channel_state_router,
			cei_toggle_record_type(),
			cei_toggle_record_cb,
			NULL)) {
		manager_cei_shutdown();
		return -1;
	}

	if (stasis_message_router_add(channel_state_router,
			cei_talk_status_type(),
			cei_talk_status_cb,
			NULL)) {
		manager_cei_shutdown();
		return -1;
	}

	if (stasis_message_router_add(channel_state_router,
			cei_moderator_type(),
			cei_moderator_cb,
			NULL)) {
		manager_cei_shutdown();
		return -1;
	}

	if (stasis_message_router_add(channel_state_router,
			cei_refresh_all_type(),
			cei_refresh_all_cb,
			NULL)) {
		manager_cei_shutdown();
		return -1;
	}

	if (stasis_message_router_add(channel_state_router,
			cei_mute_status_type(),
			cei_mute_status_cb,
			NULL)) {
		manager_cei_shutdown();
		return -1;
	}

	if (stasis_message_router_add(channel_state_router,
			cei_role_update_type(),
			cei_role_update_cb,
			NULL)) {
		manager_cei_shutdown();
		return -1;
	}

	if (stasis_message_router_add(channel_state_router,
			cei_name_update_type(),
			cei_name_update_cb,
			NULL)) {
		manager_cei_shutdown();
		return -1;
	}

	if (stasis_message_router_add(channel_state_router,
			cei_focus_status_type(),
			cei_focus_status_cb,
			NULL)) {
		manager_cei_shutdown();
		return -1;
	}

	if (stasis_message_router_add(channel_state_router,
			cei_layout_status_type(),
			cei_layout_status_cb,
			NULL)) {
		manager_cei_shutdown();
		return -1;
	}

	if (stasis_message_router_add(channel_state_router,
			cei_user_permission_type(),
			cei_user_permission_cb,
			NULL)) {
		manager_cei_shutdown();
		return -1;
	}

	if (stasis_message_router_add(channel_state_router,
			cei_gesture_status_type(),
			cei_gesture_status_cb,
			NULL)) {
		manager_cei_shutdown();
		return -1;
	}

	if (stasis_message_router_add(channel_state_router,
			cei_act_event_type(),
			cei_act_event_cb,
			NULL)) {
		manager_cei_shutdown();
		return -1;
	}

	// Add CEI Manager-CEI NOTIFY generator message routing for MCB
	struct stasis_topic *cei_mcb_process_topic;

	cei_mcb_process_topic = ast_cei_mcb_process_topic_all();
	if (!cei_mcb_process_topic) {
		ast_log(LOG_WARNING, "[WL] Failed to get cei_mcb_process_topic\n");
		manager_cei_shutdown();
		return -1;
	}

	cei_mcb_process_router = stasis_message_router_create(cei_mcb_process_topic);
	if (cei_mcb_process_router == NULL) {
		ast_log(LOG_WARNING, "[WL] Failed to create cei_mcb_process_router\n");
		manager_cei_shutdown();
		return -1;
	}

	if (stasis_message_router_add(cei_mcb_process_router,
									ast_cei_mcb_process_unsub_type(),
									cei_mcb_process_unsub_cb, NULL))
	{
		ast_log(LOG_WARNING, "[WL] Failed to subscribe to ast_cei_mcb_process_unsub_type\n");
		manager_cei_shutdown();
		return -1;
	}

	if (stasis_message_router_add(cei_mcb_process_router,
									ast_cei_mcb_process_xml_type(),
									cei_ext_mcb_process_xml_cb, NULL))
	{
		ast_log(LOG_WARNING, "[WL] Failed to subscribe to ast_cei_mcb_process_xml_type\n");
		manager_cei_shutdown();
		return -1;
	}

	if (stasis_message_router_add(cei_mcb_process_router,
									ast_cei_mcb_process_string_type(),
									cei_int_mcb_process_string_cb, NULL))
	{
		ast_log(LOG_WARNING, "[WL] Failed to subscribe to ast_cei_mcb_process_string_type\n");
		manager_cei_shutdown();
		return -1;
	}

	/*start added by zyxia for trunk conf cei*/
	struct stasis_topic *cei_etc_process_topic;

	cei_etc_process_topic = ast_cei_etc_process_topic_all();
	if (!cei_etc_process_topic) {
		ast_log(LOG_WARNING, "[WL] Failed to get cei_mcb_process_topic\n");
		manager_cei_shutdown();
		return -1;
	}

	cei_etc_process_router = stasis_message_router_create(cei_etc_process_topic);
	if (cei_etc_process_router == NULL) {
		ast_log(LOG_WARNING, "[WL] Failed to create cei_mcb_process_router\n");
		manager_cei_shutdown();
		return -1;
	}

	if (stasis_message_router_add(cei_etc_process_router,
									ast_cei_etc_process_xml_type(),
									cei_ext_trunk_conf_process_xml_cb, NULL))
	{
		ast_log(LOG_WARNING, "[WL] Failed to subscribe to ast_cei_etc_process_xml_type\n");
		manager_cei_shutdown();
		return -1;
	}

	if (stasis_message_router_add(cei_etc_process_router,
									ast_cei_etc_remove_type(),
									cei_ext_trunk_conf_remove_cb, NULL))
	{
		ast_log(LOG_WARNING, "[WL] Failed to subscribe to ast_cei_etc_remove_type\n");
		manager_cei_shutdown();
		return -1;
	}

	/* !begin Modified by pyzhong@grandstream.cn for bug[65207], 2016-11-3*/
	struct stasis_topic * cei_sub_topic = ast_cei_sub_topic_all();
	if (!cei_sub_topic) {
		ast_log(LOG_WARNING, "[WL] Failed to get cei_sub_topic\n");
		manager_cei_shutdown();
		return -1;
	}

	cei_sub_router = stasis_message_router_create(cei_sub_topic);
	if (cei_sub_router == NULL) {
		ast_log(LOG_WARNING, "[WL] Failed to create cei_sub_router\n");
		manager_cei_shutdown();
		return -1;
	}

	if (stasis_message_router_add(cei_sub_router,
								  ast_cei_refresh_type(),
								  cei_refresh_cb, NULL))
	{
		ast_log(LOG_WARNING, "[WL] Failed to subscribe to ast_cei_refresh_type\n");
		manager_cei_shutdown();
		return -1;
	}
	/* !end Modified by pyzhong@grandstream.cn for bug[65207], 2016-11-3*/

	if (stasis_message_router_add(cei_sub_router,
									ast_cei_sub_type(),
									cei_sub_cb, NULL))
	{
		ast_log(LOG_WARNING, "[WL] Failed to subscribe to ast_cei_sub_type\n");
		manager_cei_shutdown();
		return -1;
	}

	// Get conf max user capability
	conf_max_capability = get_conf_max_capability();

	return 0;
}

/*Get the web server uri*/
static char *conf_get_http_uri(void)
{
	char url[256] = {0};
	char value[64] = {0};
	int ssl = 0;
	int port = 8089;

	ast_sip_get_nvram_value("900", value, sizeof(value));
	ssl = atoi(value);

	ast_sip_get_nvram_value("901", value, sizeof(value));
	port = atoi(value);

	snprintf(url, sizeof(url), "%s://%s:%d", ssl?"https":"http", "127.0.0.1", port);
	return ast_strdup(url);
}

static struct cei_conference* build_conf_by_num(const char *conf_no) {

	struct cei_conference *conf;
	char conf_id[41];
	char database_tree[MAX_CONFNUM + 5];

	if (conf_no == NULL) {
		return NULL;
	}

	memset(conf_id, 0, sizeof(conf_id));
	snprintf(database_tree, sizeof(database_tree), "MCB/%s", conf_no);

	if (ast_db_get(database_tree, "LOCAL_IDENTITY", conf_id, sizeof(conf_id)) == 0) {
		ast_debug(LOG_DEBUG, "[WL] ROOM %s HAS LOCAL_IDENTITY %s", conf_no, conf_id);
	}
	else {
		ast_debug(LOG_DEBUG, "[WL] ROOM %s HAS LOCAL_IDENTITY NONE", conf_no);

		char unix_time[16];
		snprintf(unix_time, sizeof(unix_time), "%u", (int) time(NULL));
		char random_num[3];
		snprintf(random_num, sizeof(random_num), "%d", (rand() % 90) + 10);
		char to_hash[strlen(conf_no) + strlen(unix_time) + strlen(random_num) + 1];
		strcpy(to_hash, conf_no);
		strcpy(to_hash + strlen(to_hash), unix_time);
		strcpy(to_hash + strlen(to_hash), random_num);

		char hash_result[41];
		ast_sha1_hash(hash_result, to_hash);

		ast_copy_string(conf_id, hash_result, sizeof(conf_id));
		ast_db_put(database_tree, "LOCAL_IDENTITY", hash_result);
	}
	ast_debug(LOG_DEBUG, "[WL] conference id IS %s", conf_id);

	AST_LIST_LOCK(&confs);
	AST_LIST_TRAVERSE(&confs, conf, list) {
		if (!strcmp(conf_id, conf->conference_id)){
			AST_LIST_UNLOCK(&confs);
			ast_atomic_fetchadd_int(&conf->refcount, 1);
			ast_debug(LOG_DEBUG, "[WL] Found the confernece %s refcount %d", conf->conference_id, conf->refcount);
			return conf;
		}
	}

	if (!(conf = ao2_alloc(sizeof(struct cei_conference), NULL)) ||
		!(conf->remote_container = ao2_container_alloc(1, NULL, conference_id_cmp)) ||
		!(conf->user_container = ao2_container_alloc(1, NULL, user_hash_cmp))) {
		if (conf) {
			conf_free(conf);
		}
		ast_debug(LOG_DEBUG, "[WL] CONFERENCE %s %s CANNOT BE CREATED\n", conf_no, conf_id);
		AST_LIST_UNLOCK(&confs);
	}
	else {
		ast_copy_string(conf->conference_num, conf_no, sizeof(conf->conference_num));
		ast_copy_string(conf->conference_id, conf_id, sizeof(conf->conference_id));
		conf->locked = FALSE;
		conf->local_size = 0;
		conf->remote_size = 0;
		conf->local_user_size = 0;
		conf->remote_user_size = 0;
		conf->mcb_count = 0;
		conf->refcount = 0;
		conf->host_conf = NULL;
		conf->version = 0;
		conf->version_reset = 0;
		conf->stop_propagating = FALSE;
		conf->web_page = conf_get_http_uri();
		conf->sharing_user = NULL;
		conf->feature_id = NULL;
		conf->subject = NULL;

		AST_LIST_INSERT_HEAD(&confs, conf, list);
		ast_atomic_fetchadd_int(&conf->refcount, 2);
		ast_debug(LOG_DEBUG, "[WL] Create new conference struct %s refcount %d", conf->conference_id, conf->refcount);

		AST_LIST_UNLOCK(&confs);

		return conf;
	}

	ast_debug(LOG_DEBUG, "[WL] conference room %s can't find!", conf_no);

	return NULL;
}

static int user_hash_cmp(void *obj, void *arg, int flags) {
	struct cei_user *user = obj;
	char *user_hash = arg;

	if (!strcmp(user->user_hash, user_hash)) {
		return (CMP_MATCH | CMP_STOP);
	}

	return 0;
}

static int conference_id_cmp(void *obj, void *arg, int flags) {
	struct cei_conference *conf = obj;
	char *conference_id = arg;

	if (!strcmp(conf->conference_id, conference_id)) {
		return (CMP_MATCH | CMP_STOP);
	}

	return 0;
}

static void hash_user(struct cei_user *user) {
	char to_hash[51];

	snprintf(to_hash, sizeof(to_hash), "%s%d", user->conference_id, user->user_num);
	ast_sha1_hash(user->user_hash, to_hash);
}

static void poll_cei_conference(struct cei_conference *conf, struct ast_str **data_string) {

	char user_string[CEI_EVENT_USER_LEN];
	struct ao2_iterator user_iter;
	struct cei_user *user;
	int user_status = CEI_STATUS_UNKNOWN;

	int max_user_cnt = 0;

	// Sanity check
	if (conf == NULL) {
		return;
	}

	ao2_lock(conf->user_container);
	ao2_lock(conf->remote_container);

	/*Start added by zyxia for Bug 246333,2022.11.15*/
	if (conf->is_virtial_trunk_conf && conf->maximum_user_count)
		max_user_cnt = conf->maximum_user_count;
	else
		max_user_cnt = conf->video_conference ? sfu_max_capability() : conf_max_capability;
	/*End added by zyxia for Bug 246333,2022.11.15*/

	// Append conference information
	// notify_type|local_count|remote_count|local_user_count|remote_user_count|total_count_need_update|conf_locked|subject|free_text|when|maximum_user_count|conference_type\tconf_user_count\tuser_number\tcid_number\tcid_name\tchannel_name\tconference_id\tuser_role\tentity_status|

	ast_str_append(data_string, 0, "%d|%d|%d|%d|%d|%d|%d|%s|%ld|%s|%s|%s|%d|%s|%s|%s|%s|%d|%s|",
									conf->video_conference ? CEI_NOTIFY_FULL_EXTRA : CEI_NOTIFY_FULL,
									conf->local_size,
									conf->remote_size,
									conf->local_user_size,
									conf->remote_user_size,
									conf->is_virtial_trunk_conf?conf->local_size + conf->remote_size :conf->local_size + conf->remote_size + ao2_container_count(conf->remote_container),
									conf->locked,
									!ast_strlen_zero(conf->record_type) ? conf->record_type : "NULL",
									conf->record_start_utc_timestamp,
									!ast_strlen_zero(conf->subject) ? conf->subject : "NULL",
									"NULL",
									!ast_strlen_zero(conf->start_time) ? conf->start_time : "NULL",
									max_user_cnt,
									S_OR(conf->feature_id, "NULL"),
									S_OR(conf->conf_config, "NULL"),
									S_OR(conf->conf_layout, DEFAULT_CONF_LAYOUT),
									conf->allow_user_unmute ? "yes" : "no",
									conf->act_event_type,
									S_OR(conf->conf_creator, "--")
									);

	if (conf->video_conference) {
		/*moderator and web-page*/
		ast_str_append(data_string, 0, "%s|%s|%d|",
										conf->sharing_user ? conf->sharing_user->cid_number : "NULL",
										!ast_strlen_zero(conf->web_page) ? conf->web_page : "NULL",
										conf->muted
										);
	} else {
		ast_str_append(data_string, 0, "%d|",
										conf->muted
										);
	}

	AST_LIST_LOCK(&confs);

	// LOOP over user in conference
	user_iter = ao2_iterator_init(conf->user_container, 0);
	while ((user = ao2_iterator_next(&user_iter))) {

		// User status
		/*! begin: Modified by pyzhong@grandstream.cn for bug[71632], 2017-3-31*/
		//user_status = CEI_STATUS_CONNECTED;
		user_status = (user->user_status == CEI_STATUS_UNKNOWN) ? CEI_STATUS_CONNECTED : user->user_status;
		/*! end: Modified by pyzhong@grandstream.cn for bug[71632], 2017-3-31*/
		if (user->muted) {
			user_status = CEI_STATUS_MUTEVIAFOCUS;
		}

		// Append current conference users
		// conference_type\tconf_user_count\tuser_number\tcid_number\tcid_name\tchannel_name\t
		// conference_id\tself_conference_id\tuser_role\tentity_status|

		snprintf(user_string, CEI_EVENT_USER_LEN,
						"%d\t%d\t%d\t%.32s\t%s\t%.100s\t%s\t%s\t%d\t%d",
						/* Modified by achan, Grandstream, bug 32194 */
						CEI_CONFERENCE_LOCAL,
						0,
						user->user_num,
						user->cid_number,
						user->cid_name,
						user->channel_name,
						user->conference_id,
						(!ast_strlen_zero(user->self_conf_id)) ? user->self_conf_id : "NULL",
						user->user_role,
						user_status);

		ast_str_append(data_string, 0, user_string);

		if (conf->video_conference) {
			ast_str_append(data_string, 0, "\t%d\t%d\t%d\t%ld\t%d\t%d\t%d\t%s\t%s\t%s\t%d\t%d\t%d\t%s\t%s\t%s\t%d\t%s",
							user->muted,
							user->mic_muted,
							user->isfocus,
							user->hand_up_time,
							user->media.video_muted,
							user->media.video_capable,
							user->media.type == AST_MEDIA_TYPE_VIDEO ? 1 : 0,
							"NULL",
							!ast_strlen_zero(user->media.video_info.code_name) ? user->media.video_info.code_name : "NULL",
							video_resolution_type_to_name(user->media.video_info.resolution),
							user->media.video_info.frame_rate,
							(user->term_type == AST_TERMINAL_TYPE_WEBRTC || user->term_type == AST_TERMINAL_TYPE_GVC || user->term_type == AST_TERMINAL_TYPE_SIP || user->term_type == AST_TERMINAL_TYPE_GSWAVE_MOBILE || user->term_type == AST_TERMINAL_TYPE_ICE_TRUNK ) ? 1 : 0,
							(user->muted || user->mic_muted) ? 0 : user->talking,
							cei_terminate_type_string(user->term_type),
							!ast_strlen_zero(user->extension) ? user->extension : "NULL",
							!ast_strlen_zero(user->msid) ? user->msid : "NULL",
							user->sharing,
							!ast_strlen_zero(user->related_ext) ? user->related_ext : "NULL");
		} else {
			ast_str_append(data_string, 0, "\t%d\t%d\t%d\t%d\t%d\t%s\t%s",
							user->muted,
							user->mic_muted,
							user->media.type == AST_MEDIA_TYPE_VIDEO ? 1 : 0,
							(user->term_type == AST_TERMINAL_TYPE_WEBRTC || user->term_type == AST_TERMINAL_TYPE_GVC || user->term_type == AST_TERMINAL_TYPE_SIP || user->term_type == AST_TERMINAL_TYPE_GSWAVE_MOBILE || user->term_type == AST_TERMINAL_TYPE_ICE_TRUNK ) ? 1 : 0,
							(user->muted || user->mic_muted) ? 0 : user->talking,
							cei_terminate_type_string(user->term_type),
							!ast_strlen_zero(user->extension) ? user->extension : "NULL");

		}
		ast_str_append(data_string, 0, "|");

		ao2_ref(user, -1);
	}

	ao2_iterator_destroy(&user_iter);

#if 1
	if (ao2_container_count(conf->remote_container)) {
		struct ao2_iterator remote_conf_iter = ao2_iterator_init(conf->remote_container, 0);
		struct cei_conference *remote_conf;
		while ((remote_conf = ao2_iterator_next(&remote_conf_iter))) {
			// Append new remote conference
			// conference_type\tconf_user_count\tuser_number\tcid_number\tcid_name\tchannel_name\t
			// conference_id\tself_conference_id\tuser_role\tentity_status|

			if(!conf->is_virtial_trunk_conf) {
				snprintf(user_string, CEI_EVENT_USER_LEN,
								"%d\t%d\t%d\t%.32s\t%.32s\t%.100s\t%s\t%s\t%d\t%d|",
								/* Modified by achan, Grandstream, bug 32194 */
								CEI_CONFERENCE_REMOTE,
								remote_conf->local_user_size,
								0,
								remote_conf->conference_num,
								remote_conf->conference_num,
								"unknown",
								remote_conf->conference_id,
								"NULL",
								CEI_ROLE_CONFERENCE,
								CEI_STATUS_CONNECTED);

				ast_str_append(data_string, 0, user_string);
			}

			//if ((conf->local_size + conf->remote_size) <= MAX_NOTIFY_USERS_DISPLAY)
			{
				ao2_lock(remote_conf->user_container);
				user_iter = ao2_iterator_init(remote_conf->user_container, 0);
				while ((user = ao2_iterator_next(&user_iter))) {

					// User status
					user_status = CEI_STATUS_CONNECTED;
					if (user->muted) {
						user_status = CEI_STATUS_MUTEVIAFOCUS;
					}

					// Append current remote conference users
					// conference_type\tconf_user_count\tuser_number\tcid_number\tcid_name\tchannel_name\t
					// conference_id\tself_conference_id\tuser_role\tentity_status|
					snprintf(user_string, CEI_EVENT_USER_LEN,
									"%d\t%d\t%d\t%.32s\t%s\t%.100s\t%s\t%s\t%d\t%d",
									/* Modified by achan, Grandstream, bug 32194 */
									conf->is_virtial_trunk_conf ?CEI_CONFERENCE_VIRTUAL_LOCAL:CEI_CONFERENCE_LOCAL,
									0,
									user->user_num,
									user->cid_number,
									user->cid_name,
									user->channel_name,
									user->conference_id,
									(!ast_strlen_zero(user->self_conf_id)) ? user->self_conf_id : "NULL",
									user->user_role,
									user_status);

					ast_str_append(data_string, 0, user_string);

					if (conf->video_conference) {
						ast_str_append(data_string, 0, "\t%d\t%d\t%d\t%ld\t%d\t%d\t%d\t%s\t%s\t%s\t%d\t%d\t%d\t%s\t%s\t%s\t%d\t%s",
										user->muted,
										user->mic_muted,
										user->isfocus,
										user->hand_up_time,
										user->media.video_muted,
										user->media.video_capable,
										user->media.type == AST_MEDIA_TYPE_VIDEO ? 1 : 0,
										"NULL",
										!ast_strlen_zero(user->media.video_info.code_name) ? user->media.video_info.code_name : "NULL",
										"NULL",
										user->media.video_info.frame_rate,
										(user->term_type == AST_TERMINAL_TYPE_WEBRTC || user->term_type == AST_TERMINAL_TYPE_GVC || user->term_type == AST_TERMINAL_TYPE_SIP || user->term_type == AST_TERMINAL_TYPE_GSWAVE_MOBILE || user->term_type == AST_TERMINAL_TYPE_ICE_TRUNK) ? 1 : 0,
										1,
										cei_terminate_type_string(user->term_type),
										!ast_strlen_zero(user->extension) ? user->extension : "NULL",
										!ast_strlen_zero(user->msid) ? user->msid : "NULL",
										user->sharing,
										!ast_strlen_zero(user->related_ext) ? user->related_ext : "NULL");
					} else {
						ast_str_append(data_string, 0, "\t%d\t%d\t%d\t%d\t%d\t%s\t%s",
										user->muted,
										user->mic_muted,
										user->media.type == AST_MEDIA_TYPE_VIDEO ? 1 : 0,
										0,
										(user->muted || user->mic_muted) ? 0 : user->talking,
										cei_terminate_type_string(user->term_type),
										!ast_strlen_zero(user->extension) ? user->extension : "NULL");
					}
					ast_str_append(data_string, 0, "|");

					ao2_ref(user, -1);
				}
				ao2_unlock(remote_conf->user_container);
				ao2_iterator_destroy(&user_iter);
			}
			ao2_ref(remote_conf, -1);
		}
		ao2_iterator_destroy(&remote_conf_iter);
	}
#endif

	ao2_unlock(conf->remote_container);
	ao2_unlock(conf->user_container);
	AST_LIST_UNLOCK(&confs);
	//ast_log(LOG_DEBUG, "[WL] Exit poll_cei_conference with data: %s", ast_str_buffer(*data_string));
}

static void poll_cei_partial_notify(struct cei_conference *conf, struct cei_conference *remote_conf, struct cei_user *user,
	   																struct ast_str **data_string, enum cei_event_type event_type ) {
	char user_string[CEI_EVENT_USER_LEN];
	int user_status = CEI_STATUS_UNKNOWN;

	// Sanity check
	if (conf == NULL) {
		return;
	}
	switch (event_type)
	{
		case CEI_EVENT_JOIN:
		{
			user_status = (user_status == CEI_STATUS_UNKNOWN) ? CEI_STATUS_CONNECTED : user_status;
		}
		case CEI_EVENT_PART:
		{
			user_status = (user_status == CEI_STATUS_UNKNOWN) ? CEI_STATUS_DISCONNECTED : user_status;
		}
		case CEI_EVENT_TOGGLE_MUTE:
		case CEI_EVENT_ROLE_CHANGE:
		case CEI_EVENT_TALKING:
		case CEI_EVENT_SHARING_UPDATE:
		case CEI_EVENT_RESOLUTION_UPDATE:
		case CEI_EVENT_MEDIA_UPDATE:
		{
			if (user == NULL) {
				return;
			}

			if (user_status == CEI_STATUS_UNKNOWN) {
				if(user->muted) {
					user_status = CEI_STATUS_MUTEVIAFOCUS;
				} else {
					user_status = CEI_STATUS_CONNECTED;
				}
			}

			// Append conference information
			// notify_type|local_count|remote_count|local_user_count|remote_user_count|total_count_need_update|conf_locked|subject|free_text|when|maximum_user_count|conference_type\tconf_user_count\tuser_number\tcid_number\tcid_name\tchannel_name\tconference_id\tuser_role\tentity_status|

			ast_str_append(data_string, 0, "%d|%d|%d|%d|%d|%d|%d|%s|%ld|%s|%s|%s|%d|%s|%s|%s|%s|%d|%s|",
											conf->video_conference ? CEI_NOTIFY_PARTIAl_EXTRA : CEI_NOTIFY_PARTIAl,
											conf->local_size,
											conf->remote_size,
											conf->local_user_size,
											conf->remote_user_size,
											1,
											conf->locked,
											!ast_strlen_zero(conf->record_type) ? conf->record_type : "NULL",
											conf->record_start_utc_timestamp,
											"NULL",
											"NULL",
											"NULL",
											conf->video_conference ? sfu_max_capability() : conf_max_capability,
											S_OR(conf->feature_id, "NULL"),
											S_OR(conf->conf_config, "NULL"),
											S_OR(conf->conf_layout, DEFAULT_CONF_LAYOUT),
											conf->allow_user_unmute ? "yes" : "no",
											conf->act_event_type,
											S_OR(conf->conf_creator, "--")
											);

			if (conf->video_conference) {
				/*moderator and web-page*/
				ast_str_append(data_string, 0, "%s|%s|%d|",
												conf->sharing_user ? conf->sharing_user->cid_number : "NULL",
												!ast_strlen_zero(conf->web_page) ? conf->web_page : "NULL",
												conf->muted
												);
			} else {
				ast_str_append(data_string, 0, "%d|",
												conf->muted
												);
			}

			// Append current conference users
			// conference_type\tconf_user_count\tuser_number\tcid_number\tcid_name\tchannel_name\t
			// conference_id\tself_conference_id\tuser_role\tentity_status|
			snprintf(user_string, CEI_EVENT_USER_LEN,
							"%d\t%d\t%d\t%.32s\t%s\t%.100s\t%s\t%s\t%d\t%d",
							/* Modified by achan, Grandstream, bug 32194 */
							CEI_CONFERENCE_LOCAL,
							0,
							user->user_num,
							user->cid_number,
							user->cid_name,
							user->channel_name,
							user->conference_id,
							(!ast_strlen_zero(user->self_conf_id)) ? user->self_conf_id : "NULL",
							user->user_role,
							user_status);

			ast_str_append(data_string, 0, user_string);

			if (conf->video_conference) {
					ast_str_append(data_string, 0, "\t%d\t%d\t%d\t%ld\t%d\t%d\t%d\t%s\t%s\t%s\t%d\t%d\t%d\t%s\t%s\t%s\t%d\t%s",
									user->muted,
									user->mic_muted,
									user->isfocus,
									user->hand_up_time,
									user->media.video_muted,
									user->media.video_capable,
									user->media.type == AST_MEDIA_TYPE_VIDEO ? 1 : 0,
									"NULL",
									!ast_strlen_zero(user->media.video_info.code_name) ? user->media.video_info.code_name : "NULL",
									video_resolution_type_to_name(user->media.video_info.resolution),
									user->media.video_info.frame_rate,
									(user->term_type == AST_TERMINAL_TYPE_WEBRTC || user->term_type == AST_TERMINAL_TYPE_GVC || user->term_type == AST_TERMINAL_TYPE_SIP || user->term_type == AST_TERMINAL_TYPE_GSWAVE_MOBILE || user->term_type == AST_TERMINAL_TYPE_ICE_TRUNK ) ? 1 : 0,
									(user->muted || user->mic_muted) ? 0 : user->talking,
									cei_terminate_type_string(user->term_type),
									!ast_strlen_zero(user->extension) ? user->extension : "NULL",
									!ast_strlen_zero(user->msid) ? user->msid : "NULL",
									user->sharing,
									!ast_strlen_zero(user->related_ext) ? user->related_ext : "NULL");
			} else {
					ast_str_append(data_string, 0, "\t%d\t%d\t%d\t%d\t%d\t%s\t%s|",
									user->muted,
									user->mic_muted,
									user->media.type == AST_MEDIA_TYPE_VIDEO ? 1 : 0,
									(user->term_type == AST_TERMINAL_TYPE_WEBRTC || user->term_type == AST_TERMINAL_TYPE_GVC || user->term_type == AST_TERMINAL_TYPE_SIP || user->term_type == AST_TERMINAL_TYPE_GSWAVE_MOBILE || user->term_type == AST_TERMINAL_TYPE_ICE_TRUNK ) ? 1 : 0,
									(user->muted || user->mic_muted) ? 0 : user->talking,
									cei_terminate_type_string(user->term_type),
									!ast_strlen_zero(user->extension) ? user->extension : "NULL");
			}
		}
		break;
		case CEI_EVENT_TOGGLE_LOCK:
		case CEI_EVENT_TOGGLE_RECORD:
		{
			// Append conference information
			// notify_type|local_count|remote_count|local_user_count|remote_user_count|total_count_need_update|conf_locked|subject|free_text|when|maximum_user_count|conference_type\tconf_user_count\tuser_number\tcid_number\tcid_name\tchannel_name\tconference_id\tuser_role\tentity_status|

			ast_str_append(data_string, 0, "%d|%d|%d|%d|%d|%d|%d|%s|%ld|%s|%s|%s|%d|%s|%s|%s|%s|%d|%s|",
											conf->video_conference ? CEI_NOTIFY_PARTIAl_EXTRA : CEI_NOTIFY_PARTIAl,
											conf->local_size,
											conf->remote_size,
											conf->local_user_size,
											conf->remote_user_size,
											0,
											conf->locked,
											!ast_strlen_zero(conf->record_type) ? conf->record_type : "NULL",
											conf->record_start_utc_timestamp,
											"NULL",
											"NULL",
											"NULL",
											conf->video_conference ? sfu_max_capability() : conf_max_capability,
											S_OR(conf->feature_id, "NULL"),
											S_OR(conf->conf_config, "NULL"),
											S_OR(conf->conf_layout, DEFAULT_CONF_LAYOUT),
											conf->allow_user_unmute ? "yes" : "no",
											conf->act_event_type,
											S_OR(conf->conf_creator, "--")
											);
			if (conf->video_conference) {
				/*moderator and web-page*/
				ast_str_append(data_string, 0, "%s|%s|%d|",
												conf->sharing_user ? conf->sharing_user->cid_number : "NULL",
												!ast_strlen_zero(conf->web_page) ? conf->web_page : "NULL",
												conf->muted
												);
			} else {
				ast_str_append(data_string, 0, "%d|",
												conf->muted
												);
			}
		}
		break;
		case CEI_EVENT_MCB_USER_JOIN:
		{
			user_status = (user_status == CEI_STATUS_UNKNOWN) ? CEI_STATUS_CONNECTED : user_status;
		}
		case CEI_EVENT_MCB_USER_PART:
		{
			user_status = (user_status == CEI_STATUS_UNKNOWN) ? CEI_STATUS_DISCONNECTED : user_status;
		}
		case CEI_EVENT_MCB_USER_TOGGLE_MUTE:
		{
			if (remote_conf == NULL || user == NULL) {
				return;
			}

			if (user_status == CEI_STATUS_UNKNOWN) {
				if(user->muted) {
					user_status = CEI_STATUS_MUTEVIAFOCUS;
				} else {
					user_status = CEI_STATUS_CONNECTED;
				}
			}

			// Append conference information
			// notify_type|local_count|remote_count|local_user_count|remote_user_count|total_count_need_update|conf_locked|subject|free_text|when|maximum_user_count|conference_type\tconf_user_count\tuser_number\tcid_number\tcid_name\tchannel_name\tconference_id\tuser_role\tentity_status|

			ast_str_append(data_string, 0, "%d|%d|%d|%d|%d|%d|%d|%s|%ld|%s|%s|%s|%d|%s|%s|%s|%s|%d|%s|",
											conf->video_conference ? CEI_NOTIFY_PARTIAl_EXTRA : CEI_NOTIFY_PARTIAl,
											conf->local_size,
											conf->remote_size,
											conf->local_user_size,
											conf->remote_user_size,
											2,
											conf->locked,
											!ast_strlen_zero(conf->record_type) ? conf->record_type : "NULL",
											conf->record_start_utc_timestamp,
											"NULL",
											"NULL",
											"NULL",
											conf->video_conference ? sfu_max_capability() : conf_max_capability,
											S_OR(conf->feature_id, "NULL"),
											S_OR(conf->conf_config, "NULL"),
											S_OR(conf->conf_layout, DEFAULT_CONF_LAYOUT),
											conf->allow_user_unmute ? "yes" : "no",
											conf->act_event_type,
											S_OR(conf->conf_creator, "--")
											);

			if (conf->video_conference) {
				/*moderator and web-page*/
				ast_str_append(data_string, 0, "%s|%s|%d|",
												conf->sharing_user ? conf->sharing_user->cid_number : "NULL",
												!ast_strlen_zero(conf->web_page) ? conf->web_page : "NULL",
												conf->muted
												);
			} else {
				ast_str_append(data_string, 0, "%d|",
												conf->muted
												);
			}

			if(!conf->is_virtial_trunk_conf) {
				// Append new remote conference
				// conference_type\tconf_user_count\tuser_number\tcid_number\tcid_name\tchannel_name\t
				// conference_id\tself_conference_id\tuser_role\tentity_status|
				snprintf(user_string, CEI_EVENT_USER_LEN,
								"%d\t%d\t%d\t%.32s\t%.32s\t%.100s\t%s\t%s\t%d\t%d|",
								/* Modified by achan, Grandstream, bug 32194 */
								CEI_CONFERENCE_REMOTE,
								remote_conf->local_user_size,
								0,
								remote_conf->conference_num,
								remote_conf->conference_num,
								"unknown",
								remote_conf->conference_id,
								"NULL",
								CEI_ROLE_CONFERENCE,
								CEI_STATUS_CONNECTED);

				ast_str_append(data_string, 0, user_string);
			}

			// Append current remote conference users
			// conference_type\tconf_user_count\tuser_number\tcid_number\tcid_name\tchannel_name\t
			// conference_id\tself_conference_id\tuser_role\tentity_status|
			snprintf(user_string, CEI_EVENT_USER_LEN,
							"%d\t%d\t%d\t%.32s\t%s\t%.100s\t%s\t%s\t%d\t%d",
							/* Modified by achan, Grandstream, bug 32194 */
							CEI_CONFERENCE_LOCAL,
							0,
							user->user_num,
							user->cid_number,
							user->cid_name,
							user->channel_name,
							user->conference_id,
							(!ast_strlen_zero(user->self_conf_id)) ? user->self_conf_id : "NULL",
							user->user_role,
							user_status);

			ast_str_append(data_string, 0, user_string);



			if (conf->video_conference) {
				ast_str_append(data_string, 0, "\t%d\t%d\t%d\t%ld\t%d\t%d\t%d\t%s\t%s\t%s\t%d\t%d\t%d\t%s\t%s\t%s\t%d\t%s",
								user->muted,
								user->mic_muted,
								user->isfocus,
								user->hand_up_time,
								user->media.video_muted,
								user->media.video_capable,
								user->media.type == AST_MEDIA_TYPE_VIDEO ? 1 : 0,
								"NULL",
								!ast_strlen_zero(user->media.video_info.code_name) ? user->media.video_info.code_name : "NULL",
								video_resolution_type_to_name(user->media.video_info.resolution),
								user->media.video_info.frame_rate,
								(user->term_type == AST_TERMINAL_TYPE_WEBRTC || user->term_type == AST_TERMINAL_TYPE_GVC || user->term_type == AST_TERMINAL_TYPE_SIP || user->term_type == AST_TERMINAL_TYPE_GSWAVE_MOBILE || user->term_type == AST_TERMINAL_TYPE_ICE_TRUNK ) ? 1 : 0,
								(user->muted || user->mic_muted) ? 0 : user->talking,
								cei_terminate_type_string(user->term_type),
								!ast_strlen_zero(user->extension) ? user->extension : "NULL",
								!ast_strlen_zero(user->msid) ? user->msid : "NULL",
								user->sharing,
								!ast_strlen_zero(user->related_ext) ? user->related_ext : "NULL");
			} else {
				ast_str_append(data_string, 0, "\t%d\t%d\t%d\t%d\t%d\t%s\t%s|",
								user->muted,
								user->mic_muted,
								user->media.type == AST_MEDIA_TYPE_VIDEO ? 1 : 0,
								0,
								(user->muted || user->mic_muted) ? 0 : user->talking,
								cei_terminate_type_string(user->term_type),
								!ast_strlen_zero(user->extension) ? user->extension : "NULL");
			}
		}
		break;
		default:
		{
			ast_log(LOG_DEBUG, "[WL] Invalid event type %d !", event_type);
		}
		break;
	}

	//ast_log(LOG_DEBUG, "[WL] Exit poll_cei_partial_notify with data: %s", ast_str_buffer(*data_string));
}

static void conf_free_all(void) {
	struct cei_conference *conf = NULL;

	AST_LIST_LOCK(&confs);
	while ((conf = AST_LIST_REMOVE_HEAD(&confs, list))) {
		conf_free(conf);
	}
	AST_LIST_UNLOCK(&confs);
}

// Assume confs is locked
static int conf_free(struct cei_conference *conf)
{

	/*remove node from list*/
	AST_LIST_REMOVE(&confs, conf, list);

	struct ao2_iterator iter;
	if (conf->remote_container) {
		struct cei_conference *remote_conf = NULL;
		ao2_lock(conf->remote_container);
		iter = ao2_iterator_init(conf->remote_container, 0);
		while ((remote_conf = ao2_iterator_next(&iter))) {
			ao2_unlink(conf->remote_container, remote_conf);
			clear_conf_users(remote_conf);
			ao2_ref(remote_conf->user_container, -1);
			ao2_ref(remote_conf,-2);
		}
		ao2_iterator_destroy(&iter);
		ao2_unlock(conf->remote_container);
		ao2_ref(conf->remote_container,-1);
	}
	if (conf->user_container) {
		clear_conf_users(conf);
		ao2_ref(conf->user_container, -1);
	}
	if (conf->web_page) {
		ast_free(conf->web_page);
		conf->web_page = NULL;
	}
	if (conf->feature_id) {
		ast_free(conf->feature_id);
		conf->feature_id = NULL;
	}
	if (conf->subject) {
		ast_free(conf->subject);
		conf->subject = NULL;
	}
	if (conf->conf_config) {
		ast_free(conf->conf_config);
		conf->conf_config = NULL;
	}
	if (conf->schedule_id) {
		ast_free(conf->schedule_id);
		conf->schedule_id = NULL;
	}
	if (conf->im_service_prefix) {
		ast_free(conf->im_service_prefix);
		conf->im_service_prefix = NULL;
	}
	if (conf->conf_creator) {
		ast_free(conf->conf_creator);
		conf->conf_creator = NULL;
	}
	ao2_cleanup(conf->sharing_user);
	ao2_cleanup(conf);
	return 0;
}

static void clear_conf_users(struct cei_conference *conf) {

	struct ao2_iterator user_iter;
	struct cei_user *user;
	struct cei_user *next_user;

	ao2_lock(conf->user_container);
	user_iter = ao2_iterator_init(conf->user_container, 0);
	user = ao2_iterator_next(&user_iter);
	while (user) {
		next_user = ao2_iterator_next(&user_iter);
		ao2_unlink(conf->user_container, user);
		ast_atomic_fetchadd_int(&conf->local_size, -1);
		ast_atomic_fetchadd_int(&conf->local_user_size, (!check_role_as_conference(user))? -1 : 0);

		if (conf->host_conf) {
			ast_atomic_fetchadd_int(&conf->host_conf->remote_size, -1);
			ast_atomic_fetchadd_int(&conf->host_conf->remote_user_size, (!check_role_as_conference(user))? -1 : 0);
		}
		ast_atomic_fetchadd_int(&conf->refcount, -1);
		ast_debug(LOG_DEBUG, "clear_conf_users leave refcount %d", conf->refcount);
		cei_talking_user_remove(conf, user);
		/*refcount -2 for ao2_alloc and ao2_iterator_next in remove case*/
		ao2_ref(user, -2);
		/*Check if it is the sharing user*/
		if (conf->sharing_user == user) {
			ao2_ref(conf->sharing_user, -1);
			conf->sharing_user = NULL;
		}

		user = next_user;
	}
	ao2_iterator_destroy(&user_iter);
	ao2_unlock(conf->user_container);

}

static struct cei_conference* get_remote_conf_by_id(struct cei_conference *host_conf, const char *remote_id)
{
	ast_log(LOG_DEBUG, "[TK] Enter...%s\n", __FUNCTION__);

	if (host_conf) {
		struct cei_conference *remote_conf = NULL;

		ao2_lock(host_conf->remote_container);
		struct ao2_iterator conf_iter = ao2_iterator_init(host_conf->remote_container, 0);
		while ((remote_conf = ao2_iterator_next(&conf_iter))) {
			if (!strcmp(remote_id, remote_conf->conference_id)) {
				ao2_ref(remote_conf, -1);
				break;
			}
			ao2_ref(remote_conf, -1);
		}
		ao2_iterator_destroy(&conf_iter);
		ao2_unlock(host_conf->remote_container);

		return remote_conf;
	}

	return NULL;
}

static struct cei_conference* build_remote_conf(struct cei_conference *host_conf, const char *remote_no, const char *remote_id, const char *publisher)
{
	struct cei_conference *conf = NULL;
	ast_debug(LOG_DEBUG, "[WL]---> LOOKING FOR CONF NUM %s ID %s", remote_no, remote_id);

	ao2_lock(host_conf->remote_container);

	conf = get_remote_conf_by_id(host_conf, remote_id);

	if (conf) {
		ast_debug(LOG_DEBUG, "[WL]---> CONFERENCE %s %s ALREADY EXISTS\n", remote_no, remote_id);
	}
	else {
		if (!(conf = ao2_alloc(sizeof(struct cei_conference), NULL)) ||
			!(conf->user_container = ao2_container_alloc(1, NULL, user_hash_cmp))) {
			if (conf) {
				ao2_cleanup(conf);
			}
			ast_debug(LOG_DEBUG, "[WL] CONFERENCE %s %s CANNOT BE CREATED\n", remote_no, remote_id);

		}
		else {
			ast_copy_string(conf->conference_num, remote_no, sizeof(conf->conference_num));
			ast_copy_string(conf->conference_id, remote_id, sizeof(conf->conference_id));
			ast_copy_string(conf->internal_publisher, publisher, sizeof(conf->internal_publisher));
			conf->locked = FALSE;
			conf->local_size = 0;
			conf->remote_size = 0;
			conf->local_user_size = 0;
			conf->remote_user_size = 0;
			conf->mcb_count = 0;
			conf->refcount = 0;
			conf->host_conf = host_conf;
			conf->version = 0;
			conf->version_reset = 0;
			ast_atomic_fetchadd_int(&conf->refcount, 1);

			ao2_link(host_conf->remote_container, conf);

			ast_debug(LOG_DEBUG, "[WL]---> REMOTE CONFERENCE %s %s IN ROOM %s CREATED\n",
														conf->conference_num,
														conf->conference_id,
														host_conf->conference_num);
		}
	}

	ao2_unlock(host_conf->remote_container);

	return conf;
}

static void process_trunk_conf_xml(struct cei_conference *conf, const char *publisher_private_num, const char *xml, struct ast_str **data_string,int *full)
{
	char publisher[41] = {0};
	char *xml_content = NULL;
	xml_content = ast_strdup(xml);
	if (!xml_content) {
		return;
	}
	else {
		struct ast_xml_doc *xml_doc = NULL;
		if (!(xml_doc = ast_xml_read_memory(xml_content, strlen(xml_content))))
			ast_log(LOG_DEBUG, "ZAKU DEBUG ---> FAILED TO READ XML DOC");
		else {
			ast_log(LOG_DEBUG, "ZAKU DEBUG ---> READ THE XML DOC CORRECTLY");
			struct ast_xml_node *conference_node = ast_xml_get_root(xml_doc);
			struct ast_xml_node *child_nodes;
			struct ast_xml_node *node_iterator;
			struct ast_xml_ns *ns;
			const char *entity;
			const char *notifyType;
			const char *state;
			const char *namespace;
			const char *node_name;


			if (!conference_node) {
				ast_log(LOG_DEBUG, "ZAKU DEBUG ---> NO ROOT NODE");
			}
			else {
				/*********
				Parse data from xml
				**********/
				char remote_num[MAX_CONFNUM] = {0};
				struct cei_conference *remote_conference = NULL;
				struct cei_user *user = NULL;
				struct cei_user *user_tmp = NULL;
				struct ast_xml_node *node_publisher = NULL;
				struct ast_xml_node *node_host_info = NULL;
				struct ast_xml_node *node_conference_state = NULL;
				struct ast_xml_node *node_users = NULL;
				struct ast_xml_node *node_sidebars = NULL;
				struct ast_xml_node *node_conference_desc = NULL;
				struct ast_xml_node *node_recording_state = NULL;

				notifyType = ast_xml_get_attribute(conference_node, "state");

				if (strcmp(notifyType,"full") == 0) {
					entity = ast_xml_get_attribute(conference_node, "entity");
					char temp_entity[strlen(entity) + 1];
					strncpy(temp_entity, entity, strlen(entity));
					char *colon = strchr(temp_entity, ':');
					char *at = strchr(temp_entity, '@');
					*at = '\0';
					strncpy(remote_num, colon + 1, sizeof(remote_num)-1);
					ast_xml_free_attr(entity);

					ns = ast_xml_find_namespace(xml_doc, conference_node, NULL);
					if (ns) {
						namespace = ast_xml_get_ns_href(ns);
						ast_log(LOG_DEBUG, "ZAKU DEBUG ---> NAMESPACE IS %s", namespace);
					}

					child_nodes = ast_xml_node_get_children(conference_node);
					if (child_nodes) {
						ast_log(LOG_DEBUG, "ZAKU DEBUG ---> HAS CHILDREN");

						for (node_iterator = child_nodes; node_iterator; node_iterator = ast_xml_node_get_next(node_iterator)) {
							const char *node_name = ast_xml_node_get_name(node_iterator);
							if (check_xml_text_node(node_iterator)) {
								ast_log(LOG_DEBUG, "ZAKU DEBUG ---> HAS CHILD NODE %s", node_name);
								if (!strcmp(node_name, "publisher") && !node_publisher)
									node_publisher = node_iterator;
								else if (!strcmp(node_name, "host-info") && !node_host_info)
									node_host_info = node_iterator;
								else if (!strcmp(node_name, "conference-state") && !node_conference_state)
									node_conference_state = node_iterator;
								else if (!strcmp(node_name, "users") && !node_users)
									node_users = node_iterator;
								else if (!strcmp(node_name, "sidebars-by-val") && !node_sidebars) {
									node_sidebars = node_iterator;
								}
								else if (!strcmp(node_name, "conference-description") && !node_conference_desc) {
									node_conference_desc = node_iterator;
								}
								else if (!strcmp(node_name, "record-state") && !node_recording_state) {
									node_recording_state = node_iterator;
								}

								const char *content = ast_xml_get_text(node_iterator);
								if (content) {
									ast_log(LOG_DEBUG, "ZAKU DEBUG ---> HAS CHILD NODE CONTENT %s", content);
									ast_xml_free_text(content);
								}
								else
									ast_log(LOG_DEBUG, "ZAKU DEBUG ---> HAS CHILD NODE CONTENT NULL");

							}
							else
								ast_log(LOG_DEBUG, "ZAKU DEBUG ---> IGNORING BLANK TEXT NODE");
						}
					}

					if (node_publisher) {
						const char *content = ast_xml_get_text(node_publisher);

						if (content) {
							ast_copy_string(publisher, content, sizeof(publisher));
							ast_debug(LOG_DEBUG, "[TK] PUBLISHER IS %s", publisher);
						}

						ast_xml_free_text(content);
					}

					if (node_host_info) {
						const char *content = ast_xml_get_text(node_host_info);
						if (content) {
							char remote_id[strlen(content) + 1];
							strncpy(remote_id, content, sizeof(remote_id));
							ast_xml_free_text(content);
							char *strip_remote_id = ast_strip(remote_id);

							if (strlen(publisher) == 0) {
								ast_copy_string(publisher, strip_remote_id, sizeof(publisher));
								ast_debug(LOG_DEBUG, "[TK] PUBLISHER IS %s", publisher);
							}

							// clean up the remote conf published from publisher
							if(!conf){
								ast_debug(LOG_DEBUG, "[TK] no host conf !!!");
							}

							// Reinitialize published conferences
							delete_remote_confs_by_publisher(conf, strip_remote_id);

							ao2_lock(conf->remote_container);
							/***
							get or create the remote conf from host conf's remote container;
							the remote conf is basically this cei msg come FROM
							***/
							remote_conference = build_remote_conf(conf, publisher_private_num, strip_remote_id, publisher);
							ao2_unlock(conf->remote_container);
						}
					}

					if(node_recording_state) {
						for (node_iterator = ast_xml_node_get_children(node_recording_state); node_iterator; node_iterator = ast_xml_node_get_next(node_iterator)) {
							if (check_xml_text_node(node_iterator)) {
								const char *node_name = ast_xml_node_get_name(node_iterator);
								const char *content = ast_xml_get_text(node_iterator);
								ast_log(LOG_DEBUG, "ZAKU DEBUG ---> CONFERENCE DESC%s", node_name);
								if (content) {
									if (!strcmp(node_name, "recording") &&  !strcasecmp(content,"false")) {
										conf->video_conference = 1;
										ast_copy_string(conf->record_type, content, sizeof(conf->record_type));
									}
								}
								ast_xml_free_text(content);
							}
						}
					}

					if(node_conference_desc) {
						for (node_iterator = ast_xml_node_get_children(node_conference_desc); node_iterator; node_iterator = ast_xml_node_get_next(node_iterator)) {
							if (check_xml_text_node(node_iterator)) {
								const char *node_name = ast_xml_node_get_name(node_iterator);
								const char *content = ast_xml_get_text(node_iterator);
								ast_log(LOG_DEBUG, "ZAKU DEBUG ---> CONFERENCE DESC%s", node_name);
								if (content) {
									if (!strcmp(node_name, "type") &&  strstr(content,"video")) {
										conf->video_conference = 1;
									}
									else if (!strcmp(node_name, "subject")) {
										conf->subject = ast_strdup(content);
										ast_log(LOG_DEBUG, "ZAKU DEBUG ---> REMOTE subject %s", conf->subject);
									}
									else if (!strcmp(node_name, "when")) {
										ast_copy_string(conf->start_time, content, sizeof(conf->start_time));
										ast_log(LOG_DEBUG, "ZAKU DEBUG ---> REMOTE start time %s", conf->start_time);
									}
									else if (!strcmp(node_name, "feature-id")) {
										conf->feature_id= ast_strdup(content);
										ast_log(LOG_DEBUG, "ZAKU DEBUG ---> REMOTE feature-id %s", conf->feature_id);
									}
									/*for Bug 246333,2022.12.15*/
									else if (!strcmp(node_name, "maximum-user-count")) {
										conf->maximum_user_count= atoi(content);
										ast_log(LOG_DEBUG, "ZAKU DEBUG ---> REMOTE maximum_user_count %d", conf->maximum_user_count);
									}


								}
								ast_xml_free_text(content);
							}
						}
					}

					int accept = TRUE;
					if (remote_conference) {
						const char *version = ast_xml_get_attribute(conference_node, "version");
						if (!version) {
							ast_xml_free_attr(version);
							ast_log(LOG_DEBUG, "ZAKU DEBUG ---> DOCUMENT HAS NO VERSION");
							accept = FALSE;
						}
						else {
							int xml_version = atoi(version);

							if (xml_version <= remote_conference->version) {
								ast_xml_free_attr(version);
								ast_log(LOG_DEBUG, "ZAKU DEBUG ---> DOCUMENT VERSION %d <= %d", xml_version, remote_conference->version);
								accept = FALSE;
							}
							else {
								ast_xml_free_attr(version);
								remote_conference->version = xml_version;
							}
						}

						ao2_lock(remote_conference->user_container);

						if (accept) {
							clear_conf_users(remote_conference);

							// Get remote conference-state items
							if (node_conference_state) {
								for (node_iterator = ast_xml_node_get_children(node_conference_state); node_iterator; node_iterator = ast_xml_node_get_next(node_iterator)) {
									if (check_xml_text_node(node_iterator)) {
										const char *node_name = ast_xml_node_get_name(node_iterator);
										const char *content = ast_xml_get_text(node_iterator);
										ast_log(LOG_DEBUG, "ZAKU DEBUG ---> CONFERENCE STATE CHILD %s", node_name);
										if (content) {
											if (!strcmp(node_name, "user-count")) {
												char strip_content[strlen(content) + 1];
												strncpy(strip_content, content, strlen(content));
												ast_log(LOG_DEBUG, "ZAKU DEBUG ---> REMOTE LOCAL SIZE %d", remote_conference->local_user_size);
											}
											if (!strcmp(node_name, "locked")) {
												char strip_content[strlen(content) + 1];
												strncpy(strip_content, content, strlen(content));
												if (!strcmp(strip_content, "true"))
													remote_conference->locked = TRUE;
												else
													remote_conference->locked = FALSE;
												ast_log(LOG_DEBUG, "ZAKU DEBUG ---> REMOTE LOCKED %s", remote_conference->locked ? "TRUE" : "FALSE");
											}
										}
										ast_xml_free_text(content);
									}
								}
							}

							// Add users for remote publisher
							if (node_users) {
								process_remote_xml_users(conf, remote_conference,
														node_users);
							}

							// Add remote conference by sidebars-by-val
							if (node_sidebars) {
								// state
								accept = FALSE;
								state = ast_xml_get_attribute(node_sidebars, "state");
								if (!state || strcmp(state, "full") == 0) {
									accept = TRUE;
								}
								ast_xml_free_attr(state);

								if (accept) {
									for (node_iterator = ast_xml_node_get_children(node_sidebars); node_iterator; node_iterator = ast_xml_node_get_next(node_iterator)) {
										const char *node_name = ast_xml_node_get_name(node_iterator);
										ast_log(LOG_DEBUG, "ZAKU DEBUG ---> sidebars-by-val CHILD %s", node_name);

										if (check_xml_text_node(node_iterator) == 0) {
											continue;
										}

										if (strcmp(node_name, "entry") == 0) {
											char *head_pointer;
											char *split_pointer;
											struct ast_xml_node *entry_child;

											// state
											state = ast_xml_get_attribute(node_iterator, "state");
											if (!state || strcmp(state, "full") != 0) {
												ast_xml_free_attr(state);
												continue;
											}
											ast_xml_free_attr(state);

											// entity conf info
											entity = ast_xml_get_attribute(node_iterator, "entity");
											char entry_entity[strlen(entity) + 1];
											strncpy(entry_entity, entity, sizeof(entry_entity));
											ast_xml_free_attr(entity);
											ast_log(LOG_DEBUG, "[TK] entry entity: %s", entry_entity);

											head_pointer = entry_entity;
											split_pointer = strchr(entry_entity, '-');
											if (split_pointer == NULL) {
												continue;
											}
											*split_pointer = '\0';
											// remote_num is the remote conf num  in sider bar
											strncpy(remote_num, head_pointer, sizeof(remote_num)-1);

											head_pointer = strchr(split_pointer + 1, '@');
											if (head_pointer == NULL) {
												continue;
											}

											head_pointer = head_pointer + 1;
											split_pointer = strchr(head_pointer, '.');
											if (split_pointer == NULL) {
												continue;
											}
											*split_pointer = '\0';
											// remode_id is the remote conf id in sider bar
											char remote_id[strlen(head_pointer) + 1];
											strncpy(remote_id, head_pointer, sizeof(remote_id));
											char *strip_remote_id = ast_strip(remote_id);
											ast_log(LOG_DEBUG, "[TK] process remote container conf of this xml : %s", strip_remote_id);

											// Ignore invalid conference
											// Don't add myself as my remote conf
											if (strcmp(conf->conference_id, strip_remote_id) == 0) {
												ast_debug(LOG_DEBUG, "[TK] Ignore my own conference");
												ast_debug(LOG_DEBUG, "[TK] %s@%s", remote_num, strip_remote_id);
												continue;
											}
											// Don't add the remote conf which are my own remote conf again
											if (get_remote_conf_by_id(conf, strip_remote_id) != NULL) {
												ast_debug(LOG_DEBUG, "[TK] Ignore my own remote conference");
												ast_debug(LOG_DEBUG, "[TK] %s@%s", remote_num, strip_remote_id);
												continue;
											}

											// Get users
											node_users = NULL;
											for(entry_child = ast_xml_node_get_children(node_iterator); entry_child != NULL; entry_child = ast_xml_node_get_next(entry_child)) {
												node_name = ast_xml_node_get_name(entry_child);
												if (strcmp(node_name, "users") == 0) {
													node_users = entry_child;
													break;
												}
											}

											if (node_users == NULL) {
												ast_debug(LOG_DEBUG, "[TK] NO USERS");
												continue;
											}

											ast_log(LOG_DEBUG, "[TK] NEW REMOTE CONF %s@%s", remote_num, strip_remote_id);

											// Switch to new remote conference
											// Handle current remote conference
											ao2_unlock(remote_conference->user_container);

											// Handle new remote conference
											remote_conference = build_remote_conf(conf, remote_num, strip_remote_id, publisher);

											ao2_lock(remote_conference->user_container);
											clear_conf_users(remote_conference);

											process_remote_xml_users(conf, remote_conference,
																	node_users);
										}
									}
								}
							}
						}

						ao2_unlock(remote_conference->user_container);
					}
					poll_cei_conference(conf, data_string);
					*full = 1;
				}
				// PARTIAL NOTIFY
				else if (strcmp(notifyType,"partial") == 0) {
					entity = ast_xml_get_attribute(conference_node, "entity");
					char temp_entity[strlen(entity) + 1];
					strncpy(temp_entity, entity, strlen(entity));
					char *colon = strchr(temp_entity, ':');
					char *at = strchr(temp_entity, '@');
					*at = '\0';
					strncpy(remote_num, colon + 1, sizeof(remote_num)-1);
					ast_xml_free_attr(entity);

					ns = ast_xml_find_namespace(xml_doc, conference_node, NULL);
					if (ns) {
						namespace = ast_xml_get_ns_href(ns);
						ast_log(LOG_DEBUG, "ZAKU DEBUG ---> NAMESPACE IS %s", namespace);
					}

					child_nodes = ast_xml_node_get_children(conference_node);
					if (child_nodes) {
						ast_log(LOG_DEBUG, "ZAKU DEBUG ---> HAS CHILDREN");

						for (node_iterator = child_nodes; node_iterator; node_iterator = ast_xml_node_get_next(node_iterator)) {
							const char *node_name = ast_xml_node_get_name(node_iterator);
							if (check_xml_text_node(node_iterator)) {
								ast_log(LOG_DEBUG, "ZAKU DEBUG ---> HAS CHILD NODE %s", node_name);
								if (!strcmp(node_name, "publisher") && !node_publisher)
									node_publisher = node_iterator;
								else if (!strcmp(node_name, "host-info") && !node_host_info)
									node_host_info = node_iterator;
								else if (!strcmp(node_name, "conference-state") && !node_conference_state)
									node_conference_state = node_iterator;
								else if (!strcmp(node_name, "users") && !node_users)
									node_users = node_iterator;
								else if (!strcmp(node_name, "sidebars-by-val") && !node_sidebars) {
									node_sidebars = node_iterator;
								}
								else if (!strcmp(node_name, "conference-description") && !node_conference_desc) {
									node_conference_desc = node_iterator;
								}
								else if (!strcmp(node_name, "record-state") && !node_recording_state) {
									node_recording_state = node_iterator;
								}

								const char *content = ast_xml_get_text(node_iterator);
								if (content) {
									ast_log(LOG_DEBUG, "ZAKU DEBUG ---> HAS CHILD NODE CONTENT %s", content);
									ast_xml_free_text(content);
								}
								else
									ast_log(LOG_DEBUG, "ZAKU DEBUG ---> HAS CHILD NODE CONTENT NULL");

							}
							else
								ast_log(LOG_DEBUG, "ZAKU DEBUG ---> IGNORING BLANK TEXT NODE");
						}
					}

					if (node_publisher) {
						const char *content = ast_xml_get_text(node_publisher);

						if (content) {
							ast_copy_string(publisher, content, sizeof(publisher));
							ast_debug(LOG_DEBUG, "[TK] PUBLISHER IS %s", publisher);
						}

						ast_xml_free_text(content);
					}

					if (node_host_info) {
						const char *content = ast_xml_get_text(node_host_info);
						if (content) {
							char remote_id[strlen(content) + 1];
							strncpy(remote_id, content, sizeof(remote_id));
							ast_xml_free_text(content);
							char *strip_remote_id = ast_strip(remote_id);

							if (strlen(publisher) == 0) {
								ast_copy_string(publisher, strip_remote_id, sizeof(publisher));
								ast_debug(LOG_DEBUG, "[TK] PUBLISHER IS %s", publisher);
							}

							// clean up the remote conf published from publisher
							if(!conf){
								ast_debug(LOG_DEBUG, "[TK] no host conf !!!");
							}

							/***
							get the remote conf from host conf's remote container;
							the remote conf is basically this cei msg come FROM
							***/
							remote_conference = get_remote_conf_by_id(conf, publisher);
						}
					}
					if(node_recording_state) {
						for (node_iterator = ast_xml_node_get_children(node_recording_state); node_iterator; node_iterator = ast_xml_node_get_next(node_iterator)) {
							if (check_xml_text_node(node_iterator)) {
								const char *node_name = ast_xml_node_get_name(node_iterator);
								const char *content = ast_xml_get_text(node_iterator);
								ast_log(LOG_DEBUG, "ZAKU DEBUG ---> CONFERENCE DESC%s", node_name);
								if (content) {
									if (!strcmp(node_name, "recording") &&  !strcasecmp(content,"false")) {
										conf->video_conference = 1;
										ast_copy_string(conf->record_type, content, sizeof(conf->record_type));
									}
								}
								ast_xml_free_text(content);
							}
						}
					}

					if(node_conference_desc) {
						for (node_iterator = ast_xml_node_get_children(node_conference_desc); node_iterator; node_iterator = ast_xml_node_get_next(node_iterator)) {
							if (check_xml_text_node(node_iterator)) {
								const char *node_name = ast_xml_node_get_name(node_iterator);
								const char *content = ast_xml_get_text(node_iterator);
								ast_log(LOG_DEBUG, "ZAKU DEBUG ---> CONFERENCE DESC%s", node_name);
								if (content) {
									if (!strcmp(node_name, "type") &&  strstr(content,"video")) {
										conf->video_conference = 1;
									}
									else if (!strcmp(node_name, "subject")) {
										conf->subject = ast_strdup(content);
										ast_log(LOG_DEBUG, "ZAKU DEBUG ---> REMOTE subject %s", conf->subject);
									}
									else if (!strcmp(node_name, "when")) {
										ast_copy_string(conf->start_time, content, sizeof(conf->start_time));
										ast_log(LOG_DEBUG, "ZAKU DEBUG ---> REMOTE start time %s", conf->start_time);
									}
									else if (!strcmp(node_name, "feature-id")) {
										conf->feature_id= ast_strdup(content);
										ast_log(LOG_DEBUG, "ZAKU DEBUG ---> REMOTE feature-id %s", conf->feature_id);
									}


								}
								ast_xml_free_text(content);
							}
						}
					}

					int accept = TRUE;
					if (remote_conference) {
						const char *version = ast_xml_get_attribute(conference_node, "version");
						if (!version) {
							ast_xml_free_attr(version);
							ast_log(LOG_DEBUG, "ZAKU DEBUG ---> DOCUMENT HAS NO VERSION");
							accept = FALSE;
						}
						else {
							int xml_version = atoi(version);

							if (xml_version <= remote_conference->version) {
								ast_xml_free_attr(version);
								ast_log(LOG_DEBUG, "ZAKU DEBUG ---> DOCUMENT VERSION %d <= %d", xml_version, remote_conference->version);
								accept = FALSE;
							}
							else {
								ast_xml_free_attr(version);
								remote_conference->version = xml_version;
							}
						}


						if (accept) {

							// Get remote conference-state items
							if (node_conference_state) {
								for (node_iterator = ast_xml_node_get_children(node_conference_state); node_iterator; node_iterator = ast_xml_node_get_next(node_iterator)) {
									if (check_xml_text_node(node_iterator)) {
										const char *node_name = ast_xml_node_get_name(node_iterator);
										const char *content = ast_xml_get_text(node_iterator);
										ast_log(LOG_DEBUG, "ZAKU DEBUG ---> CONFERENCE STATE CHILD %s", node_name);
										if (content) {
											if (!strcmp(node_name, "user-count")) {
												char strip_content[strlen(content) + 1];
												strncpy(strip_content, content, strlen(content));
												ast_log(LOG_DEBUG, "ZAKU DEBUG ---> REMOTE LOCAL SIZE %d", remote_conference->local_user_size);
											}
											if (!strcmp(node_name, "locked")) {
												char strip_content[strlen(content) + 1];
												strncpy(strip_content, content, strlen(content));
												if (!strcmp(strip_content, "true"))
													remote_conference->locked = TRUE;
												else
													remote_conference->locked = FALSE;
												ast_log(LOG_DEBUG, "ZAKU DEBUG ---> REMOTE LOCKED %s", remote_conference->locked ? "TRUE" : "FALSE");
											}
										}
										ast_xml_free_text(content);
									}
								}
							}

							// Add users for remote publisher
							if (node_users) {
								for (node_iterator = ast_xml_node_get_children(node_users); node_iterator; node_iterator = ast_xml_node_get_next(node_iterator)) {
									if (check_xml_text_node(node_iterator) == 0) {
										continue;
									}
									node_name = ast_xml_node_get_name(node_iterator);
									if (!strcmp(node_name, "user")) {

										user = process_remote_xml_user(conf, node_iterator);
										if (user) {
											switch (user->user_status) {
												case CEI_STATUS_CONNECTED:
												{
													ao2_lock(remote_conference->user_container);
													//CEI_EVENT_MCB_USER_TOGGLE_MUTE unmute
													user_tmp = get_user_by_channel_name(remote_conference, user->channel_name,TRUE);
													if (user_tmp) {//if exist ,need update
														ao2_unlink(remote_conference->user_container, user_tmp);//remove the old user fisrt
														hash_user(user);
														ao2_link(remote_conference->user_container, user);

														user_tmp->muted = 0;
														user_tmp->talking = user->talking;
														poll_cei_partial_notify(conf, remote_conference, user, data_string, CEI_EVENT_MCB_USER_TOGGLE_MUTE);
														/*refcount -2 for ao2_alloc and get_user_by_channel_name in remove case*/
														ao2_ref(user_tmp, -2);
													}
													else {
														hash_user(user);
														ast_log(LOG_DEBUG, "ZAKU DEBUG ---> add user : %s to remote conf user container", user->channel_name);
														ao2_link(remote_conference->user_container, user);
														ast_atomic_fetchadd_int(&remote_conference->local_size, 1);
														ast_atomic_fetchadd_int(&remote_conference->local_user_size, (!check_role_as_conference(user))? 1 : 0);
														ast_atomic_fetchadd_int(&conf->remote_size, 1);
														ast_atomic_fetchadd_int(&conf->remote_user_size, (!check_role_as_conference(user))? 1 : 0);
														ast_atomic_fetchadd_int(&remote_conference->refcount, 1);
														poll_cei_partial_notify(conf, remote_conference, user, data_string, CEI_EVENT_MCB_USER_JOIN);
													}
													ao2_unlock(remote_conference->user_container);
												}
												break;
												case CEI_STATUS_DISCONNECTED:
												{
													ao2_lock(remote_conference->user_container);

													user_tmp = get_user_by_channel_name(remote_conference, user->channel_name,TRUE);

													if (user_tmp) {

														ast_atomic_fetchadd_int(&remote_conference->local_size, -1);
														ast_atomic_fetchadd_int(&remote_conference->local_user_size, (!check_role_as_conference(user))? -1 : 0);
														ast_atomic_fetchadd_int(&conf->remote_size, -1);
														ast_atomic_fetchadd_int(&conf->remote_user_size, (!check_role_as_conference(user))? -1 : 0);

														poll_cei_partial_notify(conf, remote_conference, user, data_string, CEI_EVENT_MCB_USER_PART);

														ao2_unlink(remote_conference->user_container, user_tmp);
														ast_atomic_fetchadd_int(&remote_conference->refcount, -1);

														ast_debug(LOG_DEBUG, "[WL] REMOVED USER WITH ADDRESS %p", user_tmp);
														/*refcount -2 for ao2_alloc and get_user_by_channel_name in remove case*/
														ao2_ref(user_tmp, -2);
													}
													else {
														ast_debug(LOG_DEBUG, "[WL] ERROR, CANNOT FIND USER  %s IN CONFERENCE %s", user->cid_number, remote_conference->conference_num);
													}
													ao2_ref(user, -1);
													ao2_unlock(remote_conference->user_container);
												}
												break;
												case CEI_STATUS_MUTEVIAFOCUS:
												{
													ao2_lock(remote_conference->user_container);
													user_tmp = get_user_by_channel_name(remote_conference, user->channel_name,TRUE);

													if (user_tmp) {
														user_tmp->muted = 1;
														poll_cei_partial_notify(conf, remote_conference, user_tmp, data_string, CEI_EVENT_MCB_USER_TOGGLE_MUTE);
														ao2_ref(user_tmp, -1);
													}
													ao2_ref(user, -1);
													ao2_unlock(remote_conference->user_container);
												}
												break;
												default:
												{

												}
												break;
											}
										}
									}
								}
							}

							// Add remote conference by sidebars-by-val
							if (node_sidebars) {
								// state
								accept = FALSE;
								state = ast_xml_get_attribute(node_sidebars, "state");
								if (!state || strcmp(state, "partial") == 0) {
									accept = TRUE;
								}
								ast_xml_free_attr(state);

								if (accept) {
									for (node_iterator = ast_xml_node_get_children(node_sidebars); node_iterator; node_iterator = ast_xml_node_get_next(node_iterator)) {
										const char *node_name = ast_xml_node_get_name(node_iterator);
										ast_log(LOG_DEBUG, "ZAKU DEBUG ---> sidebars-by-val CHILD %s", node_name);

										if (check_xml_text_node(node_iterator) == 0) {
											continue;
										}

										if (strcmp(node_name, "entry") == 0) {
											char *head_pointer;
											char *split_pointer;
											struct ast_xml_node *node_remote_users;
											struct ast_xml_node *node_remote_child_user;

											// state
											state = ast_xml_get_attribute(node_iterator, "state");

											// entity conf info
											entity = ast_xml_get_attribute(node_iterator, "entity");
											char entry_entity[strlen(entity) + 1];
											strncpy(entry_entity, entity, sizeof(entry_entity));
											ast_xml_free_attr(entity);
											ast_log(LOG_DEBUG, "[TK] entry entity: %s", entry_entity);

											head_pointer = entry_entity;
											split_pointer = strchr(entry_entity, '-');
											if (split_pointer == NULL) {
												continue;
											}
											*split_pointer = '\0';
											// remote_num is the remote conf num  in sider bar
											strncpy(remote_num, head_pointer, sizeof(remote_num)-1);

											head_pointer = strchr(split_pointer + 1, '@');
											if (head_pointer == NULL) {
												continue;
											}

											head_pointer = head_pointer + 1;
											split_pointer = strchr(head_pointer, '.');
											if (split_pointer == NULL) {
												continue;
											}
											*split_pointer = '\0';
											// remode_id is the remote conf id in sider bar
											char remote_id[strlen(head_pointer) + 1];
											strncpy(remote_id, head_pointer, sizeof(remote_id));
											char *strip_remote_id = ast_strip(remote_id);
											ast_log(LOG_DEBUG, "[TK] process remote container conf of this xml : %s", strip_remote_id);

											// Ignore invalid conference
											// Don't add myself as my remote conf
											if (strcmp(conf->conference_id, strip_remote_id) == 0) {
												ast_debug(LOG_DEBUG, "[TK] Ignore my own conference");
												ast_debug(LOG_DEBUG, "[TK] %s@%s", remote_num, strip_remote_id);
												continue;
											}

											remote_conference = get_remote_conf_by_id(conf, strip_remote_id);

											if (strcmp(state, "partial") == 0) {
												if (remote_conference != NULL) {
													// Get users
													for(node_remote_users = ast_xml_node_get_children(node_iterator); node_remote_users != NULL; node_remote_users = ast_xml_node_get_next(node_remote_users)) {
														node_name = ast_xml_node_get_name(node_remote_users);
														if (strcmp(node_name, "users") == 0) {
															break;
														}
													}

													if (node_remote_users == NULL) {
														ast_debug(LOG_DEBUG, "[TK] NO USERS");
														continue;
													}

													ast_log(LOG_DEBUG, "[TK] NEW REMOTE CONF %s@%s", remote_num, strip_remote_id);

													// Switch to new remote conference
													// Handle current remote conference

													for (node_remote_child_user = ast_xml_node_get_children(node_remote_users); node_remote_child_user; node_remote_child_user = ast_xml_node_get_next(node_remote_child_user)) {
														if (check_xml_text_node(node_remote_child_user) == 0) {
															continue;
														}
														node_name = ast_xml_node_get_name(node_remote_child_user);
														if (!strcmp(node_name, "user")) {
															user = process_remote_xml_user(conf, node_remote_child_user);
															if (user) {
																switch (user->user_status) {
																	case CEI_STATUS_CONNECTED:
																	{
																		ao2_lock(remote_conference->user_container);
																		user_tmp = get_user_by_cid_num(remote_conference, user->cid_number);
																		//CEI_EVENT_MCB_USER_TOGGLE_MUTE unmute
																		if (user_tmp) {
																			user_tmp->muted = 0;
																			poll_cei_partial_notify(conf, remote_conference, user_tmp, data_string, CEI_EVENT_MCB_USER_TOGGLE_MUTE);
																			ao2_ref(user, -1);
																			ao2_ref(user_tmp, -1);
																		}
																		//CEI_EVENT_MCB_USER_JOIN
																		else {
																			hash_user(user);
																			ast_log(LOG_DEBUG, "ZAKU DEBUG ---> add user : %s to remote conf user container", user->channel_name);
																			ao2_link(remote_conference->user_container, user);
																			ast_atomic_fetchadd_int(&remote_conference->local_size, 1);
																			ast_atomic_fetchadd_int(&remote_conference->local_user_size, (!check_role_as_conference(user))? 1 : 0);
																			ast_atomic_fetchadd_int(&conf->remote_size, 1);
																			ast_atomic_fetchadd_int(&conf->remote_user_size, (!check_role_as_conference(user))? 1 : 0);
																			ast_atomic_fetchadd_int(&remote_conference->refcount, 1);
																			poll_cei_partial_notify(conf, remote_conference, user, data_string, CEI_EVENT_MCB_USER_JOIN);
																		}
																		ao2_unlock(remote_conference->user_container);
																	}
																	break;
																	case CEI_STATUS_DISCONNECTED:
																	{
																		ao2_lock(remote_conference->user_container);

																		user_tmp = get_user_by_cid_num(remote_conference, user->cid_number);

																		if (user_tmp) {

																			ast_atomic_fetchadd_int(&remote_conference->local_size, -1);
																			ast_atomic_fetchadd_int(&remote_conference->local_user_size, (!check_role_as_conference(user))? -1 : 0);
																			ast_atomic_fetchadd_int(&conf->remote_size, -1);
																			ast_atomic_fetchadd_int(&conf->remote_user_size, (!check_role_as_conference(user))? -1 : 0);

																			poll_cei_partial_notify(conf, remote_conference, user, data_string, CEI_EVENT_MCB_USER_PART);

																			ao2_unlink(remote_conference->user_container, user_tmp);
																			ast_atomic_fetchadd_int(&remote_conference->refcount, -1);
																			/*refcount -2 for ao2_alloc and get_user_by_cid_num in remove case*/
																			ao2_ref(user_tmp, -2);
																		}
																		else {
																			ast_debug(LOG_DEBUG, "[WL] ERROR, CANNOT FIND USER  %s IN CONFERENCE %s", user->cid_number, remote_conference->conference_num);
																		}
																		ao2_ref(user, -1);
																		ao2_unlock(remote_conference->user_container);
																	}
																	break;
																	case CEI_STATUS_MUTEVIAFOCUS:
																	{
																		ao2_lock(remote_conference->user_container);

																		user_tmp = get_user_by_cid_num(remote_conference, user->cid_number);

																		if (user_tmp) {
																			user_tmp->muted = 1;
																			poll_cei_partial_notify(conf, remote_conference, user_tmp, data_string, CEI_EVENT_MCB_USER_TOGGLE_MUTE);
																			ao2_ref(user_tmp, -1);
																		}
																		ao2_ref(user, -1);
																		ao2_unlock(remote_conference->user_container);
																	}
																	break;
																	default:
																	{
																		ast_debug(LOG_DEBUG, "[WL] INVALID STATUS");
																	}
																	break;
																}
															}
														}
													}
												}

											}
											else if(strcmp(state, "deleted") == 0) {
												if (remote_conference !=  NULL) {
													delete_remote_conf_by_id(conf, publisher);
													poll_cei_conference(conf, data_string);
												}
											}
											else {
												ast_debug(LOG_DEBUG, "[WL] INVALID STATE");
											}
											ast_xml_free_attr(state);
										}
									}
								}
							}
						}

					}
				}
				ast_xml_free_attr(notifyType);
			}
		}
		ast_xml_close(xml_doc);
	}
	ast_free(xml_content);
	ast_debug(LOG_DEBUG, "ZAKU DEBUG ---> END TEST XML\n");
}

static void process_remote_xml(struct cei_conference *conf, const char *publisher_private_num, const char *xml, struct ast_str **data_string)
{
	char publisher[41] = {0};
	char *xml_content = NULL;
	xml_content = ast_strdup(xml);
	if (!xml_content) {
		return;
	}
	else {
		struct ast_xml_doc *xml_doc = NULL;
		if (!(xml_doc = ast_xml_read_memory(xml_content, strlen(xml_content))))
			ast_log(LOG_DEBUG, "ZAKU DEBUG ---> FAILED TO READ XML DOC");
		else {
			ast_log(LOG_DEBUG, "ZAKU DEBUG ---> READ THE XML DOC CORRECTLY");
			struct ast_xml_node *conference_node = ast_xml_get_root(xml_doc);
			struct ast_xml_node *child_nodes;
			struct ast_xml_node *node_iterator;
			struct ast_xml_ns *ns;
			const char *entity;
			const char *notifyType;
			const char *state;
			const char *namespace;
			const char *node_name;


			if (!conference_node) {
				ast_log(LOG_DEBUG, "ZAKU DEBUG ---> NO ROOT NODE");
			}
			else {
				/*********
				Parse data from xml
				**********/
				char remote_num[MAX_CONFNUM] = {0};
				struct cei_conference *remote_conference = NULL;
				struct cei_user *user = NULL;
				struct cei_user *user_tmp = NULL;
				struct ast_xml_node *node_publisher = NULL;
				struct ast_xml_node *node_host_info = NULL;
				struct ast_xml_node *node_conference_state = NULL;
				struct ast_xml_node *node_users = NULL;
				struct ast_xml_node *node_sidebars = NULL;

				notifyType = ast_xml_get_attribute(conference_node, "state");

				if (strcmp(notifyType,"full") == 0) {
					entity = ast_xml_get_attribute(conference_node, "entity");
					char temp_entity[strlen(entity) + 1];
					strncpy(temp_entity, entity, strlen(entity));
					char *colon = strchr(temp_entity, ':');
					char *at = strchr(temp_entity, '@');
					*at = '\0';
					strncpy(remote_num, colon + 1, sizeof(remote_num)-1);
					ast_xml_free_attr(entity);

					ns = ast_xml_find_namespace(xml_doc, conference_node, NULL);
					if (ns) {
						namespace = ast_xml_get_ns_href(ns);
						ast_log(LOG_DEBUG, "ZAKU DEBUG ---> NAMESPACE IS %s", namespace);
					}

					child_nodes = ast_xml_node_get_children(conference_node);
					if (child_nodes) {
						ast_log(LOG_DEBUG, "ZAKU DEBUG ---> HAS CHILDREN");

						for (node_iterator = child_nodes; node_iterator; node_iterator = ast_xml_node_get_next(node_iterator)) {
							const char *node_name = ast_xml_node_get_name(node_iterator);
							if (check_xml_text_node(node_iterator)) {
								ast_log(LOG_DEBUG, "ZAKU DEBUG ---> HAS CHILD NODE %s", node_name);
								if (!strcmp(node_name, "publisher") && !node_publisher)
									node_publisher = node_iterator;
								else if (!strcmp(node_name, "host-info") && !node_host_info)
									node_host_info = node_iterator;
								else if (!strcmp(node_name, "conference-state") && !node_conference_state)
									node_conference_state = node_iterator;
								else if (!strcmp(node_name, "users") && !node_users)
									node_users = node_iterator;
								else if (!strcmp(node_name, "sidebars-by-val") && !node_sidebars) {
									node_sidebars = node_iterator;
								}

								const char *content = ast_xml_get_text(node_iterator);
								if (content) {
									ast_log(LOG_DEBUG, "ZAKU DEBUG ---> HAS CHILD NODE CONTENT %s", content);
									ast_xml_free_text(content);
								}
								else
									ast_log(LOG_DEBUG, "ZAKU DEBUG ---> HAS CHILD NODE CONTENT NULL");

							}
							else
								ast_log(LOG_DEBUG, "ZAKU DEBUG ---> IGNORING BLANK TEXT NODE");
						}
					}

					if (node_publisher) {
						const char *content = ast_xml_get_text(node_publisher);

						if (content) {
							ast_copy_string(publisher, content, sizeof(publisher));
							ast_debug(LOG_DEBUG, "[TK] PUBLISHER IS %s", publisher);
						}

						ast_xml_free_text(content);
					}

					if (node_host_info) {
						const char *content = ast_xml_get_text(node_host_info);
						if (content) {
							char remote_id[strlen(content) + 1];
							strncpy(remote_id, content, sizeof(remote_id));
							ast_xml_free_text(content);
							char *strip_remote_id = ast_strip(remote_id);

							if (strlen(publisher) == 0) {
								ast_copy_string(publisher, strip_remote_id, sizeof(publisher));
								ast_debug(LOG_DEBUG, "[TK] PUBLISHER IS %s", publisher);
							}

							// clean up the remote conf published from publisher
							if(!conf){
								ast_debug(LOG_DEBUG, "[TK] no host conf !!!");
							}

							// Reinitialize published conferences
							delete_remote_confs_by_publisher(conf, strip_remote_id);

							ao2_lock(conf->remote_container);
							/***
							get or create the remote conf from host conf's remote container;
							the remote conf is basically this cei msg come FROM
							***/
							remote_conference = build_remote_conf(conf, publisher_private_num, strip_remote_id, publisher);
							ao2_unlock(conf->remote_container);
						}
					}

					int accept = TRUE;
					if (remote_conference) {
						const char *version = ast_xml_get_attribute(conference_node, "version");
						if (!version) {
							ast_xml_free_attr(version);
							ast_log(LOG_DEBUG, "ZAKU DEBUG ---> DOCUMENT HAS NO VERSION");
							accept = FALSE;
						}
						else {
							int xml_version = atoi(version);

							if (xml_version <= remote_conference->version) {
								ast_xml_free_attr(version);
								ast_log(LOG_DEBUG, "ZAKU DEBUG ---> DOCUMENT VERSION %d <= %d", xml_version, remote_conference->version);
								accept = FALSE;
							}
							else {
								ast_xml_free_attr(version);
								remote_conference->version = xml_version;
							}
						}

						ao2_lock(remote_conference->user_container);

						if (accept) {
							clear_conf_users(remote_conference);

							// Get remote conference-state items
							if (node_conference_state) {
								for (node_iterator = ast_xml_node_get_children(node_conference_state); node_iterator; node_iterator = ast_xml_node_get_next(node_iterator)) {
									if (check_xml_text_node(node_iterator)) {
										const char *node_name = ast_xml_node_get_name(node_iterator);
										const char *content = ast_xml_get_text(node_iterator);
										ast_log(LOG_DEBUG, "ZAKU DEBUG ---> CONFERENCE STATE CHILD %s", node_name);
										if (content) {
											if (!strcmp(node_name, "user-count")) {
												char strip_content[strlen(content) + 1];
												strncpy(strip_content, content, strlen(content));
												ast_log(LOG_DEBUG, "ZAKU DEBUG ---> REMOTE LOCAL SIZE %d", remote_conference->local_user_size);
											}
											if (!strcmp(node_name, "locked")) {
												char strip_content[strlen(content) + 1];
												strncpy(strip_content, content, strlen(content));
												if (!strcmp(strip_content, "true"))
													remote_conference->locked = TRUE;
												else
													remote_conference->locked = FALSE;
												ast_log(LOG_DEBUG, "ZAKU DEBUG ---> REMOTE LOCKED %s", remote_conference->locked ? "TRUE" : "FALSE");
											}
										}
										ast_xml_free_text(content);
									}
								}
							}

							// Add users for remote publisher
							if (node_users) {
								process_remote_xml_users(conf, remote_conference,
														node_users);
							}

							// Add remote conference by sidebars-by-val
							if (node_sidebars) {
								// state
								accept = FALSE;
								state = ast_xml_get_attribute(node_sidebars, "state");
								if (!state || strcmp(state, "full") == 0) {
									accept = TRUE;
								}
								ast_xml_free_attr(state);

								if (accept) {
									for (node_iterator = ast_xml_node_get_children(node_sidebars); node_iterator; node_iterator = ast_xml_node_get_next(node_iterator)) {
										const char *node_name = ast_xml_node_get_name(node_iterator);
										ast_log(LOG_DEBUG, "ZAKU DEBUG ---> sidebars-by-val CHILD %s", node_name);

										if (check_xml_text_node(node_iterator) == 0) {
											continue;
										}

										if (strcmp(node_name, "entry") == 0) {
											char *head_pointer;
											char *split_pointer;
											struct ast_xml_node *entry_child;

											// state
											state = ast_xml_get_attribute(node_iterator, "state");
											if (!state || strcmp(state, "full") != 0) {
												ast_xml_free_attr(state);
												continue;
											}
											ast_xml_free_attr(state);

											// entity conf info
											entity = ast_xml_get_attribute(node_iterator, "entity");
											char entry_entity[strlen(entity) + 1];
											strncpy(entry_entity, entity, sizeof(entry_entity));
											ast_xml_free_attr(entity);
											ast_log(LOG_DEBUG, "[TK] entry entity: %s", entry_entity);

											head_pointer = entry_entity;
											split_pointer = strchr(entry_entity, '-');
											if (split_pointer == NULL) {
												continue;
											}
											*split_pointer = '\0';
											// remote_num is the remote conf num  in sider bar
											strncpy(remote_num, head_pointer, sizeof(remote_num)-1);

											head_pointer = strchr(split_pointer + 1, '@');
											if (head_pointer == NULL) {
												continue;
											}

											head_pointer = head_pointer + 1;
											split_pointer = strchr(head_pointer, '.');
											if (split_pointer == NULL) {
												continue;
											}
											*split_pointer = '\0';
											// remode_id is the remote conf id in sider bar
											char remote_id[strlen(head_pointer) + 1];
											strncpy(remote_id, head_pointer, sizeof(remote_id));
											char *strip_remote_id = ast_strip(remote_id);
											ast_log(LOG_DEBUG, "[TK] process remote container conf of this xml : %s", strip_remote_id);

											// Ignore invalid conference
											// Don't add myself as my remote conf
											if (strcmp(conf->conference_id, strip_remote_id) == 0) {
												ast_debug(LOG_DEBUG, "[TK] Ignore my own conference");
												ast_debug(LOG_DEBUG, "[TK] %s@%s", remote_num, strip_remote_id);
												continue;
											}
											// Don't add the remote conf which are my own remote conf again
											if (get_remote_conf_by_id(conf, strip_remote_id) != NULL) {
												ast_debug(LOG_DEBUG, "[TK] Ignore my own remote conference");
												ast_debug(LOG_DEBUG, "[TK] %s@%s", remote_num, strip_remote_id);
												continue;
											}

											// Get users
											node_users = NULL;
											for(entry_child = ast_xml_node_get_children(node_iterator); entry_child != NULL; entry_child = ast_xml_node_get_next(entry_child)) {
												node_name = ast_xml_node_get_name(entry_child);
												if (strcmp(node_name, "users") == 0) {
													node_users = entry_child;
													break;
												}
											}

											if (node_users == NULL) {
												ast_debug(LOG_DEBUG, "[TK] NO USERS");
												continue;
											}

											ast_log(LOG_DEBUG, "[TK] NEW REMOTE CONF %s@%s", remote_num, strip_remote_id);

											// Switch to new remote conference
											// Handle current remote conference
											ao2_unlock(remote_conference->user_container);

											// Handle new remote conference
											remote_conference = build_remote_conf(conf, remote_num, strip_remote_id, publisher);

											ao2_lock(remote_conference->user_container);
											clear_conf_users(remote_conference);

											process_remote_xml_users(conf, remote_conference,
																	node_users);
										}
									}
								}
							}
						}

						ao2_unlock(remote_conference->user_container);
					}
					poll_cei_conference(conf, data_string);
				}
				// PARTIAL NOTIFY
				else if (strcmp(notifyType,"partial") == 0) {
					entity = ast_xml_get_attribute(conference_node, "entity");
					char temp_entity[strlen(entity) + 1];
					strncpy(temp_entity, entity, strlen(entity));
					char *colon = strchr(temp_entity, ':');
					char *at = strchr(temp_entity, '@');
					*at = '\0';
					strncpy(remote_num, colon + 1, sizeof(remote_num)-1);
					ast_xml_free_attr(entity);

					ns = ast_xml_find_namespace(xml_doc, conference_node, NULL);
					if (ns) {
						namespace = ast_xml_get_ns_href(ns);
						ast_log(LOG_DEBUG, "ZAKU DEBUG ---> NAMESPACE IS %s", namespace);
					}

					child_nodes = ast_xml_node_get_children(conference_node);
					if (child_nodes) {
						ast_log(LOG_DEBUG, "ZAKU DEBUG ---> HAS CHILDREN");

						for (node_iterator = child_nodes; node_iterator; node_iterator = ast_xml_node_get_next(node_iterator)) {
							const char *node_name = ast_xml_node_get_name(node_iterator);
							if (check_xml_text_node(node_iterator)) {
								ast_log(LOG_DEBUG, "ZAKU DEBUG ---> HAS CHILD NODE %s", node_name);
								if (!strcmp(node_name, "publisher") && !node_publisher)
									node_publisher = node_iterator;
								else if (!strcmp(node_name, "host-info") && !node_host_info)
									node_host_info = node_iterator;
								else if (!strcmp(node_name, "conference-state") && !node_conference_state)
									node_conference_state = node_iterator;
								else if (!strcmp(node_name, "users") && !node_users)
									node_users = node_iterator;
								else if (!strcmp(node_name, "sidebars-by-val") && !node_sidebars) {
									node_sidebars = node_iterator;
								}

								const char *content = ast_xml_get_text(node_iterator);
								if (content) {
									ast_log(LOG_DEBUG, "ZAKU DEBUG ---> HAS CHILD NODE CONTENT %s", content);
									ast_xml_free_text(content);
								}
								else
									ast_log(LOG_DEBUG, "ZAKU DEBUG ---> HAS CHILD NODE CONTENT NULL");

							}
							else
								ast_log(LOG_DEBUG, "ZAKU DEBUG ---> IGNORING BLANK TEXT NODE");
						}
					}

					if (node_publisher) {
						const char *content = ast_xml_get_text(node_publisher);

						if (content) {
							ast_copy_string(publisher, content, sizeof(publisher));
							ast_debug(LOG_DEBUG, "[TK] PUBLISHER IS %s", publisher);
						}

						ast_xml_free_text(content);
					}

					if (node_host_info) {
						const char *content = ast_xml_get_text(node_host_info);
						if (content) {
							char remote_id[strlen(content) + 1];
							strncpy(remote_id, content, sizeof(remote_id));
							ast_xml_free_text(content);
							char *strip_remote_id = ast_strip(remote_id);

							if (strlen(publisher) == 0) {
								ast_copy_string(publisher, strip_remote_id, sizeof(publisher));
								ast_debug(LOG_DEBUG, "[TK] PUBLISHER IS %s", publisher);
							}

							// clean up the remote conf published from publisher
							if(!conf){
								ast_debug(LOG_DEBUG, "[TK] no host conf !!!");
							}

							/***
							get the remote conf from host conf's remote container;
							the remote conf is basically this cei msg come FROM
							***/
							remote_conference = get_remote_conf_by_id(conf, publisher);
						}
					}

					int accept = TRUE;
					if (remote_conference) {
						const char *version = ast_xml_get_attribute(conference_node, "version");
						if (!version) {
							ast_xml_free_attr(version);
							ast_log(LOG_DEBUG, "ZAKU DEBUG ---> DOCUMENT HAS NO VERSION");
							accept = FALSE;
						}
						else {
							int xml_version = atoi(version);

							if (xml_version <= remote_conference->version) {
								ast_xml_free_attr(version);
								ast_log(LOG_DEBUG, "ZAKU DEBUG ---> DOCUMENT VERSION %d <= %d", xml_version, remote_conference->version);
								accept = FALSE;
							}
							else {
								ast_xml_free_attr(version);
								remote_conference->version = xml_version;
							}
						}


						if (accept) {

							// Get remote conference-state items
							if (node_conference_state) {
								for (node_iterator = ast_xml_node_get_children(node_conference_state); node_iterator; node_iterator = ast_xml_node_get_next(node_iterator)) {
									if (check_xml_text_node(node_iterator)) {
										const char *node_name = ast_xml_node_get_name(node_iterator);
										const char *content = ast_xml_get_text(node_iterator);
										ast_log(LOG_DEBUG, "ZAKU DEBUG ---> CONFERENCE STATE CHILD %s", node_name);
										if (content) {
											if (!strcmp(node_name, "user-count")) {
												char strip_content[strlen(content) + 1];
												strncpy(strip_content, content, strlen(content));
												ast_log(LOG_DEBUG, "ZAKU DEBUG ---> REMOTE LOCAL SIZE %d", remote_conference->local_user_size);
											}
											if (!strcmp(node_name, "locked")) {
												char strip_content[strlen(content) + 1];
												strncpy(strip_content, content, strlen(content));
												if (!strcmp(strip_content, "true"))
													remote_conference->locked = TRUE;
												else
													remote_conference->locked = FALSE;
												ast_log(LOG_DEBUG, "ZAKU DEBUG ---> REMOTE LOCKED %s", remote_conference->locked ? "TRUE" : "FALSE");
											}
										}
										ast_xml_free_text(content);
									}
								}
							}

							// Add users for remote publisher
							if (node_users) {
								for (node_iterator = ast_xml_node_get_children(node_users); node_iterator; node_iterator = ast_xml_node_get_next(node_iterator)) {
									if (check_xml_text_node(node_iterator) == 0) {
										continue;
									}
									node_name = ast_xml_node_get_name(node_iterator);
									if (!strcmp(node_name, "user")) {

										user = process_remote_xml_user(conf, node_iterator);
										if (user) {
											switch (user->user_status) {
												case CEI_STATUS_CONNECTED:
												{
													ao2_lock(remote_conference->user_container);
													//CEI_EVENT_MCB_USER_TOGGLE_MUTE unmute
													user_tmp = get_user_by_cid_num(remote_conference, user->cid_number);
													if (user_tmp) {
														user_tmp->muted = 0;
														user_tmp->talking = user->talking;
														poll_cei_partial_notify(conf, remote_conference, user_tmp, data_string, CEI_EVENT_MCB_USER_TOGGLE_MUTE);
														ao2_ref(user, -1);
														ao2_ref(user_tmp, -1);
													}
													else {
														hash_user(user);
														ast_log(LOG_DEBUG, "ZAKU DEBUG ---> add user : %s to remote conf user container", user->channel_name);
														ao2_link(remote_conference->user_container, user);
														ast_atomic_fetchadd_int(&remote_conference->local_size, 1);
														ast_atomic_fetchadd_int(&remote_conference->local_user_size, (!check_role_as_conference(user))? 1 : 0);
														ast_atomic_fetchadd_int(&conf->remote_size, 1);
														ast_atomic_fetchadd_int(&conf->remote_user_size, (!check_role_as_conference(user))? 1 : 0);
														ast_atomic_fetchadd_int(&remote_conference->refcount, 1);
														poll_cei_partial_notify(conf, remote_conference, user, data_string, CEI_EVENT_MCB_USER_JOIN);
													}
													ao2_unlock(remote_conference->user_container);
												}
												break;
												case CEI_STATUS_DISCONNECTED:
												{
													ao2_lock(remote_conference->user_container);

													user_tmp = get_user_by_cid_num(remote_conference, user->cid_number);

													if (user_tmp) {

														ast_atomic_fetchadd_int(&remote_conference->local_size, -1);
														ast_atomic_fetchadd_int(&remote_conference->local_user_size, (!check_role_as_conference(user))? -1 : 0);
														ast_atomic_fetchadd_int(&conf->remote_size, -1);
														ast_atomic_fetchadd_int(&conf->remote_user_size, (!check_role_as_conference(user))? -1 : 0);

														poll_cei_partial_notify(conf, remote_conference, user, data_string, CEI_EVENT_MCB_USER_PART);

														ao2_unlink(remote_conference->user_container, user_tmp);
														ast_atomic_fetchadd_int(&remote_conference->refcount, -1);

														ast_debug(LOG_DEBUG, "[WL] REMOVED USER WITH ADDRESS %p", user_tmp);
														/*refcount -2 for ao2_alloc and get_user_by_cid_num in remove case*/
														ao2_ref(user_tmp, -2);
													}
													else {
														ast_debug(LOG_DEBUG, "[WL] ERROR, CANNOT FIND USER  %s IN CONFERENCE %s", user->cid_number, remote_conference->conference_num);
													}
													ao2_ref(user, -1);
													ao2_unlock(remote_conference->user_container);
												}
												break;
												case CEI_STATUS_MUTEVIAFOCUS:
												{
													ao2_lock(remote_conference->user_container);

													user_tmp = get_user_by_cid_num(remote_conference, user->cid_number);

													if (user_tmp) {
														user_tmp->muted = 1;
														poll_cei_partial_notify(conf, remote_conference, user_tmp, data_string, CEI_EVENT_MCB_USER_TOGGLE_MUTE);
														ao2_ref(user_tmp, -1);
													}
													ao2_ref(user, -1);
													ao2_unlock(remote_conference->user_container);
												}
												break;
												default:
												{

												}
												break;
											}
										}
									}
								}
							}

							// Add remote conference by sidebars-by-val
							if (node_sidebars) {
								// state
								accept = FALSE;
								state = ast_xml_get_attribute(node_sidebars, "state");
								if (!state || strcmp(state, "partial") == 0) {
									accept = TRUE;
								}
								ast_xml_free_attr(state);

								if (accept) {
									for (node_iterator = ast_xml_node_get_children(node_sidebars); node_iterator; node_iterator = ast_xml_node_get_next(node_iterator)) {
										const char *node_name = ast_xml_node_get_name(node_iterator);
										ast_log(LOG_DEBUG, "ZAKU DEBUG ---> sidebars-by-val CHILD %s", node_name);

										if (check_xml_text_node(node_iterator) == 0) {
											continue;
										}

										if (strcmp(node_name, "entry") == 0) {
											char *head_pointer;
											char *split_pointer;
											struct ast_xml_node *node_remote_users;
											struct ast_xml_node *node_remote_child_user;

											// state
											state = ast_xml_get_attribute(node_iterator, "state");

											// entity conf info
											entity = ast_xml_get_attribute(node_iterator, "entity");
											char entry_entity[strlen(entity) + 1];
											strncpy(entry_entity, entity, sizeof(entry_entity));
											ast_xml_free_attr(entity);
											ast_log(LOG_DEBUG, "[TK] entry entity: %s", entry_entity);

											head_pointer = entry_entity;
											split_pointer = strchr(entry_entity, '-');
											if (split_pointer == NULL) {
												continue;
											}
											*split_pointer = '\0';
											// remote_num is the remote conf num  in sider bar
											strncpy(remote_num, head_pointer, sizeof(remote_num)-1);

											head_pointer = strchr(split_pointer + 1, '@');
											if (head_pointer == NULL) {
												continue;
											}

											head_pointer = head_pointer + 1;
											split_pointer = strchr(head_pointer, '.');
											if (split_pointer == NULL) {
												continue;
											}
											*split_pointer = '\0';
											// remode_id is the remote conf id in sider bar
											char remote_id[strlen(head_pointer) + 1];
											strncpy(remote_id, head_pointer, sizeof(remote_id));
											char *strip_remote_id = ast_strip(remote_id);
											ast_log(LOG_DEBUG, "[TK] process remote container conf of this xml : %s", strip_remote_id);

											// Ignore invalid conference
											// Don't add myself as my remote conf
											if (strcmp(conf->conference_id, strip_remote_id) == 0) {
												ast_debug(LOG_DEBUG, "[TK] Ignore my own conference");
												ast_debug(LOG_DEBUG, "[TK] %s@%s", remote_num, strip_remote_id);
												continue;
											}

											remote_conference = get_remote_conf_by_id(conf, strip_remote_id);

											if (strcmp(state, "partial") == 0) {
												if (remote_conference != NULL) {
													// Get users
													for(node_remote_users = ast_xml_node_get_children(node_iterator); node_remote_users != NULL; node_remote_users = ast_xml_node_get_next(node_remote_users)) {
														node_name = ast_xml_node_get_name(node_remote_users);
														if (strcmp(node_name, "users") == 0) {
															break;
														}
													}

													if (node_remote_users == NULL) {
														ast_debug(LOG_DEBUG, "[TK] NO USERS");
														continue;
													}

													ast_log(LOG_DEBUG, "[TK] NEW REMOTE CONF %s@%s", remote_num, strip_remote_id);

													// Switch to new remote conference
													// Handle current remote conference

													for (node_remote_child_user = ast_xml_node_get_children(node_remote_users); node_remote_child_user; node_remote_child_user = ast_xml_node_get_next(node_remote_child_user)) {
														if (check_xml_text_node(node_remote_child_user) == 0) {
															continue;
														}
														node_name = ast_xml_node_get_name(node_remote_child_user);
														if (!strcmp(node_name, "user")) {
															user = process_remote_xml_user(conf, node_remote_child_user);
															if (user) {
																switch (user->user_status) {
																	case CEI_STATUS_CONNECTED:
																	{
																		ao2_lock(remote_conference->user_container);
																		user_tmp = get_user_by_cid_num(remote_conference, user->cid_number);
																		//CEI_EVENT_MCB_USER_TOGGLE_MUTE unmute
																		if (user_tmp) {
																			user_tmp->muted = 0;
																			poll_cei_partial_notify(conf, remote_conference, user_tmp, data_string, CEI_EVENT_MCB_USER_TOGGLE_MUTE);
																			ao2_ref(user, -1);
																			ao2_ref(user_tmp, -1);
																		}
																		//CEI_EVENT_MCB_USER_JOIN
																		else {
																			hash_user(user);
																			ast_log(LOG_DEBUG, "ZAKU DEBUG ---> add user : %s to remote conf user container", user->channel_name);
																			ao2_link(remote_conference->user_container, user);
																			ast_atomic_fetchadd_int(&remote_conference->local_size, 1);
																			ast_atomic_fetchadd_int(&remote_conference->local_user_size, (!check_role_as_conference(user))? 1 : 0);
																			ast_atomic_fetchadd_int(&conf->remote_size, 1);
																			ast_atomic_fetchadd_int(&conf->remote_user_size, (!check_role_as_conference(user))? 1 : 0);
																			ast_atomic_fetchadd_int(&remote_conference->refcount, 1);
																			poll_cei_partial_notify(conf, remote_conference, user, data_string, CEI_EVENT_MCB_USER_JOIN);
																		}
																		ao2_unlock(remote_conference->user_container);
																	}
																	break;
																	case CEI_STATUS_DISCONNECTED:
																	{
																		ao2_lock(remote_conference->user_container);

																		user_tmp = get_user_by_cid_num(remote_conference, user->cid_number);

																		if (user_tmp) {

																			ast_atomic_fetchadd_int(&remote_conference->local_size, -1);
																			ast_atomic_fetchadd_int(&remote_conference->local_user_size, (!check_role_as_conference(user))? -1 : 0);
																			ast_atomic_fetchadd_int(&conf->remote_size, -1);
																			ast_atomic_fetchadd_int(&conf->remote_user_size, (!check_role_as_conference(user))? -1 : 0);

																			poll_cei_partial_notify(conf, remote_conference, user, data_string, CEI_EVENT_MCB_USER_PART);

																			ao2_unlink(remote_conference->user_container, user_tmp);
																			ast_atomic_fetchadd_int(&remote_conference->refcount, -1);

																			/*refcount -2 for ao2_alloc and get_user_by_cid_num in remove case*/
																			ao2_ref(user_tmp, -2);
																		}
																		else {
																			ast_debug(LOG_DEBUG, "[WL] ERROR, CANNOT FIND USER  %s IN CONFERENCE %s", user->cid_number, remote_conference->conference_num);
																		}
																		ao2_ref(user, -1);
																		ao2_unlock(remote_conference->user_container);
																	}
																	break;
																	case CEI_STATUS_MUTEVIAFOCUS:
																	{
																		ao2_lock(remote_conference->user_container);

																		user_tmp = get_user_by_cid_num(remote_conference, user->cid_number);

																		if (user_tmp) {
																			user_tmp->muted = 1;
																			poll_cei_partial_notify(conf, remote_conference, user_tmp, data_string, CEI_EVENT_MCB_USER_TOGGLE_MUTE);
																			ao2_ref(user_tmp, -1);
																		}
																		ao2_ref(user, -1);
																		ao2_unlock(remote_conference->user_container);
																	}
																	break;
																	default:
																	{
																		ast_debug(LOG_DEBUG, "[WL] INVALID STATUS");
																	}
																	break;
																}
															}
														}
													}
												}

											}
											else if(strcmp(state, "deleted") == 0) {
												if (remote_conference !=  NULL) {
													delete_remote_conf_by_id(conf, publisher);
													poll_cei_conference(conf, data_string);
												}
											}
											else {
												ast_debug(LOG_DEBUG, "[WL] INVALID STATE");
											}
											ast_xml_free_attr(state);
										}
									}
								}
							}
						}

					}
				}
				ast_xml_free_attr(notifyType);
			}
		}
		ast_xml_close(xml_doc);
	}
	ast_free(xml_content);
	ast_debug(LOG_DEBUG, "ZAKU DEBUG ---> END TEST XML\n");
}

static int check_xml_text_node(struct ast_xml_node *node)
{
	return strcmp(ast_xml_node_get_name(node), "text") ? 1 : 0;
}

static void process_remote_xml_users(struct cei_conference *conf,
									 struct cei_conference *remote_conference,
									 struct ast_xml_node *node_users)
{
	const char *node_name;
	struct cei_user *user;
	struct ast_xml_node *node_iterator = NULL;

	// Sanity check
	if (conf == NULL || remote_conference == NULL || node_users == NULL) {
		return;
	}

	for (node_iterator = ast_xml_node_get_children(node_users); node_iterator; node_iterator = ast_xml_node_get_next(node_iterator)) {
		if (check_xml_text_node(node_iterator) == 0) {
			continue;
		}

		node_name = ast_xml_node_get_name(node_iterator);
		ast_log(LOG_DEBUG, "ZAKU DEBUG ---> USERS CHILD %s", node_name);

		if (!strcmp(node_name, "user")) {
			user = process_remote_xml_user(conf, node_iterator);
			if (user == NULL) {
				continue;
			}
			ast_log(LOG_DEBUG, "ZAKU DEBUG ---> add user : %s to remote conf user container", user->channel_name);
			ao2_link(remote_conference->user_container, user);
			ast_atomic_fetchadd_int(&remote_conference->local_size, 1);
			ast_atomic_fetchadd_int(&remote_conference->local_user_size, (!check_role_as_conference(user))? 1 : 0);
			ast_atomic_fetchadd_int(&conf->remote_size, 1);
			ast_atomic_fetchadd_int(&conf->remote_user_size, (!check_role_as_conference(user))? 1 : 0);
			ast_atomic_fetchadd_int(&remote_conference->refcount, 1);
		}
	}
}

static struct cei_user *process_remote_xml_user(struct cei_conference *host_conf, struct ast_xml_node *node_iterator)
{
	char *head_pointer;
	char *split_pointer;
	const char *content;
	const char *self_conf_id;
	const char *entity;
	const char *state;
	struct cei_user *user;
	struct ast_xml_node *node_display_text = NULL;
	struct ast_xml_node *node_aors = NULL;
	struct ast_xml_node *node_roles = NULL;
	struct ast_xml_node *node_endpoint = NULL;
	struct ast_xml_node *user_inner_iterator = NULL;
	struct ast_xml_node *node_role_iterator = NULL;

	// Sanity check
	if (node_iterator == NULL || host_conf == NULL) {
		return NULL;
	}

	if (!(user = ao2_alloc(sizeof(struct cei_user), NULL))) {
		ast_debug(LOG_DEBUG, "ZAKU DEBUG ---> CANNOT CREATE NEW USER");
		return NULL;
	}

	state = ast_xml_get_attribute(node_iterator, "state");

	if (!state || strcmp(state, "full")) {
		ast_xml_free_attr(state);
		ast_log(LOG_DEBUG, "ZAKU DEBUG ---> USER STATE NOT full");
		ao2_ref(user, -1);
		return NULL;
	}
	ast_xml_free_attr(state);

	entity = ast_xml_get_attribute(node_iterator, "entity");
	if (entity == NULL) {
		ao2_ref(user, -1);
		return NULL;
	}

	char user_entity[strlen(entity) + 1];
	strncpy(user_entity, entity, sizeof(user_entity));
	ast_xml_free_attr(entity);
	head_pointer = user_entity;
	split_pointer = strchr(user_entity, '-');
	if (!split_pointer) {
		ast_log(LOG_DEBUG, "ZAKU DEBUG ---> USER ENTITY SPLIT 1");
		ao2_ref(user, -1);
		return NULL;
	}
	*split_pointer = '\0';
	strncpy(user->cid_number, head_pointer, sizeof(user->cid_number));

	head_pointer = split_pointer + 1;
	split_pointer = strchr(head_pointer, '@');
	if (!split_pointer) {
		ast_log(LOG_DEBUG, "ZAKU DEBUG ---> USER ENTITY SPLIT 2");
		ao2_ref(user, -1);
		return NULL;
	}
	*split_pointer = '\0';
	user->user_num = atoi(head_pointer);

	head_pointer = split_pointer + 1;
	split_pointer = strchr(head_pointer, '.');
	if (!split_pointer) {
		ast_log(LOG_DEBUG, "ZAKU DEBUG ---> USER ENTITY SPLIT 3");
		ao2_ref(user, -1);
		return NULL;
	}
	*split_pointer = '\0';
	strncpy(user->conference_id, head_pointer, sizeof(user->conference_id));

	ast_log(LOG_DEBUG, "ZAKU DEBUG ---> GOT USER CID NUM, CONFERENCE ID, USER NUM");

	// Remove local users in host conf
	if (strcmp(user->conference_id, host_conf->conference_id) == 0) {
		ast_log(LOG_DEBUG, "[TK] Ignore local conference users");
		ast_log(LOG_DEBUG, "[TK] %s-%d@%s",
		user->cid_number, user->user_num, user->conference_id);
		ao2_ref(user, -1);
		return NULL;
	}

	node_display_text = NULL;
	node_aors = NULL;
	node_roles = NULL;
	node_endpoint = NULL;

	for (user_inner_iterator = ast_xml_node_get_children(node_iterator); user_inner_iterator; user_inner_iterator = ast_xml_node_get_next(user_inner_iterator)) {
		if (check_xml_text_node(user_inner_iterator)) {
			const char *user_inner_node_name = ast_xml_node_get_name(user_inner_iterator);
			ast_log(LOG_DEBUG, "ZAKU DEBUG ---> USER CHILD %s", user_inner_node_name);
			if (!strcmp(user_inner_node_name, "display-text") && !node_display_text) {
				node_display_text = user_inner_iterator;
			}
			else if (!strcmp(user_inner_node_name, "associated-aors") && !node_aors) {
				node_aors = user_inner_iterator;
			}
			else if (!strcmp(user_inner_node_name, "roles") && !node_roles) {
				node_roles = user_inner_iterator;
			}
			else if (!strcmp(user_inner_node_name, "endpoint") && !node_endpoint) {
				node_endpoint = user_inner_iterator;
			}
		}
	}

	if (!node_display_text || !node_roles || !node_endpoint ) {
		ast_log(LOG_DEBUG, "ZAKU DEBUG ---> USER MISSING DISPLAY/ROLES/ENDPOINT");
		ao2_ref(user, -1);
		return NULL;
	}

	//<display-text>
	content = ast_xml_get_text(node_display_text);
	if (content) {
		strncpy(user->cid_name, content, sizeof(user->cid_name));
		ast_xml_free_text(content);
	}
	else {
		ast_log(LOG_DEBUG, "ZAKU DEBUG ---> USER EMPTY DISPLAY");
		ao2_ref(user, -1);
		return NULL;
	}

	//<associated-aors> the self conf id if conference user
	if (node_aors) {

		self_conf_id = ast_xml_get_text(node_aors);
		ast_log(LOG_DEBUG, "[WL] Got self conf id : %s",self_conf_id);
		if (self_conf_id) {
			if (strcmp(host_conf->conference_id, self_conf_id) == 0) {
				memset(user->cid_number, 0, sizeof(user->cid_number));
				strncpy(user->cid_number, host_conf->conference_num, sizeof(user->cid_number));
			}
		}
	}

	//<roles>
	user->admin = 0;
	user->can_invite = 0;
	user->user_role = CEI_ROLE_UNKNOWN;
	for (node_role_iterator = ast_xml_node_get_children(node_roles); node_role_iterator; node_role_iterator = ast_xml_node_get_next(node_role_iterator)) {
		if (check_xml_text_node(node_role_iterator) == 0){
			continue;
		}

		if (!strcmp(ast_xml_node_get_name(node_role_iterator), "entry")) {
			content = ast_xml_get_text(node_role_iterator);
			if (content) {
				if (!strcmp(content, "admin")) {
					user->admin = 1;
					user->user_role = CEI_ROLE_ADMIN;
					break;
				}
				else if (!strcmp(content, "conference")) {
					user->user_role = CEI_ROLE_CONFERENCE;
					break;
				}
				else if (!strcmp(content, "invite")) {
					user->can_invite = 1;
					break;
				}
				else {
					user->user_role = CEI_ROLE_USER_NORMAL;
				}

				ast_xml_free_text(content);
			}
		}
	}

	if (user->can_invite && (user->user_role == CEI_ROLE_USER_NORMAL)) {
		user->user_role = CEI_ROLE_USER_INVITE;
	}

	//<endpoint>
	entity = ast_xml_get_attribute(node_endpoint, "entity");
	strncpy(user->channel_name, entity, sizeof(user->channel_name));
	ast_xml_free_attr(entity);

	state = ast_xml_get_attribute(node_endpoint, "state");
	if (!state || strcmp(state, "full")) {
		ast_xml_free_attr(state);
		ast_log(LOG_DEBUG, "ZAKU DEBUG ---> ENDPOINT STATE NOT full");
		ao2_ref(user, -1);
		return NULL;
	}
	ast_xml_free_attr(state);

	// Get user status
	user->muted = 0;
	user->user_status = CEI_STATUS_UNKNOWN;

	for (user_inner_iterator = ast_xml_node_get_children(node_endpoint); user_inner_iterator; user_inner_iterator = ast_xml_node_get_next(user_inner_iterator)) {
		const char *endpoint_node_name = ast_xml_node_get_name(user_inner_iterator);
		ast_log(LOG_DEBUG, "ZAKU DEBUG ---> ENDPOINT CHILD %s", endpoint_node_name);

		// [TK] TODO: replace muted with user_status
		const char *content = ast_xml_get_text(user_inner_iterator);
		if (!content) {
			continue;
		}

		if (!strcmp(endpoint_node_name, "status")) {
			if (strstr(content, "disconnected")) {
				user->user_status = CEI_STATUS_DISCONNECTED;
			}
			else if (strstr(content, "connected")) {
				user->user_status = CEI_STATUS_CONNECTED;
			}
			else if (strstr(content, "on-hold")) {
				user->user_status = CEI_STATUS_ONHOLD;
			}
			else if (strstr(content, "muted-via-focus")) {
				user->muted = 1;
				user->user_status = CEI_STATUS_MUTEVIAFOCUS;
			}
			else if (strstr(content, "pending")) {
				user->user_status = CEI_STATUS_PENDING;
			}
			else if (strstr(content, "alerting")) {
				user->user_status = CEI_STATUS_ALERTING;
			}
			else if (strstr(content, "dialing-in")) {
				user->user_status = CEI_STATUS_DIALINGIN;
			}
			else if (strstr(content, "dialing-out")) {
				user->user_status = CEI_STATUS_DIALINGOUT;
			}
			else if (strstr(content, "disconnecting")) {
				user->user_status = CEI_STATUS_DISCONNECTING;
			}
		}else if (!strcmp(endpoint_node_name, "muted")) {
			if (!strcmp(content, "true")) {
				user->muted = 1;
			} else {
				user->muted = 0;
			}
		} else if (!strcmp(endpoint_node_name, "type")) {
			user->term_type = cei_terminate_type_by_string(content);
		} else if (!strcmp(endpoint_node_name, "talk-status")) {
			user->talking = !strcmp(content, "talking") ? 1 : 0;
		}

		ast_xml_free_text(content);
	}
	if (user->user_status == CEI_STATUS_UNKNOWN) {
		ast_log(LOG_DEBUG, "ZAKU DEBUG ---> USER EMPTY STATUS");
		ao2_ref(user, -1);
		return NULL;
	}
	return user;

}


static void delete_remote_confs_by_publisher(struct cei_conference *host_conf,
											 const char *publisher)
{
	ast_debug(LOG_DEBUG, "[WL] Deleting remote conf by publisher: %s", publisher);

	struct cei_conference *conf = NULL;
	struct ao2_iterator iter;

	if (!host_conf || !publisher) {
		return;
	}

	ao2_lock(host_conf->remote_container);
	iter = ao2_iterator_init(host_conf->remote_container, 0);
	while ((conf = ao2_iterator_next(&iter))) {
		if (strcmp(conf->internal_publisher, publisher) == 0) {
			ao2_unlink(host_conf->remote_container, conf);
			clear_conf_users(conf);
			ao2_ref(conf->user_container, -1);
			ao2_ref(conf, -1);
		}
		ao2_ref(conf, -1);
	}

	/*Start added by zyxia for Bug 248263 ,2023.1.11*/
	if(host_conf->is_virtial_trunk_conf) {
		ast_debug(LOG_DEBUG, "[WL] clear  conf  conference_num :%s remote user size", host_conf->conference_num);
		host_conf->remote_user_size = 0;
		host_conf->remote_size = 0;
	}

	/*End added by zyxia for Bug 248263 ,2023.1.11*/

	ao2_iterator_destroy(&iter);
	ao2_unlock(host_conf->remote_container);
}

static void delete_remote_conf_by_id(struct cei_conference *host_conf,
											 const char *remote_conf_id)
{
	ast_debug(LOG_DEBUG, "[WL] Deleting remote conf by id: %s", remote_conf_id);

	struct cei_conference *conf = NULL;
	struct ao2_iterator iter;

	if (!host_conf || !remote_conf_id) {
		return;
	}

	ao2_lock(host_conf->remote_container);
	iter = ao2_iterator_init(host_conf->remote_container, 0);
	while ((conf = ao2_iterator_next(&iter))) {
		if (strcmp(conf->conference_id, remote_conf_id) == 0) {
			ao2_unlink(host_conf->remote_container, conf);
			clear_conf_users(conf);
			ao2_ref(conf->user_container, -1);
			ao2_ref(conf, -1);
		}
		ao2_ref(conf, -1);
	}
	ao2_iterator_destroy(&iter);
	ao2_unlock(host_conf->remote_container);
}


static struct cei_conference* get_conf_by_id(const char *conf_id)
{
	if (conf_id) {
		struct cei_conference *conf = NULL;

		AST_LIST_LOCK(&confs);
		AST_LIST_TRAVERSE(&confs, conf, list) {
			if (!strcmp(conf_id, conf->conference_id)) {
				ast_atomic_fetchadd_int(&conf->refcount, 1);
				break;
			}
		}
		AST_LIST_UNLOCK(&confs);
		return conf;
	}
	return NULL;
}

struct cei_conference* get_conf_by_num(const char *conf_num)
{
	if (conf_num) {
		struct cei_conference *conf = NULL;

		AST_LIST_LOCK(&confs);
		AST_LIST_TRAVERSE(&confs, conf, list) {
			if (!strcmp(conf_num, conf->conference_num)) {
				ast_atomic_fetchadd_int(&conf->refcount, 1);
				break;
			}
		}

		AST_LIST_UNLOCK(&confs);
		return conf;
	}
	return NULL;
}

static struct cei_user* get_user_by_cid_num(struct cei_conference *conf, const char *cid_num)
{
	if (conf == NULL || cid_num == NULL) {
		return NULL;
	}

	struct cei_user *user = NULL;

	ao2_lock(conf->user_container);
	struct ao2_iterator user_iter = ao2_iterator_init(conf->user_container, 0);
	while ((user = ao2_iterator_next(&user_iter))) {
		if (strcmp(cid_num,user->cid_number) == 0){
			break;
		}
		ao2_ref(user, -1);
	}
	ao2_iterator_destroy(&user_iter);
	ao2_unlock(conf->user_container);
	if (user) {
		return user;
	}
	return NULL;
}

static struct cei_user* get_user_by_user_num(struct cei_conference *conf, int user_num)
{
	if (conf == NULL) {
		return NULL;
	}

	struct cei_user *user = NULL;

	ao2_lock(conf->user_container);
	struct ao2_iterator user_iter = ao2_iterator_init(conf->user_container, 0);
	while ((user = ao2_iterator_next(&user_iter))) {
		if (user->user_num == user_num) {
			break;
		}
		ao2_ref(user, -1);
	}
	ao2_iterator_destroy(&user_iter);
	ao2_unlock(conf->user_container);
	if (user) {
		return user;
	}
	return NULL;
}

/*! begin: Added by pyzhong@grandstream.cn for bug[63462], 2016-10-24*/
static struct cei_user* get_user_by_channel_name(struct cei_conference *conf, const char *channel_name, int strict_cmp)
{
	if (conf == NULL || channel_name == NULL) {
		return NULL;
	}

	struct cei_user *user = NULL;

	ao2_lock(conf->user_container);
	struct ao2_iterator user_iter = ao2_iterator_init(conf->user_container, 0);
	while ((user = ao2_iterator_next(&user_iter))) {
		if (strict_cmp) {
			if (strcmp(user->channel_name, channel_name) == 0) {
				break;
			}
		} else {
			if (ast_sip_cei_same_channel(user->channel_name, channel_name, user->cid_number) == 0) {
				break;
			}
		}
		ao2_ref(user, -1);
	}
	ao2_iterator_destroy(&user_iter);
	ao2_unlock(conf->user_container);
	if (user) {
		return user;
	}
	return NULL;
}
/*! end: Added by pyzhong@grandstream.cn for bug[63462], 2016-10-24*/

static int remote_conf_cei_set_sharing_user(const char *cid_name, struct cei_conference *remote_conference)
{
	RAII_VAR(struct cei_user *, user, NULL, ao2_cleanup);

	if (!cid_name || !remote_conference) {
		return -1;
	}

	user = get_user_by_cid_num(remote_conference, cid_name);
	if (!user) {
		/*-1 for using get_conf_by_num or build_conf_by_num*/
		ast_atomic_fetchadd_int(&remote_conference->refcount, -1);
		ast_debug(LOG_DEBUG, "remote_conf_cei_set_sharing_user leave refcount %d", remote_conference->refcount);
		return -1;
	}

	ao2_lock(remote_conference);
	if (remote_conference->sharing_user == user) {
		ast_log(LOG_DEBUG, "Sharing user '%s' is the same as before in the cei conference '%s', don't need update.\n",
			remote_conference->sharing_user->cid_name, remote_conference->conference_num);
		user->sharing = 1;
		ast_atomic_fetchadd_int(&remote_conference->refcount, -1);
		ao2_unlock(remote_conference);
		return 0;
	}
	if (remote_conference->sharing_user) {
		ast_log(LOG_WARNING, "Sharing user '%s' exists in the cei conference '%s', remove it first.\n",
			remote_conference->sharing_user->cid_name, remote_conference->conference_num);
		remote_conference->sharing_user->sharing = 0;
		ao2_ref(remote_conference->sharing_user, -1);
		remote_conference->sharing_user = NULL;
	}
	user->sharing = 1;
	remote_conference->sharing_user = ao2_bump(user);
	/*-1 for using get_conf_by_num or build_conf_by_num*/
	ast_atomic_fetchadd_int(&remote_conference->refcount, -1);
	ast_debug(LOG_DEBUG, "remote_conf_cei_set_sharing_user leave refcount %d", remote_conference->refcount);
	ao2_unlock(remote_conference);

	return 0;
}

// update struct for conf
static void process_remote_string(struct cei_conference *conf,
		const char *publisher_num, const char *publisher_id,
		char *content, struct ast_str **data_string)
{
	ast_log(LOG_DEBUG, "[TK] Enter...%s\n", __FUNCTION__);

	enum cei_notify_type notifyType = CEI_NOTIFY_UNKNOWN;
	int total_count = 0;
	int index = 0;
	char *saveptr = NULL;
	char *notifyPointer = NULL;
	char *local_count = NULL;
	char *remote_count = NULL;
	char *local_user_count = NULL;
	char *remote_user_count = NULL;
	char *total_update_count = NULL;
	char *conf_locked = NULL;
	char *subject = NULL;
	char *free_text = NULL;
	char *conf_start = NULL;
	char *conf_max_user = NULL;
	char *conference_type = NULL;
	char *conf_user_count = NULL;
	char *user_number = NULL;
	char *cid_number = NULL;
	char *cid_name = NULL;
	char *channel_name = NULL;
	char *conference_id = NULL;
	char *self_conference_id = NULL;
	char *user_role = NULL;
	char *entity_status = NULL;
	struct cei_conference *remote_conference = NULL;

	int flag_ignored_conf = FALSE;

	char *muted     = NULL;
	char *mic_muted = NULL;
	char *is_video  = NULL;
	char *transfer_host = NULL;
	char *talking   = NULL;
	char *term_type = NULL;
	char *extension = NULL;
	char *conf_muted = NULL;
	char *feature_id = NULL;
	char *conf_config = NULL;
	char *record_type = NULL;
	char *record_start_timestamp_utc = NULL;
	char *conf_layout = NULL;
	char *allow_user_unmute = NULL;
	char *act_event_type = NULL;
	char *conf_creator = NULL;

	char *sharing_user = NULL;
	char *web_page = NULL;
	char *is_focus  = NULL;
	char *hand_up_time  = NULL;
	char *video_mute  = NULL;
	char *video_capable  = NULL;
	char *video_code_name  = NULL;
	char *video_resolution  = NULL;
	char *video_frame_rate  = NULL;
	char *user_msid  = NULL;
	char *user_sharing  = NULL;
	char *user_related_exten  = NULL;

	// Sanatity check
	if (content == NULL || conf == NULL || publisher_num == NULL || publisher_id == NULL) {
		ast_log(LOG_WARNING, "[TK] Invalid CEI data");
		return;
	}

	// Notify type
	notifyPointer = strtok_r(content, "|", &saveptr);
	if (notifyPointer == NULL) {
		ast_log(LOG_DEBUG, "[TK] Invalid notify type\n");
		return;
	}
	notifyType = atoi(notifyPointer);
	ast_log(LOG_DEBUG, "[TK] notifyType: %d\n", notifyType);
	ast_log(LOG_DEBUG, "[TK] content: %s\n", content);

	local_count = strtok_r(NULL, "|", &saveptr);
	remote_count = strtok_r(NULL, "|", &saveptr);
	local_user_count = strtok_r(NULL, "|", &saveptr);
	remote_user_count = strtok_r(NULL, "|", &saveptr);
	total_update_count = strtok_r(NULL, "|", &saveptr);
	conf_locked = strtok_r(NULL, "|", &saveptr);
	record_type = strtok_r(NULL, "|", &saveptr);
	record_start_timestamp_utc = strtok_r(NULL, "|", &saveptr);
	subject = strtok_r(NULL, "|", &saveptr);
	free_text = strtok_r(NULL, "|", &saveptr);
	conf_start = strtok_r(NULL, "|", &saveptr);
	conf_max_user = strtok_r(NULL, "|", &saveptr);
	feature_id = strtok_r(NULL, "|", &saveptr);
	conf_config = strtok_r(NULL, "|", &saveptr);
	conf_layout = strtok_r(NULL, "|", &saveptr);
	allow_user_unmute = strtok_r(NULL, "|", &saveptr);
	act_event_type = strtok_r(NULL, "|", &saveptr);
	conf_creator = strtok_r(NULL, "|", &saveptr);

	// Sanity check
	if (local_count == NULL || remote_count == NULL ||
		local_user_count == NULL || remote_user_count == NULL ||
		total_update_count == NULL || conf_locked == NULL ||
		subject == NULL || free_text == NULL ||
		conf_start == NULL || conf_max_user == NULL) {
		ast_log(LOG_WARNING, "[TK] Invalid CEI conference header data");
		return;
	}

	switch(notifyType) {
		case CEI_NOTIFY_FULL:
		case CEI_NOTIFY_PARTIAl:
			conf_muted = strtok_r(NULL, "|", &saveptr);
			break;
		case CEI_NOTIFY_FULL_EXTRA:
		case CEI_NOTIFY_PARTIAl_EXTRA:
			sharing_user = strtok_r(NULL, "|", &saveptr);
			web_page = strtok_r(NULL, "|", &saveptr);
			conf_muted = strtok_r(NULL, "|", &saveptr);
			break;
		default:
			ast_debug(LOG_DEBUG, "[WL] INVALID TYPE");
	}
	switch(notifyType) {
		case CEI_NOTIFY_FULL:
		{
			// Calculate user count
			total_count = atoi(local_user_count) + atoi(remote_user_count);

			// Reinitialize published conferences
			delete_remote_confs_by_publisher(conf, publisher_id);

			ao2_lock(conf->remote_container);
			/***
			get or create the remote conf from host conf's remote container;
			the remote conf is basically this cei msg come FROM
			***/
			remote_conference = build_remote_conf(conf, publisher_num, publisher_id, publisher_id);
			ao2_unlock(conf->remote_container);

			if (remote_conference) {
				ao2_lock(remote_conference->user_container);
				clear_conf_users(remote_conference);
				remote_conference->remote_user_size = total_count;
				remote_conference->locked = atoi(conf_locked) ? TRUE : FALSE;
				remote_conference->muted = atoi(conf_muted) ? TRUE : FALSE;
				if (!remote_conference->feature_id && strcasecmp(feature_id, "NULL")) {
					remote_conference->feature_id = ast_strdup(feature_id);
				}
				if (!remote_conference->conf_config && strcasecmp(conf_config, "NULL")) {
					remote_conference->conf_config = ast_strdup(conf_config);
				}
				if (!ast_strlen_zero(record_type)) {
					strncpy(remote_conference->record_type, record_type, sizeof(remote_conference->record_type));
				}
				if (!ast_strlen_zero(record_start_timestamp_utc)) {
					remote_conference->record_start_utc_timestamp = atoi(record_start_timestamp_utc);
				}
				if (!ast_strlen_zero(conf_layout)) {
					strncpy(remote_conference->conf_layout, conf_layout, sizeof(remote_conference->conf_layout));
				}
				if (!ast_strlen_zero(act_event_type)) {
					conf->act_event_type = atoi(act_event_type);
				}
				if (!remote_conference->conf_creator && !ast_strlen_zero(conf_creator) && strcasecmp(feature_id, "--")) {
					remote_conference->conf_creator = ast_strdup(conf_creator);
				}
				if (!ast_strlen_zero(allow_user_unmute) && ast_true(allow_user_unmute)) {
					conf->allow_user_unmute = 1;
				}

				// Append users for remote publisher
				for (index = 0; index < atoi(total_update_count); index++) {
					conference_type = strtok_r(NULL, "\t", &saveptr);
					conf_user_count = strtok_r(NULL, "\t", &saveptr);
					user_number = strtok_r(NULL, "\t", &saveptr);
					cid_number = strtok_r(NULL, "\t", &saveptr);
					/*!begin Added by pyzhong@grandstream.cn for bug[68670], 2017-9-23*/
					if (saveptr[0] != '\t') {
						cid_name = strtok_r(NULL, "\t", &saveptr);
					} else {
						cid_name = "";
					}
					/*!end Added by pyzhong@grandstream.cn for bug[68670], 2017-9-23*/
					if (!ast_strlen_zero(conference_type) && atoi(conference_type) == CEI_CONFERENCE_LOCAL) {
						channel_name = strtok_r(NULL, "\t", &saveptr);
						conference_id = strtok_r(NULL, "\t", &saveptr);
						self_conference_id = strtok_r(NULL, "\t", &saveptr);
						user_role = strtok_r(NULL, "\t", &saveptr);
						entity_status = strtok_r(NULL, "\t", &saveptr);
						muted = strtok_r(NULL, "\t", &saveptr);
						mic_muted = strtok_r(NULL, "\t", &saveptr);
						is_video = strtok_r(NULL, "\t", &saveptr);
						transfer_host = strtok_r(NULL, "\t", &saveptr);
						talking = strtok_r(NULL, "\t", &saveptr);
						term_type = strtok_r(NULL, "\t", &saveptr);
						extension = strtok_r(NULL, "|", &saveptr);

						if (conference_type == NULL || user_number == NULL || conf_user_count == NULL ||
							cid_number == NULL || cid_name == NULL || channel_name == NULL ||
							conference_id == NULL || user_role == NULL ||
							entity_status == NULL || self_conference_id == NULL || transfer_host == NULL) {
							ast_log(LOG_WARNING, "[TK] Invalid CEI conference data");
							continue;
						}
					} else {
						channel_name = strtok_r(NULL, "\t", &saveptr);
						conference_id = strtok_r(NULL, "\t", &saveptr);
						self_conference_id = strtok_r(NULL, "\t", &saveptr);
						user_role = strtok_r(NULL, "\t", &saveptr);
						entity_status = strtok_r(NULL, "|", &saveptr);

						if (conference_type == NULL || user_number == NULL || conf_user_count == NULL ||
							cid_number == NULL || cid_name == NULL || channel_name == NULL ||
							conference_id == NULL || user_role == NULL ||
							entity_status == NULL || self_conference_id == NULL) {
							ast_log(LOG_WARNING, "[TK] Invalid CEI conference data");
							continue;
						}
					}

					switch (atoi(conference_type)) {
						case CEI_CONFERENCE_LOCAL:
						{
							if (flag_ignored_conf == TRUE) {
								ast_debug(LOG_DEBUG, "[WL] User of ignored remote conf, then ignor this user");
								continue;
							}
							struct cei_user *user;
							if (!(user = ao2_alloc(sizeof(struct cei_user), NULL))) {
								ast_debug(LOG_DEBUG, "ZAKU DEBUG ---> CANNOT CREATE NEW USER");
								continue;
							}

							//add info to cei_user
							user->user_num = atoi(user_number);
							ast_copy_string(user->cid_name, cid_name, sizeof(user->cid_name));
							ast_copy_string(user->cid_number, cid_number, sizeof(user->cid_number));
							ast_copy_string(user->channel_name, channel_name, sizeof(user->channel_name));
							ast_copy_string(user->conference_id, remote_conference->conference_id, sizeof(user->conference_id));
							hash_user(user);
							user->muted = (atoi(entity_status) == CEI_STATUS_MUTEVIAFOCUS) ? 1 : 0;
							user->user_role = atoi(user_role);
							user->admin = 0;
							user->can_invite = 0;
							if (atoi(user_role) == CEI_ROLE_ADMIN) {
								user->admin = 1;
							} else if (atoi(user_role) == CEI_ROLE_USER_INVITE) {
								user->can_invite = 1;
							}
							user->media.type = ((is_video && atoi(is_video)) ? AST_MEDIA_TYPE_VIDEO : AST_MEDIA_TYPE_AUDIO);
							user->talking = (talking && atoi(talking) ? TRUE : FALSE);
							user->term_type = cei_terminate_type_by_string(term_type);
							if (!ast_strlen_zero(extension) && strcasecmp(extension, "NULL")) {
								ast_copy_string(user->extension, extension, sizeof(user->extension));
							}
							user->muted = atoi(muted) ? TRUE : FALSE;
							user->mic_muted = atoi(mic_muted) ? TRUE : FALSE;

							if (atoi(entity_status) == CEI_STATUS_UNKNOWN) {
								ast_log(LOG_DEBUG, "ZAKU DEBUG ---> USER EMPTY STATUS");
								ao2_ref(user, -1);
								continue;
							}
							ao2_link(remote_conference->user_container, user);
							ast_atomic_fetchadd_int(&remote_conference->local_size, 1);
							ast_atomic_fetchadd_int(&remote_conference->local_user_size, (!check_role_as_conference(user))? 1 : 0);
							ast_atomic_fetchadd_int(&conf->remote_size, 1);
							ast_atomic_fetchadd_int(&conf->remote_user_size, (!check_role_as_conference(user))? 1 : 0);
							ast_atomic_fetchadd_int(&remote_conference->refcount, 1);
						}
						break;
						case CEI_CONFERENCE_REMOTE:
						{
							// Ignore invalid conference
							// Don't add myself as my remote conf
							if (strcmp(conf->conference_id, conference_id) == 0) {
								ast_debug(LOG_DEBUG, "[TK] Ignore my own conference");
								ast_debug(LOG_DEBUG, "[TK] %s@%s", cid_number, conference_id);
								flag_ignored_conf = TRUE;
								continue;
							}
							// Don't add the remote conf which are my own remote conf again
							if (get_remote_conf_by_id(conf, conference_id) != NULL) {
								ast_debug(LOG_DEBUG, "[TK] Ignore my own remote conference");
								ast_debug(LOG_DEBUG, "[TK] %s@%s", cid_number, conference_id);
								flag_ignored_conf = TRUE;
								continue;
							}

							ao2_unlock(remote_conference->user_container);
							/***
							get or create the remote conf from host conf's remote container;
							the remote conf is basically this cei msg come FROM
							***/
							remote_conference = build_remote_conf(conf, cid_number, conference_id, publisher_id);

							ao2_lock(remote_conference->user_container);
							clear_conf_users(remote_conference);

							flag_ignored_conf = FALSE;
						}
						break;
						default:
						{
							ast_debug(LOG_DEBUG, "[WL] INVALID TYPE");
						}
					}
				}
				ao2_unlock(remote_conference->user_container);
			}
			poll_cei_conference(conf, data_string);
		}
		break;
		case CEI_NOTIFY_PARTIAl:
		{
			// Calculate user count
			total_count = atoi(local_user_count) + atoi(remote_user_count);

			remote_conference = get_remote_conf_by_id(conf, publisher_id);

			if (remote_conference) {

				// Append users for remote publisher
				for (index = 0; index < atoi(total_update_count); index++) {
					conference_type = strtok_r(NULL, "\t", &saveptr);
					conf_user_count = strtok_r(NULL, "\t", &saveptr);
					user_number = strtok_r(NULL, "\t", &saveptr);
					cid_number = strtok_r(NULL, "\t", &saveptr);
					/*!begin Added by pyzhong@grandstream.cn for bug[68670], 2017-9-23*/
					if (saveptr[0] != '\t') {
						cid_name = strtok_r(NULL, "\t", &saveptr);
					} else {
						cid_name = "";
					}
					/*!end Added by pyzhong@grandstream.cn for bug[68670], 2017-9-23*/
					if (!ast_strlen_zero(conference_type) && atoi(conference_type) == CEI_CONFERENCE_LOCAL) {
						channel_name = strtok_r(NULL, "\t", &saveptr);
						conference_id = strtok_r(NULL, "\t", &saveptr);
						self_conference_id = strtok_r(NULL, "\t", &saveptr);
						user_role = strtok_r(NULL, "\t", &saveptr);
						entity_status = strtok_r(NULL, "\t", &saveptr);
						muted = strtok_r(NULL, "\t", &saveptr);
						mic_muted = strtok_r(NULL, "\t", &saveptr);
						is_video = strtok_r(NULL, "\t", &saveptr);
						transfer_host = strtok_r(NULL, "\t", &saveptr);
						talking = strtok_r(NULL, "\t", &saveptr);
						term_type = strtok_r(NULL, "\t", &saveptr);
						extension = strtok_r(NULL, "|", &saveptr);

						if (conference_type == NULL || user_number == NULL || conf_user_count == NULL ||
							cid_number == NULL || cid_name == NULL || channel_name == NULL ||
							conference_id == NULL || user_role == NULL ||
							entity_status == NULL || self_conference_id == NULL || transfer_host == NULL) {
							ast_log(LOG_WARNING, "[TK] Invalid CEI conference data");
							continue;
						}
					}else {
						channel_name = strtok_r(NULL, "\t", &saveptr);
						conference_id = strtok_r(NULL, "\t", &saveptr);
						self_conference_id = strtok_r(NULL, "\t", &saveptr);
						user_role = strtok_r(NULL, "\t", &saveptr);
						entity_status = strtok_r(NULL, "|", &saveptr);

						if (conference_type == NULL || user_number == NULL || conf_user_count == NULL ||
							cid_number == NULL || cid_name == NULL || channel_name == NULL ||
							conference_id == NULL || user_role == NULL ||
							entity_status == NULL || self_conference_id == NULL) {
							ast_log(LOG_WARNING, "[TK] Invalid CEI conference data");
							continue;
						}
					}
					switch (atoi(conference_type)) {
						case CEI_CONFERENCE_LOCAL:
						{
							if (flag_ignored_conf == TRUE) {
								ast_debug(LOG_DEBUG, "[WL] User of ignored remote conf, then ignor this user");
								continue;
							}
							struct cei_user *user;

							switch (atoi(entity_status)) {
								// CEI_EVENT_MCB_USER_JOIN
								case CEI_STATUS_CONNECTED:
								{
									ao2_lock(remote_conference->user_container);
									user = get_user_by_cid_num(remote_conference, cid_number);
									//CEI_EVENT_MCB_USER_TOGGLE_MUTE unmute
									if (user) {
										user->muted = 0;
										user->talking = (talking && atoi(talking) ? TRUE : FALSE);
										poll_cei_partial_notify(conf, remote_conference, user, data_string, CEI_EVENT_MCB_USER_TOGGLE_MUTE);
										ao2_ref(user, -1);
									}
									//CEI_EVENT_MCB_USER_JOIN
									else {
										if (!(user = ao2_alloc(sizeof(struct cei_user), NULL))) {
											ast_debug(LOG_DEBUG, "ZAKU DEBUG ---> CANNOT CREATE NEW USER");
											ao2_unlock(remote_conference->user_container);
											continue;
										}

										//add info to cei_user
										user->user_num = atoi(user_number);
										ast_copy_string(user->cid_name, cid_name, sizeof(user->cid_name));
										ast_copy_string(user->cid_number, cid_number, sizeof(user->cid_number));
										ast_copy_string(user->channel_name, channel_name, sizeof(user->channel_name));
										ast_copy_string(user->conference_id, remote_conference->conference_id, sizeof(user->conference_id));
										hash_user(user);
										user->muted = (atoi(entity_status) == CEI_STATUS_MUTEVIAFOCUS) ? 1 : 0;
										user->user_role = atoi(user_role);
										user->admin = 0;
										user->can_invite = 0;
										if (atoi(user_role) == CEI_ROLE_ADMIN) {
											user->admin = 1;
										} else if (atoi(user_role) == CEI_ROLE_USER_INVITE) {
											user->can_invite = 1;
										}
										user->media.type = ((is_video && atoi(is_video)) ? AST_MEDIA_TYPE_VIDEO : AST_MEDIA_TYPE_AUDIO);
										user->talking = (talking && atoi(talking) ? TRUE : FALSE);
										user->term_type = cei_terminate_type_by_string(term_type);
										if (!ast_strlen_zero(extension) && strcasecmp(extension, "NULL")) {
											ast_copy_string(user->extension, extension, sizeof(user->extension));
										}
										user->muted = atoi(muted) ? TRUE : FALSE;

										ao2_link(remote_conference->user_container, user);
										ast_atomic_fetchadd_int(&remote_conference->local_size, 1);
										ast_atomic_fetchadd_int(&remote_conference->local_user_size, (!check_role_as_conference(user))? 1 : 0);
										ast_atomic_fetchadd_int(&conf->remote_size, 1);
										ast_atomic_fetchadd_int(&conf->remote_user_size, (!check_role_as_conference(user))? 1 : 0);
										ast_atomic_fetchadd_int(&remote_conference->refcount, 1);

										poll_cei_partial_notify(conf, remote_conference, user, data_string, CEI_EVENT_MCB_USER_JOIN);
									}
									ao2_unlock(remote_conference->user_container);
								}
								break;
								//CEI_EVENT_MCB_USER_PART
								case CEI_STATUS_DISCONNECTED:
								{
									ao2_lock(remote_conference->user_container);

									user = get_user_by_cid_num(remote_conference, cid_number);

									if (user) {

										ast_atomic_fetchadd_int(&remote_conference->local_size, -1);
										ast_atomic_fetchadd_int(&remote_conference->local_user_size, (!check_role_as_conference(user))? -1 : 0);
										ast_atomic_fetchadd_int(&conf->remote_size, -1);
										ast_atomic_fetchadd_int(&conf->remote_user_size, (!check_role_as_conference(user))? -1 : 0);

										poll_cei_partial_notify(conf, remote_conference, user, data_string, CEI_EVENT_MCB_USER_PART);

										ao2_unlink(remote_conference->user_container, user);
										ast_atomic_fetchadd_int(&remote_conference->refcount, -1);

										ast_debug(LOG_DEBUG, "[WL] REMOVED USER WITH ADDRESS %p", user);
										/*refcount -2 for ao2_alloc and get_user_by_cid_num in remove case*/
										ao2_ref(user, -2);
									}
									else {
										ast_debug(LOG_DEBUG, "[WL] ERROR, CANNOT FIND USER  %s IN CONFERENCE %s", cid_number, remote_conference->conference_num);
									}
									ao2_unlock(remote_conference->user_container);
								}
								break;
								case CEI_STATUS_MUTEVIAFOCUS:
								{
									ao2_lock(remote_conference->user_container);

									user = get_user_by_cid_num(remote_conference, cid_number);

									if (user) {
										user->muted = 1;
										poll_cei_partial_notify(conf, remote_conference, user, data_string, CEI_EVENT_MCB_USER_TOGGLE_MUTE);
										ao2_ref(user, -1);
									}

									ao2_unlock(remote_conference->user_container);
								}
								break;
								default:
								{
									ast_debug(LOG_DEBUG, "[WL] INVALID STATUS");
								}
								break;
							}
						}
						break;
						case CEI_CONFERENCE_REMOTE:
						{
							// Ignore invalid conference
							// Don't add myself as my remote conf
							if (strcmp(conf->conference_id, conference_id) == 0) {
								ast_debug(LOG_DEBUG, "[TK] Ignore my own conference");
								ast_debug(LOG_DEBUG, "[TK] %s@%s", cid_number, conference_id);
								flag_ignored_conf = TRUE;
								continue;
							}

							/***
							get or create the remote conf from host conf's remote container;
							the remote conf is basically this cei msg come FROM
							***/

							remote_conference = get_remote_conf_by_id(conf, conference_id);
							if (atoi(entity_status) == CEI_STATUS_CONNECTED) {
								if (remote_conference != NULL) {
									ast_debug(LOG_DEBUG, "[TK] CEI_EVENT_MCB_USER_* event");
									continue;
								}

							} else if (atoi(entity_status) == CEI_STATUS_DISCONNECTED) {
								// CEI_EVENT_MCB_CONF_PART
								if (remote_conference != NULL) {
									delete_remote_conf_by_id(conf, publisher_id);
									poll_cei_conference(conf, data_string);
								}
							}
						}
						break;
						default:
						{
							ast_debug(LOG_DEBUG, "[WL] INVALID TYPE");
						}
					}
				}
			}
		}
		break;
		/* For video conference*/
		case CEI_NOTIFY_FULL_EXTRA:
		{
			// Calculate user count
			total_count = atoi(local_user_count) + atoi(remote_user_count);

			// Reinitialize published conferences
			delete_remote_confs_by_publisher(conf, publisher_id);

			ao2_lock(conf->remote_container);
			/***
			get or create the remote conf from host conf's remote container;
			the remote conf is basically this cei msg come FROM
			***/
			remote_conference = build_remote_conf(conf, publisher_num, publisher_id, publisher_id);
			ao2_unlock(conf->remote_container);

			if (remote_conference) {
				ao2_lock(remote_conference->user_container);
				clear_conf_users(remote_conference);
				remote_conference->remote_user_size = total_count;
				remote_conference->locked = atoi(conf_locked) ? TRUE : FALSE;
				remote_conference->muted = atoi(conf_muted) ? TRUE : FALSE;
				if (!remote_conference->feature_id && strcasecmp(feature_id, "NULL")) {
					remote_conference->feature_id = ast_strdup(feature_id);
				}
				if (!remote_conference->conf_config && strcasecmp(conf_config, "NULL")) {
					remote_conference->conf_config = ast_strdup(conf_config);
				}
				if (!ast_strlen_zero(record_type)) {
					strncpy(remote_conference->record_type, record_type, sizeof(remote_conference->record_type));
				}
				if (!ast_strlen_zero(record_start_timestamp_utc)) {
					remote_conference->record_start_utc_timestamp = atoi(record_start_timestamp_utc);
				}
				if (!ast_strlen_zero(conf_layout)) {
					strncpy(remote_conference->conf_layout, conf_layout, sizeof(remote_conference->conf_layout));
				}
				if (!ast_strlen_zero(act_event_type)) {
					remote_conference->act_event_type = atoi(act_event_type);
				}
				if (!remote_conference->conf_creator && !ast_strlen_zero(conf_creator) && strcasecmp(feature_id, "--")) {
					remote_conference->conf_creator = ast_strdup(conf_creator);
				}
				if (!ast_strlen_zero(allow_user_unmute) && ast_true(allow_user_unmute)) {
					remote_conference->allow_user_unmute = 1;
				}
				remote_conf_cei_set_sharing_user(sharing_user, remote_conference);
				if (!remote_conference->web_page && strcasecmp(web_page, "NULL")) {
					remote_conference->web_page = ast_strdup(web_page);
				}
				remote_conference->video_conference = 1;

				// Append users for remote publisher
				for (index = 0; index < atoi(total_update_count); index++) {
					conference_type = strtok_r(NULL, "\t", &saveptr);
					conf_user_count = strtok_r(NULL, "\t", &saveptr);
					user_number = strtok_r(NULL, "\t", &saveptr);
					cid_number = strtok_r(NULL, "\t", &saveptr);
					/*!begin Added by pyzhong@grandstream.cn for bug[68670], 2017-9-23*/
					if (saveptr[0] != '\t') {
						cid_name = strtok_r(NULL, "\t", &saveptr);
					} else {
						cid_name = "";
					}
					/*!end Added by pyzhong@grandstream.cn for bug[68670], 2017-9-23*/
					if (!ast_strlen_zero(conference_type) && atoi(conference_type) == CEI_CONFERENCE_LOCAL) {
						channel_name = strtok_r(NULL, "\t", &saveptr);
						conference_id = strtok_r(NULL, "\t", &saveptr);
						self_conference_id = strtok_r(NULL, "\t", &saveptr);
						user_role = strtok_r(NULL, "\t", &saveptr);
						entity_status = strtok_r(NULL, "\t", &saveptr);

						muted = strtok_r(NULL, "\t", &saveptr);
						mic_muted = strtok_r(NULL, "\t", &saveptr);
						is_focus= strtok_r(NULL, "\t", &saveptr);
						hand_up_time= strtok_r(NULL, "\t", &saveptr);
						video_mute= strtok_r(NULL, "\t", &saveptr);
						video_capable= strtok_r(NULL, "\t", &saveptr);
						is_video = strtok_r(NULL, "\t", &saveptr);
						strtok_r(NULL, "\t", &saveptr);
						video_code_name = strtok_r(NULL, "\t", &saveptr);
						video_resolution = strtok_r(NULL, "\t", &saveptr);
						video_frame_rate = strtok_r(NULL, "\t", &saveptr);
						transfer_host = strtok_r(NULL, "\t", &saveptr);
						talking = strtok_r(NULL, "\t", &saveptr);
						term_type = strtok_r(NULL, "\t", &saveptr);
						extension = strtok_r(NULL, "\t", &saveptr);
						user_msid = strtok_r(NULL, "\t", &saveptr);
						user_sharing = strtok_r(NULL, "\t", &saveptr);
						user_related_exten = strtok_r(NULL, "|", &saveptr);
					} else {
						channel_name = strtok_r(NULL, "\t", &saveptr);
						conference_id = strtok_r(NULL, "\t", &saveptr);
						self_conference_id = strtok_r(NULL, "\t", &saveptr);
						user_role = strtok_r(NULL, "\t", &saveptr);
						entity_status = strtok_r(NULL, "|", &saveptr);
					}
					if (conference_type == NULL || user_number == NULL || conf_user_count == NULL ||
						cid_number == NULL || cid_name == NULL || channel_name == NULL ||
						conference_id == NULL || user_role == NULL ||
						entity_status == NULL || self_conference_id == NULL) {
						ast_log(LOG_WARNING, "[TK] Invalid CEI conference data");
						continue;
					}

					switch (atoi(conference_type)) {
						case CEI_CONFERENCE_LOCAL:
						{
							if (flag_ignored_conf == TRUE) {
								ast_debug(LOG_DEBUG, "[WL] User of ignored remote conf, then ignor this user");
								continue;
							}
							struct cei_user *user;
							if (!(user = ao2_alloc(sizeof(struct cei_user), NULL))) {
								ast_debug(LOG_DEBUG, "ZAKU DEBUG ---> CANNOT CREATE NEW USER");
								continue;
							}
							if (atoi(entity_status) == CEI_STATUS_UNKNOWN) {
								ast_log(LOG_DEBUG, "ZAKU DEBUG ---> USER EMPTY STATUS");
								ao2_ref(user, -1);
								continue;
							}

							//add info to cei_user
							user->user_num = atoi(user_number);
							ast_copy_string(user->cid_name, cid_name, sizeof(user->cid_name));
							ast_copy_string(user->cid_number, cid_number, sizeof(user->cid_number));
							ast_copy_string(user->channel_name, channel_name, sizeof(user->channel_name));
							ast_copy_string(user->conference_id, remote_conference->conference_id, sizeof(user->conference_id));
							hash_user(user);
							user->muted = (atoi(entity_status) == CEI_STATUS_MUTEVIAFOCUS) ? 1 : 0;
							user->user_role = atoi(user_role);
							user->admin = 0;
							user->can_invite = 0;
							if (atoi(user_role) == CEI_ROLE_ADMIN) {
								user->admin = 1;
							} else if (atoi(user_role) == CEI_ROLE_USER_INVITE) {
								user->can_invite = 1;
							}

							user->muted = (muted && atoi(muted) ? TRUE : FALSE);
							user->mic_muted = (mic_muted && atoi(mic_muted) ? TRUE : FALSE);
							user->isfocus = (is_focus && atoi(is_focus) ? TRUE : FALSE);
							user->hand_up_time = hand_up_time ? atoi(hand_up_time) : 0;
							user->media.video_muted = (video_mute && atoi(video_mute) ? TRUE : FALSE);
							user->media.video_capable = video_capable ? atoi(video_capable) : 0;
							user->media.type = ((is_video && atoi(is_video)) ? AST_MEDIA_TYPE_VIDEO : AST_MEDIA_TYPE_AUDIO);
							ast_copy_string(user->media.video_info.code_name, video_code_name ? video_code_name : "NULL", sizeof(user->media.video_info.code_name));
							user->media.video_info.resolution = video_resolution_name_to_type(video_resolution);
							user->media.video_info.frame_rate = video_frame_rate ? atoi(video_frame_rate) : 0;
							user->talking = (talking && atoi(talking) ? TRUE : FALSE);
							user->term_type = cei_terminate_type_by_string(term_type);
							if (!ast_strlen_zero(extension) && strcasecmp(extension, "NULL")) {
								ast_copy_string(user->extension, extension, sizeof(user->extension));
							}
							ast_copy_string(user->msid, user_msid ? user_msid : "NULL", sizeof(user->msid));
							user->sharing = (user_sharing && atoi(user_sharing) ? TRUE : FALSE);
							ast_copy_string(user->related_ext, user_related_exten ? user_related_exten : "NULL", sizeof(user->related_ext));

							ao2_link(remote_conference->user_container, user);
							ast_atomic_fetchadd_int(&remote_conference->local_size, 1);
							ast_atomic_fetchadd_int(&remote_conference->local_user_size, (!check_role_as_conference(user))? 1 : 0);
							ast_atomic_fetchadd_int(&conf->remote_size, 1);
							ast_atomic_fetchadd_int(&conf->remote_user_size, (!check_role_as_conference(user))? 1 : 0);
							ast_atomic_fetchadd_int(&remote_conference->refcount, 1);
						}
						break;
						case CEI_CONFERENCE_REMOTE:
						{
							// Ignore invalid conference
							// Don't add myself as my remote conf
							if (strcmp(conf->conference_id, conference_id) == 0) {
								ast_debug(LOG_DEBUG, "[TK] Ignore my own conference");
								ast_debug(LOG_DEBUG, "[TK] %s@%s", cid_number, conference_id);
								flag_ignored_conf = TRUE;
								continue;
							}
							// Don't add the remote conf which are my own remote conf again
							if (get_remote_conf_by_id(conf, conference_id) != NULL) {
								ast_debug(LOG_DEBUG, "[TK] Ignore my own remote conference");
								ast_debug(LOG_DEBUG, "[TK] %s@%s", cid_number, conference_id);
								flag_ignored_conf = TRUE;
								continue;
							}

							ao2_unlock(remote_conference->user_container);
							/***
							get or create the remote conf from host conf's remote container;
							the remote conf is basically this cei msg come FROM
							***/
							remote_conference = build_remote_conf(conf, cid_number, conference_id, publisher_id);

							ao2_lock(remote_conference->user_container);
							clear_conf_users(remote_conference);

							flag_ignored_conf = FALSE;
						}
						break;
						default:
						{
							ast_debug(LOG_DEBUG, "[WL] INVALID TYPE");
						}
					}
				}
				ao2_unlock(remote_conference->user_container);
			}
			poll_cei_conference(conf, data_string);
		}
		break;
		case CEI_NOTIFY_PARTIAl_EXTRA:
		{
			// Calculate user count
			total_count = atoi(local_user_count) + atoi(remote_user_count);

			remote_conference = get_remote_conf_by_id(conf, publisher_id);

			if (remote_conference) {

				// Append users for remote publisher
				for (index = 0; index < atoi(total_update_count); index++) {
					conference_type = strtok_r(NULL, "\t", &saveptr);
					conf_user_count = strtok_r(NULL, "\t", &saveptr);
					user_number = strtok_r(NULL, "\t", &saveptr);
					cid_number = strtok_r(NULL, "\t", &saveptr);
					/*!begin Added by pyzhong@grandstream.cn for bug[68670], 2017-9-23*/
					if (saveptr[0] != '\t') {
						cid_name = strtok_r(NULL, "\t", &saveptr);
					} else {
						cid_name = "";
					}
					/*!end Added by pyzhong@grandstream.cn for bug[68670], 2017-9-23*/
					if (!ast_strlen_zero(conference_type) && atoi(conference_type) == CEI_CONFERENCE_LOCAL) {
						channel_name = strtok_r(NULL, "\t", &saveptr);
						conference_id = strtok_r(NULL, "\t", &saveptr);
						self_conference_id = strtok_r(NULL, "\t", &saveptr);
						user_role = strtok_r(NULL, "\t", &saveptr);
						entity_status = strtok_r(NULL, "\t", &saveptr);

						muted = strtok_r(NULL, "\t", &saveptr);
						mic_muted = strtok_r(NULL, "\t", &saveptr);
						is_focus= strtok_r(NULL, "\t", &saveptr);
						hand_up_time= strtok_r(NULL, "\t", &saveptr);
						video_mute= strtok_r(NULL, "\t", &saveptr);
						video_capable= strtok_r(NULL, "\t", &saveptr);
						is_video = strtok_r(NULL, "\t", &saveptr);
						strtok_r(NULL, "\t", &saveptr);
						video_code_name = strtok_r(NULL, "\t", &saveptr);
						video_resolution = strtok_r(NULL, "\t", &saveptr);
						video_frame_rate = strtok_r(NULL, "\t", &saveptr);
						transfer_host = strtok_r(NULL, "\t", &saveptr);
						talking = strtok_r(NULL, "\t", &saveptr);
						term_type = strtok_r(NULL, "\t", &saveptr);
						extension = strtok_r(NULL, "\t", &saveptr);
						user_msid = strtok_r(NULL, "\t", &saveptr);
						user_sharing = strtok_r(NULL, "\t", &saveptr);
						user_related_exten = strtok_r(NULL, "|", &saveptr);
					}else {
						channel_name = strtok_r(NULL, "\t", &saveptr);
						conference_id = strtok_r(NULL, "\t", &saveptr);
						self_conference_id = strtok_r(NULL, "\t", &saveptr);
						user_role = strtok_r(NULL, "\t", &saveptr);
						entity_status = strtok_r(NULL, "|", &saveptr);
					}
					ast_debug(LOG_DEBUG, "conference_type %s user_number %s conf_user_count %s cid_number %s cid_name %s ", conference_type, user_number, conf_user_count, cid_number, cid_name);
					ast_debug(LOG_DEBUG, "channel_name %s conference_id %s user_role %s entity_status %s self_conference_id %s transfer_host %s", channel_name, conference_id, user_role, entity_status, self_conference_id, transfer_host);

					if (conference_type == NULL || user_number == NULL || conf_user_count == NULL ||
						cid_number == NULL || cid_name == NULL || channel_name == NULL ||
						conference_id == NULL || user_role == NULL ||
						entity_status == NULL || self_conference_id == NULL || transfer_host == NULL) {
						ast_log(LOG_WARNING, "[TK] Invalid CEI conference data");
						continue;
					}
					switch (atoi(conference_type)) {
						case CEI_CONFERENCE_LOCAL:
						{
							if (flag_ignored_conf == TRUE) {
								ast_debug(LOG_DEBUG, "[WL] User of ignored remote conf, then ignor this user");
								continue;
							}
							struct cei_user *user;

							switch (atoi(entity_status)) {
								// CEI_EVENT_MCB_USER_JOIN
								case CEI_STATUS_CONNECTED:
								{
									ao2_lock(remote_conference->user_container);
									user = get_user_by_cid_num(remote_conference, cid_number);
									//CEI_EVENT_MCB_USER_TOGGLE_MUTE unmute
									if (user) {
										user->muted = 0;
										user->talking = (talking && atoi(talking) ? TRUE : FALSE);
										poll_cei_partial_notify(conf, remote_conference, user, data_string, CEI_EVENT_MCB_USER_TOGGLE_MUTE);
										ao2_ref(user, -1);
									}
									//CEI_EVENT_MCB_USER_JOIN
									else {
										if (!(user = ao2_alloc(sizeof(struct cei_user), NULL))) {
											ast_debug(LOG_DEBUG, "ZAKU DEBUG ---> CANNOT CREATE NEW USER");
											ao2_unlock(remote_conference->user_container);
											continue;
										}

										//add info to cei_user
										user->user_num = atoi(user_number);
										ast_copy_string(user->cid_name, cid_name, sizeof(user->cid_name));
										ast_copy_string(user->cid_number, cid_number, sizeof(user->cid_number));
										ast_copy_string(user->channel_name, channel_name, sizeof(user->channel_name));
										ast_copy_string(user->conference_id, remote_conference->conference_id, sizeof(user->conference_id));
										hash_user(user);
										user->muted = (atoi(entity_status) == CEI_STATUS_MUTEVIAFOCUS) ? 1 : 0;
										user->user_role = atoi(user_role);
										user->admin = 0;
										user->can_invite = 0;
										if (atoi(user_role) == CEI_ROLE_ADMIN) {
											user->admin = 1;
										} else if (atoi(user_role) == CEI_ROLE_USER_INVITE) {
											user->can_invite = 1;
										}

										user->muted = (muted && atoi(muted) ? TRUE : FALSE);
										user->mic_muted = (mic_muted && atoi(mic_muted) ? TRUE : FALSE);
										user->isfocus = (is_focus && atoi(is_focus) ? TRUE : FALSE);
										user->hand_up_time = hand_up_time ? atoi(hand_up_time) : 0;
										user->media.video_muted = (video_mute && atoi(video_mute) ? TRUE : FALSE);
										user->media.video_capable = video_capable ? atoi(video_capable) : 0;
										user->media.type = ((is_video && atoi(is_video)) ? AST_MEDIA_TYPE_VIDEO : AST_MEDIA_TYPE_AUDIO);
										ast_copy_string(user->media.video_info.code_name, video_code_name ? video_code_name : "NULL", sizeof(user->media.video_info.code_name));
										user->media.video_info.resolution = video_resolution_name_to_type(video_resolution);
										user->media.video_info.frame_rate = video_frame_rate ? atoi(video_frame_rate) : 0;
										user->talking = (talking && atoi(talking) ? TRUE : FALSE);
										user->term_type = cei_terminate_type_by_string(term_type);
										if (!ast_strlen_zero(extension) && strcasecmp(extension, "NULL")) {
											ast_copy_string(user->extension, extension, sizeof(user->extension));
										}
										ast_copy_string(user->msid, user_msid ? user_msid : "NULL", sizeof(user->msid));
										user->sharing = (user_sharing && atoi(user_sharing) ? TRUE : FALSE);
										ast_copy_string(user->related_ext, user_related_exten ? user_related_exten : "NULL", sizeof(user->related_ext));

										ao2_link(remote_conference->user_container, user);
										ast_atomic_fetchadd_int(&remote_conference->local_size, 1);
										ast_atomic_fetchadd_int(&remote_conference->local_user_size, (!check_role_as_conference(user))? 1 : 0);
										ast_atomic_fetchadd_int(&conf->remote_size, 1);
										ast_atomic_fetchadd_int(&conf->remote_user_size, (!check_role_as_conference(user))? 1 : 0);
										ast_atomic_fetchadd_int(&remote_conference->refcount, 1);

										poll_cei_partial_notify(conf, remote_conference, user, data_string, CEI_EVENT_MCB_USER_JOIN);
									}
									ao2_unlock(remote_conference->user_container);
								}
								break;
								//CEI_EVENT_MCB_USER_PART
								case CEI_STATUS_DISCONNECTED:
								{
									ao2_lock(remote_conference->user_container);

									user = get_user_by_cid_num(remote_conference, cid_number);

									if (user) {

										ast_atomic_fetchadd_int(&remote_conference->local_size, -1);
										ast_atomic_fetchadd_int(&remote_conference->local_user_size, (!check_role_as_conference(user))? -1 : 0);
										ast_atomic_fetchadd_int(&conf->remote_size, -1);
										ast_atomic_fetchadd_int(&conf->remote_user_size, (!check_role_as_conference(user))? -1 : 0);

										poll_cei_partial_notify(conf, remote_conference, user, data_string, CEI_EVENT_MCB_USER_PART);

										ao2_unlink(remote_conference->user_container, user);
										ast_atomic_fetchadd_int(&remote_conference->refcount, -1);

										ast_debug(LOG_DEBUG, "[WL] REMOVED USER WITH ADDRESS %p", user);
										/*refcount -2 for ao2_alloc and get_user_by_cid_num in remove case*/
										ao2_ref(user, -2);
									}
									else {
										ast_debug(LOG_DEBUG, "[WL] ERROR, CANNOT FIND USER	%s IN CONFERENCE %s", cid_number, remote_conference->conference_num);
									}

									ao2_unlock(remote_conference->user_container);
								}
								break;
								case CEI_STATUS_MUTEVIAFOCUS:
								{
									ao2_lock(remote_conference->user_container);

									user = get_user_by_cid_num(remote_conference, cid_number);

									if (user) {
										user->muted = 1;
										poll_cei_partial_notify(conf, remote_conference, user, data_string, CEI_EVENT_MCB_USER_TOGGLE_MUTE);
										ao2_ref(user, -1);
									}

									ao2_unlock(remote_conference->user_container);
								}
								break;
								default:
								{
									ast_debug(LOG_DEBUG, "[WL] INVALID STATUS");
								}
								break;
							}
						}
						break;
						case CEI_CONFERENCE_REMOTE:
						{
							// Ignore invalid conference
							// Don't add myself as my remote conf
							if (strcmp(conf->conference_id, conference_id) == 0) {
								ast_debug(LOG_DEBUG, "[TK] Ignore my own conference");
								ast_debug(LOG_DEBUG, "[TK] %s@%s", cid_number, conference_id);
								flag_ignored_conf = TRUE;
								continue;
							}

							/***
							get or create the remote conf from host conf's remote container;
							the remote conf is basically this cei msg come FROM
							***/

							remote_conference = get_remote_conf_by_id(conf, conference_id);
							if (atoi(entity_status) == CEI_STATUS_CONNECTED) {
								if (remote_conference != NULL) {
									ast_debug(LOG_DEBUG, "[TK] CEI_EVENT_MCB_USER_* event");
									continue;
								}

							} else if (atoi(entity_status) == CEI_STATUS_DISCONNECTED) {
								// CEI_EVENT_MCB_CONF_PART
								if (remote_conference != NULL) {
									delete_remote_conf_by_id(conf, publisher_id);
									poll_cei_conference(conf, data_string);
								}
							}
						}
						break;
						default:
						{
							ast_debug(LOG_DEBUG, "[WL] INVALID TYPE");
						}
					}
				}
			}
		}
		break;
		default:
		{
			ast_debug(LOG_DEBUG, "[WL] INVALID TYPE");
		}
	}
	ast_log(LOG_DEBUG, "[TK] Exit...%s\n", __FUNCTION__);

	return;
}

static int get_conf_max_capability(void)
{
	// Get device model
	char devModel[16] = {0};
	ast_safe_nvram_get(":dev_alias", devModel, sizeof(devModel));

	// Get max user count for conference according to system
	int maxUserCount = 0;
	if (strcmp(devModel, "UCM6301") == 0) {
		maxUserCount = 75;
	} else if (strcmp(devModel, "UCM6302") == 0) {
		maxUserCount = 150;
	} else if ( strcmp(devModel, "UCM6304") == 0 ) {
		maxUserCount = 200;
	} else {
		maxUserCount = 300;
	}

	ast_copy_string(dev_info, devModel, sizeof(dev_info));

	return maxUserCount;
}

void conf_get_ucm_dec_info(char *buffer, int buffer_size)
{
	// Get device model
	memset(buffer, 0, buffer_size);
	ast_copy_string(buffer, dev_info, buffer_size > sizeof(dev_info) ? sizeof(dev_info) : buffer_size);
}

static int check_role_as_conference(struct cei_user *user)
{
		if (user == NULL) {
			return 0;
		}

		if (user->user_role == CEI_ROLE_CONFERENCE) {
			return 1;
		} else {
			return 0;
		}
}

#if 0
static void reset_conference(struct cei_conference *conf)
{
	if (conf == NULL) {
		return;
	}

	ao2_lock(conf);
	if (ast_atomic_dec_and_test(&conf->refcount)) {
		//reset conf
		conf->locked = FALSE;
		conf->muted = FALSE;
		conf->version = 0;
		conf->version_reset = 0;
		conf->stop_propagating = FALSE;
		ast_atomic_fetchadd_int(&conf->refcount, 1);
		if (conf->web_page) {
			ast_free(conf->web_page);
			conf->web_page = NULL;
		}
		memset(conf->start_time, 0, sizeof(conf->start_time));
		if (conf->feature_id) {
			ast_free(conf->feature_id);
			conf->feature_id = NULL;
		}
		if (conf->subject) {
			ast_free(conf->subject);
			conf->subject = NULL;
		}
		if (conf->conf_config) {
			ast_free(conf->conf_config);
			conf->conf_config = NULL;
		}
	} else {
		ast_atomic_fetchadd_int(&conf->refcount, 1);
	}
	ao2_unlock(conf);
}
#endif

static void remove_conference(struct cei_conference *conf)
{
	if (conf == NULL) {
		ast_debug(LOG_DEBUG, "[WL] cei_conference conf null！");
		return;
	}

	AST_LIST_LOCK(&confs);
	ast_debug(LOG_DEBUG, "[WL] confernece %s conf->refcount %d", conf->conference_id, conf->refcount);
	/*when conf->refcount=1 we need remove conference, because build conference conf->refcount init val is 2*/
	if (ast_atomic_dec_and_test(&conf->refcount)) {
		ast_debug(LOG_DEBUG, "[WL] do conf_free now!");
		cei_sche_conf_user_remove(conf->conference_num);
		conf_free(conf);
	} else {
		ast_atomic_fetchadd_int(&conf->refcount, 1);
	}
	AST_LIST_UNLOCK(&confs);

}

void remove_virtual_conference(struct cei_conference *conf)
{
	if (conf == NULL) {
		ast_debug(LOG_DEBUG, "[WL] cei_conference conf null！");
		return;
	}

	AST_LIST_LOCK(&confs);
	ast_debug(LOG_DEBUG, "[WL] confernece %s conf->refcount %d", conf->conference_id, conf->refcount);
	ast_debug(LOG_DEBUG, "[WL] do conf_free now!");
	cei_sche_conf_user_remove(conf->conference_num);
	conf_free(conf);

	AST_LIST_UNLOCK(&confs);

}

static int cei_sharing_user_update_event(struct cei_conference *conf, struct cei_user *user)
{
	RAII_VAR(struct ast_json *, json_object, NULL, ast_json_unref);
	struct ast_str *full_text = ast_str_create(CEI_EVENT_DATA_LEN);
	struct ast_str *partial_text = ast_str_create(CEI_EVENT_DATA_LEN);

	poll_cei_conference(conf, &full_text);
	poll_cei_partial_notify(conf, NULL, user, &partial_text, CEI_EVENT_SHARING_UPDATE);

	json_object = ast_json_pack("{s: s, s: i, s: s, s: s, s: s}",
			"conference", conf->conference_num,
			"version_reset", conf->version_reset,
			"full_text", ast_str_buffer(full_text),
			"partial_text", ast_str_buffer(partial_text),
			"event_type", CEI_NOTIFY_EVENT_SHARING_UPDATE
	);

	cei_publish_manager_event(json_object);

	ast_free(full_text);
	ast_free(partial_text);
	return 0;
}

int conf_cei_add_sharing_user(const char *channel_name, const char *conf_number, int share_audio)
{
	struct cei_conference *conf = NULL;
	RAII_VAR(struct cei_user *, user, NULL, ao2_cleanup);

	conf = get_conf_by_num(conf_number);
	if (!conf) {
		return -1;
	}

	user = get_user_by_channel_name(conf, channel_name, 1);
	if (!user) {
		/*-1 for using get_conf_by_num or build_conf_by_num*/
		ast_atomic_fetchadd_int(&conf->refcount, -1);
		ast_debug(LOG_DEBUG, "conf_cei_add_sharing_user leave refcount %d", conf->refcount);
		return -1;
	}

	ao2_lock(conf);
	user->sharing = 1;
	user->share_audio = share_audio;
	if (conf->sharing_user == user) {
		ast_log(LOG_DEBUG, "Sharing user '%s' is the same as before in the cei conference '%s', don't need update.\n",
			conf->sharing_user->channel_name, conf_number);
		ast_atomic_fetchadd_int(&conf->refcount, -1);
		ao2_unlock(conf);
		return 0;
	}
	if (conf->sharing_user) {
		ast_log(LOG_WARNING, "Sharing user '%s' exists in the cei conference '%s', remove it first.\n",
			conf->sharing_user->channel_name, conf_number);
		conf->sharing_user->sharing = 0;
		conf->sharing_user->share_audio = 0;
		ao2_ref(conf->sharing_user, -1);
		conf->sharing_user = NULL;
	}
	conf->sharing_user = ao2_bump(user);
	/*-1 for using get_conf_by_num or build_conf_by_num*/
	ast_atomic_fetchadd_int(&conf->refcount, -1);
	ast_debug(LOG_DEBUG, "conf_cei_add_sharing_user leave refcount %d", conf->refcount);
	ao2_unlock(conf);

	cei_sharing_user_update_event(conf, user);

	return 0;
}

int conf_cei_delete_sharing_user(const char *channel_name, const char *conf_number)
{
	struct cei_conference *conf = NULL;
	struct cei_user *user = NULL;

	conf = get_conf_by_num(conf_number);
	if (!conf) {
		return -1;
	}

	ao2_lock(conf);
	if (conf->sharing_user && !strcmp(conf->sharing_user->channel_name, channel_name)) {
		user = ao2_bump(conf->sharing_user);
		conf->sharing_user->sharing = 0;
		conf->sharing_user->share_audio = 0;
		ao2_ref(conf->sharing_user, -1);
		conf->sharing_user = NULL;
	}
	/*-1 for using get_conf_by_num or build_conf_by_num*/
	ast_atomic_fetchadd_int(&conf->refcount, -1);
	ast_debug(LOG_DEBUG, "conf_cei_delete_sharing_user leave refcount %d", conf->refcount);
	ao2_unlock(conf);

	if (user) {
		cei_sharing_user_update_event(conf, user);
		ao2_ref(user, -1);
	}

	return 0;
}

int conf_cei_video_resolution_update(const char *channel_name, const char *conf_number, int resolution)
{
	RAII_VAR(struct ast_json *, json_object, NULL, ast_json_unref);
	struct ast_str *full_text = ast_str_create(CEI_EVENT_DATA_LEN);
	struct ast_str *partial_text = ast_str_create(CEI_EVENT_DATA_LEN);
	struct cei_conference *conf = NULL;
	RAII_VAR(struct cei_user *, user, NULL, ao2_cleanup);

	conf = get_conf_by_num(conf_number);
	if (!conf) {
		ast_free(full_text);
		ast_free(partial_text);
		return -1;
	}

	ao2_lock(conf->user_container);
	user = get_user_by_channel_name(conf, channel_name, 1);
	if (user) {
		user->media.video_info.resolution = resolution;
	} else {
		ast_debug(LOG_WARNING, "[WL] ERROR, CANNOT FIND USER NUMBER %s IN CONFERENCE %s", channel_name, conf->conference_num);
		ao2_unlock(conf->user_container);
		ast_free(full_text);
		ast_free(partial_text);
		/*-1 for using get_conf_by_num or build_conf_by_num*/
		ast_atomic_fetchadd_int(&conf->refcount, -1);
		ast_debug(LOG_DEBUG, "conf_cei_video_resolution_update leave refcount %d", conf->refcount);
		return -1;
	}
	ao2_unlock(conf->user_container);

	poll_cei_conference(conf, &full_text);
	poll_cei_partial_notify(conf, NULL, user, &partial_text, CEI_EVENT_RESOLUTION_UPDATE);
	json_object = ast_json_pack("{s: s, s: i, s: s, s: s, s: s}",
			"conference", conf->conference_num,
			"version_reset", conf->version_reset,
			"full_text", ast_str_buffer(full_text),
			"partial_text", ast_str_buffer(partial_text),
			"event_type", CEI_NOTIFY_EVENT_RESOLUTION_UPDATE
	);

	cei_publish_manager_event(json_object);
	/*-1 for using get_conf_by_num or build_conf_by_num*/
	ast_atomic_fetchadd_int(&conf->refcount, -1);
	ast_debug(LOG_DEBUG, "conf_cei_video_resolution_update leave refcount %d", conf->refcount);

	ast_free(full_text);
	ast_free(partial_text);

	return 0;
}

int conf_cei_media_update(const char *channel_name, const char *conf_number, int is_video)
{
	RAII_VAR(struct ast_json *, json_object, NULL, ast_json_unref);
	struct ast_str *full_text = ast_str_create(CEI_EVENT_DATA_LEN);
	struct ast_str *partial_text = ast_str_create(CEI_EVENT_DATA_LEN);
	struct cei_conference *conf = NULL;
	RAII_VAR(struct cei_user *, user, NULL, ao2_cleanup);


	conf = get_conf_by_num(conf_number);
	if (!conf) {
		ast_free(full_text);
		ast_free(partial_text);
		return -1;
	}

	ao2_lock(conf->user_container);
	user = get_user_by_channel_name(conf, channel_name, 1);
	if (user) {
		user->media.type = is_video ? AST_MEDIA_TYPE_VIDEO : AST_MEDIA_TYPE_AUDIO;
	} else {
		ast_debug(LOG_WARNING, "[WL] ERROR, CANNOT FIND USER NUMBER %s IN CONFERENCE %s", channel_name, conf->conference_num);
		ao2_unlock(conf->user_container);
		ast_free(full_text);
		ast_free(partial_text);
		/*-1 for using get_conf_by_num or build_conf_by_num*/
		ast_atomic_fetchadd_int(&conf->refcount, -1);
		ast_debug(LOG_DEBUG, "conf_cei_media_update leave refcount %d", conf->refcount);
		return -1;
	}
	ao2_unlock(conf->user_container);

	if (is_video) {
		cei_get_msid(user->channel_name, user->msid, sizeof(user->msid)-1);
		cei_update_media_info(user, conf_number);
	}

	poll_cei_conference(conf, &full_text);
	poll_cei_partial_notify(conf, NULL, user, &partial_text, CEI_EVENT_MEDIA_UPDATE);
	json_object = ast_json_pack("{s: s, s: i, s: s, s: s, s: s}",
			"conference", conf->conference_num,
			"version_reset", conf->version_reset,
			"full_text", ast_str_buffer(full_text),
			"partial_text", ast_str_buffer(partial_text),
			"event_type", CEI_NOTIFY_EVENT_MEDIA_UPDATE
	);

	cei_publish_manager_event(json_object);
	/*-1 for using get_conf_by_num or build_conf_by_num*/
	ast_atomic_fetchadd_int(&conf->refcount, -1);
	ast_debug(LOG_DEBUG, "conf_cei_media_update leave refcount %d", conf->refcount);

	ast_free(full_text);
	ast_free(partial_text);

	return 0;
}


#endif
